#ifndef __XDXGPU_DRM_H__
#define __XDXGPU_DRM_H__

#if defined(__KERNEL__)
#include <drm/drm.h>
#else
#include <drm.h>
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* dc supported drm_property */
enum drm_xdx_degamma_mode {
	XDX_DEGAMMA_DISABLE = 0,
	XDX_DEGAMMA_BT709 = 1,
	XDX_DEGAMMA_BT2020 = 2,
};

enum drm_xdx_sync_dc_mode {
	XDX_SINGLE_DC = 0,
	XDX_MULTI_DC_PRIMARY = 1,
	XDX_MULTI_DC_SECONDARY = 2,
};

enum drm_xdx_mmu_prefetch_mode {
	XDX_MMU_PREFETCH_DISABLE = 0,
	XDX_MMU_PREFETCH_ENABLE = 1,
};

struct drm_xdx_watermark {
	__u32 watermark;
	__u8 qos_low;
	__u8 qos_high;
};

struct drm_xdx_color_mgmt {
	__u32 colorkey;
	__u32 colorkey_high;
	__u32 clear_value;
	bool clear_enable;
	bool transparency;
};

struct drm_xdx_roi {
	bool enable;
	__u16 roi_x;
	__u16 roi_y;
	__u16 roi_w;
	__u16 roi_h;
};

struct drm_pvr_srvkm_cmd {
	__u32 bridge_id;
	__u32 bridge_func_id;
	__u64 in_data_ptr;
	__u64 out_data_ptr;
	__u32 in_data_size;
	__u32 out_data_size;
};

struct pvr_sync_rename_ioctl_data {
	char szName[32];
};

struct pvr_sw_sync_create_fence_data {
	char name[32];
	__s32 fence;
	__u32 pad;
	__u64 sync_pt_idx;
};

struct pvr_sw_timeline_advance_data {
	__u64 sync_pt_idx;
};

#define PVR_SRVKM_SERVICES_INIT  1
#define PVR_SRVKM_SYNC_INIT 2
struct drm_pvr_srvkm_init_data {
	__u32 init_module;
    __u32 _pad;
};

struct drm_xdxgpu_gem_query_in {
	/** the GEM object handle */
	__u32 handle;
	__u32 _pad;
};

struct drm_xdxgpu_gem_query_out {
	__u64 size;
	/** mmap offset from the vma offset manager */
	__u64 addr_ptr;
};

union drm_xdxgpu_gem_query {
	struct drm_xdxgpu_gem_query_in   in;
	struct drm_xdxgpu_gem_query_out out;
};

enum xdxgpu_gem_mapping_flag {
	/* specifies that the memory may be read by the GPU */
	XDXGPU_GEM_MAPPING_GPU_READABLE				= (1 << 0),
	/* specifies that the memory may be written by the GPU */
	XDXGPU_GEM_MAPPING_GPU_WRITEABLE			= (1 << 1),
	/* indicates whether an allocation can be mapped as GPU readable in another GPU memory context. */
	XDXGPU_GEM_MAPPING_GPU_READ_PERMITTED		= (1 << 2),
	/* indicates whether an allocation can be mapped as GPU writable in another GPU memory context. */
	XDXGPU_GEM_MAPPING_GPU_WRITE_PERMITTED		= (1 << 3),
	/* indicates that an allocation is mapped as readable to the CPU. */
	XDXGPU_GEM_MAPPING_CPU_READABLE				= (1 << 4),
	/* indicates that an allocation is mapped as writable to the CPU */
	XDXGPU_GEM_MAPPING_CPU_WRITEABLE			= (1 << 5),
	/* indicates whether an allocation can be mapped as CPU readable in another CPU memory context */
	XDXGPU_GEM_MAPPING_CPU_READ_PERMITTED		= (1 << 6),
	/* indicates whether an allocation can be mapped as CPU writable in another CPU memory context. */
	XDXGPU_GEM_MAPPING_CPU_WRITE_PERMITTED		= (1 << 7),
};

#define XDXGPU_GEM_MAPPING_GPU_RW (XDXGPU_GEM_MAPPING_GPU_READABLE | XDXGPU_GEM_MAPPING_GPU_WRITEABLE)
#define XDXGPU_GEM_MAPPING_CPU_RW (XDXGPU_GEM_MAPPING_CPU_READABLE | XDXGPU_GEM_MAPPING_CPU_WRITEABLE)

enum xdxgpu_gem_gpu_cache {
	/**
	 * GPU domain. Use write combiner (if supported) to combine sequential writes
	 * together to reduce memory access by doing burst writes.
	 */
	XDXGPU_GEM_GPU_UNCACHED_WC = 0,
	/**
	 * GPU domain. Flag indicating uncached memory. This means that any writes to memory
	 * allocated with this flag are written straight to memory and thus are
	 * coherent for any device in the system.
	 * */
	XDXGPU_GEM_GPU_UNCACHED,
	/**
	 * The allocation will be cached. The GPU cache is snooping the CPU cache.
	 * If coherency is not supported the caller is responsible to ensure
	 * the caches are up to date.
	 */
	XDXGPU_GEM_GPU_CACHE_COHERENT,
	/**
	 * Request cached memory, but not coherent (i.e. no cache
	 * snooping). drivers will flush the GPU internal caches after every GPU
	 * task so no cache maintenance requests from the users are necessary.
	 */
	XDXGPU_GEM_GPU_CACHE_INCOHERENT
};

enum xdxgpu_gem_cpu_cache {
	/**
	 * Don't cache read accesses, but use write combiner (if supported)
	 * to combine sequential writes together to reduce memory access by
	 * doing burst writes.
	 */
	XDXGPU_GEM_CPU_UNCACHED_WC = 0,
	/**
	 * CPU domain. Flag indicating uncached memory. This means that any writes to memory
	 * allocated with this flag are written straight to memory and thus are
	 * coherent for any device in the system.
	 * */
	XDXGPU_GEM_CPU_UNCACHED,
	/**
	 * CPU domain. This flag affects the CPU MMU protection flags.
	 * The allocation will be cached.
	 * driver will try to set the coherent bit in the CPU MMU tables so the
	 * CPU cache is snooping the GPU cache. If coherency is not supported the
	 * caller is responsible to ensure the caches are up to date.
	 */
	XDXGPU_GEM_CPU_CACHE_COHERENT,
	/**
	 * CPU domain. Request cached memory, but not coherent (i.e. no cache
	 * snooping). This means that if the allocation needs to transition from
	 * one device to another services has to be informed so it can
	 * flush/invalidate the appropriate caches.
	 */
	XDXGPU_GEM_CPU_CACHE_INCOHERENT,
};

enum xdxgpu_gem_alloc_flags {
	/* clear alloction with 0 when allocated */
	XDXGPU_GEM_ALLOC_CLEARED			= (1 << 0),
	/**
	 * indicate that physical pages of the allocation should be
	 * contiguous
	 * */
	XDXGPU_GEM_ALLOC_CONTIGUOUS			= (1 << 1),
	XDXGPU_GEM_ALLOC_VPU				= (1 << 2),
};

enum xdxgpu_gem_placement {
	XDXGPU_PL_VRAM = 0, /* CPU visable VRAM */
	XDXGPU_PL_PRIV, /* CPU invisable VRAM */
	XDXGPU_PL_TT, /* GPU visable system memory */
	XDXGPU_PL_SYSTEM /* GPU invisable system memory */
};

struct drm_xdxgpu_gem_create_in {
	/** the requested memory size */
	__u64 bo_size;
	/** physical start_addr alignment in bytes for some HW requirements */
	__u64 alignment;
	enum xdxgpu_gem_placement placement;
	enum xdxgpu_gem_gpu_cache gpu_cache;
	enum xdxgpu_gem_cpu_cache cpu_cache;
	/** the mapping flags of GPU and CPU domain */
	uint32_t mapping;
	/** alloc flags */
	uint32_t flags;
};

struct drm_xdxgpu_gem_create_out {
	/** returned GEM object handle */
	__u32 handle;
	__u32 _pad;
};

union drm_xdxgpu_gem_create {
	struct drm_xdxgpu_gem_create_in in;
	struct drm_xdxgpu_gem_create_out out;
};

struct drm_xdxgpu_gem_export_to_pvr_in {
	/* gem object handle */
	__u32 handle;
	__u32 _pad;
};

struct drm_xdxgpu_gem_export_to_pvr_out {
	/* pvr object handle */
	__u32 handle;
	__u32 _pad;
};

union drm_xdxgpu_gem_export_to_pvr {
	struct drm_xdxgpu_gem_export_to_pvr_in in;
	struct drm_xdxgpu_gem_export_to_pvr_out out;
};

struct drm_xdxgpu_gem_import_from_pvr_in {
	/* pvr object handle */
	__u32 handle;
	__u32 _pad;
};

struct drm_xdxgpu_gem_import_from_pvr_out {
	/* gem object handle */
	__u32 handle;
	__u32 _pad;
};

union drm_xdxgpu_gem_import_from_pvr {
	struct drm_xdxgpu_gem_import_from_pvr_in in;
	struct drm_xdxgpu_gem_import_from_pvr_out out;
};

struct drm_xdxgpu_connection_query_out {
	__u32 connection_id;
	__u32 _pad;
};

union drm_xdxgpu_connection_query {
	struct drm_xdxgpu_connection_query_out out;
};

struct drm_xdxgpu_info_device {
	__u32 asic_id;
	__u32 _pad;
};

/* Input structure for the INFO ioctl */
struct drm_xdxgpu_info {
	/* Where the return value will be stored */
	__u64 return_pointer;
	/* The size of the return value. Just like "size" in "snprintf",
	 * it limits how many bytes the kernel can write. */
	__u32 return_size;
	/* The query request id. */
	__u32 query;
};

#if defined(SUPPORT_XDXGPU_FBC)
struct drm_xdxgpu_fbc_buffer_get {
       /* On-screen buffer handler */
       __u32 fb_handle;
       /* The width of on-screen buffer */
       __u32 fb_width;
       /* The height of on-screen buffer */
       __u32 fb_height;
       /* The fourcc of on-screen buffer */
       __u32 fb_fourcc;
};

struct drm_xdxgpu_fbc_buffer_put {
       /* On-screen buffer handler */
       __u32 fb_handle;
};
#endif

/* query id definition */
#define XDXGPU_INFO_DEVICE_INFO		0x00
#define XDXGPU_INFO_PROCESS_INFO	0x01
#define XDXGPU_INFO_UTIL_INFO   	0x02

/* Values used to configure the PVRSRV_DEVICE_INIT_MODE tunable (Linux-only) */
#define PVRSRV_LINUX_DEV_INIT_ON_PROBE   1
#define PVRSRV_LINUX_DEV_INIT_ON_OPEN    2
#define PVRSRV_LINUX_DEV_INIT_ON_CONNECT 3

/* PVR Services command */
#define DRM_PVR_SRVKM_CMD                   0

/* PVR Sync commands */
#define DRM_PVR_SYNC_RENAME_CMD             1
#define DRM_PVR_SYNC_FORCE_SW_ONLY_CMD      2

/* PVR Software Sync commands */
#define DRM_PVR_SW_SYNC_CREATE_FENCE_CMD    3
#define DRM_PVR_SW_SYNC_INC_CMD             4

/* PVR Services Render Device Init command */
#define DRM_PVR_SRVKM_INIT                  5

/* create GEM buffer object */
#define DRM_XDXGPU_GEM_CREATE               6

/* query mmap offset from the vma offset manager */
#define DRM_XDXGPU_GEM_QUERY                7

/* export gem handle to pvr handle */
#define DRM_XDXGPU_EXPORT_TO_PVR_HANDLE     8

/* import gem handle from pvr handle */
#define DRM_XDXGPU_IMPORT_FROME_PVR_HANDLE  9

/* query connection information */
#define DRM_XDXGPU_CONNECTION_QUERY		10

/* query driver or device information */
#define DRM_XDXGPU_INFO_QUERY			11

/* Temporary workaround for gpu driver information query,
 * due to DRM_XDXGPU_INFO_QUERY binded to device-only function.
 */
#define DRM_XDXGPU_INFO			12

#if defined(SUPPORT_XDXGPU_FBC)
/* define xdxFBC IOCTL */
#define DRM_XDXGPU_FBC_BASE                     20
#define DRM_XDXGPU_FBC_BUFFER_GET               DRM_XDXGPU_FBC_BASE + 0
#define DRM_XDXGPU_FBC_BUFFER_PUT               DRM_XDXGPU_FBC_BASE + 1
#endif

#define DRM_XDXGPU_VPU_BASE			30
#define DRM_XDXGPU_VPU_GEM_CLOSE		DRM_XDXGPU_VPU_BASE + 1
#define DRM_XDXGPU_VPU_GEM_FLINK		DRM_XDXGPU_VPU_BASE + 2
#define DRM_XDXGPU_VPU_GEM_OPEN			DRM_XDXGPU_VPU_BASE + 3
#define DRM_XDXGPU_VPU_PRIME_HANDLE_TO_FD	DRM_XDXGPU_VPU_BASE + 4
#define DRM_XDXGPU_VPU_PRIME_FD_TO_HANDLE	DRM_XDXGPU_VPU_BASE + 5
#define DRM_XDXGPU_VPU_MODE_ADDFB		DRM_XDXGPU_VPU_BASE + 6
#define DRM_XDXGPU_VPU_MODE_ADDFB2		DRM_XDXGPU_VPU_BASE + 7
#define DRM_XDXGPU_VPU_MODE_CREATE_DUMB		DRM_XDXGPU_VPU_BASE + 8
#define DRM_XDXGPU_VPU_MODE_MAP_DUMB		DRM_XDXGPU_VPU_BASE + 9
#define DRM_XDXGPU_VPU_MODE_DESTROY_DUMB	DRM_XDXGPU_VPU_BASE + 10

#define DRM_XDXGPU_VPU_HANTRO_TESTCMD		DRM_XDXGPU_VPU_BASE + 11
#define DRM_XDXGPU_VPU_HANTRO_GETPADDR		DRM_XDXGPU_VPU_BASE + 12
#define DRM_XDXGPU_VPU_HANTRO_HWCFG		DRM_XDXGPU_VPU_BASE + 13
#define DRM_XDXGPU_VPU_HANTRO_TESTREADY		DRM_XDXGPU_VPU_BASE + 14
#define DRM_XDXGPU_VPU_HANTRO_SETDOMAIN		DRM_XDXGPU_VPU_BASE + 15
#define DRM_XDXGPU_VPU_HANTRO_ACQUIREBUF	DRM_XDXGPU_VPU_BASE + 16
#define DRM_XDXGPU_VPU_HANTRO_RELEASEBUF	DRM_XDXGPU_VPU_BASE + 17
#define DRM_XDXGPU_VPU_HANTRO_GETPRIMEADDR	DRM_XDXGPU_VPU_BASE + 18
#define DRM_XDXGPU_VPU_HANTRO_PTR_PHYADDR	DRM_XDXGPU_VPU_BASE + 19
#define DRM_XDXGPU_VPU_HANTRO_QUERY_METADATA	DRM_XDXGPU_VPU_BASE + 20
#define DRM_XDXGPU_VPU_HANTRO_UPDATE_METADATA	DRM_XDXGPU_VPU_BASE + 21
#define DRM_XDXGPU_VPU_HANTRO_GET_SLICENUM	DRM_XDXGPU_VPU_BASE + 22
#define DRM_XDXGPU_VPU_HANTRO_GET_VCMDSUP	DRM_XDXGPU_VPU_BASE + 23
#define DRM_XDXGPU_VPU_HANTRO_CODEC		DRM_XDXGPU_VPU_BASE + 24

/* These defines must be prefixed with "DRM_IOCTL_". */
#define DRM_IOCTL_PVR_SRVKM_CMD				DRM_IOWR(DRM_COMMAND_BASE + DRM_PVR_SRVKM_CMD, struct drm_pvr_srvkm_cmd)
#define DRM_IOCTL_PVR_SYNC_RENAME_CMD			DRM_IOW(DRM_COMMAND_BASE + DRM_PVR_SYNC_RENAME_CMD, struct pvr_sync_rename_ioctl_data)
#define DRM_IOCTL_PVR_SYNC_FORCE_SW_ONLY_CMD		DRM_IO(DRM_COMMAND_BASE + DRM_PVR_SYNC_FORCE_SW_ONLY_CMD)
#define DRM_IOCTL_PVR_SW_SYNC_CREATE_FENCE_CMD		DRM_IOWR(DRM_COMMAND_BASE + DRM_PVR_SW_SYNC_CREATE_FENCE_CMD, struct pvr_sw_sync_create_fence_data)
#define DRM_IOCTL_PVR_SW_SYNC_INC_CMD			DRM_IOR(DRM_COMMAND_BASE + DRM_PVR_SW_SYNC_INC_CMD, struct pvr_sw_timeline_advance_data)
#define DRM_IOCTL_PVR_SRVKM_INIT			DRM_IOW(DRM_COMMAND_BASE + DRM_PVR_SRVKM_INIT, struct drm_pvr_srvkm_init_data)
#define DRM_IOCTL_XDXGPU_GEM_CREATE			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_GEM_CREATE, union drm_xdxgpu_gem_create)
#define DRM_IOCTL_XDXGPU_GEM_QUERY			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_GEM_QUERY, union drm_xdxgpu_gem_query)
#define DRM_IOCTL_XDXGPU_EXPORT_TO_PVR_HANDLE		DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_EXPORT_TO_PVR_HANDLE, union drm_xdxgpu_gem_export_to_pvr)
#define DRM_IOCTL_XDXGPU_IMPORT_FROME_PVR_HANDLE	DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_IMPORT_FROME_PVR_HANDLE, union drm_xdxgpu_gem_import_from_pvr)
#define DRM_IOCTL_XDXGPU_CONNECTION_QUERY		DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_CONNECTION_QUERY, union drm_xdxgpu_connection_query)
#define DRM_IOCTL_XDXGPU_INFO_QUERY			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_INFO_QUERY, struct drm_xdxgpu_info)
#define DRM_IOCTL_XDXGPU_INFO			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_INFO, struct drm_xdxgpu_info)

#if defined(SUPPORT_XDXGPU_FBC)
#define DRM_IOCTL_XDXGPU_FBC_BUFFER_GET			DRM_IOR(DRM_COMMAND_BASE + DRM_XDXGPU_FBC_BUFFER_GET, struct drm_xdxgpu_fbc_buffer_get)
#define DRM_IOCTL_XDXGPU_FBC_BUFFER_PUT			DRM_IOW(DRM_COMMAND_BASE + DRM_XDXGPU_FBC_BUFFER_PUT, struct drm_xdxgpu_fbc_buffer_put)
#endif

#define DRM_IOCTL_XDXGPU_VPU_PRIME_HANDLE_TO_FD				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_PRIME_HANDLE_TO_FD, struct drm_prime_handle)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_TESTCMD				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_TESTCMD, unsigned int)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_GETPADDR				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_GETPADDR, struct hantro_addrmap)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_HWCFG                               DRM_IO(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_HWCFG)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_TESTREADY				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_TESTREADY, struct hantro_fencecheck)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_SETDOMAIN				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_SETDOMAIN, struct hantro_domainset)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_ACQUIREBUF				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_ACQUIREBUF, struct hantro_acquirebuf)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_RELEASEBUF				DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_RELEASEBUF, struct hantro_releasebuf)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_GETPRIMEADDR			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_GETPRIMEADDR, unsigned long *)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_PTR_PHYADDR                         DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_PTR_PHYADDR, unsigned long *)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_QUERY_METADATA			DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_QUERY_METADATA, struct hantro_metadata_params)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_UPDATE_METADATA                     DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_UPDATE_METADATA, struct hantro_metadata_params)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_GET_SLICENUM                        DRM_IO(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_GET_SLICENUM)
#define DRM_IOCTL_XDXGPU_VPU_HANTRO_GET_VCMDSUP                         DRM_IOWR(DRM_COMMAND_BASE + DRM_XDXGPU_VPU_HANTRO_GET_VCMDSUP, unsigned int *)

#define HANTRO_IOC_MAGIC 'h'
#define HANTRO_IOCTL_START HANTRO_IOC_MAGIC
/* hantro enc related */
#define HANTROENC_IOC_START _IO(HANTRO_IOCTL_START, 17+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTROENC_IOCGHWOFFSET                                                 \
	_IOR(HANTRO_IOCTL_START, 17 + DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define HANTROENC_IOCGHWIOSIZE _IOWR(HANTRO_IOCTL_START, 18+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long *)
#define HANTROENC_IOC_CLI _IO(HANTRO_IOCTL_START, 19+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTROENC_IOC_STI _IO(HANTRO_IOCTL_START, 20+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTROENC_IOCHARDRESET _IO(HANTRO_IOCTL_START, 21+DRM_XDXGPU_VPU_HANTRO_CODEC) /* for debugging */
#define HANTROENC_IOCGSRAMOFFSET                                               \
	_IOR(HANTRO_IOCTL_START, 22+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define HANTROENC_IOCGSRAMEIOSIZE _IOR(HANTRO_IOCTL_START, 23+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned int *)
#define HANTROENC_IOCH_ENC_RESERVE                                             \
	_IOWR(HANTRO_IOCTL_START, 24+DRM_XDXGPU_VPU_HANTRO_CODEC, struct nor32_parameter *)
#define HANTROENC_IOCH_ENC_RELEASE                                             \
	_IOW(HANTRO_IOCTL_START, 25+DRM_XDXGPU_VPU_HANTRO_CODEC, struct nor32_parameter *)
#define HANTROENC_IOCG_CORE_NUM _IO(HANTRO_IOCTL_START, 26+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTROENC_IOCG_CORE_WAIT                                               \
	_IOWR(HANTRO_IOCTL_START, 27+DRM_XDXGPU_VPU_HANTRO_CODEC, struct nor32_parameter *)
#define HANTROENC_IOCG_CORE_INFO                                               \
	_IOR(HANTRO_IOCTL_START, 28+DRM_XDXGPU_VPU_HANTRO_CODEC, SUBSYS_CORE_INFO *)
#define HANTROENC_IOCG_ANYCORE_WAIT                                            \
	_IOR(HANTRO_IOCTL_START, 29+DRM_XDXGPU_VPU_HANTRO_CODEC, CORE_WAIT_OUT *)
#define HANTROENC_IOC_END _IO(HANTRO_IOCTL_START, 32+DRM_XDXGPU_VPU_HANTRO_CODEC)

/* hantro dec related */
#define HANTRODEC_IOC_START _IO(HANTRO_IOCTL_START, 41+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_PP_INSTANCE _IO(HANTRO_IOCTL_START, 41+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_HW_PERFORMANCE _IO(HANTRO_IOCTL_START, 42+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCGHWOFFSET                                                 \
	_IOWR(HANTRO_IOCTL_START, 43+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define HANTRODEC_IOCGHWIOSIZE _IOWR(HANTRO_IOCTL_START, 44+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned int *)
#define HANTRODEC_IOC_CLI _IO(HANTRO_IOCTL_START, 45+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOC_STI _IO(HANTRO_IOCTL_START, 46+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOC_MC_OFFSETS                                               \
	_IOWR(HANTRO_IOCTL_START, 47+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define HANTRODEC_IOC_MC_CORES _IO(HANTRO_IOCTL_START, 48+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCS_DEC_PUSH_REG                                            \
	_IOW(HANTRO_IOCTL_START, 49+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCS_PP_PUSH_REG                                             \
	_IOW(HANTRO_IOCTL_START, 50+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCH_DEC_RESERVE                                             \
	_IOW(HANTRO_IOCTL_START, 51+DRM_XDXGPU_VPU_HANTRO_CODEC, struct nor32_parameter *)
#define HANTRODEC_IOCT_DEC_RELEASE _IO(HANTRO_IOCTL_START, 52+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCQ_PP_RESERVE _IO(HANTRO_IOCTL_START, 53+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCT_PP_RELEASE _IO(HANTRO_IOCTL_START, 54+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCX_DEC_WAIT _IOW(HANTRO_IOCTL_START, 55+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCX_PP_WAIT _IOWR(HANTRO_IOCTL_START, 56+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCS_DEC_PULL_REG                                            \
	_IOWR(HANTRO_IOCTL_START, 57+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCS_PP_PULL_REG                                             \
	_IOWR(HANTRO_IOCTL_START, 58+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCG_CORE_WAIT _IO(HANTRO_IOCTL_START, 59+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOX_ASIC_ID _IO(HANTRO_IOCTL_START, 60+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOCG_CORE_ID                                                 \
	_IOW(HANTRO_IOCTL_START, 61+DRM_XDXGPU_VPU_HANTRO_CODEC, struct nor32_parameter *)
#define HANTRODEC_IOCS_DEC_WRITE_REG                                           \
	_IOW(HANTRO_IOCTL_START, 62+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_IOCS_DEC_READ_REG                                            \
	_IOWR(HANTRO_IOCTL_START, 63+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define HANTRODEC_DEBUG_STATUS _IO(HANTRO_IOCTL_START, 64+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRODEC_IOX_ASIC_BUILD_ID                                            \
	_IOWR(HANTRO_IOCTL_START, 65+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned int *)
#define HANTRODEC_IOX_IOCX_POLL _IOWR(HANTRO_IOCTL_START, 66+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned int *)

#define HANTRODEC_IOC_END _IO(HANTRO_IOCTL_START, 79+DRM_XDXGPU_VPU_HANTRO_CODEC)

/* hantro cache related */
#define HANTROCACHE_IOC_START _IO(HANTRO_IOCTL_START, 80+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define CACHE_IOCGHWOFFSET _IOR(HANTRO_IOCTL_START, 80+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define CACHE_IOCGHWIOSIZE _IO(HANTRO_IOCTL_START, 81+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define CACHE_IOCHARDRESET _IO(HANTRO_IOCTL_START, 82+DRM_XDXGPU_VPU_HANTRO_CODEC) /* debugging tool */
#define CACHE_IOCH_HW_RESERVE _IOW(HANTRO_IOCTL_START, 83+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define CACHE_IOCH_HW_RELEASE _IO(HANTRO_IOCTL_START, 84+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define CACHE_IOCG_CORE_NUM _IO(HANTRO_IOCTL_START, 85+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define CACHE_IOCG_ABORT_WAIT _IO(HANTRO_IOCTL_START, 86+DRM_XDXGPU_VPU_HANTRO_CODEC)
//#define CACHE_IOCGBUFBUSADDRESS _IOR(CACHE_IOC_MAGIC,  87, unsigned long *)
//#define CACHE_IOCGBUFSIZE       _IOR(CACHE_IOC_MAGIC,  88, unsigned int *)
#define HANTROCACHE_IOC_END _IO(HANTRO_IOCTL_START, 89+DRM_XDXGPU_VPU_HANTRO_CODEC)

/* hantro dec400 related */
#define HANTRODEC400_IOC_START _IO(HANTRO_IOCTL_START, 90+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define DEC400_IOCGHWIOSIZE _IO(HANTRO_IOCTL_START, 90+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define DEC400_IOCS_DEC_WRITE_REG                                              \
	_IOW(HANTRO_IOCTL_START, 91+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define DEC400_IOCS_DEC_READ_REG                                               \
	_IOWR(HANTRO_IOCTL_START, 92+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define DEC400_IOCS_DEC_PUSH_REG                                               \
	_IOW(HANTRO_IOCTL_START, 93+DRM_XDXGPU_VPU_HANTRO_CODEC, struct core_desc *)
#define DEC400_IOCGHWOFFSET _IOWR(HANTRO_IOCTL_START, 94+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long long *)
#define HANTRODEC400_IOC_END _IO(HANTRO_IOCTL_START, 99+DRM_XDXGPU_VPU_HANTRO_CODEC)

/* hantro mmu related */
#define HANTROMMU_IOC_START _IO(HANTRO_IOCTL_START, 100+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define HANTRO_IOCS_MMU_MEM_MAP                                                \
	_IOWR(HANTRO_IOCTL_START, 101+DRM_XDXGPU_VPU_HANTRO_CODEC, struct addr_desc *)
#define HANTRO_IOCS_MMU_MEM_UNMAP                                              \
	_IOWR(HANTRO_IOCTL_START, 102+DRM_XDXGPU_VPU_HANTRO_CODEC, struct addr_desc *)
#define HANTRO_IOCS_MMU_FLUSH _IOW(HANTRO_IOCTL_START, 103+DRM_XDXGPU_VPU_HANTRO_CODEC, struct addr_desc *)
#define HANTROMMU_IOC_END _IO(HANTRO_IOCTL_START, 109+DRM_XDXGPU_VPU_HANTRO_CODEC)

#define VCMD_IOC_START _IO(HANTRO_IOCTL_START, 110+DRM_XDXGPU_VPU_HANTRO_CODEC)
#define VCMD_IOCH_GET_CMDBUF_PARAMETER                                         \
	_IOWR(HANTRO_IOCTL_START, 111+DRM_XDXGPU_VPU_HANTRO_CODEC, struct cmdbuf_mem_parameter *)
#define VCMD_IOCH_GET_CMDBUF_POOL_SIZE                                         \
	_IOWR(HANTRO_IOCTL_START, 112+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long)
#define VCMD_IOCH_SET_CMDBUF_POOL_BASE                                         \
	_IOWR(HANTRO_IOCTL_START, 113+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned long)
#define VCMD_IOCH_GET_VCMD_PARAMETER                                           \
	_IOWR(HANTRO_IOCTL_START, 114+DRM_XDXGPU_VPU_HANTRO_CODEC, struct vcmd_cfg_par *)
#define VCMD_IOCH_RESERVE_CMDBUF                                               \
	_IOWR(HANTRO_IOCTL_START, 115+DRM_XDXGPU_VPU_HANTRO_CODEC, struct exchange_parameter *)
#define VCMD_IOCH_LINK_RUN_CMDBUF                                              \
	_IOR(HANTRO_IOCTL_START, 116+DRM_XDXGPU_VPU_HANTRO_CODEC, struct exchange_parameter *)
#define VCMD_IOCH_WAIT_CMDBUF                                                  \
	_IOR(HANTRO_IOCTL_START, 117+DRM_XDXGPU_VPU_HANTRO_CODEC, struct cmdbuf_id_parameter *)
#define VCMD_IOCH_RELEASE_CMDBUF                                               \
	_IOR(HANTRO_IOCTL_START, 118+DRM_XDXGPU_VPU_HANTRO_CODEC, struct cmdbuf_id_parameter *)
#define VCMD_IOCH_POLLING_CMDBUF _IOR(HANTRO_IOCTL_START, 119+DRM_XDXGPU_VPU_HANTRO_CODEC, unsigned int *)
#define VCMD_IOC_END _IO(HANTRO_IOCTL_START, 124+DRM_XDXGPU_VPU_HANTRO_CODEC)

#define HANTRO_IOC_AXIFE_CONFIG                                                \
	_IOR(HANTRO_IOCTL_START, 125+DRM_XDXGPU_VPU_HANTRO_CODEC, struct axife_cfg *)

#define HANTRO_IOCTL_COUNT 125+DRM_XDXGPU_VPU_HANTRO_CODEC                                                
#if defined(__cplusplus)
}
#endif

#endif /* __XDXGPU_DRM_H__ */
