#pragma once
/*****************************************************************************************/
/*                                Include Header Files                                   */
/*****************************************************************************************/
#include <sound/core.h>
#include <linux/pci.h>
#include <sound/pcm.h>
#include <linux/workqueue.h>
#include <sound/initval.h>
#include "../display/output/dp/xdx_dp.h"
#include "../xdx_device.h"
#include "../xdx_irq.h"

/*****************************************************************************************/
/*                                Public Macros                                          */
/*****************************************************************************************/
#define XDX_VAUDIO_VENDOR                   (0x1eed)
#define XDX_VAUDIO_DEVICE                   (0x1101)

#define XDX_AUDIO_BAR4_16M					16*1024*1024
#define XDX_AUDIO_BAR4_1M					1*1024*1024
/*****************************************************************************************/
/*                                Data Structure Define                                  */
/*****************************************************************************************/
enum PANGU_INTERRUPT {
    PANGU_INTERRUPT_H1_START    = 0,
    PANGU_INTERRUPT_VD0MCU 	    = 0,
    PANGU_INTERRUPT_VD1MCU	    = 1,
    PANGU_INTERRUPT_CMCU        = 2,
    PANGU_INTERRUPT_ACPU   	    = 3,
    PANGU_INTERRUPT_RESERVE   	= 4,
    PANGU_INTERRUPT_MAX,
};

typedef enum
{
    PCM_SAMPLEBRATE_32K		= 32000,
    PCM_SAMPLEBRATE_441K	= 44100,
    PCM_SAMPLEBRATE_48K		= 48000,
    PCM_SAMPLEBRATE_64K		= 64000,
    PCM_SAMPLEBRATE_882K	= 88200,
    PCM_SAMPLEBRATE_96K		= 96000,
    PCM_SAMPLEBRATE_128K	= 128000,
    PCM_SAMPLEBRATE_1764K	= 176400,
    PCM_SAMPLEBRATE_192K	= 192000,
    PCM_SAMPLEBRATE_256K	= 256000,
    PCM_SAMPLEBRATE_3528K	= 352800,
    PCM_SAMPLEBRATE_384K	= 384000,
    PCM_SAMPLEBRATE_MAX,
} PCM_SAMPLERATE_MODE;

struct pangu_interrupt_handler {
    bool enabled;
    void (*handler_function)(void *data);
    void *handler_data;
};

typedef enum {
	XDX_AUDIO_MODULE_NULL 		= 0,
	XDX_AUDIO_MODULE_HDMI 		= 1,
	XDX_AUDIO_MODULE_DP   		= 2,
	XDX_AUDIO_MODULE_DOUBLE     	= 3,
} PlayDrv;

typedef enum {
	XDX_AUDIO_MODULE_DISCONNECT = 0,
	XDX_AUDIO_MODULE_CONNECT 	= 1,
} PlayCon;

typedef enum {
	XDX_AUDIO_MODULE_DRV_DISCONNECT = 0,
	XDX_AUDIO_MODULE_DRV_CONNECT 	= 1,
} PlayDisCon;

struct snd_xdx_hdmi_audio {
    struct xdx_device *xdev;
    struct snd_card *card;
    struct pci_dev *pci;
    struct snd_pcm *pcm;
    struct device* dev;
    struct hdmi_dev* hdmi;
    struct dp_dev *dp;
    int irq;
    struct xdx_irq_src *irq_src_audio;
    struct xdx_irq_src *irq_src_vdma1;


    spinlock_t    lock_audio;
    spinlock_t    lock_vdma1ch3;
    spinlock_t    lock;
    struct mutex open_mutex;

    struct pangu_interrupt_handler interrupt_handlers[PANGU_INTERRUPT_MAX];
    struct workqueue_struct *pWorkQueue;
    struct work_struct audioIntWork;
    //AudioAcpuHostCmd_T * audioAcpuHostCmd;
    u32 *audioAcpuHostInt;
    u32 *audioAcpuHostClearInt;

    unsigned long iomem;
    unsigned long irqcount;
    unsigned long inIRQ;
    void __iomem *iobase;
    void __iomem *iobase_audio0;
    void __iomem *iobase_audio1;
    void __iomem *iobase_vdma1;
    void __iomem *iobase_hdmi0;
    void __iomem *iobase_hdmi1;
    void __iomem *iobase_dp0;
    void __iomem *iobase_dp1;
    void __iomem *iobase_acpu;

    u8 *pcmbufferData;
    u8 *periodBufPtr;
    u32 periodBufPhy;
    u8 *tempBufPtr;
    uint32_t periodBufSize;
    struct snd_dma_buffer dma_period;
    struct snd_dma_buffer tempBuf;

    size_t periodsize;
    int channels;
    PCM_SAMPLERATE_MODE rate;
    uint32_t video_type;
    uint32_t bit_width;
    uint32_t sample_rate;
    int currentBuffer;
    uint32_t dmaTag;
    volatile uint32_t pos;
    volatile uint32_t fifo_bytes;
    volatile uint32_t fifo_empty;
    volatile uint32_t status;
    volatile uint32_t dma_pending_cnt;
    volatile uint32_t dma_bytes;
    volatile uint32_t vdma1ch3_finish;

    struct snd_pcm_substream *playback_substream;
    struct snd_pcm_substream *capture_substream;

    pid_t capture_pid;
    pid_t playback_pid;

    //int running;
    //int idleMonitorOn;           // indicates whether the card is in idle monitor mode.
    //u32 cmdRetryCount;           // tracks how many times we have retried sending to the card.

    int opencnt;		     // Open/Close count
    int setcnt;		     // SetupForPlay count
    int playcnt;		     // TriggerPlay count
    int errorcnt;		     // Error Count
    unsigned long totalerrorcnt; // Total Error Count

    PlayDrv  audioif_video_connect;        //0:NULL; 1:hdmi; 2:dp
    PlayDrv  audioif_hdmi_connect;
    PlayDrv  audioif_dp_connect;
    PlayDrv  audioif_double_connect;
    PlayDisCon audioif_dp_disconnect;      //0:connect 1:disconnect
    PlayDisCon audioif_hdmi_disconnect;
    uint32_t flag;
    struct mutex *transfer_mutex;
    struct mutex *hw_mutex;
    int transfer_flag;
    int blkSize;
    uint32_t last_bytes;
    uint32_t first_enable;
    uint32_t buf_ptr;
    uint32_t ptr;
};

/*****************************************************************************************/
/*                                Public Function Statement                              */
/*****************************************************************************************/
int snd_xdx_hdmi_audio_create(struct snd_card *card, struct xdx_device *xdev, struct snd_xdx_hdmi_audio **rchip);
int xdx_hdmi_audio_setRate(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr);
int xdx_audio_drv_hotPlug(struct xdx_device *xdev, PlayDrv module, PlayCon status);
int xdx_audio_resume(struct xdx_device *xdev);
int xdxaudio_isr_install(struct xdx_device *xdev);
int xdxaudio_isr_uninstall(struct xdx_device *xdev);
void pangu_a0_db_interrupt(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr);
