#include "xdx_device.h"
#include "xdx_irq.h"
#include "dm_irq.h"

static int dc_irq_set(struct xdx_device *xdev, struct xdx_irq_src *src,
		      unsigned type, enum xdx_interrupt_state state)
{
	// TODO: enable/disable IRQ
	return -1;
}

static int dc_irq_process(struct xdx_device *xdev, struct xdx_irq_src *src,
			  struct xdx_irq_entry *entry)
{
	struct xdx_dm *dm = xdev_to_dm(xdev);
	struct dm_dc *dc;
	int ret = 0;

	if (src->id == XDX_IRQ_CLIENTID_DC0)
		dc = dm->dc[0];
	else if (src->id == XDX_IRQ_CLIENTID_DC1 && dm->dc_num == 2)
		dc = dm->dc[1];
	else
		return -EINVAL;

	if (dc->funcs->interrupt_handle) {
		dc->funcs->interrupt_handle(dc);
	} else {
		dev_err(xdev->dev, "interrupt handle is null\n");
		ret = -EINVAL;
	}

	return ret;
}

static struct xdx_irq_src_funcs dc_src_funcs = {
	.set = dc_irq_set,
	.process = dc_irq_process,
};

int dm_dc_isr_install(struct dm_dc *dc)
{
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0)) 
        struct drm_device *ddev = xdev_to_drm(xdev);
#endif	
	struct xdx_irq_src *irq_src;

	irq_src = kmalloc(sizeof(*irq_src), GFP_KERNEL);
	if (!irq_src)
		return -ENOMEM;

	if (dc->id == 0)
		irq_src->id = XDX_IRQ_CLIENTID_DC0;
	else if (dc->id == 1 && xdev->asic_type == CHIP_PANGU_B0)
		irq_src->id = XDX_IRQ_CLIENTID_DC1;
	else
		return -EINVAL;

	irq_src->funcs = &dc_src_funcs;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0))
	ddev->irq_enabled = true;
#endif
	xdx_irq_register_src(xdev, irq_src);

	dc->irq_src = irq_src;

	return 0;
}

int dm_dc_isr_uninstall(struct dm_dc *dc)
{
	struct xdx_device *xdev = dm_to_xdev(dc->dm);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0))
        struct drm_device *ddev = xdev_to_drm(xdev);
#endif
	int ret;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0))
	ddev->irq_enabled = false;
#endif
	ret = xdx_irq_unregister_src(xdev, dc->irq_src);
	if (ret) {
		dev_err(xdev->dev, "failed to unregister irq source\n");
		return ret;
	}

	kfree(dc->irq_src);
	dc->irq_src = NULL;

	return 0;
}
