/*
 * Copyright (c) 2016 Synopsys, Inc.
 *
 * Synopsys DP TX Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 *
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 *
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef __XDX_DPTX_AVGEN_H__
#define __XDX_DPTX_AVGEN_H__
#include "pix.h"

#define AUDIO_INFOFREAME_HEADER 0x441B8400
struct dptx;

struct audio_params {
	u8 iec_channel_numcl0;
	u8 iec_channel_numcr0;
	u8 use_lut;
	u8 iec_samp_freq;
	u8 iec_word_length;
	u8 iec_orig_samp_freq;
	u8 data_width;
	u8 num_channels;
	u8 inf_type;
	u8 mute;
	u8 ats_ver;
};

enum audio_sample_freq {
	SAMPLE_FREQ_32 = 0,
	SAMPLE_FREQ_44_1 = 1,
	SAMPLE_FREQ_48 = 2,
	SAMPLE_FREQ_88_2 = 3,
	SAMPLE_FREQ_96 = 4,
	SAMPLE_FREQ_176_4 = 5,
	SAMPLE_FREQ_192 = 6
};

struct audio_short_desc
{
	u8 max_num_of_channels;
	enum audio_sample_freq max_sampling_freq; 
	u8 max_bit_per_sample;
};

void dptx_audio_params_reset(struct audio_params *aparams);
void dptx_audio_config(struct dptx *dptx);
void dptx_audio_core_config(struct dptx *dptx);
void dptx_audio_gen_config(struct dptx *dptx);
void dptx_en_audio_channel(struct dptx *dptx, int ch_num, int enable);
void dptx_audio_mute(struct dptx *dptx);

void dptx_audio_inf_type_change(struct dptx *dptx);
void dptx_audio_num_ch_change(struct dptx *dptx);
void dptx_audio_data_width_change(struct dptx *dptx);

enum pattern_mode {
	TILE = 0,
	RAMP = 1,
	CHESS = 2,
	COLRAMP = 3
};

enum dynamic_range_type {
	CEA = 1,
	VESA = 2
};

enum video_format_type {
	VCEA = 0,
	CVT = 1,
	DMT = 2
};

struct dtd {
	u16 pixel_repetition_input;
	int pixel_clock;
	/** 1 for interlaced, 0 progressive */
	u8 interlaced;
	u16 h_active;
	u16 h_blanking;
	u16 h_image_size;
	u16 h_sync_offset;
	u16 h_sync_pulse_width;
	u8 h_sync_polarity;
	u16 v_active;
	u16 v_blanking;
	u16 v_image_size;
	u16 v_sync_offset;
	u16 v_sync_pulse_width;
	u8 v_sync_polarity;
};

int dptx_dtd_fill(struct dtd *mdtd, u8 code, u32 refresh_rate,
			u8 video_format);
void dwc_dptx_dtd_reset(struct dtd *mdtd);

struct video_params {
	u8 pix_enc;
	u8 pattern_mode;
	struct dtd mdtd;
	u8 mode;
	u8 bpc;
	u8 colorimetry;
	u8 dynamic_range;
	u8 aver_bytes_per_tu;
	u8 aver_bytes_per_tu_frac;
	u8 init_threshold;
	u32 refresh_rate;
	u8 video_format;
	// DSC staff here TODO: move to special DSC struct
	u16 slice_width;
	u16 chunk_size;
	u16 slice_height;
	int encoders;
	u8 first_line_bpg_offset;
	u32 minRateBufferSize;
	u16 dsc_bpp;
	u16 dsc_bpc;
	u32 hrddelay;
	u32 initial_dec_delay;
	u16 initial_scale_value;
	u32 scale_decrement_interval;
};

u8 dptx_bit_field(const u16 data, u8 shift, u8 width);
u16 dptx_concat_bits(u8 bhi, u8 ohi, u8 nhi, u8 blo, u8 olo, u8 nlo);
u16 dptx_byte_to_word(const u8 hi, const u8 lo);
u32 dptx_byte_to_dword(u8 b3, u8 b2, u8 b1, u8 b0);

int dptx_calculate_hblank_interval(struct dptx* dptx);
void dptx_calc_dsc_lsteer_xmit_delay(struct dptx* dptx);
u8 dptx_calculate_dsc_init_threshold(struct dptx* dptx);

void dptx_set_ram_ctr(struct dptx *dptx, int stream);
void dptx_set_ram_data(struct dptx *dptx, u32 value, int count, int stream);
void dptx_set_ramp_pattern(struct dptx *dptx, int column_num, int stream);
void dptx_set_col_ramp_pattern(struct dptx *dptx, int stream);
void dptx_set_col_ycbcr_ramp_pattern(struct dptx *dptx, int stream);

int dptx_dtd_parse(struct dptx *dptx, struct dtd *mdtd, u8 data[18]);
void dptx_video_params_reset(struct dptx *dptx);

void dptx_video_timing_change(struct dptx *dptx, int stream);
int dptx_video_mode_change(struct dptx *dptx, u8 vmode, int stream);
int dptx_video_config(struct dptx *dptx, int stream);
void dptx_video_core_config(struct dptx *dptx, int stream);
void dptx_video_gen_config(struct dptx *dptx, int stream);
void dptx_video_set_core_bpc(struct dptx *dptx, int stream);
void dptx_video_set_gen_bpc(struct dptx *dptx, int stream);
void dptx_video_set_sink_bpc(struct dptx *dptx, int stream);
void dptx_video_set_sink_col(struct dptx *dpx, int stream);
void dptx_video_pattern_change(struct dptx *dptx, int stream);
void dptx_video_bpc_change(struct dptx *dptx, int stream);
void dptx_program_dsc_bits_perpixel(struct dptx* dptx);
void dptx_video_ycc_mapping_change(struct dptx *dptx, int stream);
void dptx_video_ts_change(struct dptx *dptx, int stream);
int dptx_video_ts_calculate(struct dptx *dptx, int lane_num, int rate,
									int bpc, int encoding, int pixel_clock);
void dptx_video_pattern_set(struct dptx *dptx, u8 pattern, int stream);
void dptx_enable_default_video_stream(struct dptx *dptx, int stream);
void dptx_disable_default_video_stream(struct dptx *dptx, int stream);
void dptx_video_reset(struct dptx *dptx, int enable, int stream);
void dptx_audio_samp_freq_config(struct dptx *dptx);
void dptx_audio_infoframe_sdp_send(struct dptx *dptx);
void dptx_vsd_ycbcr420_send(struct dptx *dptx, u8 enable) ;
#endif
