/*
 * Copyright (c) 2016 Synopsys, Inc.
 *
 * Synopsys DP TX Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 *
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 *
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#include "xdx_dp.h"
#include "xdx_dp_accel.h"
#include "core.h"
#include "dm.h"
#include "xdx_device.h"
#include "xdx_aux.h"

#define DPTX_NO_DEBUG_REG
static int dptx_handle_aux_reply(struct dptx *dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 auxsts;
	u32 status;
	u32 auxm;
	u32 br;
	int count;
	count = 0;
	while(1) {
		auxsts = dptx_readl(dptx, DPTX_AUX_STS);

		if (!(auxsts & DPTX_AUX_STS_REPLY_RECEIVED))
			break;

		count ++;
		if (count > 5000)
			return -ETIMEDOUT;

		udelay(1);
	};


	auxsts = dptx_readl(dptx, DPTX_AUX_STS);

	status = (auxsts & DPTX_AUX_STS_STATUS_MASK) >>
		DPTX_AUX_STS_STATUS_SHIFT;

	auxm = (auxsts & DPTX_AUX_STS_AUXM_MASK) >>
		DPTX_AUX_STS_AUXM_SHIFT;
	br = (auxsts & DPTX_AUX_STS_BYTES_READ_MASK) >>
		DPTX_AUX_STS_BYTES_READ_SHIFT;

	//dev_dbg(xdev->dev, "%s: 0x%08x: sts=%d, auxm=%d, br=%d, replyrcvd=%d,
	//		replyerr=%d, timeout=%d, disconn=%d\n",
	//		__func__, auxsts, status, auxm, br,
	//		(auxsts & DPTX_AUX_STS_REPLY_RECEIVED),
	//		(auxsts & DPTX_AUX_STS_REPLY_ERR),
	//		(auxsts & DPTX_AUX_STS_TIMEOUT),
	//		(auxsts & DPTX_AUX_STS_SINK_DWA));

	switch (status) {
	case DPTX_AUX_STS_STATUS_ACK:
		dev_dbg(xdev->dev, "%s: DPTX_AUX_STS_STATUS_ACK\n", __func__);
		break;
	case DPTX_AUX_STS_STATUS_NACK:
		dev_dbg(xdev->dev, "%s: DPTX_AUX_STS_STATUS_NACK\n", __func__);
		break;
	case DPTX_AUX_STS_STATUS_DEFER:
		dev_dbg(xdev->dev, "%s: DPTX_AUX_STS_STATUS_DEFER\n", __func__);
		break;
	case DPTX_AUX_STS_STATUS_I2C_NACK:
		dev_dbg(xdev->dev, "%s: DPTX_AUX_STS_STATUS_I2C_NACK\n",
			__func__);
		break;
	case DPTX_AUX_STS_STATUS_I2C_DEFER:
		dev_dbg(xdev->dev, "%s: DPTX_AUX_STS_STATUS_I2C_DEFER\n",
			__func__);
		break;
	default:
		dev_err(xdev->dev, "Invalid AUX status 0x%x\n", status);
		break;
	}

	dptx->aux.data[0] = dptx_readl(dptx, DPTX_AUX_DATA0);
	//dev_dbg(xdev->dev, "aux.data[0]: 0x%x\n", dptx->aux.data[0]);
	dptx->aux.data[1] = dptx_readl(dptx, DPTX_AUX_DATA1);
	//dev_dbg(xdev->dev, "aux.data[1]: 0x%x\n", dptx->aux.data[1]);
	dptx->aux.data[2] = dptx_readl(dptx, DPTX_AUX_DATA2);
	//dev_dbg(xdev->dev, "aux.data[2]: 0x%x\n", dptx->aux.data[2]);
	dptx->aux.data[3] = dptx_readl(dptx, DPTX_AUX_DATA3);
	//dev_dbg(xdev->dev, "aux.data[3]: 0x%x\n", dptx->aux.data[3]);
	dptx->aux.sts = auxsts;

	return 0;
}

static void dptx_aux_clear_data(struct dptx *dptx)
{
	dptx_writel(dptx, DPTX_AUX_DATA0, 0);
	dptx_writel(dptx, DPTX_AUX_DATA1, 0);
	dptx_writel(dptx, DPTX_AUX_DATA2, 0);
	dptx_writel(dptx, DPTX_AUX_DATA3, 0);
}

static int dptx_aux_read_data(struct dptx *dptx, u8 *bytes, unsigned int len)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	unsigned int i;

	u32 *data = dptx->aux.data;

	for (i = 0; i < len; i++) {
		bytes[i] = (data[i / 4] >> ((i % 4) * 8)) & 0xff;
		dev_dbg(xdev->dev, "bytes[%d]: 0x%x\n", i, bytes[i]);
	}

	return len;
}

static int dptx_aux_write_data(struct dptx *dptx, u8 const *bytes,
				unsigned int len)
{
	unsigned int i;
	u32 data[4];

	memset(data, 0, sizeof(u32) * 4);

	for (i = 0; i < len; i++)
		data[i / 4] |= (bytes[i] << ((i % 4) * 8));

	dptx_writel(dptx, DPTX_AUX_DATA0, data[0]);
	dptx_writel(dptx, DPTX_AUX_DATA1, data[1]);
	dptx_writel(dptx, DPTX_AUX_DATA2, data[2]);
	dptx_writel(dptx, DPTX_AUX_DATA3, data[3]);

	return len;
}

static int dptx_aux_rw(struct dptx *dptx,
			bool rw,
			bool i2c,
			bool mot,
			bool addr_only,
			u32 addr,
			u8 *bytes,
			unsigned int len)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int retval;
	int tries = 0;
	u32 auxcmd;
	u32 type;
	unsigned int status;
	unsigned int br;
	u32 intsts;
#if 0
	u32 count;
#endif

again:
	mdelay(1);
	tries++;
	if (tries > 20)
		return -EINVAL;

	dev_dbg(xdev->dev, "%s: addr=0x%08x, len=%d, try=%d\n", __func__, addr, len, tries);

	if (WARN((len > 16) || (len == 0),
		"AUX read/write len must be 1-15, len=%d\n", len))
		return -EINVAL;

	type = rw ? DPTX_AUX_CMD_TYPE_READ : DPTX_AUX_CMD_TYPE_WRITE;

	if (!i2c)
		type |= DPTX_AUX_CMD_TYPE_NATIVE;

	if (i2c && mot)
		type |= DPTX_AUX_CMD_TYPE_MOT;

	dptx_aux_clear_data(dptx);

    	//dev_dbg(xdev->dev,"rw buffer addr:%llx\n",bytes);
	if (!rw)
		dptx_aux_write_data(dptx, bytes, len);

	auxcmd = ((type << DPTX_AUX_CMD_TYPE_SHIFT) |
			(addr << DPTX_AUX_CMD_ADDR_SHIFT) |
			((len - 1) << DPTX_AUX_CMD_REQ_LEN_SHIFT));

	if (addr_only)
		auxcmd |= DPTX_AUX_CMD_I2C_ADDR_ONLY;

	dptx_writel(dptx, DPTX_AUX_CMD, auxcmd);
#if 0
	count = 0;
	while(1){
		intsts =  dptx_readl(dptx, DPTX_ISTS);
		if(intsts & DPTX_ISTS_AUX_REPLY){
			dptx_writel(dptx, DPTX_AUX_CMD, auxcmd);
			break;
		}
		count ++;
		if (count > 5000)
			goto again;
		udelay(1);
	}
#endif
	retval = dptx_handle_aux_reply(dptx);
	if (retval)
		return retval;

	if (retval == -ETIMEDOUT) {
		dev_dbg(xdev->dev, "AUX timed out\n");
		return retval;
	}

	if (retval == -ESHUTDOWN) {
		dev_dbg(xdev->dev, "AUX aborted on driver shutdown\n");
		return retval;
	}

	if (atomic_read(&dptx->aux.abort)) {
		dev_dbg(xdev->dev, "AUX aborted\n");
		return -ETIMEDOUT;
	}

	status = (dptx->aux.sts & DPTX_AUX_STS_STATUS_MASK) >>
		DPTX_AUX_STS_STATUS_SHIFT;

	br = (dptx->aux.sts & DPTX_AUX_STS_BYTES_READ_MASK) >>
		DPTX_AUX_STS_BYTES_READ_SHIFT;

	switch (status) {
		case DPTX_AUX_STS_STATUS_ACK:
			dev_dbg(xdev->dev, "AUX Success\n");
			if (br == 0) {
				dev_dbg(xdev->dev, "BR=0, Retry\n");
				dptx_soft_reset(dptx, DPTX_SRST_CTRL_AUX);
				goto again;
			}
			break;

		case DPTX_AUX_STS_STATUS_NACK:
		case DPTX_AUX_STS_STATUS_I2C_NACK:
			dev_dbg(xdev->dev, "AUX Nack\n");
			return -EINVAL;

		case DPTX_AUX_STS_STATUS_I2C_DEFER:
		case DPTX_AUX_STS_STATUS_DEFER:
			dev_dbg(xdev->dev, "AUX Defer\n");
			goto again;

		default:
			dev_err(xdev->dev, "AUX Status Invalid\n");
			dptx_soft_reset(dptx, DPTX_SRST_CTRL_AUX);
			goto again;
	}

	intsts = dptx_readl(dptx, DPTX_ISTS);
    dptx_writel(dptx, DPTX_ISTS, intsts);

	if (rw)
		dptx_aux_read_data(dptx, bytes, len);

	udelay(50);
	return 0;
}

int dptx_aux_rw_bytes(struct dptx *dptx,
		      bool rw,
		      bool i2c,
		      u32 addr,
		      u8 *bytes,
		      unsigned int len)
{
	int retval;
	unsigned int i;
	//u8 bytes1[1];

	for (i = 0; i < len; ) {
		unsigned int curlen;

		curlen = min_t(unsigned int, len - i, 16);

		if (!i2c) {
			retval = dptx_aux_rw(dptx, rw, i2c, true, false,
					     addr + i, &bytes[i], curlen);
		}
		else {
			retval = dptx_aux_rw(dptx, rw, i2c, true, false,
					     addr, &bytes[i], curlen);
		}
		if (retval)
			return retval;

		i += curlen;
	}

//	if (i2c & rw) {
//	dev_dbg{xdev->devtx, "something harutk \n ");
//	retval = dptx_aux_rw(dptx, 1, i2c, false, true,
//				addr, &bytes1[0], 1);
//		if (retval)
//	return retval;
//	}
	return retval;
}

int dptx_read_bytes_from_i2c(struct dptx *dptx,
				u32 device_addr,
				u8 *bytes,
				u32 len)
{
	return dptx_aux_rw_bytes(dptx, true, true,
				device_addr, bytes, len);
}

int dptx_i2c_address_only(struct dptx *dptx,
				unsigned int device_addr)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u8 bytes[1];
	dev_dbg(xdev->dev, "something harutk \n ");
	return dptx_aux_rw(dptx, 0, true, false, true,
				device_addr, &bytes[0], 1);
}

int dptx_write_bytes_to_i2c(struct dptx *dptx,
				u32 device_addr,
				u8 *bytes,
				u32 len)
{
	return dptx_aux_rw_bytes(dptx, false, true,
				 device_addr, bytes, len);
}

int __dptx_read_bytes_from_dpcd(struct dptx *dptx,
				u32 reg_addr,
				u8 *bytes,
				u32 len)
{
	return dptx_aux_rw_bytes(dptx, true, false,
				 reg_addr, bytes, len);
}

int __dptx_write_bytes_to_dpcd(struct dptx *dptx,
				u32 reg_addr,
				u8 *bytes,
				u32 len)
{
	return dptx_aux_rw_bytes(dptx, false, false,
				 reg_addr, bytes, len);
}

int __dptx_read_dpcd(struct dptx *dptx, u32 addr, u8 *byte)
{
	return __dptx_read_bytes_from_dpcd(dptx, addr, byte, 1);
}

int __dptx_write_dpcd(struct dptx *dptx, u32 addr, u8 byte)
{
	return __dptx_write_bytes_to_dpcd(dptx, addr, &byte, 1);
}

static int dp_i2c_master_xfer(struct i2c_adapter *adap,
					struct i2c_msg *msgs, int num)
{
	struct dp_dev *dp = i2c_get_adapdata(adap);
	struct xdx_output* output = dp_to_output(dp);
	struct xdx_dm* dm = output_to_dm(output);
	struct xdx_device* xdev = dm_to_xdev(dm);
	int i, j;

	for (i = 0; i < num; i++) {
		struct i2c_msg *msg = &msgs[i];
		dev_dbg(xdev->dev, "[dp msg: %d out of %d][len: %d][type: %s][slave addr: 0x%x][buf0: 0x%x]\n",
			i+1, num, msg->len, (msg->flags & I2C_M_RD) ? "read" : "write", msg->addr, msg->buf[0]);

		// for debug
		if (!(msg->flags & I2C_M_RD)) {
			for (j=0; j< msg->len; j++) {
				dev_dbg(xdev->dev, "tx buf[%d]: 0x%x\n", j, msg->buf[j]);
			}
		}

		if (msg->flags & I2C_M_RD) {
			dptx_read_bytes_from_i2c(dp->dptx, msg->addr, msg->buf, msg->len);
			dptx_i2c_address_only(dp->dptx, msg->addr);
		}
		else {
			dptx_write_bytes_to_i2c(dp->dptx, msg->addr, msg->buf, msg->len);
			dptx_i2c_address_only(dp->dptx, msg->addr);
		}
	}

	return i;
}


static u32 dp_i2c_func(struct i2c_adapter *adapter)
{
	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
}


static const struct i2c_algorithm dp_algo = {
	.master_xfer	= dp_i2c_master_xfer,
	.functionality	= dp_i2c_func,
};

static ssize_t
xdx_dp_aux_transfer(struct drm_dp_aux *aux, struct drm_dp_aux_msg *msg)
{
	struct dp_dev *dp = (struct dp_dev *)aux->ddc.dev.driver_data;
	struct xdx_output* output = dp_to_output(dp);
	struct xdx_dm* dm = output_to_dm(output);
	struct xdx_device* xdev = dm_to_xdev(dm);
	int ret;

	dev_dbg(xdev->dev,"aux transfer pointer:%p dptx pointer:%p rw buffer address:%p\n",dp,dp->dptx,msg->buffer);
	switch (msg->request & ~DP_AUX_I2C_MOT) {
	case DP_AUX_NATIVE_WRITE:
	case DP_AUX_I2C_WRITE:
	case DP_AUX_I2C_WRITE_STATUS_UPDATE:
	ret = __dptx_write_bytes_to_dpcd(dp->dptx,msg->address,msg->buffer,msg->size);
		dev_dbg(xdev->dev,"aux write addr:%x value:%x",msg->address,*(u8 *)msg->buffer);
		break;
	case DP_AUX_NATIVE_READ:
	case DP_AUX_I2C_READ:	    
		ret = __dptx_read_bytes_from_dpcd(dp->dptx,msg->address,msg->buffer,msg->size);
		dev_dbg(xdev->dev,"aux read addr:%x value:%x",msg->address,*(u8 *)msg->buffer);
		break;
	default:
		ret = -EINVAL;
		break;
	}
	dev_dbg(xdev->dev,"ret:%d size:%lx\n",ret,msg->size);
	if(ret == 0){
		msg->reply = 0;
	}else{
		msg->reply--;
	}
	return msg->size;    
}

int xdx_dp_aux_init(struct dp_dev *dp)
{
	struct xdx_output* output = dp_to_output(dp);
	struct xdx_dm* dm = output_to_dm(output);
	struct xdx_device* xdev = dm_to_xdev(dm);
	int ret;

	dp->aux = kzalloc(sizeof(struct drm_dp_aux), GFP_KERNEL);
	if (!dp->aux)
		return -ENOMEM;

	dp->aux->dev = xdev->dev;
	dp->aux->ddc.algo = &dp_algo;
	dp->aux->ddc.algo_data = dp->aux;
	dp->aux->ddc.retries = 3;
	dp->aux->transfer = xdx_dp_aux_transfer;
	mutex_init(&dp->aux->hw_mutex);
	#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 14, 0)
	dp->aux->drm_dev = xdev_to_drm(xdev);
	#endif

	i2c_set_adapdata(&dp->aux->ddc, dp);

	ret = drm_dp_aux_register(dp->aux);
	if (ret)
		dev_err(xdev->dev, "failed to register dp aux\n");

	return ret;
}
