#ifndef __XDX_DP_DRV_H__
#define __XDX_DP_DRV_H__

#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/version.h>

#include <drm/drm_gem.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0)
#include <drm/drmP.h>
#endif
#include <drm/drm_crtc.h>
#include <drm/drm_crtc_helper.h>
#include <drm/drm_of.h>
#include <drm/drm_edid.h>

#include <drm/drm_atomic_helper.h>
#if((LINUX_VERSION_CODE>=KERNEL_VERSION(5,18,0))&&(LINUX_VERSION_CODE<KERNEL_VERSION(6,0,0)))
#include <drm/dp/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0))
#include <drm/drm_probe_helper.h>
#endif

#include "reg.h"
#include "doorbell.h"
#include "avgen.h"
#include "dm.h"
#define XDX_DEVICE_NAME_DP			"xdx_dp"

#define DPTX_COMBO_PHY
#define DP_DEBUG
//#define DPTX_DEBUG_AUX
// Rounding up to the nearest multiple of a number
#define ROUND_UP_TO_NEAREST(numToRound, mult) ((((numToRound+(mult)-1) / (mult)) * (mult)))


//DSC definitions coming from RC files of DSC C model 
#define RC_MODEL_SIZE			8192
#define INITIAL_OFFSET			6144
#define INITIAL_DELAY			512
#define PIXELS_PER_GROUP		3
#define PIXEL_HOLD_DELAY		5
#define PIXEL_GROUP_DELAY		3
#define MUXER_INITIAL_BUFFERING_DELAY	9
#define DSC_MAX_AUX_ORIG_WIDTH		24
#define PIXEL_FLATNESSBUF_DELAY		DSC_MAX_AUX_ORIG_WIDTH
#define FLATNESS_MIN_QP			3
#define	FLATNESS_MAX_QP			12
#define RC_EDGE_FACTOR			6
#define RC_QUANT_INCR_LIMIT0		11
#define RC_QUANT_INCR_LIMIT1		11
#define RC_TGT_OFFSET_HIGH		3
#define RC_TGT_OFFSET_LOW		3
#define INITIAL_SCALE_VALUE_SHIFT	3
#define OFFSET_FRACTIONAL_BITS		11

/* The max rate and lanes supported by the core */
#define DPTX_MAX_LINK_RATE DPTX_PHYIF_CTRL_RATE_HBR3
#define DPTX_MAX_LINK_LANES 4

/* The default rate and lanes to use for link training */
#define DPTX_DEFAULT_LINK_RATE DPTX_MAX_LINK_RATE
#define DPTX_DEFAULT_LINK_LANES DPTX_MAX_LINK_LANES
#define DPTX_RECEIVER_CAP_SIZE		0x100

struct dptx_aux {
	u32 sts;
	u32 data[4];
	atomic_t abort;
};

struct dptx_link {
	//u8 status[DP_LINK_STATUS_SIZE];
	u8 status[4];
	bool trained;
	u8 rate;
	u8 lanes;
	u8 preemp_level[4];
	u8 vswing_level[4];
	u8 cr_preemp_level[4];
	u8 cr_vswing_level[4];
};


enum established_timings {
	DMT_640x480_60hz,
	DMT_800x600_60hz,
	DMT_1024x768_60hz,
	None
};

#define DPTX_HDCP_REG_DPK_CRC_ORIG	0x331c1169
#define DPTX_HDCP_MAX_AUTH_RETRY	10

struct hdcp_aksv {
	u32 lsb;
	u32 msb;
};

struct hdcp_dpk {
	u32 lsb;
	u32 msb;
};

struct hdcp_params {
	struct hdcp_aksv aksv;
	struct hdcp_dpk dpk[40];
	u32 enc_key;
	u32 crc32;
	u8 auth_fail_count;
	int hdcp13_is_en;
	int hdcp22_is_en;
};

struct dptx {
	struct mutex mutex; /* generic mutex for dptx */

	struct {
		u8 multipixel;
		u8 streams;
		bool gen2phy;
		bool dsc;
	} hwparams;

	u32 version;

	u8 max_rate;
	u8 max_lanes;
		bool ycbcr420;
	u8 streams;

	bool mst;
	int active_mst_links;

	bool cr_fail;// harutk need to remove

	u8 multipixel;
		u8 bstatus;

	bool ssc_en;
	bool scramble_dis;
	bool fec;
	bool dsc;
	u8 dp_pcs_width;
	
	int vcp_id;
	u16 pbn;
	bool need_rad;
	bool logic_port;
	u8 rad_port;
	u8 port[2];

	bool dummy_dtds_present;
	enum established_timings selected_est_timing;

	//struct device *dev;
	//struct dentry *root;
	//struct debugfs_regset32 *regset;

	struct video_params vparams;
	//struct audio_params aparams;
	//struct audio_short_desc audio_desc;
	struct hdcp_params hparams;

	wait_queue_head_t waitq;

	atomic_t shutdown;
	atomic_t c_connect;
	atomic_t sink_request;

	u8 rx_caps[DP_RECEIVER_CAP_SIZE];

	// linux kernel v5.4.x have no "DP_MAX_DOWNSTREAM_PORTS"
	u8 downstream_ports[0x10];

	u8 *edid;
	u8 *edid_second;
#define DPTX_DEFAULT_EDID_BUFLEN 128

	//struct sdp_full_data sdp_list[DPTX_SDP_NUM];//struct dptx_aux aux;
	struct dptx_aux aux;
	struct dptx_link link;
	struct xdx_output *output;
	struct dp_dev *dp;
};

struct xdx_i2c_chan {
	struct i2c_adapter adapter;
	struct drm_dp_aux aux;
	bool has_aux;
};

struct dp_dev {
	struct drm_connector connector;
	struct drm_encoder encoder;

	struct workqueue_struct *workq;
	struct work_struct hpd_work;
	struct dptx *dptx;

	/* Video parameters */
	struct xdx_output  *output;

	struct drm_display_info dp_data;
	struct edid *edid;
	struct drm_display_mode *dp_mode;
	struct drm_display_mode *native_mode;
	unsigned int vrefresh;
	struct drm_dp_aux *aux;

	bool hpd_detect;
	bool hpd_changed;
	bool phy_enabled;
#ifdef CONFIG_DEBUG_FS
	struct dentry **dirs;
	u32 dirs_num;
	u32 debugfs_dpcd_address;
	u32 debugfs_dpcd_size;
#endif
	u32 id;
	struct xdx_irq_src *irq_src;
};

struct dp_trans_para{
        int clock;
        int hdisplay;
        int hsync_start;
        int hsync_end;
        int htotal;
        int hskew;
        int vdisplay;
        int vsync_start;
        int vsync_end;
        int vtotal;
        int vscan;
        int vrefresh;
        u32 flag;

        u32 colordeep;
        u32 colorformat;
        u32 vic_code;
        u32 pixclock;

        u32 mode;
        u32 bpc;
        u32 multipixel;
        u32 mode_flag_pvsync;
        u32 lanes;
        u32 rate;
        u32 dc_idx;
        enum hdmi_picture_aspect picture_aspect_ratio;
};

static inline struct xdx_output *dp_to_output(struct dp_dev *dp)
{
	return dp->output;
}


static inline struct dp_dev *connector_to_dp(struct drm_connector *connector) {
	return container_of(connector, struct dp_dev, connector);
}

#define xdx_connector_to_dp(c) \
	struct dp_dev *dp = container_of((c), struct dp_dev, connector)
#define xdx_encoder_to_dp(e) \
	struct dp_dev *dp = container_of((e), struct dp_dev, encoder)

int xdx_dp_phy_power_down(struct dp_dev *dp);
void dp_video_params_init(struct dp_dev *dp);

void xdx_dp_trans_para_fill(struct dptx *dptx, struct dp_trans_para *pass_para, struct drm_display_mode *mode, void* para_sram_vaddr);
int xdx_dp_phy_startup_fw(struct dptx *dptx, struct drm_display_mode *mode);
int xdx_dp_preinit(struct dptx *dptx);

int xdx_dp_hw_init(struct xdx_dm *dm, u32 id);
int xdx_dp_deinit(struct xdx_dm *dm, u32 id);
int xdx_dp_init(struct xdx_dm *dm, u32 id);

#endif /* __XDX_DP_DRV_H__ */
