// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2010 Synopsys, Inc. and/or its affiliates
 * Synopsys DesignWare HDMIQP TX controller driver
 */
#include <linux/version.h>
#include <linux/component.h>
#include <linux/module.h>

#include <linux/hdmi.h>

#include <drm/drm_crtc.h>
#include <drm/drm_crtc_helper.h>
#include <drm/drm_of.h>
#include <drm/drm_edid.h>

#include <drm/drm_atomic_helper.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0))
#include <drm/drm_probe_helper.h>
#endif

#include "xdx_hdmi.h"
#include "xdx_hdmi_reg.h"
#include "xdx_hdmi_accel.h"
#include "fc_packets.h"
#include "dtd.h"
#include "fc_avi.h"
#include "dm.h"
#include "xdx_device.h"

#define VERTICAL_BAR_PRESENT_OFFSET	2
#define RGB_QUANTIZATION_RANGE_OFFSET	2
#define HORIZONTAL_BAR_PRESENT_OFFSET	3
#define ACTIVE_FORMAT_PRESENT_OFFSET	4
#define PICTURE_ASPECT_RATIO_OFFSET	4
#define EXTCOLORIMETRY_OFFSET		4
#define ITCONTENT_OFFSET		4
#define RGBYCC_INDICATION_OFFSET	5
#define COLORIMETRY_OFFSET		6
#define YCC_QUANTIZATION_RANGE_OFFSET	6
#define ITCONTENT_PRESENT_OFFSET	7
/*
 * Autogenerated from the DMT spec.
 * This table is copied from xfree86/modes/xf86EdidModes.c.
 */
void fc_avi_config(struct hdmi_dev *dev)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u8 buff[32] = {0};	// 8 registers of 32bits
	videoParams_t *videoParams = &dev->mode->pVideo;
	//dtd_t *dtd = &videoParams->mDisplayMode.dtd;
	u8 avi_version, len, pic = 0, *ptr;
	u32 mCode = 0;
	u32 i;
	u16 mHImageSize,mVImageSize;
	mCode = get_mcode(dev->hdmi_mode->clock, dev->hdmi_mode->hdisplay);
	dev_dbg(xdev->dev,"mCode %d ratio:%d\n",mCode,dev->connector.state->picture_aspect_ratio);
	switch(dev->hdmi_mode->picture_aspect_ratio){
		case HDMI_PICTURE_ASPECT_4_3:{
			mHImageSize = 4;
			mVImageSize = 3;
			break;
		}
		case HDMI_PICTURE_ASPECT_16_9:{
			mHImageSize = 16;
			mVImageSize = 9;
			break;
		}
		case HDMI_PICTURE_ASPECT_64_27:{
			mHImageSize = 64;
			mVImageSize = 27;
			break;
		}
		case HDMI_PICTURE_ASPECT_256_135:{
			mHImageSize = 256;
			mVImageSize = 135;
			break;
		}
		default:
			mHImageSize = 4;
			mVImageSize = 3;
			break;
	}
	//mCode = 1;
	//mHImageSize = 4;
	//mVImageSize = 3;
	if ((videoParams->mColorimetry == EXTENDED_COLORIMETRY) && (videoParams->mExtColorimetry == ADDITIONAL_COLORIMETRY)) {
		avi_version = 0x4;
		len = 14;
	} else if (mCode >= 128) {
		avi_version = 0x3;
		len = 13;
	} else {
		avi_version = 0x2;
		len = 13;
	}

	buff[0] = 0x82;
	buff[1] = avi_version;
	buff[2] = len;

	// PB1
	ptr = &buff[PB1_OFFSET];
	*ptr = videoParams->mScanInfo & 0x03;
	if ((videoParams->mEndTopBar != (u16)-1) || (videoParams->mStartBottomBar != (u16)-1))
		*ptr |= 1 << VERTICAL_BAR_PRESENT_OFFSET;
	if ((videoParams->mEndLeftBar != (u16)-1) || (videoParams->mStartRightBar != (u16)-1))
		*ptr |= 1 << HORIZONTAL_BAR_PRESENT_OFFSET;
	if (videoParams->mActiveFormatAspectRatio) {
		pic = (mHImageSize * 10) % mVImageSize;
		*ptr |= 1 << ACTIVE_FORMAT_PRESENT_OFFSET;
	}
	*ptr |= (videoParams->mEncoding & 0x7) << RGBYCC_INDICATION_OFFSET;

	// PB2
	ptr++;
	*ptr = videoParams->mActiveFormatAspectRatio & 0xF;
	if (pic)
		*ptr |= ((pic > 5) ? 2 : 1) << PICTURE_ASPECT_RATIO_OFFSET;
	*ptr |= (videoParams->mColorimetry & 0x3) << COLORIMETRY_OFFSET;

	// PB3
	ptr++;
	*ptr = videoParams->mNonUniformScaling & 0x3;
	*ptr |= (videoParams->mRgbQuantizationRange & 0x3) << RGB_QUANTIZATION_RANGE_OFFSET;
	*ptr |= (videoParams->mExtColorimetry & 0x7) << EXTCOLORIMETRY_OFFSET;
	if (videoParams->mItContent)
		*ptr |= 1 << ITCONTENT_PRESENT_OFFSET;

	// PB4
	ptr++;
	if (videoParams->mHdmiVideoFormat == HDMI_NORMAL_FORMAT)
		*ptr = mCode;

	// PB5
	ptr++;
	*ptr = (((videoParams->mPixelRepetitionInput + 1) * (videoParams->mPixelRepetitionOutput + 1) - 1) & 0xF);
	*ptr |= (videoParams->mItContent & 0x3) << ITCONTENT_OFFSET;
	// TODO: ADD YCC QUANTIZATION RANGE
	//*ptr |= ( & 0x3) << YCC_QUANTIZATION_RANGE_OFFSET;

	// PB6 and PB7
	ptr++;
	*ptr = videoParams->mEndTopBar;

	// PB8 and PB9
	ptr += 2;
	*ptr = videoParams->mStartBottomBar;

	// PB10 and PB11
	ptr += 2;
	*ptr = videoParams->mEndLeftBar;

	// PB12 and PB13
	ptr += 2;
	*ptr = videoParams->mStartRightBar;

	// PB0
	buff[CHECKSUM_OFFSET] = infoframe_checksum(buff);

	dev_write_mask(dev, PKT_AVI_CONTENTS0, 0x00FFFF00, *((u32*)&buff[0]));
	dev_dbg(xdev->dev, "PKT_AVI_CONTENTS0%d:%x\n",PKT_AVI_CONTENTS0,*((u32*)&buff[0]));
	for (i = 0; i <= (PKT_AVI_CONTENTS7 - PKT_AVI_CONTENTS1); i += 4){
		xdx_hdmicore_write(dev, PKT_AVI_CONTENTS1 + i, *((u32*)&buff[PB0_OFFSET + i]));
	        dev_dbg(xdev->dev, "PKT_AVI_CONTENTS%d:%x\n",i,*((u32*)&buff[PB0_OFFSET + i]));
	}
}
