// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2010 Synopsys, Inc. and/or its affiliates
 * Synopsys DesignWare HDMIQP TX controller driver
 */

#include "xdx_hdmi_accel.h"
#include "xdx_hdmi.h"

#include "xdx_hdmi_reg.h"
#include "fc_packets.h"

#define NEXT_FIELD_OFFSET	7

static u8 fc_gamut_CurrentSeqNo(struct hdmi_dev *dev)
{
	u32 val = dev_read_mask(dev, PKT_GMD_CONTENTS0, PKT_CONTENTS0_HB2_MASK);
	return (u8)(val & 0xF);
}

void fc_gamut_config(struct hdmi_dev *dev, const u8 *gbdContent, u8 length)
{
	u8 buff[32] = {0};	// 8 registers of 32bits
	u8 i, affected_seq, *ptr;

	affected_seq = (fc_gamut_CurrentSeqNo(dev) + 1) & 0xF;

	buff[0] = 0xA;
	// Profile P0
	buff[1] = affected_seq | 1 << NEXT_FIELD_OFFSET;
	buff[2] = 0;

	// PB0++
	ptr = &buff[PB0_OFFSET];
	if (gbdContent && length)
		memcpy(ptr, gbdContent, length);

	dev_write_mask(dev, PKT_GMD_CONTENTS0, 0x00FFFF00, *((u32*)&buff[0]));
	for (i = 0; i <= (PKT_GMD_CONTENTS7 - PKT_GMD_CONTENTS1); i += 4)
		xdx_hdmicore_write(dev, PKT_GMD_CONTENTS1 + i, *((u32*)&buff[PB0_OFFSET + i]));
}
