// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2010 Synopsys, Inc. and/or its affiliates
 * Synopsys DesignWare HDMIQP TX controller driver
 */

#include "xdx_hdmi.h"
#include "xdx_hdmi_reg.h"
#include "xdx_hdmi_accel.h"
#include "fc_packets.h"

#define HDMI_INFOFRAME_HEADER_SIZE	4
#define TYPE2MASK(type)			(1 << (type))

static void fc_packets_AutoSend(struct hdmi_dev *dev, u32 mask)
{
	u32 config3_mask = mask & PKTSCHED_PKT_CONFIG3_ALL_MASK;
	u32 en_mask = mask & PKTSCHED_PKT_EN_ALL_MASK;
	dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG3, config3_mask, 0);
	dev_write_mask_shifted(dev, PKTSCHED_PKT_EN, en_mask, 1);
}

static void fc_packets_ManualSend(struct hdmi_dev *dev, u32 mask)
{
	u32 config3_mask = mask & PKTSCHED_PKT_CONFIG3_ALL_MASK;
	u32 send_mask = mask & PKTSCHED_PKT_SEND_ALL_MASK;
	u32 en_mask = mask & PKTSCHED_PKT_EN_ALL_MASK;
	if (config3_mask && send_mask) {
		dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG3, config3_mask, 1);
		dev_write_mask_shifted(dev, PKTSCHED_PKT_EN, en_mask, 1);
		dev_write_mask_shifted(dev, PKTSCHED_PKT_SEND, send_mask, 1);
	}
}

void fc_packets_GCP_enable(struct hdmi_dev *dev)
{
	u32 mask = PKTSCHED_PKT_EN_GCP_MASK;
	dev_write_mask(dev, PKTSCHED_PKT_EN, mask, mask);
}

void fc_packets_enable(struct hdmi_dev *dev, enum pkt_type type, u8 mode)
{
	u32 mask = TYPE2MASK(type);
	if (mode == AUTO)
		fc_packets_AutoSend(dev, mask);
	else
		fc_packets_ManualSend(dev, mask);

	if (type == GHDMI1)
		dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG2,
			       PKTSCHED_PKT_CONFIG2_GHDMI1_RATE_MASK, 1);
	else if (type == GHDMI2)
		dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG2,
			       PKTSCHED_PKT_CONFIG2_GHDMI2_RATE_MASK, 1);
}

void fc_packets_disable(struct hdmi_dev *dev, enum pkt_type type)
{
	u32 mask = TYPE2MASK(type);
	dev_write_mask_shifted(dev, PKTSCHED_PKT_EN, mask, 0);
}

void fc_packets_disable_all(struct hdmi_dev *dev)
{
	dev_write_mask(dev, PKTSCHED_PKT_EN, PKTSCHED_PKT_EN_ALL_MASK, 0);
	dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG2,
			       PKTSCHED_PKT_CONFIG2_GHDMI1_RATE_MASK, 0);
	dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG2,
			       PKTSCHED_PKT_CONFIG2_GHDMI2_RATE_MASK, 0);
}

u8 infoframe_checksum(u8 *buff)
{
	u8 csum = 0;
	size_t i, size;

	size = HDMI_INFOFRAME_HEADER_SIZE + buff[2];
	/* compute checksum */
	for (i = 0; i < size; i++)
		csum += buff[i];

	return 256 - csum;
}
