// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2010 Synopsys, Inc. and/or its affiliates
 * Synopsys DesignWare HDMIQP TX controller driver
 */

#include "xdx_hdmi.h"
#include "xdx_hdmi_reg.h"
#include "xdx_hdmi_accel.h"
#include "fc_packets.h"
#include "fc_avi.h"
#include "fc_vsi.h"
#include "dtd.h"
#include "dm.h"
#include "xdx_device.h"

#define ISRC_PACKET_SIZE 	16
int packets_Initialize(struct hdmi_dev *dev);
void packets_StopSendACRandAUDS(struct hdmi_dev *dev);
void packets_StartSendACRandAUDS(struct hdmi_dev *dev);
void packets_StopSendAudi(struct hdmi_dev *dev);
void packets_StopSendAmd(struct hdmi_dev *dev);
u32 packets_get_AvMute_status(struct hdmi_dev *dev);
void packets_disable_AvMute(struct hdmi_dev *dev);
int videoParams_GetHdmiVicCode(u8 cea_code);

int packets_Initialize(struct hdmi_dev *dev)
{
	fc_packets_disable_all(dev);
	return TRUE;
}

static void packets_AVI(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, AVI);

	fc_avi_config(dev);

	fc_packets_enable(dev, AVI, AUTO);
}

static void packets_SPDI(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, SPDI);

	fc_spdi_config(dev);

	fc_packets_enable(dev, SPDI, AUTO);
}

static void packets_Gamut(struct hdmi_dev *dev)
{
	u8 gamut_metadata[28] = {0};
	int gdb_color_space = 0;
    videoParams_t *video = &dev->mode->pVideo;
	fc_packets_disable(dev, GMD);

	if (video->mColorimetry == EXTENDED_COLORIMETRY) {
		if (video->mExtColorimetry == XV_YCC601)
			gdb_color_space = 1;
		else if (video->mExtColorimetry == XV_YCC709)
			gdb_color_space = 2;
		else if (video->mExtColorimetry == S_YCC601)
			gdb_color_space = 3;
		else if (video->mExtColorimetry == ADOBE_YCC601)
			gdb_color_space = 3;
		else if (video->mExtColorimetry == ADOBE_RGB)
			gdb_color_space = 3;

		if (video->mColorimetryDataBlock == TRUE) {
			gamut_metadata[0] = (1 << 7) | gdb_color_space;
			fc_gamut_config(dev, gamut_metadata, (sizeof(gamut_metadata) / sizeof(u8)));
		}

		fc_packets_enable(dev, GMD, AUTO);
	}
}

void packets_StopSendACRandAUDS(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, ACR);
	fc_packets_disable(dev, AUDS);
}

void packets_StartSendACRandAUDS(struct hdmi_dev *dev)
{
	fc_packets_enable(dev, ACR, AUTO);
	fc_packets_enable(dev, AUDS, AUTO);
}

void packets_StopSendAudi(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, AUDI);
}

//void packets_Audi(struct hdmi_dev *dev, audioParams_t *audio)
//{
//	packets_StopSendAudi(dev);

//	fc_audi_config(dev, audio);

//	fc_packets_enable(dev, AUDI, AUTO);
//}

void packets_StopSendAmd(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, AMD);
}

#if 0
void packets_Amd(struct hdmi_dev *dev, audioParams_t *audio)
{
	packets_StopSendAmd(dev);

	fc_amd_config(dev, audio);

	fc_packets_enable(dev, AMD, AUTO);
}

void packets_StopSendAcp(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, ACP);
}

void packets_AudioContentProtection(struct hdmi_dev *dev, u8 type, const u8 *fields, u8 length)
{
	packets_StopSendAcp(dev);

	fc_acp_config(dev, type, fields, length);

	fc_packets_enable(dev, ACP, AUTO);
}


void packets_StopSendIsrc(struct hdmi_dev *dev)
{
	fc_packets_disable(dev, ISRC1);
	fc_packets_disable(dev, ISRC2);
}

void packets_IsrcPackets(struct hdmi_dev *dev, u8 initStatus, const u8 *codes, u8 length)
{
	packets_StopSendIsrc(dev);

	if (length > ISRC_PACKET_SIZE) {
		fc_isrc2_config(dev, codes + ISRC_PACKET_SIZE, length - ISRC_PACKET_SIZE);
		fc_isrc1_config(dev, initStatus, 1, 1, codes, ISRC_PACKET_SIZE);

		fc_packets_enable(dev, ISRC2, AUTO);
	} else {
		fc_isrc2_config(dev, NULL, 0);	// Clear ISRC2 packets
		fc_isrc1_config(dev, initStatus, 1, 0, codes, length);
	}

	fc_packets_enable(dev, ISRC1, AUTO);
}
#endif

void packets_AvMute(struct hdmi_dev *dev, u8 enable)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	dev_write_mask(dev, PKTSCHED_PKT_CONTROL0,
		       PKTSCHED_PKT_CONTROL0_GCP_SET_AVMUTE_MASK |
		       PKTSCHED_PKT_CONTROL0_GCP_CLEAR_AVMUTE_MASK,
		       (enable ? 0x1 : 0x2));
	dev_dbg(xdev->dev,"bypass hdcp\n");
	dev_write_mask_shifted(dev, HDCP14_CONFIG0, HDCP14_CONFIG0_BYPASS_MASK, 1);
	dev_write_mask_shifted(dev, HDCP2_CONFIG, HDCP2_CONFIG_BYPASS_MASK, 1);
}

u32 packets_get_AvMute_status(struct hdmi_dev *dev)
{
	return dev_read_mask(dev, PKTSCHED_PKT_STATUS1,
			     PKTSCHED_PKT_STATUS1_GCP_AVMUTE_SENT_STS_MASK);
}

void packets_disable_AvMute(struct hdmi_dev *dev)
{
	dev_write_mask(dev, PKTSCHED_PKT_CONTROL0,
		PKTSCHED_PKT_CONTROL0_GCP_SET_AVMUTE_MASK |
		PKTSCHED_PKT_CONTROL0_GCP_CLEAR_AVMUTE_MASK, 0x0);
}

int videoParams_GetHdmiVicCode(u8 cea_code)
{
	switch (cea_code) {
	case 95:
		return 1;
	case 94:
		return 2;
	case 93:
		return 3;
	case 98:
		return 4;
	default:
		break;
	}
	return -1;
}


int packets_Configure(struct hdmi_dev *dev)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u8 packet_data[3] = {0}; //PB4-PB6
	u8 mcode;
	videoParams_t *video = &dev->mode->pVideo;
	dev_dbg(xdev->dev, "packet configure\n");
	mcode = get_mcode(dev->hdmi_mode->clock,dev->hdmi_mode->hdisplay);
	if (video->mHdmiVideoFormat == HDMI_3D_FORMAT) {
		dev_dbg(xdev->dev, "%s: 3D packet configuration", __func__);
		fc_packets_disable(dev, VSI);	/* prevent sending half the info. */

		// frame packing || tab || sbs
		if ((video->m3dStructure == FRAME_PACKING_3D)  ||
		    (video->m3dStructure == TOP_AND_BOTTOM_3D) ||
		    (video->m3dStructure == SIDE_BY_SIDE_3D)) {
			packet_data[0] = set8(0, PACKET_HDMIVIDEOFORMAT_MASK, video->mHdmiVideoFormat);
			packet_data[1] = set8(0, PACKET_3D_STRUCTURE_MASK, video->m3dStructure);
			packet_data[2] = set8(0, PACKET_3D_EXT_DATA_MASK,  video->m3dExtData);

			fc_vsi_config(dev, packet_data, sizeof(packet_data));
		} else {
			dev_err(xdev->dev, "%s: 3D structure not supported %d", __func__, video->m3dStructure);
			return FALSE;
		}

		fc_packets_enable(dev, VSI, AUTO);
	} else if (video->mHdmiVideoFormat == HDMI_EXT_RES_FORMAT) {
		dev_dbg(xdev->dev, "%s: 4K packet configuration", __func__);

		fc_packets_disable(dev, VSI);	/* prevent sending half the info. */

		packet_data[0] = set8(0, PACKET_HDMIVIDEOFORMAT_MASK, video->mHdmiVideoFormat);
		packet_data[1] = set8(0, PACKET_HDMI_VIC_MASK, videoParams_GetHdmiVicCode(mcode));
		packet_data[2] = 0;

		fc_vsi_config(dev, packet_data, sizeof(packet_data));

		fc_packets_enable(dev, VSI, AUTO);
	} else {
		fc_packets_disable(dev, VSI);
	}

	//fc_packets_disable(dev, VSI);

	// Enable GCP packets
	packets_AvMute(dev, TRUE);
	fc_packets_GCP_enable(dev);

	// Configure AVI packet and respective rate
	packets_AVI(dev);
	dev_write_mask_shifted(dev, PKTSCHED_PKT_CONFIG1,
			       PKTSCHED_PKT_CONFIG1_AVI_FIELDRATE_MASK,
			       //dev->hdmi_dev.avi_fieldrate);
                   1);

	packets_SPDI(dev);

	packets_Gamut(dev);

	return TRUE;
}
