#include "phyconfig.h"
#include "xdx_hdmi_accel.h"
#include "xdx_hdmi.h"
#include "xdx_hdmi_reg.h"
#include "xdx_tile_reg.h"
#include "doorbell.h"
#define HDMI_PMA 1

int xdx_hdmi_phy_reset(struct hdmi_dev *hdmi)
{
	u32 TimeOut, DBValue;
	DPDBCmd_T_NOSINK    PowerConfig;
	/* For HDMI TX 1.4 PHY, power down by placing PHY in reset */
	//need add code
	PowerConfig.cmd = cDPCmdHDMIPhyReset;
	xdx_hdmi_h2c_doorbell_w(hdmi,(u64)PowerConfig.all);
	TimeOut = 10000;
	while(1)
	{
		DBValue = xdx_hdmi_h2c_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0)
			return -1;
		udelay(1000);
	}
	return 0;
}

int xdx_hdmi_phy_reset_release(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 TimeOut, DBValue;
	DPDBCmd_T_NOSINK    PowerConfig;
	u32 tiledata = 0;

	/* For HDMI TX 1.4 PHY, power down by placing PHY in reset */
	xdx_hdmicore_read(hdmi,0);
	PowerConfig.cmd = cDPCmdHDMIPhyResetRelease;
	xdx_hdmi_h2c_doorbell_w(hdmi,(u64)PowerConfig.all);
	TimeOut = 10000;
	while(1){
		DBValue = xdx_hdmi_h2c_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0){
			dev_err(xdev->dev,"powerup fail\n");
		return -1;
		}
		udelay(1000);
	}
	dev_dbg(xdev->dev,"wait sram ready\n");
	//PHY ready
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if((tiledata & 0x1) == 1)
			break;
		if(TimeOut == 0){
			dev_err(xdev->dev,"wait PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(50);
	}
/*	tiledata = 1<<1;
	xdx_hdmitile_write(hdmi,ALL_HDMIDP_PHY_IF_CLR,tiledata);
	tiledata = 0;
	while(1){
	   tiledata =  xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
	   if(((tiledata >> 1) & 0x1) == 0)
		  break;
	}
*/
	return 0;
}

int xdx_hdmi_phy_startup(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 PixClock = hdmi->hdmi_mode->clock;
	u32 ColorDeep = hdmi->hdmi_data.bpc;
	u32 Format = hdmi->hdmi_data.color_formats;
	HDMIDBCmd_T_NOSINK    PhyConfig;
	u32 TimeOut, DBValue;
	u32 data;
	u32 value;
	int ret;
#if HDMI_PMA
	u32 tiledata;
	xdx_hdmi_phy_reset(hdmi);
#endif
	//ENABLE PLL
	data = xdx_hdmitile_read(hdmi,PHYREF_PLL_EN);
	data = data | BIT(0);
	xdx_hdmitile_write(hdmi,PHYREF_PLL_EN,data);
	TimeOut = 10000;
	while(1){
		data = xdx_hdmitile_read(hdmi,PHYREF_PLL_LOCK);
		if( data & 1)
			break;
		if(TimeOut == 0){
			dev_err(xdev->dev,"wait PHYREF_PLL_LOCK timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
	dev_dbg(xdev->dev,"HDMI PHY SRAM bypass\n");
	value = xdx_hdmitile_read(hdmi,0x0);
	value = (value | (1<<1));
	xdx_hdmitile_write(hdmi,0x0,value);
#if HDMI_PMA
	ret = xdx_hdmi_phy_reset_release(hdmi);
	if(ret){
		dev_err(xdev->dev,"hdmi phy reset release fail\n");
		return -1;
	}
#endif
	value = xdx_hdmicore_read(hdmi,0xaac);
	value = (value & 0xfffffffd) | 0x1;
	xdx_hdmicore_write(hdmi,0xaac,value);
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
	/* Preferred encoding type is RGB444 */
	if (Format & DRM_COLOR_FORMAT_RGB444)
		Format = RGB;
	else if (Format & DRM_COLOR_FORMAT_YCRCB444)
		Format = YCBCR444;
	else if (Format & DRM_COLOR_FORMAT_YCRCB422)
		Format = YCBCR422;
	else
		Format = YCBCR420;
#else
	if (Format & DRM_COLOR_FORMAT_RGB444)
		Format = RGB;
	else if (Format & DRM_COLOR_FORMAT_YCBCR444)
		Format = YCBCR444;
	else if (Format & DRM_COLOR_FORMAT_YCBCR422)
		Format = YCBCR422;
	else
		Format = YCBCR420;
#endif
	if(ColorDeep == 8){
		switch(Format){
			case RGB:
				xdx_hdmitile_write(hdmi,0x1c,0x50);
				dev_dbg(xdev->dev,"pix format 8BIT RGB\n");
				break;
			case YCBCR422:
				xdx_hdmitile_write(hdmi,0x1c,0x51);
				dev_dbg(xdev->dev,"pix format 8BIT YUV422\n");
				break;
			case YCBCR444:
				xdx_hdmitile_write(hdmi,0x1c,0x52);
				dev_dbg(xdev->dev,"pix format 8BIT YUV444\n");
				break;
			case YCBCR420:
				xdx_hdmitile_write(hdmi,0x1c,0x53);
				dev_dbg(xdev->dev,"pix format 8BIT YUV420\n");
				break;
			default:
				xdx_hdmitile_write(hdmi,0x1c,0x50);
				break;
		}
	}
	if(ColorDeep == 10){
		switch(Format){
			case RGB:
				xdx_hdmitile_write(hdmi,0x1c,0x60);
				dev_dbg(xdev->dev,"pix format 10BIT RGB\n");
				break;
			case YCBCR422:
				xdx_hdmitile_write(hdmi,0x1c,0x61);
				dev_dbg(xdev->dev,"pix format 10BIT YUV422\n");
				break;
			case YCBCR444:
				xdx_hdmitile_write(hdmi,0x1c,0x62);
				dev_dbg(xdev->dev,"pix format 10BIT YUV444\n");
				break;
			case YCBCR420:
				xdx_hdmitile_write(hdmi,0x1c,0x63);
				dev_dbg(xdev->dev,"pix format 10BIT YUV420\n");
				break;
			 default:
				xdx_hdmitile_write(hdmi,0x1c,0x60);
				break;
		}
	}

	if(Format == YCBCR422){
		PhyConfig.colorDepth = 8;
	}else{
		PhyConfig.colorDepth = ColorDeep;
	}
	PhyConfig.colorFormat = Format;
	PhyConfig.screenMode = PixClock;
	PhyConfig.cmd = cHDMIDBCmdPhyInit;
	xdx_hdmi_h2a_doorbell_w(hdmi,(u64)PhyConfig.all);
	TimeOut = 10000;
	while(1){
		DBValue = xdx_hdmi_h2a_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0){
			dev_dbg(xdev->dev,"HDMI phyinit timeout\n");
			return -1;
		}
		udelay(1000);
	}

#if HDMI_PMA
	//p3 status
	dev_dbg(xdev->dev,"phy to p3\n");
	value = xdx_hdmitile_read(hdmi,ALL_HDMI_PCS_CTRL);
	value = (value | (0x3 << 28));
	xdx_hdmitile_write(hdmi,ALL_HDMI_PCS_CTRL,value);
	dev_dbg(xdev->dev,"HDMI PCS Cntl:%x\n",xdx_hdmitile_read(hdmi,0x08));
	TimeOut = 10000;
	while(1){
		value = xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if(((value >> 3) & 0xf) == 0xf){
			break;
		}
		if(TimeOut == 0){
			dev_err(xdev->dev,"hdmi p0 to p3 wai PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
#endif
#if HDMI_PMA
	xdx_hdmitile_write(hdmi, ALL_HDMIDP_PHY_IF_CLR, 0xf<<3);
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi, ALL_HDMIDP_PHY_IF_RAW);
		if(((tiledata>>3) & 0xf) == 0){
			break;
		}
		xdx_hdmitile_write(hdmi, ALL_HDMIDP_PHY_IF_CLR, 0xf<<3);
		if(TimeOut == 0){
			dev_err(xdev->dev,"p0->p3 clear irq state timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
#endif

#if HDMI_PMA
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi,ALL_HDMI_PHY_CTRL);
		if(((tiledata >> 28) & 0xf) == 0){
			break;
		}
		if(TimeOut == 0){
			dev_err(xdev->dev,"p0->p3 wai PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
#endif
	//ALL__HDMI_PCS_CTRL p3->p0
	dev_dbg(xdev->dev,"HDMI tile Cntl bit[30:28] 0\n");
	value = xdx_hdmitile_read(hdmi,ALL_HDMI_PCS_CTRL);
	value = (value & (~(0xf << 28)));
	xdx_hdmitile_write(hdmi,ALL_HDMI_PCS_CTRL,value);
#if HDMI_PMA
	dev_dbg(xdev->dev,"HDMI PCS Cntl:%x\n",xdx_hdmitile_read(hdmi,0x08));
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if(((tiledata >> 3) & 0xf) == 0xf){
			break;
		}
		if(TimeOut == 0){
			dev_err(xdev->dev,"p3->p0 wai PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
	tiledata = 0xf<<3;
	xdx_hdmitile_write(hdmi,ALL_HDMIDP_PHY_IF_CLR,tiledata);
	tiledata = 0;
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if(((tiledata >> 3)&0xf) == 0)
			break;
		xdx_hdmitile_write(hdmi,ALL_HDMIDP_PHY_IF_CLR,0xf<<3);
		if(TimeOut == 0){
			dev_err(xdev->dev,"p3->p0 wait clear irq state timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
	//PHY ready
	tiledata = 0;
	TimeOut = 10000;
	while(1){
		tiledata = xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if((tiledata >> 1 & 0x1) == 1)
			break;
		if(TimeOut == 0){
			dev_err(xdev->dev,"phy ready wait PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
	tiledata = 1<<1;
	xdx_hdmitile_write(hdmi,ALL_HDMIDP_PHY_IF_CLR,tiledata);
	tiledata = 0;
	TimeOut = 10000;
	while(1){
		tiledata =  xdx_hdmitile_read(hdmi,ALL_HDMIDP_PHY_IF_RAW);
		if(((tiledata >> 1) & 0x1) == 0)
			break;
		if(TimeOut == 0){
			dev_err(xdev->dev,"phy init done wait PHY_IF_RAW timeout\n");
			return -1;
		}
		TimeOut--;
		udelay(10);
	}
	value = xdx_hdmicore_read(hdmi,0xaac);
	value = (value & 0xffffffe) | 0x2;
	xdx_hdmicore_write(hdmi,0xaac,value);
#endif
	dev_dbg(xdev->dev,"hdmi phy init done\n");
	return 0;
}

int xdx_phy_power_down(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 TimeOut, DBValue;
	DPDBCmd_T_NOSINK    PowerConfig;
	/* For HDMI TX 1.4 PHY, power down by placing PHY in reset */
	PowerConfig.cmd = cDPCmdHDMIPowerdown;
	xdx_hdmi_h2c_doorbell_w(hdmi,(u64)PowerConfig.all);
	TimeOut = 10000;
	while(1){
		DBValue = xdx_hdmi_h2c_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0){
			dev_err(xdev->dev,"hdmi power down fail\n");
			return -1;
		}
		udelay(1000);
	}
	return 0;
}

int xdx_phy_power_down_sink(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 TimeOut, DBValue;
	HDMIDBCmd_T_SINK    PhyConfig;
	int ret;
	/* For HDMI TX 1.4 PHY, power down by placing PHY in reset */
   	struct hdmi_trans_para *pass_para = kmalloc(sizeof(struct hdmi_trans_para), GFP_KERNEL);

	if(output->psDoorbellShareMemDesc[hdmi->id] == NULL){
		ret = DPDoorbellShareMemAllocate(output,hdmi->id);
		if(ret){
			dev_err(xdev->dev,"hdmi share ram allocate fail\n");
			return ret;
		}
	}
	xdx_hdmi_trans_para_fill(hdmi, pass_para, output->psDoorbellVirAddr[hdmi->id]);

	PhyConfig.all = 0;
	PhyConfig.cmd = cH02ADisplayDBCmdHDMI;
	PhyConfig.subCmd = cH02AHDMISubCmdPowerDown;
	PhyConfig.dword = output->psDoorbellPhyAddr[hdmi->id].uiAddr;
	xdx_hdmi_h2a_doorbell_w(hdmi,(u64)PhyConfig.all);
	TimeOut = 10000;
	while(1){
		DBValue = xdx_hdmi_h2a_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0){
			dev_dbg(xdev->dev,"HDMI phyinit timeout\n");
			return -1;
		}
		udelay(1000);
	}
	PhyConfig.all = DBValue;
	kfree(pass_para);
	return PhyConfig.ret;
}	

int xdx_phy_power_up(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 TimeOut, DBValue;
	DPDBCmd_T_NOSINK    PowerConfig;
	/* For HDMI TX 1.4 PHY, power down by placing PHY in reset */
	xdx_hdmicore_read(hdmi,0);
	PowerConfig.cmd = cDPCmdHDMIPoweron;
	xdx_hdmi_h2c_doorbell_w(hdmi,(u64)PowerConfig.all);
	TimeOut = 10000;
	while(1){
		DBValue = xdx_hdmi_h2c_doorbell_r(hdmi);
		if((DBValue & 1) == 1)
			break;
		TimeOut--;
		if(TimeOut == 0){
			dev_dbg(xdev->dev,"powerup fail\n");
			return -1;
	}
	udelay(1000);
	}
	return 0;
}	
