/*-*- linux-c -*-
 *  linux/drivers/video/i810_accel.c -- Hardware Acceleration
 *
 *      Copyright (C) 2001 Antonino Daplas<adaplas@pol.net>
 *      All Rights Reserved      
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */
#include <linux/kernel.h>
#include "xdx_hdmi_reg.h"
#include "xdx_hdmi.h"
#include "xdx_hdmi_accel.h"
#include "xdx_device.h"
#include "dm.h"

/* *************************************************************************
 * Data Manipulation and Access
 * *************************************************************************/
/**
 * Find first (least significant) bit set
 * @param[in] data word to search
 * @return bit position or 32 if none is set
 */

typedef struct irq_vector {
	irq_sources_t source;
	uint32_t status_reg;
	uint32_t mask_reg;
	uint32_t clear_reg;
	uint32_t force_reg;
} irq_vector_t;

static irq_vector_t irq_vec[] = {
	{MU_INT0,   MU_INT0_STATUS,   MU_INT0_MASK,   MU_INT0_CLEAR,   MU_INT0_FORCE},
	{MU_INT1,   MU_INT1_STATUS,   MU_INT1_MASK,   MU_INT1_CLEAR,   MU_INT1_FORCE},
	{AVP_INT0,  AVP_INT0_STATUS,  AVP_INT0_MASK,  AVP_INT0_CLEAR,  AVP_INT0_FORCE},
	{AVP_INT1,  AVP_INT1_STATUS,  AVP_INT1_MASK,  AVP_INT1_CLEAR,  AVP_INT1_FORCE},
	{AVP_INT2,  AVP_INT2_STATUS,  AVP_INT2_MASK,  AVP_INT2_CLEAR,  AVP_INT2_FORCE},
	{AVP_INT3,  AVP_INT3_STATUS,  AVP_INT3_MASK,  AVP_INT3_CLEAR,  AVP_INT3_FORCE},
	{AVP_INT4,  AVP_INT4_STATUS,  AVP_INT4_MASK,  AVP_INT4_CLEAR,  AVP_INT4_FORCE},
	{AVP_INT5,  AVP_INT5_STATUS,  AVP_INT5_MASK,  AVP_INT5_CLEAR,  AVP_INT5_FORCE},
	{AVP_INT6,  AVP_INT6_STATUS,  AVP_INT6_MASK,  AVP_INT6_CLEAR,  AVP_INT6_FORCE},
//	{CEC_INT,   CEC_INT_STATUS,   CEC_INT_MASK,   CEC_INT_CLEAR,   CEC_INT_FORCE},
//	{EARC_INT0, EARC_INT0_STATUS, EARC_INT0_MASK, EARC_INT0_CLEAR, EARC_INT0_FORCE},
//	{EARC_INT1, EARC_INT1_STATUS, EARC_INT1_MASK, EARC_INT1_CLEAR, EARC_INT1_FORCE},
	{0, 0, 0},
};

static inline uint32_t first_bit_set(uint32_t data)
{
	uint32_t n = 0;

	if (data != 0) {
		for (n = 0; (data & 1) == 0; n++) {
			data >>= 1;
		}
	}
	return n;
}

/*u16 xdx_hdmipcs_read(struct hdmi_dev *hdmi, u32 offset)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev, xdev->hw_info.hdmi_pcs_regs[hdmi->id].base + offset);
}

void xdx_hdmipcs_write(struct hdmi_dev *hdmi, u32 offset, u16 value)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev, xdev->hw_info.hdmi_pcs_regs[hdmi->id].base + offset, value);
}*/

u32 xdx_hdmitile_read(struct hdmi_dev *hdmi, u32 offset)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev, xdev->hw_info.hdmi_tile_regs[hdmi->id].base + offset);
}

void xdx_hdmitile_write(struct hdmi_dev *hdmi, u32 offset, u32 value)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev, xdev->hw_info.hdmi_tile_regs[hdmi->id].base + offset, value);
}

u32 xdx_hdmicore_read(struct hdmi_dev *hdmi, u32 offset)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev, xdev->hw_info.hdmi_core_regs[hdmi->id].base + offset);
}

void xdx_hdmicore_write(struct hdmi_dev *hdmi, u32 offset, u32 value)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev, xdev->hw_info.hdmi_core_regs[hdmi->id].base + offset, value);
}

void dev_write_mask(struct hdmi_dev *dev, u32 addr, u32 mask, u32 data)
{
	u32 temp = 0;

	temp = xdx_hdmicore_read(dev, addr);
	temp &= ~(mask);
	temp |= (mask & data);
	xdx_hdmicore_write(dev, addr, temp);
}

void dev_write_mask_shifted(struct hdmi_dev  *dev, u32 addr, u32 mask, u32 data)
{
	u32 shift = first_bit_set(mask);

	dev_write_mask(dev, addr, mask, data << shift);
}

u32 dev_read_mask(struct hdmi_dev *dev, u32 addr, u32 mask)
{
	u32 shift = first_bit_set(mask);
	return ((xdx_hdmicore_read(dev, addr) & mask) >> shift);
}


int irq_clear_bit(struct hdmi_dev  *dev, irq_sources_t irq_source, u32 bit_mask)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i = 0;

	for (i = 0; irq_vec[i].source != 0; i++) {
		if (irq_vec[i].source == irq_source) {
				dev_write_mask_shifted(dev, irq_vec[i].clear_reg,
						       bit_mask, 1);
			return TRUE;
		}
	}
	dev_dbg(xdev->dev, "IRQ source [%d] is not supported", irq_source);
	return FALSE;
}

int irq_read_stat(struct hdmi_dev *dev, irq_sources_t irq_source, u32 *stat)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i = 0;
	for (i = 0; irq_vec[i].source != 0; i++) {
		if (irq_vec[i].source == irq_source) {
			*stat = xdx_hdmicore_read(dev, irq_vec[i].status_reg);
			return TRUE;
		}
	}
	dev_dbg(xdev->dev, "IRQ source [%d] is not supported", irq_source);
	*stat = 0;
	return FALSE;
}

uint8_t set8(uint8_t data, uint8_t mask, uint8_t value)
{
	return (((value << first_bit_set(mask)) & mask) | (data & ~mask));
}

void xdx_hdmi_h2a_doorbell_w(struct hdmi_dev *hdmi, u64 value)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev,
			SYS_PANGU_REG_HDMIDOORBELL + 8,
			value & 0xffffffff);

	xdx_mmio_writel(xdev,
			SYS_PANGU_REG_HDMIDOORBELL + 0xc,
			(value >> 32) & 0xffffffff);
}

u32 xdx_hdmi_h2a_doorbell_r(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev,
			SYS_PANGU_REG_HDMIDOORBELL + 8);
}

void xdx_hdmi_h2c_doorbell_w(struct hdmi_dev *hdmi, u64 value)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev,
			SYS_PANGU_REG_HDMIDOORBELL,
			value & 0xffffffff);

	xdx_mmio_writel(xdev,
			SYS_PANGU_REG_HDMIDOORBELL + 4,
			(value >> 32) & 0xffffffff);
}

u32 xdx_hdmi_h2c_doorbell_r(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev,
			SYS_PANGU_REG_HDMIDOORBELL);
}
