/*************************************************************************/ /*!
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

@Copyright      Portions Copyright (c) Synopsys Ltd. All Rights Reserved
@License        Synopsys Permissive License

The Synopsys Software Driver and documentation (hereinafter "Software")
is an unsupported proprietary work of Synopsys, Inc. unless otherwise
expressly agreed to in writing between  Synopsys and you.

The Software IS NOT an item of Licensed Software or Licensed Product under
any End User Software License Agreement or Agreement for Licensed Product
with Synopsys or any supplement thereto.  Permission is hereby granted,
free of charge, to any person obtaining a copy of this software annotated
with this license and the Software, to deal in the Software without
restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject
to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT     LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

*/ /**************************************************************************/
#include <linux/i2c.h>
#include <linux/delay.h>

#include "xdx_hdmi_reg.h"
#include "xdx_hdmi.h"
#include "xdx_hdmi_accel.h"
#include "xdx_device.h"
#include "dm.h"

/*************************************************************************/ /*!
 EDID Information
*/ /**************************************************************************/
#define EDID_SIGNATURE                      0x00ffffffffffff00ull
#define EDID_BLOCK_SIZE                     128
#define EDID_CHECKSUM                       0 // per VESA spec, 8 bit checksum of base block
#define EDID_SLAVE_ADDRESS                  0x50
#define EDID_SEGMENT_ADDRESS                0x30
#define EDID_REFRESH_RATE                   60000
#define EDID_I2C_TIMEOUT_MS                 1000
#define AUDIO_READ_ADDR                     0x64

/*static void i2c_bus_clear(struct hdmi_dev *dev)
{
	dev_write_mask(dev, I2CM_CONTROL, I2CM_CONTROL_BUS_CLEAR_MASK, 1);
}*/

static int _write(struct hdmi_dev *dev, u8 src_addr, u8 *data, u8 slave_addr, u8 count)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i, timeout = I2CDDC_TIMEOUT;
	u32 status = 0, control0;
	//u8  slave = EDID_SLAVE_ADDRESS;
	u8 slave = slave_addr;
	u8 nbytes = count;
	u32 wrd0, wrd1;


	u32 mask[4] = {I2CM_INTERFACE_RDDATA0_0_MASK, I2CM_INTERFACE_RDDATA0_1_MASK,
		       I2CM_INTERFACE_RDDATA0_2_MASK, I2CM_INTERFACE_RDDATA0_3_MASK};

	dev_dbg(xdev->dev,"%s: I2C DDC Write for slave_ddr: 0x%x, src_addr: 0x%x, nbytes: %d\n", __func__, slave_addr, src_addr, nbytes);
	// Clear status
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_DONE_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_NACK_RCVD_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_ARB_LOST_MASK);

	nbytes = nbytes-1;

	control0 = ((u32)(nbytes - 1) << 20) & I2CM_INTERFACE_CONTROL0_NBYTES_MASK;
	control0 |= ((u8)src_addr << 12) & I2CM_INTERFACE_CONTROL0_ADDRESS_MASK;
	control0 |= ((u32)slave << 5) & I2CM_INTERFACE_CONTROL0_SLAVE_ADDR_MASK;
	control0 |= I2CM_INTERFACE_CONTROL0_WRITE_MASK;

	for (i = 0; i < nbytes; i++)
		dev_write_mask_shifted(dev, I2CM_INTERFACE_WRDATA0 + (4 * (i / 4)), mask[i % 4], data[i+1]);

	wrd0 = xdx_hdmicore_read(dev, I2CM_INTERFACE_WRDATA0);
	dev_dbg(xdev->dev,"wrd0: 0x%x\n", wrd0);
	wrd1 = xdx_hdmicore_read(dev, I2CM_INTERFACE_WRDATA1);
	dev_dbg(xdev->dev,"wrd1: 0x%x\n", wrd1);

	dev_write_mask(dev, I2CM_INTERFACE_CONTROL0, I2CM_INTERFACE_CONTROL0_OPERATION_MASK, control0);
	do{
		udelay(50);
		irq_read_stat(dev, MU_INT1, &status);
		status &= (MU_INT1_I2CM_DONE_MASK |
			MU_INT1_I2CM_NACK_RCVD_MASK |
			MU_INT1_I2CM_ARB_LOST_MASK);
	} while (status == 0 && (timeout--));

	// Clear status
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_DONE_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_NACK_RCVD_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_ARB_LOST_MASK);
	if (status & MU_INT1_I2CM_ARB_LOST_MASK || status & MU_INT1_I2CM_NACK_RCVD_MASK) {
		dev_dbg(xdev->dev,"%s: I2C DDC Write failed for i2cAddr 0x%x addr 0x%x nbytes %u",
			__func__, slave_addr, src_addr, nbytes);
		return -1;
	}

	if (status & MU_INT1_I2CM_DONE_MASK) {
		dev_dbg(xdev->dev,"%s done\n", __func__);
		return 0;
	}

	dev_dbg(xdev->dev,"%s: I2C DDC Write timeout", __func__);
	return -1;

}

static int _read(struct hdmi_dev *dev, u16 src_addr, u8 *value, u8 slave_addr, u8 count)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i, timeout = I2CDDC_TIMEOUT;
	u32 status = 0, control0, control1;
	u8 segment = 0;
	u8 slave = slave_addr;
	//u8 nbytes = 1;
	u8 nbytes = count;
	u8 pointer = 0;
	u32 mask[4] = {I2CM_INTERFACE_RDDATA0_0_MASK, I2CM_INTERFACE_RDDATA0_1_MASK,
		I2CM_INTERFACE_RDDATA0_2_MASK, I2CM_INTERFACE_RDDATA0_3_MASK};
	if(slave == EDID_SEGMENT_ADDRESS){
		segment = EDID_SEGMENT_ADDRESS;
	}
	dev_dbg(xdev->dev,"%s: I2C DDC Read for slave_ddr: 0x%x, src_addr: 0x%x, nbytes: %d,segment:0x%x\n", __func__, slave_addr, src_addr, nbytes, segment);
	// Clear status
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_DONE_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_NACK_RCVD_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_ARB_LOST_MASK);
	control1 = ((u32)pointer << 7) & I2CM_INTERFACE_CONTROL1_SEG_PTR_MASK;
	control1 |= (segment & I2CM_INTERFACE_CONTROL1_SEG_ADDR_MASK);
	control0 = ((u32)(nbytes - 1) << 20) & I2CM_INTERFACE_CONTROL0_NBYTES_MASK;
	control0 |= ((u32)src_addr << 12) & I2CM_INTERFACE_CONTROL0_ADDRESS_MASK;
	control0 |= ((u32)slave << 5) & I2CM_INTERFACE_CONTROL0_SLAVE_ADDR_MASK;

	if (pointer)
		control0 |= I2CM_INTERFACE_CONTROL0_EXTENDED_READ_MASK;
	else
		control0 |= I2CM_INTERFACE_CONTROL0_READ_MASK;

	xdx_hdmicore_write(dev, I2CM_INTERFACE_CONTROL1, control1);
        for (i = 0; i < nbytes; i++)
	    dev_write_mask(dev, I2CM_INTERFACE_CONTROL0,
			I2CM_INTERFACE_CONTROL0_OPERATION_MASK, control0);
	do {
		udelay(10);
		irq_read_stat(dev, MU_INT1, &status);
		status &= (MU_INT1_I2CM_DONE_MASK |
			   MU_INT1_I2CM_NACK_RCVD_MASK |
			   MU_INT1_I2CM_ARB_LOST_MASK);
	} while (status == 0 && (timeout--));
	// Clear status
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_DONE_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_NACK_RCVD_MASK);
	irq_clear_bit(dev, MU_INT1, MU_INT1_I2CM_ARB_LOST_MASK);

	if (status & MU_INT1_I2CM_ARB_LOST_MASK || status & MU_INT1_I2CM_NACK_RCVD_MASK) {
		dev_dbg(xdev->dev,"%s: I2C DDC Read failed for i2cAddr 0x%x seg 0x%x pointer 0x%x src_addr 0x%x nbytes %u",
		       __func__, slave, segment, pointer, src_addr, nbytes);
		return -1;
	}

	if (status & MU_INT1_I2CM_DONE_MASK) {
		for (i = 0; i < nbytes; i++)
			value[i] = (u8)dev_read_mask(dev, I2CM_INTERFACE_RDDATA0 + (4 * (i / 4)),
				mask[i % 4]);
		return 0;
	}

	dev_dbg(xdev->dev,"%s: I2C DDC Read timeout", __func__);
	return -1;
}

static int hdmi_i2c_read(struct hdmi_dev *dev, struct i2c_msg *msg, u16 src_addr)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm); 
	unsigned int i, nbytes;
	int tries, status = 0;
	u8 *msg_data = msg->buf;
	u32 len =msg->len;
	dev_dbg(xdev->dev,"i2c read: src_addr: %x len:%x msg->addr:%x",src_addr, len, msg->addr);
	for (i = 0; i < len;){
		tries = 3;
		if ((len - i) >= 16)
			nbytes = 16;
		else
			nbytes = len - i;

		do {
			status = _read(dev, src_addr + i, &(msg_data[i]), msg->addr, nbytes);
		} while (status && tries--);
		if (status) //Error after 3 failed writes
			return status;
		i += nbytes;
	}
	return 0;
}



static int hdmi_i2c_write(struct hdmi_dev *dev, struct i2c_msg *msg)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	unsigned int i, nbytes;
	int tries, status = 0;
	u8 *msg_data = msg->buf;
	u16 len = msg->len;

	u8 src_addr = msg->buf[0];

	//dev_dbg(xdev->dev,"i2c write: src_addr %x len %x data %x", offset, msg_len, *msg_data);
	for (i = 0; i < len;) {
		tries = 3;
		if ((len - i) >= 16)
			nbytes = 16;
		else
			nbytes = len - i;

		dev_dbg(xdev->dev,"nbytes: %d\n", nbytes);
		do {
			status = _write(dev, src_addr, &msg_data[i], msg->addr, nbytes);
		} while (status && tries--);

		if (status) //Error after 3 failed writes
			return status;

		i += nbytes;
	}

	return 0;
}

static void _standard_mode_config(struct hdmi_dev *dev, u16 high, u16 low)
{
	u32 val = (((u32)high << 16) & 0xFFFF0000) | ((u32)low & 0x0000FFFF);

	xdx_hdmicore_write(dev, I2CM_SM_SCL_CONFIG, val);
}

static void _i2cddc_fast_mode(struct hdmi_dev *dev)
{
	dev_write_mask_shifted(dev, I2CM_INTERFACE_CONTROL0,
			       I2CM_INTERFACE_CONTROL0_FAST_MODE_MASK, dev->i2c->i2cm_fast_mode);
	dev_write_mask_shifted(dev, SCDC_CONFIG,
			       SCDC_CONFIG_I2C_FM_EN_MASK, dev->i2c->scdc_fast_mode);
}

static void _fast_mode_config(struct hdmi_dev *dev, u16 high, u16 low)
{
	u32 val = (((u32)high << 16) & 0xFFFF0000) | ((u32)low & 0x0000FFFF);

	xdx_hdmicore_write(dev, I2CM_FM_SCL_CONFIG, val);
}

static void i2cddc_clk_config(struct hdmi_dev *dev)
{
	u16 s_low, s_high, f_low, f_high;
	u32 tmp;

	tmp = REFCLK_HZ / dev->i2c->slow_speed;

	s_high = tmp / 2;
	s_low = tmp / 2;
	if (tmp % 2 != 0)
		s_low++;

	tmp = REFCLK_HZ / dev->i2c->fast_speed;

	f_high = tmp / 2;
	f_low = tmp / 2;
	if (tmp % 2 != 0)
		f_low++;
        s_high = 0x5dc;
	s_low = 0x5dc;
	f_high = 0x177;
	f_low = 0x177;
	_standard_mode_config(dev, s_high, s_low);
	_fast_mode_config(dev, f_high, f_low);
	_i2cddc_fast_mode(dev);
}

static int hdmi_i2c_master_xfer(struct i2c_adapter *adap, struct i2c_msg *msgs, int num)
{
	struct hdmi_dev *hdmi = i2c_get_adapdata(adap);
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i, j;
	int ret = 0;
	u16 src_addr = 0;

	spin_lock(&hdmi->hdmi_reg_lock);
	for (i = 0; i < num; i++) {
		struct i2c_msg *msg = &msgs[i];
		dev_dbg(xdev->dev, "[msg: %d out of %d][len: %d][type: %s][slave addr: 0x%x][buf0: 0x%x][src_addr:0x%x]\n",
			i+1, num, msg->len, (msg->flags & I2C_M_RD) ? "read" : "write", msg->addr, msg->buf[0],src_addr);
#if 1
		for (j=0; j< msg->len; j++) {
			dev_dbg(xdev->dev, "buf[%d]: 0x%x\n", j, msg->buf[j]);
		}
#endif
		if (msg->flags & I2C_M_RD){
			//ret = hdmi_i2c_read(hdmi, msg, src_addr, segment,msg->len,msg->buf);
			ret = hdmi_i2c_read(hdmi, msg, src_addr);
		} else {
			src_addr = msg->buf[0];
			// no need msg-len
			if(msg->len > 1){
				//ret = hdmi_i2c_write(hdmi, msg, src_addr, segment, msg->len-1, &msg->buf[1]);
				ret = hdmi_i2c_write(hdmi, msg);
			}
		}
	}

	if(!ret)
	{
	    spin_unlock(&hdmi->hdmi_reg_lock);
	    return num;
	}
	spin_unlock(&hdmi->hdmi_reg_lock);

	return ret;
}

static u32 hdmi_i2c_func(struct i2c_adapter *adapter)
{
	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
}

static const struct i2c_algorithm hdmi_algo = {
	.master_xfer	= hdmi_i2c_master_xfer,
	.functionality	= hdmi_i2c_func,
};

int xdx_hdmi_i2c_init(struct hdmi_dev *hdmi)
{
	struct xdx_output *output = hdmi_to_output(hdmi);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int ret;

	mutex_init(&hdmi->i2c->lock);
	init_completion(&hdmi->i2c->comp);
	hdmi->i2c->slow_speed = 100000;
	hdmi->i2c->fast_speed = 200000;
	hdmi->i2c->i2cm_fast_mode = FALSE;
	hdmi->i2c->scdc_fast_mode = FALSE;
	i2cddc_clk_config(hdmi);

	hdmi->i2c->adap.class = I2C_CLASS_DDC;
	hdmi->i2c->adap.owner = THIS_MODULE;
	hdmi->i2c->adap.dev.parent = xdev->dev;
	hdmi->i2c->adap.dev.of_node = xdev->dev->of_node;
	hdmi->i2c->adap.algo = &hdmi_algo;
	snprintf(hdmi->i2c->adap.name, sizeof(hdmi->i2c->adap.name),
		 "XDX HDMI");
	i2c_set_adapdata(&hdmi->i2c->adap, hdmi);

	ret = i2c_add_adapter(&hdmi->i2c->adap);
	if (ret) {
		dev_err(xdev->dev, "Failed to add %s i2c adapter\n",
			hdmi->i2c->adap.name);
		return ret;
	}

	hdmi->i2c->segment_ptr = 0;
	hdmi->i2c->ddc_addr = 0;

	return ret;
}

void xdx_hdmi_i2c_deinit(struct hdmi_dev *hdmi)
{
	i2c_del_adapter(&hdmi->i2c->adap);
}
