// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2010 Synopsys, Inc. and/or its affiliates
 * Synopsys DesignWare HDMIQP TX controller driver
 */

#include "xdx_hdmi_reg.h"
#include "xdx_hdmi.h"
#include "xdx_hdmi_accel.h"
#include "main_unit.h"


static void mu_avp_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_MASK, bit);
}

static void mu_avp_datapath_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_MASK, bit);
}

static void mu_avp_datapath_video_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_VIDEO_MASK, bit);
}

static void mu_avp_datapath_frl_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_FRL_MASK, bit);
}

static void mu_avp_datapath_tmds_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_TMDS_MASK, bit);
}

static void mu_packets_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_PACKET_MASK, bit);
}

static void mu_audio_packets_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_PACKET_AUDIO_MASK, bit);
}

static void mu_metadata_packets_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_AVP_DATAPATH_PACKET_METADATA_MASK, bit);
}

static void mu_cec_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_CEC_MASK, bit);
}

static void mu_cec_ctrl_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_CEC_CTRL_MASK, bit);
}

static void mu_earc_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_EARCRX_MASK, bit);
}

static void mu_earc_ctrl_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_EARCRX_CTRL_MASK, bit);
}

static u32 mu_get_cmu_status(struct hdmi_dev *dev)
{
	return xdx_hdmicore_read(dev, MU_CMU_STATUS);
}

void mu_i2c_enable(struct hdmi_dev *dev, u8 bit)
{
	dev_write_mask_shifted(dev, MU_SWDIS, MU_SWDIS_I2C_MASK, bit);
}

void mu_reset_avp(struct hdmi_dev *dev)
{
	dev_write_mask_shifted(dev, MU_SWRSTREQ, MU_SWRSTREQ_AVP_MASK, 1);
}

void xdx_reset_avp(struct hdmi_dev *dev)
{
	xdx_hdmicore_write(dev, MU_SWRSTREQ, 0x4b);
}
void xdx_enable_scramb(struct hdmi_dev *dev,bool istrue)
{
     if(istrue)
         xdx_hdmicore_write(dev,SCRAMB_CONFIG0,1);
     else
         xdx_hdmicore_write(dev,SCRAMB_CONFIG0,0);
}

void mu_reset_avp_datapath(struct hdmi_dev *dev)
{
	dev_write_mask_shifted(dev, MU_SWRSTREQ, MU_SWRSTREQ_AVP_DATAPATH_MASK, 1);
}

void mu_disable(struct hdmi_dev *dev)
{
	mu_avp_enable(dev, 1);
	mu_avp_datapath_enable(dev, 1);
	mu_avp_datapath_video_enable(dev, 1);
	mu_avp_datapath_frl_enable(dev, 1);
	mu_avp_datapath_tmds_enable(dev, 1);
	mu_packets_enable(dev, 1);
	mu_audio_packets_enable(dev, 1);
	mu_metadata_packets_enable(dev, 1);
	mu_i2c_enable(dev, 1);
}

void mu_enable(struct hdmi_dev *dev)
{
	dev_write_mask_shifted(dev, MU_CMU_CONFIG1,
			       MU_CMU_CONFIG1_IPICLK_MARGIN_MASK, 2);
	dev_write_mask_shifted(dev, MU_CMU_CONFIG1,
			       MU_CMU_CONFIG1_VIDQPCLK_MARGIN_MASK, 2);
	dev_write_mask_shifted(dev, MU_CMU_CONFIG2,
			       MU_CMU_CONFIG2_LINKQPCLK_MARGIN_MASK, 2);

	mu_avp_enable(dev, 0);
	mu_avp_datapath_enable(dev, 0);
	//mu_avp_datapath_video_enable(dev, dev->frl_on ? 1 : 0);
	//mu_avp_datapath_frl_enable(dev, dev->frl_on ? 0 : 1);
	//mu_avp_datapath_tmds_enable(dev, dev->frl_on ? 1 : 0);
	mu_avp_datapath_video_enable(dev,  0);
	mu_avp_datapath_frl_enable(dev, 1);
	mu_avp_datapath_tmds_enable(dev, 0);
	mu_packets_enable(dev, 0);
	//mu_audio_packets_enable(dev, dev->audio_on ? 0 : 1);
	mu_audio_packets_enable(dev, 1);
	mu_metadata_packets_enable(dev, 0);
	mu_i2c_enable(dev, 0);
}

void mu_enable_frl(struct hdmi_dev *dev)
{
	mu_avp_datapath_video_enable(dev, 0);
}

void mu_disable_frl(struct hdmi_dev *dev)
{
	mu_avp_datapath_video_enable(dev, 1);
}

void mu_enable_tmds(struct hdmi_dev *dev)
{
	mu_avp_datapath_frl_enable(dev, 1);
	mu_avp_datapath_tmds_enable(dev, 0);
	mu_avp_datapath_video_enable(dev, 0);
}

void mu_enable_cec(struct hdmi_dev *dev)
{
	mu_cec_enable(dev, 0);
	mu_cec_ctrl_enable(dev, 0);
}

void mu_disable_cec(struct hdmi_dev *dev)
{
	mu_cec_enable(dev, 1);
	mu_cec_ctrl_enable(dev, 1);
}

void mu_enable_earc(struct hdmi_dev *dev)
{
	mu_earc_enable(dev, 0);
	mu_earc_ctrl_enable(dev, 0);
}

void mu_disable_earc(struct hdmi_dev *dev)
{
	mu_earc_enable(dev, 1);
	mu_earc_ctrl_enable(dev, 1);
}

bool mu_core_regbank_ready(struct hdmi_dev *dev)
{
	u32 state;

	irq_read_stat(dev, MU_INT0, &state);

	return (state & MU_INT0_APB_REGBANK_READY_MASK) ? TRUE : FALSE;
}

void mu_core_timerbase_config(struct hdmi_dev *dev)
{
	dev_write_mask(dev, MU_TIMER_BASE_CONFIG,
		       MU_TIMER_BASE_CONFIG_TIMER_BASE_MASK, REFCLK_HZ);
}

bool mu_core_timerbase_ready(struct hdmi_dev *dev)
{
	return xdx_hdmicore_read(dev, MU_TIMER_BASE_STATUS);
}

bool mu_check_link_clock_status(struct hdmi_dev *dev)
{
	u32 cmu_lock;

	cmu_lock = mu_get_cmu_status(dev);

	if (cmu_lock & MU_CMU_STATUS_LINKQPCLK_OFF_MASK ||
	    (!(cmu_lock & MU_CMU_STATUS_LINKQPCLK_LOCK_MASK))) {
		return FALSE;
	}

	return TRUE;
}

bool mu_check_avp_clocks_status(struct hdmi_dev *dev)
{
	u32 cmu_lock;

	cmu_lock = mu_get_cmu_status(dev);

	if (cmu_lock & MU_CMU_STATUS_IPICLK_OFF_MASK ||
	    (!(cmu_lock & MU_CMU_STATUS_IPICLK_LOCK_MASK))) {
		return FALSE;
	}
	if (cmu_lock & MU_CMU_STATUS_VIDQPCLK_OFF_MASK ||
	    (!(cmu_lock & MU_CMU_STATUS_VIDQPCLK_LOCK_MASK))) {
		return FALSE;
	}
	if (cmu_lock & MU_CMU_STATUS_LINKQPCLK_OFF_MASK ||
	    (!(cmu_lock & MU_CMU_STATUS_LINKQPCLK_LOCK_MASK))) {
		return FALSE;
	}
	//lqs audio
	//if (dev->audio_on) {
	//	if (cmu_lock & MU_CMU_STATUS_AUDCLK_OFF_MASK ||
	//	    (!(cmu_lock & MU_CMU_STATUS_AUDCLK_LOCK_MASK))) {
	//		return FALSE;
	//	}
	//}

	return TRUE;
}
#if 0
void mu_core_handler(struct hdmi_dev *dev)
{
	struct xdx_output *output = hdmi_to_output(dev);
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 irq_state, cmu_lock;

	irq_read_stat(dev, MU_INT0, &irq_state);
	cmu_lock = mu_get_cmu_status(dev);

	if (irq_state & MU_INT0_IPICLK_LOCKED_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_IPICLK_LOCK_MASK)
			dev_dbg(xdev->dev, "%s: IPICLK LOCKED CHANGED - "GREEN"LOCKED"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: IPICLK LOCKED CHANGED - "RED"NOT LOCKED"BLACK, __func__);
	}

	if (irq_state & MU_INT0_IPICLK_OFF_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_IPICLK_OFF_MASK)
			dev_dbg(xdev->dev, "%s: IPICLK OFF CHANGED - "RED"OFF"BLACK, __func__);
		else
			dev_err(xdev->dev, "%s: IPICLK OFF CHANGED - "GREEN"NOT OFF"BLACK, __func__);
	}

	if (irq_state & MU_INT0_VIDQPCLK_LOCKED_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_VIDQPCLK_LOCK_MASK)
			dev_dbg(xdev->dev, "%s: VIDQPCLK LOCKED CHANGED - "GREEN"LOCKED"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: VIDQPCLK LOCKED CHANGED - "RED"NOT LOCKED"BLACK, __func__);
	}

	if (irq_state & MU_INT0_VIDQPCLK_OFF_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_VIDQPCLK_OFF_MASK)
			dev_dbg(xdev->dev, "%s: VIDQPCLK OFF CHANGED - "RED"OFF"BLACK, __func__);
		else
			dev_err(xdev->dev, "%s: VIDQPCLK OFF CHANGED - "GREEN"NOT OFF"BLACK, __func__);
	}

	if (irq_state & MU_INT0_LINKQPCLK_LOCKED_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_LINKQPCLK_LOCK_MASK)
			dev_dbg(xdev->dev, "%s: LINKQPCLK LOCKED CHANGED - "GREEN"LOCKED"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: LINKQPCLK LOCKED CHANGED - "RED"NOT LOCKED"BLACK, __func__);
	}

	if (irq_state & MU_INT0_LINKQPCLK_OFF_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_LINKQPCLK_OFF_MASK)
			dev_dbg(xdev->dev, "%s: LINKQPCLK OFF CHANGED - "RED"OFF"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: LINKQPCLK OFF CHANGED - "GREEN"NOT OFF"BLACK, __func__);
	}

	if (irq_state & MU_INT0_AUDCLK_LOCKED_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_AUDCLK_LOCK_MASK)
			dev_dbg(xdev->dev, "%s: AUDCLK LOCKED CHANGED - "GREEN"LOCKED"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: AUDCLK LOCKED CHANGED - "RED"NOT LOCKED"BLACK, __func__);
	}

	if (irq_state & MU_INT0_AUDCLK_OFF_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_AUDCLK_OFF_MASK)
			dev_dbg(xdev->dev, "%s: AUDCLK OFF CHANGED - "RED"OFF"BLACK, __func__);
		else
			dev_err(xdev->dev, "%s: AUDCLK OFF CHANGED - "GREEN"NOT OFF"BLACK, __func__);
	}

	if (irq_state & MU_INT0_EARC_BPCLK_LOCKED_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_EARC_BPCLK_LOCK_MASK)
			dev_dbg(xdev->dev, "%s: BPCLK LOCKED CHANGED - "GREEN"LOCKED"BLACK, __func__);
		else
			dev_dbg(xdev->dev, "%s: BPCLK LOCKED CHANGED - "RED"NOT LOCKED"BLACK, __func__);
	}

	if (irq_state & MU_INT0_EARC_BPCLK_OFF_CHG_MASK) {
		if (cmu_lock & MU_CMU_STATUS_EARC_BPCLK_OFF_MASK)
			dev_dbg(xdev->dev, "%s: BPCLK OFF CHANGED - "RED"OFF"BLACK, __func__);
		else
			dev_err(xdev->dev, "%s: BPCLK OFF CHANGED - "GREEN"NOT OFF"BLACK, __func__);
	}

	if (irq_state & MU_INT0_EARC_BPCLK_VALID_STABLE_CHG_MASK) {
		if (dev_read_mask(dev, MU_STATUS, MU_STATUS_BPCLK_VALID_MASK)) {
			dev_dbg(xdev->dev, "%s: BPCLK VALID CHANGED - "GREEN"STABLE"BLACK, __func__);
			earc_dmac_ctrl_enable(dev, TRUE);
		} else {
			dev_dbg(xdev->dev, "%s: BPCLK VALID CHANGED - "RED"NOT STABLE"BLACK, __func__);
		}
	}

	if (irq_state & MU_INT0_TIMER_BASE_LOCKED_MASK)
		dev_dbg(xdev->dev, "%s: TIMER BASE - "GREEN"LOCKED"BLACK, __func__);

	if (irq_state & MU_INT0_RESET_MANAGER_STATUS_END_MASK)
		dev_dbg(xdev->dev, "%s: RESET MANAGER STATUS END", __func__);

	if (irq_state & MU_INT0_APB_REGBANK_READY_MASK)
		dev_dbg(xdev->dev, "%s: REGBANK - "GREEN"READY"BLACK, __func__);
}
#endif
