#ifndef __XDX_HDMI_REG_H__
#define __XDX_HDMI_REG_H__

/*****************************************************************************
 *                                                                           *
 *                          Identification Registers                         *
 *                                                                           *
 *****************************************************************************/

//Core Identification Register
#define CORE_ID					0x00000000

//IP Version Number Register
#define VER_NUMBER				0x00000004

//IP Version Type Register
#define VER_TYPE				0x00000008

//Core Configuration Identification Register
#define CONFIG_REG				0x0000000C
#define CONFIG_REG_AVP_MASK			0x00000001
#define CONFIG_REG_PR_MASK			0x00000010
#define CONFIG_REG_HDCP14_MASK			0x00000100
#define CONFIG_REG_HDCP_DKSET_MASK		0x00000200
#define CONFIG_REG_HDCP_SW_SHA1_MASK		0x00000400
#define CONFIG_REG_FRL_MASK			0x00001000
#define CONFIG_REG_EEMP_MASK			0x00002000
#define CONFIG_REG_VTEM_MASK			0x00004000
#define CONFIG_REG_CVTEM_MASK			0x00008000
#define CONFIG_REG_AUD_3D_MASK			0x00100000
#define CONFIG_REG_AUD_HBR_MASK			0x00200000
#define CONFIG_REG_AUD_MS_MASK			0x00400000
#define CONFIG_REG_AUD_UD_MASK			0x00800000
#define CONFIG_REG_EARCRX_MASK			0x01000000
#define CONFIG_REG_CEC_MASK			0x10000000

//Core Timestamp HHMM Register
#define CORE_TIMESTAMP_HHMM			0x00000014

//Core Timestamp MMDD Register
#define CORE_TIMESTAMP_MMDD			0x00000018

//Core Timestamp YYYY Register
#define CORE_TIMESTAMP_YYYY			0x0000001C

/*****************************************************************************
 *                                                                           *
 *                         Main Controller Registers                         *
 *                                                                           *
 *****************************************************************************/

//Main Unit Global Software Reset Request Register
#define MU_SWRSTREQ						0x00000040
#define MU_SWRSTREQ_MASTER_MASK					0x00000001
#define MU_SWRSTREQ_MASTER_NORB_MASK				0x00000002
#define MU_SWRSTREQ_AVP_MASK					0x00000020
#define MU_SWRSTREQ_AVP_DATAPATH_MASK				0x00000040
#define MU_SWRSTREQ_AVP_DATAPATH_VIDEO_MASK			0x00000080
#define MU_SWRSTREQ_AVP_DATAPATH_HDCP_KEY_MASK			0x00000100
#define MU_SWRSTREQ_AVP_DATAPATH_PACKET_MASK			0x00000200
#define MU_SWRSTREQ_AVP_DATAPATH_PACKET_AUDIO_MASK		0x00000400
#define MU_SWRSTREQ_AVP_DATAPATH_PACKET_METADATA_MASK		0x00000800
#define MU_SWRSTREQ_I2C_MASK					0x00010000
#define MU_SWRSTREQ_CEC_MASK					0x00020000
#define MU_SWRSTREQ_CEC_CTRL_MASK				0x00040000
#define MU_SWRSTREQ_EARCRX_MASK					0x02000000
#define MU_SWRSTREQ_EARCRX_DMAC_MASK				0x04000000
#define MU_SWRSTREQ_EARCRX_CMDC_MASK				0x08000000

//Main Unit Global SW Disable Register
#define MU_SWDIS						0x00000044
#define MU_SWDIS_AVP_MASK					0x00000010
#define MU_SWDIS_AVP_DATAPATH_MASK				0x00000020
#define MU_SWDIS_AVP_DATAPATH_VIDEO_MASK			0x00000040
#define MU_SWDIS_AVP_DATAPATH_FRL_MASK				0x00000080
#define MU_SWDIS_AVP_DATAPATH_TMDS_MASK				0x00000100
#define MU_SWDIS_AVP_DATAPATH_PACKET_MASK			0x00000200
#define MU_SWDIS_AVP_DATAPATH_PACKET_AUDIO_MASK			0x00000400
#define MU_SWDIS_AVP_DATAPATH_PACKET_METADATA_MASK		0x00000800
#define MU_SWDIS_I2C_MASK					0x00010000
#define MU_SWDIS_CEC_MASK					0x00020000
#define MU_SWDIS_CEC_CTRL_MASK					0x00040000
#define MU_SWDIS_EARCRX_MASK					0x02000000
#define MU_SWDIS_EARCRX_CTRL_MASK				0x04000000

//Main Unit Reset Manager Config 0 Register
#define MU_RSTMGR_CFG0						0x00000048
#define MU_RSTMGR_CFG0_IPICLK_OVR_EN_MASK			0x00000001
#define MU_RSTMGR_CFG0_VIDQPCLK_OVR_EN_MASK			0x00000002
#define MU_RSTMGR_CFG0_LINKQPCLK_OVR_EN_MASK			0x00000004
#define MU_RSTMGR_CFG0_AUDCLK_OVR_EN_MASK			0x00000008
#define MU_RSTMGR_CFG0_EARC_BPCLK_OVR_EN_MASK			0x00000010
#define MU_RSTMGR_CFG0_IPICLK_OVR_VAL_MASK			0x00000100
#define MU_RSTMGR_CFG0_VIDQPCLK_OVR_VAL_MASK			0x00000200
#define MU_RSTMGR_CFG0_LINKQPCLK_OVR_VAL_MASK			0x00000400
#define MU_RSTMGR_CFG0_AUDCLK_OVR_VAL_MASK			0x00000800
#define MU_RSTMGR_CFG0_EARC_BPCLK_OVR_VAL_MASK			0x00001000

//Main Unit Reset Manager Status 0 Register
#define MU_RSTMGR_STS0						0x00000050
#define MU_RSTMGR_STS0_MASTER_REFCLK_MASK			0x00000001
#define MU_RSTMGR_STS0_MASTER_IPICLK_MASK			0x00000002
#define MU_RSTMGR_STS0_MASTER_VIDQPCLK_MASK			0x00000004
#define MU_RSTMGR_STS0_MASTER_LINKQPCLK_MASK			0x00000008
#define MU_RSTMGR_STS0_MASTER_AUDCLK_MASK			0x00000010
#define MU_RSTMGR_STS0_MASTER_EARC_BPCLK_MASK			0x00000020
#define MU_RSTMGR_STS0_MASTER_APB_PCLK_MASK			0x00000040
#define MU_RSTMGR_STS0_MASTER_NORB_REFCLK_MASK			0x00000080
#define MU_RSTMGR_STS0_MASTER_NORB_IPICLK_MASK			0x00000100
#define MU_RSTMGR_STS0_MASTER_NORB_VIDQPCLK_MASK		0x00000200
#define MU_RSTMGR_STS0_MASTER_NORB_LINKQPCLK_MASK		0x00000400
#define MU_RSTMGR_STS0_MASTER_NORB_AUDQPCLK_MASK		0x00000800
#define MU_RSTMGR_STS0_MASTER_NORB_EARC_BPCLK_MASK		0x00001000
#define MU_RSTMGR_STS0_MASTER_NORB_APB_PCLK_MASK		0x00002000
#define MU_RSTMGR_STS0_AVP_REFCLK_MASK				0x00004000
#define MU_RSTMGR_STS0_AVP_IPICLK_MASK				0x00008000
#define MU_RSTMGR_STS0_AVP_VIDQPCLK_MASK			0x00010000
#define MU_RSTMGR_STS0_AVP_LINKQPCLK_MASK			0x00020000
#define MU_RSTMGR_STS0_AVP_AUDCLK_MASK				0x00040000
#define MU_RSTMGR_STS0_AVP_APB_PCLK_MASK			0x00080000
#define MU_RSTMGR_STS0_AVP_DATAPATH_REFCLK_MASK			0x00100000
#define MU_RSTMGR_STS0_AVP_DATAPATH_IPICLK_MASK			0x00200000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDQPCLK_MASK		0x00400000
#define MU_RSTMGR_STS0_AVP_DATAPATH_LINKQPCLK_MASK		0x00800000
#define MU_RSTMGR_STS0_AVP_DATAPATH_AUDCLK_MASK			0x01000000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDEO_REFCLK_MASK		0x02000000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDEO_IPICLK_MASK		0x04000000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDEO_VIDQPCLK_MASK		0x08000000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDEO_LINKQPCLK_MASK	0x10000000
#define MU_RSTMGR_STS0_AVP_DATAPATH_VIDEO_AUDCLK_MASK		0x20000000

//Main Unit Reset Manager Status 1 Register
#define MU_RSTMGR_STS1						0x00000054
#define MU_RSTMGR_STS1_AVP_DATAPATH_FRL_VIDQPCLK_MASK		0x00000001
#define MU_RSTMGR_STS1_AVP_DATAPATH_FRL_LINKQPCLK_MASK		0x00000002
#define MU_RSTMGR_STS1_AVP_DATAPATH_TMDS_VIDQPCLK_MASK		0x00000004
#define MU_RSTMGR_STS1_AVP_DATAPATH_TMDS_LINKQPCLK_MASK		0x00000008
#define MU_RSTMGR_STS1_AVP_DATAPATH_HDCP_LINKQPCLK_MASK		0x00000010
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_VIDQPCLK_MASK	0x00000020
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_AUDCLK_MASK		0x00000040
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_LINKQPCLK_MASK	0x00000080
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_AUDIO_AUDCLK_MASK	0x00000100
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_AUDIO_VIDQPCLK_MASK	0x00000200
#define MU_RSTMGR_STS1_AVP_DATAPATH_PACKET_MTDATA_VIDQPCLK_MASK	0x00000400
#define MU_RSTMGR_STS1_I2C_REFCLK_MASK				0x00000800
#define MU_RSTMGR_STS1_I2C_LINKQPCLK_MASK			0x00001000
#define MU_RSTMGR_STS1_CEC_REFCLK_MASK				0x00002000
#define MU_RSTMGR_STS1_CEC_CTRL_REFCLK_MASK			0x00004000
#define MU_RSTMGR_STS1_EARC_BPCLK_MASK				0x00008000
#define MU_RSTMGR_STS1_EARC_REFCLK_MASK				0x00010000
#define MU_RSTMGR_STS1_EARC_APB_PCLK_MASK			0x00020000
#define MU_RSTMGR_STS1_EARC_CTRL_BPCLK_MASK			0x00040000
#define MU_RSTMGR_STS1_EARC_CTRL_REFCLK_MASK			0x00080000
#define MU_RSTMGR_STS1_EARC_DMAC_BPCLK_MASK			0x00100000
#define MU_RSTMGR_STS1_EARC_CMDC_REFCLK_MASK			0x00200000
#define MU_RSTMGR_STS1_APB_APB_PCLK_MASK			0x00400000
#define MU_RSTMGR_STS1_APB_IPICLK_MASK				0x00800000
#define MU_RSTMGR_STS1_APB_VIDQPCLK_MASK			0x01000000
#define MU_RSTMGR_STS1_APB_LINKQPCLK_MASK			0x02000000
#define MU_RSTMGR_STS1_APB_AUDCLK_MASK				0x04000000
#define MU_RSTMGR_STS1_APB_EARC_BPCLK_MASK			0x08000000
#define MU_RSTMGR_STS1_APB_REFCLK_MASK				0x10000000

//Main Unit Reset Manager Status 2 Register
#define MU_RSTMGR_STS2						0x00000058
#define MU_RSTMGR_STS2_IPICLK_OVR_STAT_MASK			0x00000001
#define MU_RSTMGR_STS2_VIDQPCLK_OVR_STAT_MASK			0x00000002
#define MU_RSTMGR_STS2_LINKQP_OVR_STAT_MASK			0x00000004
#define MU_RSTMGR_STS2_AUDCLK_OVR_STAT_MASK			0x00000008
#define MU_RSTMGR_STS2_EARC_BPCLK_OVR_STAT_MASK			0x00000010

//Main Unit Timer Base Config Register
#define MU_TIMER_BASE_CONFIG					0x00000080
#define MU_TIMER_BASE_CONFIG_TIMER_BASE_MASK			0x1FFFFFFF

//Main Unit Timer Base Status Register
#define MU_TIMER_BASE_STATUS					0x00000084
#define MU_TIMER_BASE_STATUS_TIMER_BASE_LOCK_MASK		0x00000001

//Main Unit CMU Config 0 Register
#define MU_CMU_CONFIG0						0x000000A0
#define MU_CMU_CONFIG0_CMU_EN_MASK				0x00000001

//Main Unit CMU Config 1 Register
#define MU_CMU_CONFIG1						0x000000A4
#define MU_CMU_CONFIG1_IPICLK_MARGIN_MASK			0x00007FFF
#define MU_CMU_CONFIG1_VIDQPCLK_MARGIN_MASK			0x7FFF0000

//Main Unit CMU Config 2 Register
#define MU_CMU_CONFIG2						0x000000A8
#define MU_CMU_CONFIG2_LINKQPCLK_MARGIN_MASK			0x00007FFF
#define MU_CMU_CONFIG2_AUDCLK_MARGIN_MASK			0x7FFF0000

//Main Unit CMU Config 3 Register
#define MU_CMU_CONFIG3						0x000000AC
#define MU_CMU_CONFIG3_EARC_BPCLK_MARGIN_MASK			0x00007FFF

//Main Unit CMU Status Register
#define MU_CMU_STATUS						0x000000B0
#define MU_CMU_STATUS_IPICLK_LOCK_MASK				0x00000001
#define MU_CMU_STATUS_IPICLK_OFF_MASK				0x00000002
#define MU_CMU_STATUS_VIDQPCLK_LOCK_MASK			0x00000004
#define MU_CMU_STATUS_VIDQPCLK_OFF_MASK				0x00000008
#define MU_CMU_STATUS_LINKQPCLK_LOCK_MASK			0x00000010
#define MU_CMU_STATUS_LINKQPCLK_OFF_MASK			0x00000020
#define MU_CMU_STATUS_AUDCLK_LOCK_MASK				0x00000040
#define MU_CMU_STATUS_AUDCLK_OFF_MASK				0x00000080
#define MU_CMU_STATUS_EARC_BPCLK_LOCK_MASK			0x00000100
#define MU_CMU_STATUS_EARC_BPCLK_OFF_MASK			0x00000200
#define MU_CMU_STATUS_ALL_MASK					0x000003FF

//Main Unit CMU IPICLK Freq Register
#define MU_CMU_IPICLK_FREQ					0x000000B4
#define MU_CMU_IPICLK_FREQ_MASK					0x000FFFFF

//Main Unit CMU VIDQPCLK Freq Register
#define MU_CMU_VIDQPCLK_FREQ					0x000000B8
#define MU_CMU_VIDQPCLK_FREQ_MASK				0x000FFFFF

//Main Unit CMU LINKQPCLK Freq Register
#define MU_CMU_LINKQPCLK_FREQ					0x000000BC
#define MU_CMU_LINKQPCLK_FREQ_MASK				0x000FFFFF

//Main Unit CMU AUDCLK Freq Register
#define MU_CMU_AUDCLK_FREQ					0x000000C0
#define MU_CMU_AUDCLK_FREQ_MASK					0x000FFFFF

//Main Unit CMU eARC BPCLK Freq Register
#define MU_CMU_EARC_BPCLK_FREQ					0x000000C4
#define MU_CMU_EARC_BPCLK_FREQ_MASK				0x000FFFFF

//Main Unit Status Register
#define MU_STATUS						0x00000180
#define MU_STATUS_BPCLK_VALID_MASK				0x00000001

/*****************************************************************************
 *                                                                           *
 *                              E-DDC Registers                              *
 *                                                                           *
 *****************************************************************************/

//I2C Master Standard Mode Configuraiton Register
#define I2CM_SM_SCL_CONFIG				0x000000E0
#define I2CM_SM_SCL_CONFIG_LOW_MASK			0x0000FFFF
#define I2CM_SM_SCL_CONFIG_HIGH_MASK			0xFFFF0000

//I2C Master Fast Mode Configuraiton Register
#define I2CM_FM_SCL_CONFIG				0x000000E4
#define I2CM_FM_SCL_CONFIG_LOW_MASK			0x0000FFFF
#define I2CM_FM_SCL_CONFIG_HIGH_MASK			0xFFFF0000

//I2C Master Configuration Register
#define I2CM_CONFIG					0x000000E8
#define I2CM_CONFIG_RXFILTER_PULSE_WIDTH_MASK		0x0000001F
#define I2CM_CONFIG_SDA_HOLD_MASK			0x0000FF00

//I2C Master Control Register
#define I2CM_CONTROL					0x000000EC
#define I2CM_CONTROL_BUS_CLEAR_MASK			0x00000001

//I2C Master Status Register
#define I2CM_STATUS					0x000000F0
#define I2CM_STATUS_ARBITRATION_MASK			0x00000001

//I2C Master Interface Control 0 Register
#define I2CM_INTERFACE_CONTROL0				0x000000F4
#define I2CM_INTERFACE_CONTROL0_FAST_MODE_MASK		0x00000001
#define I2CM_INTERFACE_CONTROL0_WRITE_MASK		0x00000002
#define I2CM_INTERFACE_CONTROL0_READ_MASK		0x00000004
#define I2CM_INTERFACE_CONTROL0_SHORT_READ_MASK		0x00000008
#define I2CM_INTERFACE_CONTROL0_EXTENDED_READ_MASK	0x00000010
#define I2CM_INTERFACE_CONTROL0_SLAVE_ADDR_MASK		0x00000FE0
#define I2CM_INTERFACE_CONTROL0_ADDRESS_MASK		0x000FF000
#define I2CM_INTERFACE_CONTROL0_NBYTES_MASK		0x00F00000
#define I2CM_INTERFACE_CONTROL0_OPERATION_MASK		0x00FFFFFE

//I2C Master Interface Control 1 Register
#define I2CM_INTERFACE_CONTROL1				0x000000F8
#define I2CM_INTERFACE_CONTROL1_SEG_ADDR_MASK		0x0000007F
#define I2CM_INTERFACE_CONTROL1_SEG_PTR_MASK		0x00007F80

//I2C Master Interface Write Data 0 (0-3) Register
#define I2CM_INTERFACE_WRDATA0				0x000000FC
#define I2CM_INTERFACE_WRDATA0_0_MASK			0x000000FF
#define I2CM_INTERFACE_WRDATA0_1_MASK			0x0000FF00
#define I2CM_INTERFACE_WRDATA0_2_MASK			0x00FF0000
#define I2CM_INTERFACE_WRDATA0_3_MASK			0xFF000000

//I2C Master Interface Write Data 1 (4-7) Register
#define I2CM_INTERFACE_WRDATA1				0x00000100
#define I2CM_INTERFACE_WRDATA1_4_MASK			0x000000FF
#define I2CM_INTERFACE_WRDATA1_5_MASK			0x0000FF00
#define I2CM_INTERFACE_WRDATA1_6_MASK			0x00FF0000
#define I2CM_INTERFACE_WRDATA1_7_MASK			0xFF000000

//I2C Master Interface Write Data 2 (8-11) Register
#define I2CM_INTERFACE_WRDATA2				0x00000104
#define I2CM_INTERFACE_WRDATA2_8_MASK			0x000000FF
#define I2CM_INTERFACE_WRDATA2_9_MASK			0x0000FF00
#define I2CM_INTERFACE_WRDATA2_10_MASK			0x00FF0000
#define I2CM_INTERFACE_WRDATA2_11_MASK			0xFF000000

//I2C Master Interface Write Data 3 (12-15) Register
#define I2CM_INTERFACE_WRDATA3				0x00000108
#define I2CM_INTERFACE_WRDATA3_12_MASK			0x000000FF
#define I2CM_INTERFACE_WRDATA3_13_MASK			0x0000FF00
#define I2CM_INTERFACE_WRDATA3_14_MASK			0x00FF0000
#define I2CM_INTERFACE_WRDATA3_15_MASK			0xFF000000

//I2C Master Interface Read Data 0 (0-3) Register
#define I2CM_INTERFACE_RDDATA0				0x0000010C
#define I2CM_INTERFACE_RDDATA0_0_MASK			0x000000FF
#define I2CM_INTERFACE_RDDATA0_1_MASK			0x0000FF00
#define I2CM_INTERFACE_RDDATA0_2_MASK			0x00FF0000
#define I2CM_INTERFACE_RDDATA0_3_MASK			0xFF000000

//I2C Master Interface Read Data 1 (4-7) Register
#define I2CM_INTERFACE_RDDATA1				0x00000110
#define I2CM_INTERFACE_RDDATA1_4_MASK			0x000000FF
#define I2CM_INTERFACE_RDDATA1_5_MASK			0x0000FF00
#define I2CM_INTERFACE_RDDATA1_6_MASK			0x00FF0000
#define I2CM_INTERFACE_RDDATA1_7_MASK			0xFF000000

//I2C Master Interface Read Data 2 (8-11) Register
#define I2CM_INTERFACE_RDDATA2				0x00000114
#define I2CM_INTERFACE_RDDATA2_8_MASK			0x000000FF
#define I2CM_INTERFACE_RDDATA2_9_MASK			0x0000FF00
#define I2CM_INTERFACE_RDDATA2_10_MASK			0x00FF0000
#define I2CM_INTERFACE_RDDATA2_11_MASK			0xFF000000

//I2C Master Interface Read Data 3 (12-15) Register
#define I2CM_INTERFACE_RDDATA3				0x00000118
#define I2CM_INTERFACE_RDDATA3_12_MASK			0x000000FF
#define I2CM_INTERFACE_RDDATA3_13_MASK			0x0000FF00
#define I2CM_INTERFACE_RDDATA3_14_MASK			0x00FF0000
#define I2CM_INTERFACE_RDDATA3_15_MASK			0xFF000000

//SCDC Configuration Register
#define SCDC_CONFIG					0x00000140
#define SCDC_CONFIG_RR_EN_MASK				0x00000001
#define SCDC_CONFIG_RR_REPLY_WITH_STOP_MASK		0x00000002
#define SCDC_CONFIG_UPD_FLAGS_POLL_EN_MASK		0x00000010
#define SCDC_CONFIG_UPD_FLAGS_AUTO_CLR_MASK		0x00000040
#define SCDC_CONFIG_UPD_FLAGS_RD_CLR_NBYTES_MASK	0x00000080
#define SCDC_CONFIG_I2C_FM_EN_MASK			0x00001000
#define SCDC_CONFIG_UPD_FLAGS_POLL_MODE_MASK		0x00030000
#define SCDC_CONFIG_UPD_FLAGS_POLL_TIMER_PERIOD_MASK	0x0FF00000

//SCDC Control Register
#define SCDC_CONTROL					0x00000148
#define SCDC_CONTROL_UPD_FLAGS_RD_REQ_MASK		0x00000001
#define SCDC_CONTROL_UPD_FLAGS_CLR_REQ_MASK		0x00000002
#define SCDC_CONTROL_UPD_FLAGS_POLL_CONT_REQ_MASK	0x00000004

//SCDC Status Register
#define SCDC_STATUS					0x00000150
#define SCDC_STATUS_UPD_FLAGS0_MASK			0x000000FF
#define SCDC_STATUS_UPD_FLAGS1_MASK			0x0000FF00
#define SCDC_STATUS_UPD_FLAGS_MASK			0x0000FFFF

//FRL Link Training Configuration 0 Register
#define FLT_CONFIG0					0x00000160
#define FLT_CONFIG0_FFE_LEVELS_MASK			0x0000000F
#define FLT_CONFIG0_NO_TIMEOUT_MASK			0x00000100
#define FLT_CONFIG0_TFLT_MASK				0x0001F000

//FRL Link Training Configuration 1 Register
#define FLT_CONFIG1					0x00000164
#define FLT_CONFIG1_LTP_LANE0_OVR_EN_MASK		0x00000001
#define FLT_CONFIG1_LTP_LANE1_OVR_EN_MASK		0x00000002
#define FLT_CONFIG1_LTP_LANE2_OVR_EN_MASK		0x00000004
#define FLT_CONFIG1_LTP_LANE3_OVR_EN_MASK		0x00000008
#define FLT_CONFIG1_LTP_ALL_LANES_OVR_EN_MASK		0x0000000F
#define FLT_CONFIG1_LTP_LANE0_OVR_VALUE_MASK		0x000000F0
#define FLT_CONFIG1_LTP_LANE1_OVR_VALUE_MASK		0x00000F00
#define FLT_CONFIG1_LTP_LANE2_OVR_VALUE_MASK		0x0000F000
#define FLT_CONFIG1_LTP_LANE3_OVR_VALUE_MASK		0x000F0000

//FRL Link Training Configuration 2 Register
#define FLT_CONFIG2					0x00000168
#define FLT_CONFIG2_TXFFE_LANE0_OVR_EN_MASK		0x00000001
#define FLT_CONFIG2_TXFFE_LANE1_OVR_EN_MASK		0x00000002
#define FLT_CONFIG2_TXFFE_LANE2_OVR_EN_MASK		0x00000004
#define FLT_CONFIG2_TXFFE_LANE3_OVR_EN_MASK		0x00000008
#define FLT_CONFIG2_TXFFE_ALL_LANES_OVR_EN_MASK		0x0000000F
#define FLT_CONFIG2_TXFFE_LANE0_OVR_VALUE_MASK		0x000000F0
#define FLT_CONFIG2_TXFFE_LANE1_OVR_VALUE_MASK		0x00000F00
#define FLT_CONFIG2_TXFFE_LANE2_OVR_VALUE_MASK		0x0000F000
#define FLT_CONFIG2_TXFFE_LANE3_OVR_VALUE_MASK		0x000F0000

//FRL Link Training Control Register
#define FLT_CONTROL					0x00000170
#define FLT_CONTROL_FLT_START_MASK			0x00000001

/*****************************************************************************
 *                                                                           *
 *                          AVP Datapath Registers                           *
 *                                                                           *
 *****************************************************************************/

//Video Interface Configuration 0 Register
#define VIDEO_INTERFACE_CONFIG0				0x00000800
#define VIDEO_INTERFACE_CONFIG0_OVR_EN_MASK		0x00000001
#define VIDEO_INTERFACE_CONFIG0_BYTE0_OVR_VAL_MASK	0x0000000E
#define VIDEO_INTERFACE_CONFIG0_BYTE1_OVR_VAL_MASK	0x00000070
#define VIDEO_INTERFACE_CONFIG0_BYTE2_OVR_VAL_MASK	0x00000380
#define VIDEO_INTERFACE_CONFIG0_BYTE3_OVR_VAL_MASK	0x00001C00
#define VIDEO_INTERFACE_CONFIG0_BYTE4_OVR_VAL_MASK	0x0000E000
#define VIDEO_INTERFACE_CONFIG0_BYTE5_OVR_VAL_MASK	0x00070000
#define VIDEO_INTERFACE_CONFIG0_ENDIANNESS_OVR_VAL_MASK	0x00080000
#define VIDEO_INTERFACE_CONFIG0_HDMI_CD_OVR_VAL_MASK	0x00F00000
#define VIDEO_INTERFACE_CONFIG0_HDMI_FMT_OVR_VAL_MASK	0x07000000
#define VIDEO_INTERFACE_CONFIG0_DSC_24bit_OVR_VAL_MASK	0x08000000
#define VIDEO_INTERFACE_CONFIG0_DSC_48bit_OVR_VAL_MASK	0x10000000

//Video Interface Configuration 1 Register
#define VIDEO_INTERFACE_CONFIG1				0x00000804
#define VIDEO_INTERFACE_CONFIG1_PREP_MASK		0x0000000F

//Video Interface Configuration 2 Register
#define VIDEO_INTERFACE_CONFIG2				0x00000808
#define VIDEO_INTERFACE_CONFIG2_FIFO_INIT_ON_OVF_MASK	0x00000001
#define VIDEO_INTERFACE_CONFIG2_IPI_DESERIALIZER_MASK	0x00000010

//Video Interface Control Register
#define VIDEO_INTERFACE_CONTROL				0x0000080C
#define VIDEO_INTERFACE_CONTROL_FIFO_CLEAR_MASK		0x00000001

//Video Interface Status Register
#define VIDEO_INTERFACE_STATUS				0x00000814
#define VIDEO_INTERFACE_STATUS_IPI_FMT_MASK		0x0000000F
#define VIDEO_INTERFACE_STATUS_IPI_CD_MASK		0x000000F0
#define VIDEO_INTERFACE_STATUS_HDMI_CD_MASK		0x00000F00
#define VIDEO_INTERFACE_STATUS_HDMI_FMT_MASK		0x00007000
#define VIDEO_INTERFACE_STATUS_VESA_24bit_MASK		0x00008000
#define VIDEO_INTERFACE_STATUS_VESA_48bit_MASK		0x00010000
#define VIDEO_INTERFACE_STATUS_VSYNC_POL_MASK		0x00020000
#define VIDEO_INTERFACE_STATUS_HSYNC_POL_MASK		0x00040000

//Video Packing Config Register
#define VIDEO_PACKING_CONFIG				0x0000081C
#define VIDEO_PACKING_CONFIG_GCP_PHASE_MASK		0x00000001

/*****************************************************************************
 *                                                                           *
 *                         Audio Interface Registers                         *
 *                                                                           *
 *****************************************************************************/

//Audio Interface Configuration 0 Register
#define AUDIO_INTERFACE_CONFIG0						0x00000820
#define AUDIO_INTERFACE_CONFIG0_INTERFACE_SELECT_MASK			0x00000003
#define AUDIO_INTERFACE_CONFIG0_FIFO_INIT_ON_OVF_MASK			0x00000004
#define AUDIO_INTERFACE_CONFIG0_I2S_LINES_EN_MASK			0x000000F0
#define AUDIO_INTERFACE_CONFIG0_I2S_BPCUV_EN_MASK			0x00001000
#define AUDIO_INTERFACE_CONFIG0_I2S_BBIT_RIGHT_MASK			0x00002000
#define AUDIO_INTERFACE_CONFIG0_SPDIF_LINES_EN_MASK			0x000F0000
#define AUDIO_INTERFACE_CONFIG0_SPDIF_PAIR_SEL_MASK			0x00300000
#define AUDIO_INTERFACE_CONFIG0_AUD_FORMAT_MASK				0x07000000
#define AUDIO_INTERFACE_CONFIG0_PAI_FORMAT_OVR_EN_MASK			0x08000000
#define AUDIO_INTERFACE_CONFIG0_PAI_DISABLE_SAMPLE_PRESENT_LEFT_MASK	0x10000000

//Audio Interface Configuration 1 Register
#define AUDIO_INTERFACE_CONFIG1						0x00000824
#define AUDIO_INTERFACE_CONFIG1_PAI_PAIR_LAYOUT_OVR_VAL_MASK		0x0000FFFF
#define AUDIO_INTERFACE_CONFIG1_PAI_PAIR_LAYOUT_OVR_EN_MASK		0x00010000
#define AUDIO_INTERFACE_CONFIG1_PAI_CHANNELSPREAD_DETECT_OVR_VAL_MASK	0x00040000
#define AUDIO_INTERFACE_CONFIG1_PAI_CHANNELSPREAD_DETECT_OVR_EN_MASK	0x00080000
#define AUDIO_INTERFACE_CONFIG1_PAI_CHANNELSPREAD_THRESH_OVR_VAL_MASK	0x0FF00000
#define AUDIO_INTERFACE_CONFIG1_PAI_CHANNELSPREAD_THRESH_OVR_EN_MASK	0x10000000

//Audio Interface Control 0 Register
#define AUDIO_INTERFACE_CONTROL						0x0000082C
#define AUDIO_INTERFACE_CONTROL_FIFO_CLR_MASK				0x00000001

//Audio Interface Status Register
#define AUDIO_INTERFACE_STATUS						0x00000834
#define AUDIO_INTERFACE_STATUS_PAIR_LAYOUT_MASK				0x0000FFFF
#define AUDIO_INTERFACE_STATUS_PAI_FORMAT_MASK				0x00070000
#define AUDIO_INTERFACE_STATUS_PAI_CHANNELSPREAD_DETECTED_MASK		0x00080000
#define AUDIO_INTERFACE_STATUS_SPDIF_LINE0_LOCK_MASK			0x00100000
#define AUDIO_INTERFACE_STATUS_SPDIF_LINE1_LOCK_MASK			0x00200000
#define AUDIO_INTERFACE_STATUS_SPDIF_LINE2_LOCK_MASK			0x00400000
#define AUDIO_INTERFACE_STATUS_SPDIF_LINE3_LOCK_MASK			0x00800000

/*****************************************************************************
 *                                                                           *
 *                          Frame Composer Registers                         *
 *                                                                           *
 *****************************************************************************/

//Frame Composer Configuration 0 Register
#define FC_CONFIG0					0x00000840
#define FC_CONFIG0_FAPA_START_LOCATION_MASK		0x00000001
#define FC_CONFIG0_SCRAMBLER_UPC_LINE_MASK		0x00000004
#define FC_CONFIG0_VIDEO_MEAS_OVR_EN_MASK		0x00000008
#define FC_CONFIG0_MTW_FREQ_CFG_MASK			0x00000030
#define FC_CONFIG0_EXTCTL_DURATION_MASK			0x07F00000
#define FC_CONFIG0_EXTCTL_SPACING_SEL_MASK		0xF0000080

//Frame Composer Configuration 1 Register
#define FC_CONFIG1					0x00000844
#define FC_CONFIG1_MAX_PACKETS_ACTIVE_MASK		0x0000FF00
#define FC_CONFIG1_MAX_PACKETS_BLANK_MASK		0xFFF00000

//Frame Composer Configuration 3 Register
#define FC_CONFIG3					0x0000084C
#define FC_CONFIG3_HBLANK_OVR_VAL_MASK			0xFFFF0000

//Frame Composer Configuration 4 Register
#define FC_CONFIG4					0x00000850
#define FC_CONFIG4_HTOTAL_OVR_VAL_MASK			0xFFFF0000

//Frame Composer Configuration 6 Register
#define FC_CONFIG6					0x00000858
#define FC_CONFIG6_VBLANK_OVR_VAL_MASK			0xFFFF0000

//Frame Composer Configuration 7 Register
#define FC_CONFIG7					0x0000085C
#define FC_CONFIG7_VTOTAL_OVR_VAL_MASK			0xFFFF0000

//Frame Composer Configuration 8 Register
#define FC_CONFIG8					0x00000860
#define FC_CONFIG8_KEEPOUT_WOO_START_MARGIN_MASK	0x000000FF
#define FC_CONFIG8_KEEPOUT_WOO_END_MARGIN_MASK		0x0000FF00
#define FC_CONFIG8_KEEPOUT_WOO_CFG_MASK			0x00030000

//Frame Composer Configuration 9 Register
#define FC_CONFIG9					0x00000864
#define FC_CONFIG9_KEEPOUT_REKEY_END_MARGIN_MASK	0x000000FF
#define FC_CONFIG9_KEEPOUT_REKEY_CFG_MASK		0x00000300

//Frame Composer Control 0 Register
#define FC_CONTROL0					0x0000086C
#define FC_CONTROL0_KEEPOUT_HDCP_AUTH_OVR_EN_MASK	0x00000001
#define FC_CONTROL0_KEEPOUT_HDCP_AUTH_OVR_VAL_MASK	0x00000002

/*****************************************************************************
 *                                                                           *
 *                          Video Monitor Registers                          *
 *                                                                           *
 *****************************************************************************/

//Video Monitor Config 0 Register
#define VM_CONFIG0					0x00000880
#define VM_CONFIG0_IRQ_HTOTAL_EN_MASK			0x00000001
#define VM_CONFIG0_IRQ_HFRONT_EN_MASK			0x00000002
#define VM_CONFIG0_IRQ_HSYNC_EN_MASK			0x00000004
#define VM_CONFIG0_IRQ_HBACK_EN_MASK			0x00000008
#define VM_CONFIG0_IRQ_HACTIVE_EN_MASK			0x00000010
#define VM_CONFIG0_IRQ_VTOTAL_EN_MASK			0x00000020
#define VM_CONFIG0_IRQ_VFRONT_EN_MASK			0x00000040
#define VM_CONFIG0_IRQ_VSYNC_EN_MASK			0x00000080
#define VM_CONFIG0_IRQ_VBACK_EN_MASK			0x00000100
#define VM_CONFIG0_IRQ_VACTIVE_EN_MASK			0x00000200

//Video Monitor Statuc 0 Register
#define VM_STATUS0					0x00000884
#define VM_STATUS0_HFRONT_MASK				0x0000FFFF
#define VM_STATUS0_HSYNC_MASK				0xFFFF0000

//Video Monitor Statuc 1 Register
#define VM_STATUS1					0x00000888
#define VM_STATUS1_HBACK_MASK				0x0000FFFF
#define VM_STATUS1_HBLANK_MASK				0xFFFF0000

//Video Monitor Statuc 2 Register
#define VM_STATUS2					0x0000088C
#define VM_STATUS2_HACTIVE_MASK				0x0000FFFF
#define VM_STATUS2_HTOTAL_MASK				0xFFFF0000

//Video Monitor Statuc 3 Register
#define VM_STATUS3					0x00000890
#define VM_STATUS3_VFRONT_MASK				0x0000FFFF
#define VM_STATUS3_VSYNC_MASK				0xFFFF0000

//Video Monitor Statuc 4 Register
#define VM_STATUS4					0x00000894
#define VM_STATUS4_VBACK_MASK				0x0000FFFF
#define VM_STATUS4_VBLANK_MASK				0xFFFF0000

//Video Monitor Statuc 5 Register
#define VM_STATUS5					0x00000898
#define VM_STATUS5_VACTIVE_MASK				0x0000FFFF
#define VM_STATUS5_VTOTAL_MASK				0xFFFF0000

//Video Monitor Statuc 6 Register
#define VM_STATUS6					0x0000089C
#define VM_STATUS6_ALT_DETECT_MASK			0x00000001
#define VM_STATUS6_ILACE_DETECT_MASK			0x00000002

/*****************************************************************************
 *                                                                           *
 *                              HDCP Registers                               *
 *                                                                           *
 *****************************************************************************/

//HDCP2 Logic Configuration Register
#define HDCP2_CONFIG					0x000008E0
#define HDCP2_CONFIG_BYPASS_MASK			0x00000001
#define HDCP2_CONFIG_AVMUTE_OVR_EN_MASK			0x00000002
#define HDCP2_CONFIG_AVMUTE_OVR_VALUE_MASK		0x00000004
#define HDCP2_CONFIG_OPMODE_OVR_EN_MASK			0x00000008
#define HDCP2_CONFIG_OPMODE_OVR_VALUE_MASK		0x00000010

//HDCP2 Logic ESM GPIO Input Configuration Register
#define HDCP2_GPIOIN					0x000008E4
#define HDCP2_GPIOIN_GLOBAL_GPIO_IN_MASK		0x0000000F
#define HDCP2_GPIOIN_P0_GPIO_IN_MASK			0x000000F0
#define HDCP2_GPIOIN_P0_GPIO_IN_HPD_MASK		0x00000010

//HDCP2 Logic ESM GPIO Output Configuration Register
#define HDCP2_GPIOOUT					0x000008E8
#define HDCP2_GPIOOUT_GLOBAL_GPIO_OUT_MASK		0x0000000F
#define HDCP2_GPIOOUT_P0_GPIO_OUT_MASK			0x000FFFF0

//HDCP14 Configuration 0 Register
#define HDCP14_CONFIG0					0x00000900
#define HDCP14_CONFIG0_HPD_MASK				0x00000001
#define HDCP14_CONFIG0_BYPASS_MASK			0x00000002
#define HDCP14_CONFIG0_ENCRYPTION_ENABLE_MASK		0x00000004
#define HDCP14_CONFIG0_I2C_FAST_MODE_MASK		0x00000008
#define HDCP14_CONFIG0_I2C_SHORT_READ_DISABLE_MASK	0x00000010
#define HDCP14_CONFIG0_I2C_NACK_THRESHOLD_MASK		0x000001E0
#define HDCP14_CONFIG0_FEATURES11_ENABLE_MASK		0x00000200
#define HDCP14_CONFIG0_KEY_DESCRYPTION_EN_MASK		0x00000400
#define HDCP14_CONFIG0_AN_CYPHER_INPUT_MASK		0x00000800
#define HDCP14_CONFIG0_AN_BYPASS_MASK			0x00001000
#define HDCP14_CONFIG0_OESS_ESSS_OVR_EN_MASK		0x00002000
#define HDCP14_CONFIG0_OESS_ESSS_OVR_VALUE_MASK		0x00004000
#define HDCP14_CONFIG0_OESS_CTL3_PULSE_SIZE_MASK	0x00038000
#define HDCP14_CONFIG0_RI_READ_MODE_MASK		0x00040000
#define HDCP14_CONFIG0_PREWOO_OVR_EN_MASK		0x00400000
#define HDCP14_CONFIG0_PREWOO_OVR_VALUE_MASK		0x7F800000

//HDCP14 Control 0 Register
#define HDCP14_CONTROL0					0x00000904
#define HDCP14_CONTROL0_RESTART_MASK			0x00000001
#define HDCP14_CONTROL0_KSV_LIST_REVOCATED_MASK		0x00000002
#define HDCP14_CONTROL0_DISABLE_SHA1_MASK		0x00000004
#define HDCP14_CONTROL0_SHA1_MSG_CORRECT_MASK		0x00000008
#define HDCP14_CONTROL0_SHA1_MSG_FAIL_MASK		0x00000010

//HDCP14 Configuration 1 Register
#define HDCP14_CONFIG1					0x00000908
#define HDCP14_CONFIG1_HDCP14_R0_TIMER_VALUE_MASK	0x0000001E
#define HDCP14_CONFIG1_HDCP14_RI_TIMER_VALUE_MASK	0x0000FF00
#define HDCP14_CONFIG1_HDCP14_READY_TIMER_VALUE_MASK	0x0FF80000

//HDCP14 Configuration 2 Register
#define HDCP14_CONFIG2					0x0000090C
#define HDCP14_CONFIG2_HDCP14_I2C_TIMER_OVR_EN_MASK	0x00000001
#define HDCP14_CONFIG2_HDCP14_I2C_TIMER_OVR_VALUE_MASK	0x00007FFE
#define HDCP14_CONFIG2_HDCP14_POLL_MODE_MASK		0x00020000
#define HDCP14_CONFIG2_HDCP14_POLL_TIMER_VALUE_MASK	0x07FC0000
#define HDCP14_CONFIG2_HDCP14_PJ_TIMER_VALUE_MASK	0xF8000000

//HDCP14 Key Decrypt Seed Register
#define HDCP14_KEY_SEED					0x00000914
#define HDCP14_KEY_SEED_MASK				0x0000FFFF

//HDCP14 Key High Configuration Register
#define HDCP14_KEY_H					0x00000918
#define HDCP14_KEY_H_MASK				0x00FFFFFF

//HDCP14 Key Low Configuration Register
#define HDCP14_KEY_L					0x0000091C

//HDCP14 Key Status Register
#define HDCP14_KEY_STATUS				0x00000920
#define HDCP14_KEY_STATUS_KEY_INDEX_MASK		0x0000003F
#define HDCP14_KEY_STATUS_KEY_WRITE_OK_MASK		0x00000100

//HDCP14 AKSV High Configuration Register
#define HDCP14_AKSV_H					0x00000924
#define HDCP14_AKSV_H_MASK				0x000000FF

//HDCP14 AKSV Low Configuration Register
#define HDCP14_AKSV_L					0x00000928

//HDCP14 An High Configuration Register
#define HDCP14_AN_H					0x0000092C

//HDCP14 An Low Configuration Register
#define HDCP14_AN_L					0x00000930

//HDCP14 Status 0 Register
#define HDCP14_STATUS0					0x00000934
#define HDCP14_STATUS0_OESS_ESSS_MASK			0x00000001
#define HDCP14_STATUS0_SHA1_MSG_ST_MASK			0x00000002
#define HDCP14_STATUS0_AUTH_ST_MASK			0x00000004
#define HDCP14_STATUS0_RCV_HDMI_MODE_MASK		0x00000008
#define HDCP14_STATUS0_RPT_MAX_CASCADE_EXCEEDED_MASK	0x00000010
#define HDCP14_STATUS0_RPT_DEPTH_MASK			0x000000E0
#define HDCP14_STATUS0_RPT_MAX_DEVS_EXCEEDED_MASK	0x00000100
#define HDCP14_STATUS0_RPT_DEVICE_COUNT_MASK		0x0000FE00
#define HDCP14_STATUS0_BKSV_REVOC_FAIL_ST_MASK		0x00010000
#define HDCP14_STATUS0_I2C_NACK_CNT_MASK		0x00F00000

//HDCP14 Status 1 Register
#define HDCP14_STATUS1					0x00000938
#define HDCP14_STATUS1_RCV_FAST_REAUTH_MASK		0x00000001
#define HDCP14_STATUS1_RCV_FEATURES11_MASK		0x00000002
#define HDCP14_STATUS1_RCV_FAST_MODE_MASK		0x00000010
#define HDCP14_STATUS1_RCV_KSV_FIFO_READY_MASK		0x00000020
#define HDCP14_STATUS1_RCV_REPEATER_MASK		0x00000040

/*****************************************************************************
 *                                                                           *
 *                              LINK Registers                               *
 *                                                                           *
 *****************************************************************************/

//Scrambler Configuration 0 Register
#define SCRAMB_CONFIG0					0x00000960
#define SCRAMB_CONFIG0_ENABLE_MASK			0x00000001

//Link Configuration 0 Register
#define LINK_CONFIG0					0x00000968
#define LINK_CONFIG0_OPMODE_FRL_MASK			0x00000001
#define LINK_CONFIG0_OPMODE_DVI_MASK			0x00000010
#define LINK_CONFIG0_OPMODE_FRL_4LANES_MASK		0x00000100
#define LINK_CONFIG0_BIT_REVERSE_MASK			0x00001000
#define LINK_CONFIG0_PHY_WORD_REVERSE_MASK		0x00010000
#define LINK_CONFIG0_PHY_LANE_CH_REVERSE_MASK		0x00100000

//TMDS FIFO Configuration 0 Register
#define TMDS_FIFO_CONFIG0				0x00000970
#define TMDS_FIFO_CONFIG0_FIFO_INIT_ON_OVF_MASK		0x00000001

//TMDS FIFO Control 0 Register
#define TMDS_FIFO_CONTROL0				0x00000974
#define TMDS_FIFO_CONTROL0_FIFO_CLR_MASK		0x00000001

/*****************************************************************************
 *                                                                           *
 *                             Packets Registers                             *
 *                                                                           *
 *****************************************************************************/

//FRL Packetizer Configuration 0 Register
#define FRL_PKTZ_CONFIG0				0x00000A40
#define FRL_PKTZ_CONFIG0_FIFO_INIT_ON_OVF_MASK		0x00000001

//FRL Packetizer Status 1 Register
#define FRL_PKTZ_STATUS1				0x00000A54

//Packet Scheduler Configuration 1 Register
#define PKTSCHED_PKT_CONFIG1				0x00000A9C
#define PKTSCHED_PKT_CONFIG1_AMD_FIELD_FORCE_MASK	0x00000100
#define PKTSCHED_PKT_CONFIG1_AUDI_FIELD_FORCE_MASK	0x00000200
#define PKTSCHED_PKT_CONFIG1_AMD_BEFORE_NEW_AUDS_MASK	0x00000400
#define PKTSCHED_PKT_CONFIG1_AUDI_BEFORE_NEW_AUDS_MASK	0x00000800
#define PKTSCHED_PKT_CONFIG1_AVI_FIELDRATE_MASK		0x00001000
#define PKTSCHED_PKT_CONFIG1_DRMI_FIELDRATE_MASK	0x00002000
#define PKTSCHED_PKT_CONFIG1_VSI_FIELDRATE_MASK		0x00004000
#define PKTSCHED_PKT_CONFIG1_EMPFAPA_NOINTERLEAVE_MASK	0x00008000

//Packet Scheduler Configuration 2 Register
#define PKTSCHED_PKT_CONFIG2				0x00000AA0
#define PKTSCHED_PKT_CONFIG2_GHDMI1_RATE_MASK		0x0000000F
#define PKTSCHED_PKT_CONFIG2_GHDMI1_UNIT_MASK		0x00000010
#define PKTSCHED_PKT_CONFIG2_GHDMI1_PKT_OVR_VAL_MASK	0x00001F00
#define PKTSCHED_PKT_CONFIG2_GHDMI2_RATE_MASK		0x000F0000
#define PKTSCHED_PKT_CONFIG2_GHDMI2_UNIT_MASK		0x00100000
#define PKTSCHED_PKT_CONFIG2_GHDMI2_PKT_OVR_VAL_MASK	0x1F000000

//Packet Scheduler Configuration 3 Register
#define PKTSCHED_PKT_CONFIG3				0x00000AA4
#define PKTSCHED_PKT_CONFIG3_ACP_MASK			0x00000010
#define PKTSCHED_PKT_CONFIG3_ISCR_MASK			0x00000020
#define PKTSCHED_PKT_CONFIG3_GMD_MASK			0x00000080
#define PKTSCHED_PKT_CONFIG3_VSI_MASK			0x00001000
#define PKTSCHED_PKT_CONFIG3_AVI_MASK			0x00002000
#define PKTSCHED_PKT_CONFIG3_SPDI_MASK			0x00004000
#define PKTSCHED_PKT_CONFIG3_AUDI_MASK			0x00008000
#define PKTSCHED_PKT_CONFIG3_NVI_MASK			0x00010000
#define PKTSCHED_PKT_CONFIG3_DRMI_MASK			0x00020000
#define PKTSCHED_PKT_CONFIG3_GHDMI1_MASK		0x00040000
#define PKTSCHED_PKT_CONFIG3_GHDMI2_MASK		0x00080000
#define PKTSCHED_PKT_CONFIG3_ALL_MASK			0x000FF0B0

//Packet Scheduler Enable Register
#define PKTSCHED_PKT_EN					0x00000AA8
#define PKTSCHED_PKT_EN_NULL_MASK			0x00000001
#define PKTSCHED_PKT_EN_ACR_MASK			0x00000002
#define PKTSCHED_PKT_EN_AUDS_MASK			0x00000004
#define PKTSCHED_PKT_EN_GCP_MASK			0x00000008
#define PKTSCHED_PKT_EN_ACP_MASK			0x00000010
#define PKTSCHED_PKT_EN_ISCR1_MASK			0x00000020
#define PKTSCHED_PKT_EN_ISCR2_MASK			0x00000040
#define PKTSCHED_PKT_EN_GMD_MASK			0x00000080
#define PKTSCHED_PKT_EN_AMD_MASK			0x00000100
#define PKTSCHED_PKT_EN_VTEM_MASK			0x00000200
#define PKTSCHED_PKT_EN_CVTEM_MASK			0x00000400
#define PKTSCHED_PKT_EN_EXTMEM_MASK			0x00000800
#define PKTSCHED_PKT_EN_VSI_MASK			0x00001000
#define PKTSCHED_PKT_EN_AVI_MASK			0x00002000
#define PKTSCHED_PKT_EN_SPDI_MASK			0x00004000
#define PKTSCHED_PKT_EN_AUDI_MASK			0x00008000
#define PKTSCHED_PKT_EN_NVI_MASK			0x00010000
#define PKTSCHED_PKT_EN_DRMI_MASK			0x00020000
#define PKTSCHED_PKT_EN_GHDMI1_MASK			0x00040000
#define PKTSCHED_PKT_EN_GHDMI2_MASK			0x00080000
#define PKTSCHED_PKT_EN_ALL_MASK			0x000FFFFF

//Packet Scheduler Control 0 Register
#define PKTSCHED_PKT_CONTROL0				0x00000AAC
#define PKTSCHED_PKT_CONTROL0_GCP_SET_AVMUTE_MASK	0x00000001
#define PKTSCHED_PKT_CONTROL0_GCP_CLEAR_AVMUTE_MASK	0x00000002

//Packet Scheduler Send Request Register
#define PKTSCHED_PKT_SEND				0x00000AB0
#define PKTSCHED_PKT_SEND_ACP_MASK			0x00000010
#define PKTSCHED_PKT_SEND_ISRC1_MASK			0x00000020
#define PKTSCHED_PKT_SEND_ISRC2_MASK			0x00000040
#define PKTSCHED_PKT_SEND_GMD_MASK			0x00000080
#define PKTSCHED_PKT_SEND_VSI_MASK			0x00001000
#define PKTSCHED_PKT_SEND_AVI_MASK			0x00002000
#define PKTSCHED_PKT_SEND_SPDI_MASK			0x00004000
#define PKTSCHED_PKT_SEND_AUDI_MASK			0x00008000
#define PKTSCHED_PKT_SEND_NVI_MASK			0x00010000
#define PKTSCHED_PKT_SEND_DRMI_MASK			0x00020000
#define PKTSCHED_PKT_SEND_GHDMI1_MASK			0x00040000
#define PKTSCHED_PKT_SEND_GHDMI2_MASK			0x00080000
#define PKTSCHED_PKT_SEND_ALL_MASK			0x000FF0F0

//Packet Scheduler Status 0 Register
#define PKTSCHED_PKT_STATUS0				0x00000AB4
#define PKTSCHED_PKT_STATUS0_ACP_MASK			0x00000010
#define PKTSCHED_PKT_STATUS0_ISRC1_MASK			0x00000020
#define PKTSCHED_PKT_STATUS0_ISRC2_MASK			0x00000040
#define PKTSCHED_PKT_STATUS0_GMD_MASK			0x00000080
#define PKTSCHED_PKT_STATUS0_VSI_MASK			0x00001000
#define PKTSCHED_PKT_STATUS0_AVI_MASK			0x00002000
#define PKTSCHED_PKT_STATUS0_SPDI_MASK			0x00004000
#define PKTSCHED_PKT_STATUS0_AUDI_MASK			0x00008000
#define PKTSCHED_PKT_STATUS0_NVI_MASK			0x00010000
#define PKTSCHED_PKT_STATUS0_DRMI_MASK			0x00020000
#define PKTSCHED_PKT_STATUS0_GHDMI1_MASK		0x00040000
#define PKTSCHED_PKT_STATUS0_GHDMI2_MASK		0x00080000

//Packet Scheduler Status 1 Register
#define PKTSCHED_PKT_STATUS1				0x00000AB8
#define PKTSCHED_PKT_STATUS1_GCP_AVMUTE_SENT_STS_MASK	0x00000001

//Audio Content Protection Contents 0 Register
#define PKT_ACP_CONTENTS0				0x00000B20
#define PKT_CONTENTS0_HB1_MASK				0x0000FF00
#define PKT_CONTENTS0_HB2_MASK				0x00FF0000

//Audio Content Protection Contents 1 Register
#define PKT_ACP_CONTENTS1				0x00000B24

//Audio Content Protection Contents 2 Register
#define PKT_ACP_CONTENTS2				0x00000B28

//Audio Content Protection Contents 3 Register
#define PKT_ACP_CONTENTS3				0x00000B2C

//Audio Content Protection Contents 4 Register
#define PKT_ACP_CONTENTS4				0x00000B30

//Audio Content Protection Contents 5 Register
#define PKT_ACP_CONTENTS5				0x00000B34

//Audio Content Protection Contents 6 Register
#define PKT_ACP_CONTENTS6				0x00000B38

//Audio Content Protection Contents 7 Register
#define PKT_ACP_CONTENTS7				0x00000B3C

//Intl Std Recording Code 1 Contents 0 Register
#define PKT_ISRC1_CONTENTS0				0x00000B40

//Intl Std Recording Code 1 Contents 1 Register
#define PKT_ISRC1_CONTENTS1				0x00000B44

//Intl Std Recording Code 1 Contents 2 Register
#define PKT_ISRC1_CONTENTS2				0x00000B48

//Intl Std Recording Code 1 Contents 3 Register
#define PKT_ISRC1_CONTENTS3				0x00000B4C

//Intl Std Recording Code 1 Contents 4 Register
#define PKT_ISRC1_CONTENTS4				0x00000B50

//Intl Std Recording Code 1 Contents 5 Register
#define PKT_ISRC1_CONTENTS5				0x00000B54

//Intl Std Recording Code 1 Contents 6 Register
#define PKT_ISRC1_CONTENTS6				0x00000B58

//Intl Std Recording Code 1 Contents 7 Register
#define PKT_ISRC1_CONTENTS7				0x00000B5C

//Intl Std Recording Code 2 Contents 0 Register
#define PKT_ISRC2_CONTENTS0				0x00000B60

//Intl Std Recording Code 2 Contents 1 Register
#define PKT_ISRC2_CONTENTS1				0x00000B64

//Intl Std Recording Code 2 Contents 2 Register
#define PKT_ISRC2_CONTENTS2				0x00000B68

//Intl Std Recording Code 2 Contents 3 Register
#define PKT_ISRC2_CONTENTS3				0x00000B6C

//Intl Std Recording Code 2 Contents 4 Register
#define PKT_ISRC2_CONTENTS4				0x00000B70

//Intl Std Recording Code 2 Contents 5 Register
#define PKT_ISRC2_CONTENTS5				0x00000B74

//Intl Std Recording Code 2 Contents 6 Register
#define PKT_ISRC2_CONTENTS6				0x00000B78

//Intl Std Recording Code 2 Contents 7 Register
#define PKT_ISRC2_CONTENTS7				0x00000B7C

//Gamut Metadata Contents 0 Register
#define PKT_GMD_CONTENTS0				0x00000B80

//Gamut Metadata Contents 1 Register
#define PKT_GMD_CONTENTS1				0x00000B84

//Gamut Metadata Contents 2 Register
#define PKT_GMD_CONTENTS2				0x00000B88

//Gamut Metadata Contents 3 Register
#define PKT_GMD_CONTENTS3				0x00000B8C

//Gamut Metadata Contents 4 Register
#define PKT_GMD_CONTENTS4				0x00000B90

//Gamut Metadata Contents 5 Register
#define PKT_GMD_CONTENTS5				0x00000B94

//Gamut Metadata Contents 6 Register
#define PKT_GMD_CONTENTS6				0x00000B98

//Gamut Metadata Contents 7 Register
#define PKT_GMD_CONTENTS7				0x00000B9C

//Audio Metadata Contents 0 Register
#define PKT_AMD_CONTENTS0				0x00000BA0

//Audio Metadata Contents 1 Register
#define PKT_AMD_CONTENTS1				0x00000BA4

//Audio Metadata Contents 2 Register
#define PKT_AMD_CONTENTS2				0x00000BA8

//Audio Metadata Contents 3 Register
#define PKT_AMD_CONTENTS3				0x00000BAC

//Audio Metadata Contents 4 Register
#define PKT_AMD_CONTENTS4				0x00000BB0

//Audio Metadata Contents 5 Register
#define PKT_AMD_CONTENTS5				0x00000BB4

//Audio Metadata Contents 6 Register
#define PKT_AMD_CONTENTS6				0x00000BB8

//Audio Metadata Contents 7 Register
#define PKT_AMD_CONTENTS7				0x00000BBC

//Vendor Specific Infoframe Contents 0 Register
#define PKT_VSI_CONTENTS0				0x00000BC0

//Vendor Specific Infoframe Contents 1 Register
#define PKT_VSI_CONTENTS1				0x00000BC4

//Vendor Specific Infoframe Contents 2 Register
#define PKT_VSI_CONTENTS2				0x00000BC8

//Vendor Specific Infoframe Contents 3 Register
#define PKT_VSI_CONTENTS3				0x00000BCC

//Vendor Specific Infoframe Contents 4 Register
#define PKT_VSI_CONTENTS4				0x00000BD0

//Vendor Specific Infoframe Contents 5 Register
#define PKT_VSI_CONTENTS5				0x00000BD4

//Vendor Specific Infoframe Contents 6 Register
#define PKT_VSI_CONTENTS6				0x00000BD8

//Vendor Specific Infoframe Contents 7 Register
#define PKT_VSI_CONTENTS7				0x00000BDC

//Auxiliary Video Infoframe Contents 0 Register
#define PKT_AVI_CONTENTS0				0x00000BE0

//Auxiliary Video Infoframe Contents 1 Register
#define PKT_AVI_CONTENTS1				0x00000BE4

//Auxiliary Video Infoframe Contents 2 Register
#define PKT_AVI_CONTENTS2				0x00000BE8

//Auxiliary Video Infoframe Contents 3 Register
#define PKT_AVI_CONTENTS3				0x00000BEC

//Auxiliary Video Infoframe Contents 4 Register
#define PKT_AVI_CONTENTS4				0x00000BF0

//Auxiliary Video Infoframe Contents 5 Register
#define PKT_AVI_CONTENTS5				0x00000BF4

//Auxiliary Video Infoframe Contents 6 Register
#define PKT_AVI_CONTENTS6				0x00000BF8

//Auxiliary Video Infoframe Contents 7 Register
#define PKT_AVI_CONTENTS7				0x00000BFC

//Source Product Description Infoframe Contents 0 Register
#define PKT_SPDI_CONTENTS0				0x00000C00

//Source Product Description Infoframe Contents 1 Register
#define PKT_SPDI_CONTENTS1				0x00000C04

//Source Product Description Infoframe Contents 2 Register
#define PKT_SPDI_CONTENTS2				0x00000C08

//Source Product Description Infoframe Contents 3 Register
#define PKT_SPDI_CONTENTS3				0x00000C0C

//Source Product Description Infoframe Contents 4 Register
#define PKT_SPDI_CONTENTS4				0x00000C10

//Source Product Description Infoframe Contents 5 Register
#define PKT_SPDI_CONTENTS5				0x00000C14

//Source Product Description Infoframe Contents 6 Register
#define PKT_SPDI_CONTENTS6				0x00000C18

//Source Product Description Infoframe Contents 7 Register
#define PKT_SPDI_CONTENTS7				0x00000C1C

//Audio Infoframe Packet Contents 0 Register
#define PKT_AUDI_CONTENTS0				0x00000C20

//Audio Infoframe Packet Contents 1 Register
#define PKT_AUDI_CONTENTS1				0x00000C24

//Audio Infoframe Packet Contents 2 Register
#define PKT_AUDI_CONTENTS2				0x00000C28

//Audio Infoframe Packet Contents 3 Register
#define PKT_AUDI_CONTENTS3				0x00000C2C

//Audio Infoframe Packet Contents 4 Register
#define PKT_AUDI_CONTENTS4				0x00000C30

//Audio Infoframe Packet Contents 5 Register
#define PKT_AUDI_CONTENTS5				0x00000C34

//Audio Infoframe Packet Contents 6 Register
#define PKT_AUDI_CONTENTS6				0x00000C38

//Audio Infoframe Packet Contents 7 Register
#define PKT_AUDI_CONTENTS7				0x00000C3C

/*****************************************************************************
 *                                                                           *
 *                         Audio Packetizer Registers                        *
 *                                                                           *
 *****************************************************************************/

//Audio Packetizer Control 0 Register
#define AUDPKT_CONTROL0					0x00000E20
#define AUDPKT_CONTROL0_CHSTATUS_OVR_EN_MASK		0x00000001
#define AUDPKT_CONTROL0_USRDATA_OVR_EN_MASK		0x00000002
#define AUDPKT_CONTROL0_USRDATA_SPREAD_EN_OVR_VAL_MASK	0x00000004
#define AUDPKT_CONTROL0_VBIT_OVR_EN_MASK		0x00000008
#define AUDPKT_CONTROL0_DBL_DST_OVR_VALUE_MASK		0x00000010
#define AUDPKT_CONTROL0_DBL_DST_EN_MASK			0x00000020
#define AUDPKT_CONTROL0_LAYOUT_OVR_VALUE_MASK		0x00000100
#define AUDPKT_CONTROL0_LAYOUT_OVR_EN_MASK		0x00000200
#define AUDPKT_CONTROL0_PBIT_FORCE_EN_MASK		0x00001000
#define AUDPKT_CONTROL0_SAMPLE_PRESENT_OVR_EN_MASK	0x00010000
#define AUDPKT_CONTROL0_SAMPLE_FLAT_OVR_EN_MASK		0x00100000
#define AUDPKT_CONTROL0_SAMPLE_START_FORCE_EN_MASK	0x01000000
#define AUDPKT_CONTROL0_FILLER_3D_MINIMAP_FORCE_EN_MASK	0x10000000

//Audio Packetizer Control 1 Register
#define AUDPKT_CONTROL1					0x00000E24
#define AUDPKT_CONTROL1_SAMPLE_PRESET_OVR_VAL_MASK	0x0000FFFF
#define AUDPKT_CONTROL1_SAMPLE_FLAT_OVR_VAL_MASK	0xFFFF0000

//Audio Packetizer ACR Control 0 Register
#define AUDPKT_ACR_CONTROL0				0x00000E40
#define AUDPKT_ACR_CONTROL0_ACR_N_VALUE_MASK		0x000FFFFF
#define AUDPKT_ACR_CONTROL0_PLL4X_OVR_EN_MASK		0x01000000
#define AUDPKT_ACR_CONTROL0_PLL4X_OVR_VAL_MASK		0x02000000

//Audio Packetizer ACR Status 0 Register
#define AUDPKT_ACR_STATUS0				0x00000E4C
#define AUDPKT_ACT_STATUS0_CTS_VALUE_MASK		0x000FFFFF

//Audio Packetizer Channel Status Override 0 Register
#define AUDPKT_CHSTATUS_OVR0				0x00000E60

//Audio Packetizer Channel Status Override 1 Register
#define AUDPKT_CHSTATUS_OVR1				0x00000E64

/*****************************************************************************
 *                                                                           *
 *                               CEC Registers                               *
 *                                                                           *
 *****************************************************************************/

//CEC TX Control Register
#define CEC_TX_CONTROL						0x00001000
#define CEC_TX_CONTROL_SEND_SET_MASK				0x00000001
#define CEC_TX_CONTROL_SEND_CLR_MASK				0x00000002

//CEC Status Register
#define CEC_STATUS						0x00001004
#define CEC_STATUS_SEND_PENDING_MASK				0x00000001
#define CEC_STATUS_RX_BUSY_MASK					0x00000010
#define CEC_STATUS_TX_BUSY_MASK					0x00000100

//CEC Configuration Register
#define CEC_CONFIG						0x00001008
#define CEC_CONFIG_TXREQ_DISCARD_IF_RXBUSY_MASK			0x00000001
#define CEC_CONFIG_BROADCAST_NACK_MASK				0x00000002
#define CEC_CONFIG_FRAME_NRETRANS_MASK				0x00000030
#define CEC_CONFIG_CECFILTER_MASK				0x00000100
#define CEC_CONFIG_RX_AUTO_DRIVE_ACK_MASK			0x00000200
#define CEC_CONFIG_SIGNAL_FREE_TIME_MASK			0x00003000

//CEC Logical Address Register
#define CEC_ADDR						0x0000100C

//CEC TX Buffer Counter Register
#define CEC_TX_CNT						0x00001020
#define CEC_TX_CNT_CEC_TXBUFFER_CNT_MASK			0x0000000F

//CEC TX Data 0 [3-0] Register
#define CEC_TX_DATA0						0x00001024

//CEC TX Data 1 [7-4] Register
#define CEC_TX_DATA1						0x00001028

//CEC TX Data 2 [11-8] Register
#define CEC_TX_DATA2						0x0000102C

//CEC TX Data 3 [15-12] Register
#define CEC_TX_DATA3						0x00001030

//CEC RX Count Status Register
#define CEC_RX_CNT_STATUS					0x00001040
#define CEC_RX_CNT_STATUS_CEC_RXBUFFER_CNT_MASK			0x0000000F
#define CEC_RX_CNT_STATUS_LOCKED_RXBUFFER_MASK			0x00000100

//CEC RX Data 0 [3-0] Register
#define CEC_RX_DATA0						0x00001044

//CEC RX Data 1 [7-4] Register
#define CEC_RX_DATA1						0x00001048

//CEC RX Data 2 [11-8] Register
#define CEC_RX_DATA2						0x0000104C

//CEC RX Data 3 [15-12] Register
#define CEC_RX_DATA3						0x00001050

//CEC Lock Control Register
#define CEC_LOCK_CONTROL					0x00001054
#define CEC_LOCK_CONTROL_LOCKED_RXBUFFER_CLR_MASK		0x00000001

//CEC RX Bit Qualifier Timmings Register
#define CEC_RX_BITIME_CONFIG0					0x00001060
#define CEC_RX_BITIME_CONFIG0_TIMET2_EARLRETHIGH_MASK		0x00000007
#define CEC_RX_BITIME_CONFIG0_TIMET3_LATERETHIGH_MASK		0x00000070
#define CEC_RX_BITIME_CONFIG0_TIMET6_EARLRETHIGH_MASK		0x00000700
#define CEC_RX_BITIME_CONFIG0_TIMET7_LATERETHIGH_MASK		0x00007000
#define CEC_RX_BITIME_CONFIG0_TIMET8_EARLSTART_MASK		0x00070000

//CEC RX Bit Qualifier Timmings Register
#define CEC_RX_BITIME_CONFIG1					0x00001064
#define CEC_RX_BITIME_CONFIG1_TIMET9_ENDFRAME_MASK		0x00000007
#define CEC_RX_BITIME_CONFIG0_TIMET1_EARLRETHIGH_MASK		0x00000070
#define CEC_RX_BITIME_CONFIG1_TIMET2_LATERETHIGH_MASK		0x00000700
#define CEC_RX_BITIME_CONFIG1_TIMET3_EARLEND_MASK		0x00007000
#define CEC_RX_BITIME_CONFIG1_TIMET4_LATEND_MASK		0x00070000
#define CEC_RX_BITIME_CONFIG1_TIMET9_AUTO_ACK_MASK		0x00700000

//CEC TX Bit Qualifier Timmings Register
#define CEC_TX_BITIME_CONFIG					0x00001068
#define CEC_TX_BITIME_CONFIG_BIT_TIME_LH_TRANS_MASK		0x00000007
#define CEC_TX_BITIME_CONFIG_BIT_TIME_RETHIGH_MASK		0x00000070
#define CEC_TX_BITIME_CONFIG_BIT_TIME_PERIOD_MASK		0x00000700
#define CEC_TX_BITIME_CONFIG_ERR_NOTIFY_TIME_MASK		0x00007000
#define CEC_TX_BITIME_CONFIG_STARTBIT_TIME_LH_TRANS_MASK	0x00070000
#define CEC_TX_BITIME_CONFIG_STARTBIT_TIME_TOTALDURATION_MASK	0x00700000
#define CEC_TX_BITIME_CONFIG_STARTBIT_TIME_RISESAMPLE_MASK	0x07000000
#define CEC_TX_BITIME_CONFIG_ROUND_TRIP_DELAY_MASK		0x70000000

/*****************************************************************************
 *                                                                           *
 *                               eARC Registers                              *
 *                                                                           *
 *****************************************************************************/

//eARC RX CMDC Configuration 0 Register
#define EARC_CMDC_CONFIG0					0x00001800
#define EARC_CMDC_CONFIG0_RXPKT_BADECC_TOLERANT_MASK		0x00000001
#define EARC_CMDC_CONFIG0_RXPKT_BADPAR_TOLERANT_MASK		0x00000002
#define EARC_CMDC_CONFIG0_RXPKT_ECC_ERRCORRECNT_EN_MASK		0x00000004
#define EARC_CMDC_CONFIG0_ECC_INV_SYNBITORDER_MASK		0x00000008
#define EARC_CMDC_CONFIG0_WHITELIST_EN_MASK			0x00000010
#define EARC_CMDC_CONFIG0_RX_XACT_TIMEOUT_MASK			0x00007F00
#define EARC_CMDC_CONFIG0_RX_COMMA_NOCOUNT_MASK			0x00010000
#define EARC_CMDC_CONFIG0_XACT_RESTART_EN_MASK			0x00040000
#define EARC_CMDC_CONFIG0_XACT_OVERFLOW_END_EN_MASK		0x00080000
#define EARC_CMDC_CONFIG0_DSCVR_EARCVALID0_TO_DISC1_MASK	0x01000000
#define EARC_CMDC_CONFIG0_XACTREAD_RETRY_CFG_MASK		0x02000000
#define EARC_CMDC_CONFIG0_XACTREAD_STOP_CFG_MASK		0x04000000

//eARC RX CMDC Configuration 1 Register
#define EARC_CMDC_CONFIG1					0x00001804
#define EARC_CMDC_CONFIG1_HEARTBEAT_DEVICEID_MASK		0x000000FF
#define EARC_CMDC_CONFIG1_HEARTBEAT_RDOFFSET_MASK		0x0000FF00
#define EARC_CMDC_CONFIG1_HEARTBEAT_WROFFSET_MASK		0x00FF0000

//eARC RX CMDC Control Register
#define EARC_CMDC_CONTROL					0x00001808
#define EARC_CMDC_CONTROL_H14BARC_ACTIVE_MASK			0x00000001
#define EARC_CMDC_CONTROL_CONNECTOR_HPD_MASK			0x00000002
#define EARC_CMDC_CONTROL_CMDC_DISCOVERY_EN_MASK		0x00000008
#define EARC_CMDC_CONTROL_CMDC_HEARTBEAT_LOSS_EN_MASK		0x00000010
#define EARC_CMDC_CONTROL_CMDC_DSCVR_FORCE_CONNECTED_MASK	0x00000020

//eARC RX CMDC Whitelist Configuration 0 Register
#define EARC_CMDC_WHITELIST0_CONFIG				0x0000180C
#define EARC_CMDC_WHITELIST0_CONFIG_DEVICEID0_MASK		0x000000FF
#define EARC_CMDC_WHITELIST0_CONFIG_OFFSET0_MASK		0x0000FF00
#define EARC_CMDC_WHITELIST0_CONFIG_DEVICEID1_MASK		0x00FF0000
#define EARC_CMDC_WHITELIST0_CONFIG_OFFSET1_MASK		0xFF000000

//eARC RX CMDC Whitelist Configuration 1 Register
#define EARC_CMDC_WHITELIST1_CONFIG				0x00001810
#define EARC_CMDC_WHITELIST1_CONFIG_DEVICEID2_MASK		0x000000FF
#define EARC_CMDC_WHITELIST1_CONFIG_OFFSET2_MASK		0x0000FF00
#define EARC_CMDC_WHITELIST1_CONFIG_DEVICEID3_MASK		0x00FF0000
#define EARC_CMDC_WHITELIST1_CONFIG_OFFSET3_MASK		0xFF000000

//eARC RX CMDC Whitelist Configuration 2 Register
#define EARC_CMDC_WHITELIST2_CONFIG				0x00001814
#define EARC_CMDC_WHITELIST2_CONFIG_DEVICEID4_MASK		0x000000FF
#define EARC_CMDC_WHITELIST2_CONFIG_OFFSET4_MASK		0x0000FF00
#define EARC_CMDC_WHITELIST2_CONFIG_DEVICEID5_MASK		0x00FF0000
#define EARC_CMDC_WHITELIST2_CONFIG_OFFSET5_MASK		0xFF000000

//eARC RX CMDC Whitelist Configuration 3 Register
#define EARC_CMDC_WHITELIST3_CONFIG				0x00001818
#define EARC_CMDC_WHITELIST3_CONFIG_DEVICEID6_MASK		0x000000FF
#define EARC_CMDC_WHITELIST3_CONFIG_OFFSET6_MASK		0x0000FF00
#define EARC_CMDC_WHITELIST3_CONFIG_DEVICEID7_MASK		0x00FF0000
#define EARC_CMDC_WHITELIST3_CONFIG_OFFSET7_MASK		0xFF000000

//eARC RX CMDC Status Register
#define EARC_CMDC_STATUS					0x0000181C
#define EARC_CMDC_STATUS_STUCKHIGH_MASK				0x00000001
#define EARC_CMDC_STATUS_DISCOVERY_STATE_MASK			0x0000000E
#define EARC_CMDC_STATUS_XACT_FAILED_STOP_MASK			0x00000020
#define EARC_CMDC_STATUS_XACT_FAILED_DEVICEID_MASK		0x00000040
#define EARC_CMDC_STATUS_XACT_FAILED_OFFSET_MASK		0x00000080
#define EARC_CMDC_STATUS_XACT_FINISHED_MASK			0x00000100
#define EARC_CMDC_STATUS_XACT_SUCCESSFUL_MASK			0x00000200
#define EARC_CMDC_STATUS_XACT_FAILED_UNEXPCMD_MASK		0x00000400
#define EARC_CMDC_STATUS_XACT_FAILED_UNCORRECTERR_MASK		0x00000800
#define EARC_CMDC_STATUS_XACT_TIMEOUT_MASK			0x00001000
#define EARC_CMDC_STATUS_XACT_CORRECTED_ERRNUM_MASK		0x0000E000
#define EARC_CMDC_STATUS_XACT_RESTARTED_MASK			0x00010000
#define EARC_CMDC_STATUS_XACT_DATA_OVERFLOW_MASK		0x00020000

//eARC RX CMDC Transaction Information Register
#define EARC_CMDC_XACT_INFO					0x00001820
#define EARC_CMDC_XACT_INFO_TYPE_MASK				0x00000001
#define EARC_CMDC_XACT_INFO_DEVICEID_MASK			0x0000FF00
#define EARC_CMDC_XACT_INFO_OFFSET_MASK				0x00FF0000
#define EARC_CMDC_XACT_INFO_SIZE_MASK				0xFF000000

//eARC RX CMDC Transaction Action Register
#define EARC_CMDC_XACT_ACTION					0x00001824
#define EARC_CMDC_XACT_ACTION_RDDATA_AVAIL_MASK			0x00000001
#define EARC_CMDC_XACT_ACTION_WRDATA_STORED_MASK		0x00000002

//eARC RX CMDC HeartBeat RXSTAT Register
#define EARC_CMDC_HEARTBEAT_RXSTAT				0x00001828
#define EARC_CMDC_HEARTBEAT_RXSTAT_EARC_HPD_MASK		0x00000001
#define EARC_CMDC_HEARTBEAT_RXSTAT_BIT1_MASK			0x00000002
#define EARC_CMDC_HEARTBEAT_RXSTAT_BIT2_MASK			0x00000004
#define EARC_CMDC_HEARTBEAT_RXSTAT_CAP_CHNG_MASK		0x00000008
#define EARC_CMDC_HEARTBEAT_RXSTAT_STAT_CHNG_MASK		0x00000010
#define EARC_CMDC_HEARTBEAT_RXSTAT_BIT5_MASK			0x00000020
#define EARC_CMDC_HEARTBEAT_RXSTAT_BIT6_MASK			0x00000040
#define EARC_CMDC_HEARTBEAT_RXSTAT_BIT7_MASK			0x00000080

//eARC RX CMDC HeartBeat Status Register
#define EARC_CMDC_HEARTBEAT_STATUS				0x0000182C
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_EARC_HPD_MASK		0x00000001
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_BIT1_MASK		0x00000002
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_BIT2_MASK		0x00000004
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_CAP_CHNG_MASK		0x00000008
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_STAT_CHNG_MASK	0x00000010
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_BIT5_MASK		0x00000020
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_BIT6_MASK		0x00000040
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_BIT7_MASK		0x00000080
#define EARC_CMDC_HEARTBEAT_STATUS_RXSTAT_ALL_MASK		0x000000FF
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_HDMI_HPD_MASK		0x00000100
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_BIT1_MASK		0x00000200
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_BIT2_MASK		0x00000400
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_CAP_CHNG_CONF_MASK	0x00000800
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_STAT_CHNG_CONF_MASK	0x00001000
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_BIT5_MASK		0x00002000
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_BIT6_MASK		0x00004000
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_EARC_VALID_MASK	0x00008000
#define EARC_CMDC_HEARTBEAT_STATUS_TXSTAT_ALL_MASK		0x0000FF00

//eARC RX CMDC Configuration 2 Register
#define EARC_CMDC_CONFIG2					0x00001830
#define EARC_CMDC_CONFIG2_NEARC_RX_CONN_COMMA_COUNT_MAX_MASK	0x0000000F
#define EARC_CMDC_CONFIG2_EARC_TGL_CM_MASK			0x00000700
#define EARC_CMDC_CONFIG2_THREEQUARTER_BIT_MASK			0x00070000
#define EARC_CMDC_CONFIG2_STUCK_HIGH_MASK			0x1F000000

//eARC RX CMDC Configuration 3 Register
#define EARC_CMDC_CONFIG3					0x00001834
#define EARC_CMDC_CONFIG3_CM_TURNOVER_MASK			0x0000000F
#define EARC_CMDC_CONFIG3_HPD_LOW_MIN_MASK			0x0000FF00
#define EARC_CMDC_CONFIG3_LOST_HEARTBEAT_MASK			0x00FF0000
#define EARC_CMDC_CONFIG3_RX_TIMEOUT_MASK			0x1F000000

//eARC RX CMDC Configuration 4 Register
#define EARC_CMDC_CONFIG4					0x00001838
#define EARC_CMDC_CONFIG4_RX_CONN_COMMA_MASK			0x0000007F

//eARC RX CMDC Write Transaction 0 Register
#define EARC_CMDC_XACT_WR0					0x00001840

//eARC RX CMDC Write Transaction 1 Register
#define EARC_CMDC_XACT_WR1					0x00001844

//eARC RX CMDC Write Transaction 2 Register
#define EARC_CMDC_XACT_WR2					0x00001848

//eARC RX CMDC Write Transaction 3 Register
#define EARC_CMDC_XACT_WR3					0x0000184C

//eARC RX CMDC Write Transaction 4 Register
#define EARC_CMDC_XACT_WR4					0x00001850

//eARC RX CMDC Write Transaction 5 Register
#define EARC_CMDC_XACT_WR5					0x00001854

//eARC RX CMDC Write Transaction 6 Register
#define EARC_CMDC_XACT_WR6					0x00001858

//eARC RX CMDC Write Transaction 7 Register
#define EARC_CMDC_XACT_WR7					0x0000185C

//eARC RX CMDC Write Transaction 8 Register
#define EARC_CMDC_XACT_WR8					0x00001860

//eARC RX CMDC Write Transaction 9 Register
#define EARC_CMDC_XACT_WR9					0x00001864

//eARC RX CMDC Write Transaction 10 Register
#define EARC_CMDC_XACT_WR10					0x00001868

//eARC RX CMDC Write Transaction 11 Register
#define EARC_CMDC_XACT_WR11					0x0000186C

//eARC RX CMDC Write Transaction 12 Register
#define EARC_CMDC_XACT_WR12					0x00001870

//eARC RX CMDC Write Transaction 13 Register
#define EARC_CMDC_XACT_WR13					0x00001874

//eARC RX CMDC Write Transaction 14 Register
#define EARC_CMDC_XACT_WR14					0x00001878

//eARC RX CMDC Write Transaction 15 Register
#define EARC_CMDC_XACT_WR15					0x0000187C

//eARC RX CMDC Write Transaction 16 Register
#define EARC_CMDC_XACT_WR16					0x00001880

//eARC RX CMDC Write Transaction 17 Register
#define EARC_CMDC_XACT_WR17					0x00001884

//eARC RX CMDC Write Transaction 18 Register
#define EARC_CMDC_XACT_WR18					0x00001888

//eARC RX CMDC Write Transaction 19 Register
#define EARC_CMDC_XACT_WR19					0x0000188C

//eARC RX CMDC Write Transaction 20 Register
#define EARC_CMDC_XACT_WR20					0x00001890

//eARC RX CMDC Write Transaction 21 Register
#define EARC_CMDC_XACT_WR21					0x00001894

//eARC RX CMDC Write Transaction 22 Register
#define EARC_CMDC_XACT_WR22					0x00001898

//eARC RX CMDC Write Transaction 23 Register
#define EARC_CMDC_XACT_WR23					0x0000189C

//eARC RX CMDC Write Transaction 24 Register
#define EARC_CMDC_XACT_WR24					0x000018A0

//eARC RX CMDC Write Transaction 25 Register
#define EARC_CMDC_XACT_WR25					0x000018A4

//eARC RX CMDC Write Transaction 26 Register
#define EARC_CMDC_XACT_WR26					0x000018A8

//eARC RX CMDC Write Transaction 27 Register
#define EARC_CMDC_XACT_WR27					0x000018AC

//eARC RX CMDC Write Transaction 28 Register
#define EARC_CMDC_XACT_WR28					0x000018B0

//eARC RX CMDC Write Transaction 29 Register
#define EARC_CMDC_XACT_WR29					0x000018B4

//eARC RX CMDC Write Transaction 30 Register
#define EARC_CMDC_XACT_WR30					0x000018B8

//eARC RX CMDC Write Transaction 31 Register
#define EARC_CMDC_XACT_WR31					0x000018BC

//eARC RX CMDC Write Transaction 32 Register
#define EARC_CMDC_XACT_WR32					0x000018C0

//eARC RX CMDC Write Transaction 33 Register
#define EARC_CMDC_XACT_WR33					0x000018C4

//eARC RX CMDC Write Transaction 34 Register
#define EARC_CMDC_XACT_WR34					0x000018C8

//eARC RX CMDC Write Transaction 35 Register
#define EARC_CMDC_XACT_WR35					0x000018CC

//eARC RX CMDC Write Transaction 36 Register
#define EARC_CMDC_XACT_WR36					0x000018D0

//eARC RX CMDC Write Transaction 37 Register
#define EARC_CMDC_XACT_WR37					0x000018D4

//eARC RX CMDC Write Transaction 38 Register
#define EARC_CMDC_XACT_WR38					0x000018D8

//eARC RX CMDC Write Transaction 39 Register
#define EARC_CMDC_XACT_WR39					0x000018DC

//eARC RX CMDC Write Transaction 40 Register
#define EARC_CMDC_XACT_WR40					0x000018E0

//eARC RX CMDC Write Transaction 41 Register
#define EARC_CMDC_XACT_WR41					0x000018E4

//eARC RX CMDC Write Transaction 42 Register
#define EARC_CMDC_XACT_WR42					0x000018E8

//eARC RX CMDC Write Transaction 43 Register
#define EARC_CMDC_XACT_WR43					0x000018EC

//eARC RX CMDC Write Transaction 44 Register
#define EARC_CMDC_XACT_WR44					0x000018F0

//eARC RX CMDC Write Transaction 45 Register
#define EARC_CMDC_XACT_WR45					0x000018F4

//eARC RX CMDC Write Transaction 46 Register
#define EARC_CMDC_XACT_WR46					0x000018F8

//eARC RX CMDC Write Transaction 47 Register
#define EARC_CMDC_XACT_WR47					0x000018FC

//eARC RX CMDC Write Transaction 48 Register
#define EARC_CMDC_XACT_WR48					0x00001900

//eARC RX CMDC Write Transaction 49 Register
#define EARC_CMDC_XACT_WR49					0x00001904

//eARC RX CMDC Write Transaction 50 Register
#define EARC_CMDC_XACT_WR50					0x00001908

//eARC RX CMDC Write Transaction 51 Register
#define EARC_CMDC_XACT_WR51					0x0000190C

//eARC RX CMDC Write Transaction 52 Register
#define EARC_CMDC_XACT_WR52					0x00001910

//eARC RX CMDC Write Transaction 53 Register
#define EARC_CMDC_XACT_WR53					0x00001914

//eARC RX CMDC Write Transaction 54 Register
#define EARC_CMDC_XACT_WR54					0x00001918

//eARC RX CMDC Write Transaction 55 Register
#define EARC_CMDC_XACT_WR55					0x0000191C

//eARC RX CMDC Write Transaction 56 Register
#define EARC_CMDC_XACT_WR56					0x00001920

//eARC RX CMDC Write Transaction 57 Register
#define EARC_CMDC_XACT_WR57					0x00001924

//eARC RX CMDC Write Transaction 58 Register
#define EARC_CMDC_XACT_WR58					0x00001928

//eARC RX CMDC Write Transaction 59 Register
#define EARC_CMDC_XACT_WR59					0x0000192C

//eARC RX CMDC Write Transaction 60 Register
#define EARC_CMDC_XACT_WR60					0x00001930

//eARC RX CMDC Write Transaction 61 Register
#define EARC_CMDC_XACT_WR61					0x00001934

//eARC RX CMDC Write Transaction 62 Register
#define EARC_CMDC_XACT_WR62					0x00001938

//eARC RX CMDC Write Transaction 63 Register
#define EARC_CMDC_XACT_WR63					0x0000193C

//eARC RX CMDC Write Transaction 64 Register
#define EARC_CMDC_XACT_WR64					0x00001940

//eARC RX CMDC Read Transaction 0 Register
#define EARC_CMDC_XACT_RD0					0x00001960

//eARC RX CMDC Read Transaction 1 Register
#define EARC_CMDC_XACT_RD1					0x00001964

//eARC RX CMDC Read Transaction 2 Register
#define EARC_CMDC_XACT_RD2					0x00001968

//eARC RX CMDC Read Transaction 3 Register
#define EARC_CMDC_XACT_RD3					0x0000196C

//eARC RX CMDC Read Transaction 4 Register
#define EARC_CMDC_XACT_RD4					0x00001970

//eARC RX CMDC Read Transaction 5 Register
#define EARC_CMDC_XACT_RD5					0x00001974

//eARC RX CMDC Read Transaction 6 Register
#define EARC_CMDC_XACT_RD6					0x00001978

//eARC RX CMDC Read Transaction 7 Register
#define EARC_CMDC_XACT_RD7					0x0000197C

//eARC RX CMDC Read Transaction 8 Register
#define EARC_CMDC_XACT_RD8					0x00001980

//eARC RX CMDC Read Transaction 9 Register
#define EARC_CMDC_XACT_RD9					0x00001984

//eARC RX CMDC Read Transaction 10 Register
#define EARC_CMDC_XACT_RD10					0x00001988

//eARC RX CMDC Read Transaction 11 Register
#define EARC_CMDC_XACT_RD11					0x0000198C

//eARC RX CMDC Read Transaction 12 Register
#define EARC_CMDC_XACT_RD12					0x00001990

//eARC RX CMDC Read Transaction 13 Register
#define EARC_CMDC_XACT_RD13					0x00001994

//eARC RX CMDC Read Transaction 14 Register
#define EARC_CMDC_XACT_RD14					0x00001998

//eARC RX CMDC Read Transaction 15 Register
#define EARC_CMDC_XACT_RD15					0x0000199C

//eARC RX CMDC Read Transaction 16 Register
#define EARC_CMDC_XACT_RD16					0x000019A0

//eARC RX CMDC Read Transaction 17 Register
#define EARC_CMDC_XACT_RD17					0x000019A4

//eARC RX CMDC Read Transaction 18 Register
#define EARC_CMDC_XACT_RD18					0x000019A8

//eARC RX CMDC Read Transaction 19 Register
#define EARC_CMDC_XACT_RD19					0x000019AC

//eARC RX CMDC Read Transaction 20 Register
#define EARC_CMDC_XACT_RD20					0x000019B0

//eARC RX CMDC Read Transaction 21 Register
#define EARC_CMDC_XACT_RD21					0x000019B4

//eARC RX CMDC Read Transaction 22 Register
#define EARC_CMDC_XACT_RD22					0x000019B8

//eARC RX CMDC Read Transaction 23 Register
#define EARC_CMDC_XACT_RD23					0x000019BC

//eARC RX CMDC Read Transaction 24 Register
#define EARC_CMDC_XACT_RD24					0x000019C0

//eARC RX CMDC Read Transaction 25 Register
#define EARC_CMDC_XACT_RD25					0x000019C4

//eARC RX CMDC Read Transaction 26 Register
#define EARC_CMDC_XACT_RD26					0x000019C8

//eARC RX CMDC Read Transaction 27 Register
#define EARC_CMDC_XACT_RD27					0x000019CC

//eARC RX CMDC Read Transaction 28 Register
#define EARC_CMDC_XACT_RD28					0x000019D0

//eARC RX CMDC Read Transaction 29 Register
#define EARC_CMDC_XACT_RD29					0x000019D4

//eARC RX CMDC Read Transaction 30 Register
#define EARC_CMDC_XACT_RD30					0x000019D8

//eARC RX CMDC Read Transaction 31 Register
#define EARC_CMDC_XACT_RD31					0x000019DC

//eARC RX CMDC Read Transaction 32 Register
#define EARC_CMDC_XACT_RD32					0x000019E0

//eARC RX CMDC Read Transaction 33 Register
#define EARC_CMDC_XACT_RD33					0x000019E4

//eARC RX CMDC Read Transaction 34 Register
#define EARC_CMDC_XACT_RD34					0x000019E8

//eARC RX CMDC Read Transaction 35 Register
#define EARC_CMDC_XACT_RD35					0x000019EC

//eARC RX CMDC Read Transaction 36 Register
#define EARC_CMDC_XACT_RD36					0x000019F0

//eARC RX CMDC Read Transaction 37 Register
#define EARC_CMDC_XACT_RD37					0x000019F4

//eARC RX CMDC Read Transaction 38 Register
#define EARC_CMDC_XACT_RD38					0x000019F8

//eARC RX CMDC Read Transaction 39 Register
#define EARC_CMDC_XACT_RD39					0x000019FC

//eARC RX CMDC Read Transaction 40 Register
#define EARC_CMDC_XACT_RD40					0x00001A00

//eARC RX CMDC Read Transaction 41 Register
#define EARC_CMDC_XACT_RD41					0x00001A04

//eARC RX CMDC Read Transaction 42 Register
#define EARC_CMDC_XACT_RD42					0x00001A08

//eARC RX CMDC Read Transaction 43 Register
#define EARC_CMDC_XACT_RD43					0x00001A0C

//eARC RX CMDC Read Transaction 44 Register
#define EARC_CMDC_XACT_RD44					0x00001A10

//eARC RX CMDC Read Transaction 45 Register
#define EARC_CMDC_XACT_RD45					0x00001A14

//eARC RX CMDC Read Transaction 46 Register
#define EARC_CMDC_XACT_RD46					0x00001A18

//eARC RX CMDC Read Transaction 47 Register
#define EARC_CMDC_XACT_RD47					0x00001A1C

//eARC RX CMDC Read Transaction 48 Register
#define EARC_CMDC_XACT_RD48					0x00001A20

//eARC RX CMDC Read Transaction 49 Register
#define EARC_CMDC_XACT_RD49					0x00001A24

//eARC RX CMDC Read Transaction 50 Register
#define EARC_CMDC_XACT_RD50					0x00001A28

//eARC RX CMDC Read Transaction 51 Register
#define EARC_CMDC_XACT_RD51					0x00001A2C

//eARC RX CMDC Read Transaction 52 Register
#define EARC_CMDC_XACT_RD52					0x00001A30

//eARC RX CMDC Read Transaction 53 Register
#define EARC_CMDC_XACT_RD53					0x00001A34

//eARC RX CMDC Read Transaction 54 Register
#define EARC_CMDC_XACT_RD54					0x00001A38

//eARC RX CMDC Read Transaction 55 Register
#define EARC_CMDC_XACT_RD55					0x00001A3C

//eARC RX CMDC Read Transaction 56 Register
#define EARC_CMDC_XACT_RD56					0x00001A40

//eARC RX CMDC Read Transaction 57 Register
#define EARC_CMDC_XACT_RD57					0x00001A44

//eARC RX CMDC Read Transaction 58 Register
#define EARC_CMDC_XACT_RD58					0x00001A48

//eARC RX CMDC Read Transaction 59 Register
#define EARC_CMDC_XACT_RD59					0x00001A4C

//eARC RX CMDC Read Transaction 60 Register
#define EARC_CMDC_XACT_RD60					0x00001A50

//eARC RX CMDC Read Transaction 61 Register
#define EARC_CMDC_XACT_RD61					0x00001A54

//eARC RX CMDC Read Transaction 62 Register
#define EARC_CMDC_XACT_RD62					0x00001A58

//eARC RX CMDC Read Transaction 63 Register
#define EARC_CMDC_XACT_RD63					0x00001A5C

//eARC RX CMDC Read Transaction 64 Register
#define EARC_CMDC_XACT_RD64					0x00001A60

//eARC RX CMDC Synchronization Configuration Register
#define EARC_CMDC_SYNC_CONFIG					0x00001B00
#define EARC_CMDC_SYNC_CONFIG_PULSE_FILTER_DATAIN_MASK		0x000000FF
#define EARC_CMDC_SYNC_CONFIG_PULSE_FILTER_DATAOUT_MASK		0x0000FF00

//eARC RX DMAC PHY Control Register
#define EARC_DMAC_PHY_CONTROL					0x00001C00
#define EARC_DMAC_PHY_CONTROL_RCV_EN_MASK			0x00000001

//eARC RX DMAC Configuration Register
#define EARC_DMAC_CONFIG					0x00001C08
#define EARC_DMAC_CONFIG_ECC_ERRCORRECT_EN_MASK			0x00000001
#define EARC_DMAC_CONFIG_ECC_INV_SYNDBITORDER_MASK		0x00000002
#define EARC_DMAC_CONFIG_PAO_EN_MASK				0x00000010
#define EARC_DMAC_CONFIG_SAO_SPDIF_EN_MASK			0x00000020
#define EARC_DMAC_CONFIG_SAO_I2S_EN_MASK			0x00000040
#define EARC_DMAC_CONFIG_SAO_I2S_BPCUV_EN_MASK			0x00000080

//eARC RX DMAC Control 0 Register
#define EARC_DMAC_CONTROL0					0x00001C0C
#define EARC_DMAC_CONTROL0_DMAC_EN_MASK				0x00000001
#define EARC_DMAC_CONTROL0_DMAC_AUDIO_EN_MASK			0x00000002
#define EARC_DMAC_CONTROL0_DMAC_AUDIO_TYPE_OVR_EN_MASK		0x00000010
#define EARC_DMAC_CONTROL0_DMAC_AUDIO_TYPE_OVR_VAL_MASK		0x000003E0

//eARC RX DMAC Control 1 Register
#define EARC_DMAC_CONTROL1					0x00001C10

//eARC RX DMAC Status Register
#define EARC_DMAC_STATUS					0x00001C14
#define EARC_DMAC_STATUS_AUDIO_TYPE_MASK			0x0000001F
#define EARC_DMAC_STATUS_AUDIO_MUTE_MASK			0x00000100
#define EARC_DMAC_STATUS_BPCLK_MFACTOR_MASK			0x00003000

//eARC RX DMAC Audio Channel Status 0 Register
#define EARC_DMAC_CHSTATUS0					0x00001C18

//eARC RX DMAC Audio Channel Status 1 Register
#define EARC_DMAC_CHSTATUS1					0x00001C1C

//eARC RX DMAC Audio Channel Status 2 Register
#define EARC_DMAC_CHSTATUS2					0x00001C20

//eARC RX DMAC Audio Channel Status 3 Register
#define EARC_DMAC_CHSTATUS3					0x00001C24

//eARC RX DMAC Audio Channel Status 4 Register
#define EARC_DMAC_CHSTATUS4					0x00001C28

//eARC RX DMAC Audio Channel Status 5 Register
#define EARC_DMAC_CHSTATUS5					0x00001C2C

//eARC RX DMAC User Data H14b ACP Payload 0 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT0			0x00001C30

//eARC RX DMAC User Data H14b ACP Payload 1 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT1			0x00001C34

//eARC RX DMAC User Data H14b ACP Payload 2 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT2			0x00001C38

//eARC RX DMAC User Data H14b ACP Payload 3 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT3			0x00001C3C

//eARC RX DMAC User Data H14b ACP Payload 4 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT4			0x00001C40

//eARC RX DMAC User Data H14b ACP Payload 5 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT5			0x00001C44

//eARC RX DMAC User Data H14b ACP Payload 6 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT6			0x00001C48

//eARC RX DMAC User Data H14b ACP Payload 7 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT7			0x00001C4C

//eARC RX DMAC User Data H14b ACP Payload 8 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT8			0x00001C50

//eARC RX DMAC User Data H14b ACP Payload 9 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT9			0x00001C54

//eARC RX DMAC User Data H14b ACP Payload 10 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT10			0x00001C58

//eARC RX DMAC User Data H14b ACP Payload 11 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ACP_PKT11			0x00001C5C

//eARC RX DMAC User Data H14b ISRC1 Payload 0 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT0			0x00001C60

//eARC RX DMAC User Data H14b ISRC1 Payload 1 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT1			0x00001C64

//eARC RX DMAC User Data H14b ISRC1 Payload 2 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT2			0x00001C68

//eARC RX DMAC User Data H14b ISRC1 Payload 3 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT3			0x00001C6C

//eARC RX DMAC User Data H14b ISRC1 Payload 4 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT4			0x00001C70

//eARC RX DMAC User Data H14b ISRC1 Payload 5 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT5			0x00001C74

//eARC RX DMAC User Data H14b ISRC1 Payload 6 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT6			0x00001C78

//eARC RX DMAC User Data H14b ISRC1 Payload 7 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT7			0x00001C7C

//eARC RX DMAC User Data H14b ISRC1 Payload 8 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT8			0x00001C80

//eARC RX DMAC User Data H14b ISRC1 Payload 9 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT9			0x00001C84

//eARC RX DMAC User Data H14b ISRC1 Payload 10 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT10			0x00001C88

//eARC RX DMAC User Data H14b ISRC1 Payload 11 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC1_PKT11			0x00001C8C

//eARC RX DMAC User Data H14b ISRC2 Payload 0 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT0			0x00001C90

//eARC RX DMAC User Data H14b ISRC2 Payload 1 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT1			0x00001C94

//eARC RX DMAC User Data H14b ISRC2 Payload 2 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT2			0x00001C98

//eARC RX DMAC User Data H14b ISRC2 Payload 3 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT3			0x00001C9C

//eARC RX DMAC User Data H14b ISRC2 Payload 4 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT4			0x00001CA0

//eARC RX DMAC User Data H14b ISRC2 Payload 5 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT5			0x00001CA4

//eARC RX DMAC User Data H14b ISRC2 Payload 6 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT6			0x00001CA8

//eARC RX DMAC User Data H14b ISRC2 Payload 7 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT7			0x00001CAC

//eARC RX DMAC User Data H14b ISRC2 Payload 8 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT8			0x00001CB0

//eARC RX DMAC User Data H14b ISRC2 Payload 9 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT9			0x00001CB4

//eARC RX DMAC User Data H14b ISRC2 Payload 10 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT10			0x00001CB8

//eARC RX DMAC User Data H14b ISRC2 Payload 11 Register
#define EARC_DMAC_USRDATA_MSG_HDMI_ISRC2_PKT11			0x00001CBC

//eARC RX DMAC User Data Generic Message Payload 0 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC0				0x00001CC0

//eARC RX DMAC User Data Generic Message Payload 1 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC1				0x00001CC4

//eARC RX DMAC User Data Generic Message Payload 2 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC2				0x00001CC8

//eARC RX DMAC User Data Generic Message Payload 3 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC3				0x00001CCC

//eARC RX DMAC User Data Generic Message Payload 4 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC4				0x00001CD0

//eARC RX DMAC User Data Generic Message Payload 5 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC5				0x00001CD4

//eARC RX DMAC User Data Generic Message Payload 6 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC6				0x00001CD8

//eARC RX DMAC User Data Generic Message Payload 7 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC7				0x00001CDC

//eARC RX DMAC User Data Generic Message Payload 8 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC8				0x00001CE0

//eARC RX DMAC User Data Generic Message Payload 9 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC9				0x00001CE4

//eARC RX DMAC User Data Generic Message Payload 10 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC10				0x00001CE8

//eARC RX DMAC User Data Generic Message Payload 11 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC11				0x00001CEC

//eARC RX DMAC User Data Generic Message Payload 12 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC12				0x00001CF0

//eARC RX DMAC User Data Generic Message Payload 13 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC13				0x00001CF4

//eARC RX DMAC User Data Generic Message Payload 14 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC14				0x00001CF8

//eARC RX DMAC User Data Generic Message Payload 15 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC15				0x00001CFC

//eARC RX DMAC User Data Generic Message Payload 16 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC16				0x00001D00

//eARC RX DMAC User Data Generic Message Payload 17 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC17				0x00001D04

//eARC RX DMAC User Data Generic Message Payload 18 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC18				0x00001D08

//eARC RX DMAC User Data Generic Message Payload 19 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC19				0x00001D0C

//eARC RX DMAC User Data Generic Message Payload 20 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC20				0x00001D10

//eARC RX DMAC User Data Generic Message Payload 21 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC21				0x00001D14

//eARC RX DMAC User Data Generic Message Payload 22 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC22				0x00001D18

//eARC RX DMAC User Data Generic Message Payload 23 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC23				0x00001D1C

//eARC RX DMAC User Data Generic Message Payload 24 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC24				0x00001D20

//eARC RX DMAC User Data Generic Message Payload 25 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC25				0x00001D24

//eARC RX DMAC User Data Generic Message Payload 26 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC26				0x00001D28

//eARC RX DMAC User Data Generic Message Payload 27 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC27				0x00001D2C

//eARC RX DMAC User Data Generic Message Payload 28 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC28				0x00001D30

//eARC RX DMAC User Data Generic Message Payload 29 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC29				0x00001D34

//eARC RX DMAC User Data Generic Message Payload 30 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC30				0x00001D38

//eARC RX DMAC User Data Generic Message Payload 31 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC31				0x00001D3C

//eARC RX DMAC User Data Generic Message Payload 32 Register
#define EARC_DMAC_USRDATA_MSG_GENERIC32				0x00001D40

//eARC RX DMAC Audio Channel Status Streamer 0 Register
#define EARC_DMAC_CHSTATUS_STREAMER0				0x00001D44

//eARC RX DMAC Audio Channel Status Streamer 1 Register
#define EARC_DMAC_CHSTATUS_STREAMER1				0x00001D48

//eARC RX DMAC Audio Channel Status Streamer 2 Register
#define EARC_DMAC_CHSTATUS_STREAMER2				0x00001D4C

//eARC RX DMAC Audio Channel Status Streamer 3 Register
#define EARC_DMAC_CHSTATUS_STREAMER3				0x00001D50

//eARC RX DMAC Audio Channel Status Streamer 4 Register
#define EARC_DMAC_CHSTATUS_STREAMER4				0x00001D54

//eARC RX DMAC Audio Channel Status Streamer 5 Register
#define EARC_DMAC_CHSTATUS_STREAMER5				0x00001D58

//eARC RX DMAC Audio Channel Status Streamer 6 Register
#define EARC_DMAC_CHSTATUS_STREAMER6				0x00001D5C

//eARC RX DMAC Audio Channel Status Streamer 7 Register
#define EARC_DMAC_CHSTATUS_STREAMER7				0x00001D60

//eARC RX DMAC Audio Channel Status Streamer 8 Register
#define EARC_DMAC_CHSTATUS_STREAMER8				0x00001D64

//eARC RX DMAC Audio Channel Status Streamer 9 Register
#define EARC_DMAC_CHSTATUS_STREAMER9				0x00001D68

//eARC RX DMAC Audio Channel Status Streamer 10 Register
#define EARC_DMAC_CHSTATUS_STREAMER10				0x00001D6C

//eARC RX DMAC Audio Channel Status Streamer 11 Register
#define EARC_DMAC_CHSTATUS_STREAMER11				0x00001D70

//eARC RX DMAC Audio Channel Status Streamer 12 Register
#define EARC_DMAC_CHSTATUS_STREAMER12				0x00001D74

//eARC RX DMAC Audio Channel Status Streamer 13 Register
#define EARC_DMAC_CHSTATUS_STREAMER13				0x00001D78

//eARC RX DMAC Audio Channel Status Streamer 14 Register
#define EARC_DMAC_CHSTATUS_STREAMER14				0x00001D7C

//eARC RX DMAC Audio User Data Streamer 0 Register
#define EARC_DMAC_USRDATA_STREAMER0				0x00001D80

/*****************************************************************************
 *                                                                           *
 *                            Interrupt Registers                            *
 *                                                                           *
 *****************************************************************************/

//Main Unit Interrupt Vector Table Register
#define MU_INTVEC_INDEX					0x00003000
#define MU_INTVEC_INDEX_0_MASK				0x00000001
#define MU_INTVEC_INDEX_1_MASK				0x00000002

//Main Unit Interrupt Status 0 Register
#define MU_INT0_STATUS					0x00003010
#define MU_INT0_IPICLK_LOCKED_CHG_MASK			0x00000001
#define MU_INT0_IPICLK_OFF_CHG_MASK			0x00000002
#define MU_INT0_VIDQPCLK_LOCKED_CHG_MASK		0x00000004
#define MU_INT0_VIDQPCLK_OFF_CHG_MASK			0x00000008
#define MU_INT0_LINKQPCLK_LOCKED_CHG_MASK		0x00000010
#define MU_INT0_LINKQPCLK_OFF_CHG_MASK			0x00000020
#define MU_INT0_AUDCLK_LOCKED_CHG_MASK			0x00000040
#define MU_INT0_AUDCLK_OFF_CHG_MASK			0x00000080
#define MU_INT0_EARC_BPCLK_LOCKED_CHG_MASK		0x00100000
#define MU_INT0_EARC_BPCLK_OFF_CHG_MASK			0x00200000
#define MU_INT0_EARC_BPCLK_VALID_STABLE_CHG_MASK	0x00400000
#define MU_INT0_TIMER_BASE_LOCKED_MASK			0x01000000
#define MU_INT0_RESET_MANAGER_STATUS_END_MASK		0x20000000
#define MU_INT0_APB_REGBANK_READY_MASK			0x80000080
#define MU_INT0_ALL_MASK				0xA17000FF

//Main Unit Interrupt Mask 0 Register
#define MU_INT0_MASK					0x00003014

//Main Unit Interrupt Clear 0 Register
#define MU_INT0_CLEAR					0x00003018

//Main Unit Interrupt Force 0 Register
#define MU_INT0_FORCE					0x0000301C

//Main Unit Interrupt Status 1 Register
#define MU_INT1_STATUS					0x00003020
#define MU_INT1_I2CM_DONE_MASK				0x00000001
#define MU_INT1_I2CM_RR_MASK				0x00000002
#define MU_INT1_I2CM_NACK_RCVD_MASK			0x00000004
#define MU_INT1_I2CM_ARB_LOST_MASK			0x00000008
#define MU_INT1_SCDC_UPD_FLAGS_RD_MASK			0x00000100
#define MU_INT1_SCDC_UPD_FLAGS_CHG_MASK			0x00000200
#define MU_INT1_SCDC_UPD_FLAGS_CLR_MASK			0x00000400
#define MU_INT1_SCDC_RR_REPLY_STOP_MASK			0x00000800
#define MU_INT1_SCDC_NACK_RCVD_MASK			0x00001000
#define MU_INT1_FLT_EXIT_TO_LTSP_MASK			0x00100000
#define MU_INT1_FLT_EXIT_TO_LTS4_MASK			0x00200000
#define MU_INT1_FLT_EXIT_TO_LTSL_MASK			0x00400000
#define MU_INT1_FLT_ALL_MASK				0x00700000
#define MU_INT1_ALL_MASK				0x00701F07

//Main Unit Interrupt Mask 1 Register
#define MU_INT1_MASK					0x00003024

//Main Unit Interrupt Clear 1 Register
#define MU_INT1_CLEAR					0x00003028

//Main Unit Interrupt Force 1 Register
#define MU_INT1_FORCE					0x0000302C

//AVP Interrupt Vector Table Register
#define AVP_INTVEC_INDEX				0x00003800
#define AVP_INTVEC_INDEX_0_MASK				0x00000001
#define AVP_INTVEC_INDEX_1_MASK				0x00000002
#define AVP_INTVEC_INDEX_2_MASK				0x00000004
#define AVP_INTVEC_INDEX_3_MASK				0x00000008
#define AVP_INTVEC_INDEX_4_MASK				0x00000010
#define AVP_INTVEC_INDEX_5_MASK				0x00000020
#define AVP_INTVEC_INDEX_6_MASK				0x00000040

//AVP Interrupt Status 0 Register
#define AVP_INT0_STATUS					0x00003810
#define AVP_INT0_IPI_NOT_SUPPORTED_MASK			0x00000001
#define AVP_INT0_DEFAULT_PHASE_VIO_MASK			0x00000010
#define AVP_INT0_EMP_VTEM_TX_DONE_MASK			0x00000020
#define AVP_INT0_EMP_CVTEM_TX_DONE_MASK			0x00000040
#define AVP_INT0_EMP_EXTMEM_TX_DONE_MASK		0x00000080
#define AVP_INT0_EMP_TX_DONE_MASK			0x00000100
#define AVP_INT0_EMP_TX_MTW_VIO_MASK			0x00000200
#define AVP_INT0_EMP_EXTMEM_THR_PASS_MASK		0x00000400
#define AVP_INT0_TMDS_FIFO_OVF_MASK			0x00001000
#define AVP_INT0_AUDIO_FIFO_UDF_MASK			0x00010000
#define AVP_INT0_PKTZ_FRLCHAR_FIFO_OVF_MASK		0x00100000
#define AVP_INT0_AUDPKT_INVALID_SAMPLE_PRESENT_MASK	0x01000000
#define AVP_INT0_AUDPKT_3DAUD_PKT0_MISSING_MASK		0x02000000
#define AVP_INT0_AUDPKT_3DAUD_PKT1_MISSING_MASK		0x04000000
#define AVP_INT0_AUDPKT_3DAUD_PKT2_MISSING_MASK		0x08000000
#define AVP_INT0_AUDPKT_3DAUD_PKT3_MISSING_MASK		0x10000000

//AVP Interrupt Mask 0 Register
#define AVP_INT0_MASK					0x00003814

//AVP Interrupt Clear 0 Register
#define AVP_INT0_CLEAR					0x00003818

//AVP Interrupt Force 0 Register
#define AVP_INT0_FORCE					0x0000381C

//AVP Interrupt Status 1 Register
#define AVP_INT1_STATUS					0x00003820
#define AVP_INT1_DKSET_DONE_MASK			0x00000001
#define AVP_INT1_KSV_LIST_DONE_MASK			0x00000002
#define AVP_INT1_I2C_TIMEOUT_MASK			0x00000004
#define AVP_INT1_I2C_NACK_RCV_MASK			0x00000008
#define AVP_INT1_BKSV_REVOC_FAIL_MASK			0x00000010
#define AVP_INT1_SHA1_MSG_MASK				0x00000020
#define AVP_INT1_AUTH_CHG_MASK				0x00000040
#define AVP_INT1_AUDPKT_ACR_CTS_CALC_MASK		0x00000100
#define AVP_INT1_FRL_RSFEC_ALIGN_BUFFER_FULL_MASK	0x00001000
#define AVP_INT1_TMDS_FIFO_UDF_MASK			0x00010000
#define AVP_INT1_PKTZ_FRLCHAR_FIFO_UDF_MASK		0x00100000
#define AVP_INT1_OPERATION_MODE_FAILED_MASK		0x01000000
#define AVP_INT1_BKSV_INVALID_MASK			0x02000000
#define AVP_INT1_INTEGRITY_INVALID_MASK			0x04000000
#define AVP_INT1_TOPOLOGY_INVALID_MASK			0x08000000
#define AVP_INT1_SYNC_INVALID_MASK			0x10000000
#define AVP_INT1_READY_BIT_FAIL_MASK			0x20000000
#define AVP_INT1_ALL_HDCP_MASK				0x3F00007F
#define AVP_INT1_ALL_OTHERS_MASK			0x00111100

//AVP Interrupt Mask 1 Register
#define AVP_INT1_MASK					0x00003824

//AVP Interrupt Clear 1 Register
#define AVP_INT1_CLEAR					0x00003828

//AVP Interrupt Force 1 Register
#define AVP_INT1_FORCE					0x0000382C

//AVP Interrupt Status 2 Register
#define AVP_INT2_STATUS					0x00003830
#define AVP_INT2_NULL_SENT_MASK				0x00000001
#define AVP_INT2_ACR_SENT_MASK				0x00000002
#define AVP_INT2_AUDS_SENT_MASK				0x00000004
#define AVP_INT2_GCP_SENT_MASK				0x00000008
#define AVP_INT2_ACP_SENT_MASK				0x00000010
#define AVP_INT2_ISRC1_SENT_MASK			0x00000020
#define AVP_INT2_ISRC2_SENT_MASK			0x00000040
#define AVP_INT2_GMD_SENT_MASK				0x00000080
#define AVP_INT2_AMD_SENT_MASK				0x00000100
#define AVP_INT2_EMP_SENT_MASK				0x00000200
#define AVP_INT2_VSI_SENT_MASK				0x00000400
#define AVP_INT2_AVI_SENT_MASK				0x00000800
#define AVP_INT2_SPDI_SENT_MASK				0x00001000
#define AVP_INT2_AUDI_SENT_MASK				0x00002000
#define AVP_INT2_NVI_SENT_MASK				0x00004000
#define AVP_INT2_DRMI_SENT_MASK				0x00008000
#define AVP_INT2_GHDMI1_SENT_MASK			0x00010000
#define AVP_INT2_GHDMI2_SENT_MASK			0x00020000
#define AVP_INT2_ACP_TO_MASK				0x00040000
#define AVP_INT2_SPDI_TO_MASK				0x00080000
#define AVP_INT2_ISRC_TO_MASK				0x00100000
#define AVP_INT2_ACR_TO_MASK				0x00200000
#define AVP_INT2_GHDMI1_UNSENT_MASK			0x00400000
#define AVP_INT2_GHDMI2_UNSENT_MASK			0x00800000
#define AVP_INT2_GMD_P0PKT_UNSENT_MASK			0x01000000

//AVP Interrupt Mask 2 Register
#define AVP_INT2_MASK					0x00003834

//AVP Interrupt Clear 2 Register
#define AVP_INT2_CLEAR					0x00003838

//AVP Interrupt Force 2 Register
#define AVP_INT2_FORCE					0x0000383C

//AVP Interrupt Status 3 Register
#define AVP_INT3_STATUS					0x00003840
#define AVP_INT3_ESM_GLOBAL_GPIO_OUT3_CHG_MASK		0x00000001
#define AVP_INT3_ESM_GLOBAL_GPIO_OUT2_CHG_MASK		0x00000002
#define AVP_INT3_ESM_GLOBAL_GPIO_OUT1_CHG_MASK		0x00000004
#define AVP_INT3_ESM_GLOBAL_GPIO_OUT0_CHG_MASK		0x00000008
#define AVP_INT3_ESM_P0_GPIO_OUT15_CHG_MASK		0x00000010
#define AVP_INT3_ESM_P0_GPIO_OUT14_CHG_MASK		0x00000020
#define AVP_INT3_ESM_P0_GPIO_OUT13_CHG_MASK		0x00000040
#define AVP_INT3_ESM_P0_GPIO_OUT12_CHG_MASK		0x00000080
#define AVP_INT3_ESM_P0_GPIO_OUT11_CHG_MASK		0x00000100
#define AVP_INT3_ESM_P0_GPIO_OUT10_CHG_MASK		0x00000200
#define AVP_INT3_ESM_P0_GPIO_OUT9_CHG_MASK		0x00000400
#define AVP_INT3_ESM_P0_GPIO_OUT8_CHG_MASK		0x00000800
#define AVP_INT3_ESM_P0_GPIO_OUT7_CHG_MASK		0x00001000
#define AVP_INT3_ESM_P0_GPIO_OUT6_CHG_MASK		0x00002000
#define AVP_INT3_ESM_P0_GPIO_OUT5_CHG_MASK		0x00004000
#define AVP_INT3_ESM_P0_GPIO_OUT4_CHG_MASK		0x00008000
#define AVP_INT3_ESM_P0_GPIO_OUT3_CHG_MASK		0x00010000
#define AVP_INT3_ESM_P0_GPIO_OUT2_CHG_MASK		0x00020000
#define AVP_INT3_ESM_P0_GPIO_OUT1_CHG_MASK		0x00040000
#define AVP_INT3_ESM_P0_GPIO_OUT0_CHG_MASK		0x00080000
#define AVP_INT3_ESM_TX_MASK				0x000F8000

//AVP Interrupt Mask 3 Register
#define AVP_INT3_MASK					0x00003844

//AVP Interrupt Clear 3 Register
#define AVP_INT3_CLEAR					0x00003848

//AVP Interrupt Force 3 Register
#define AVP_INT3_FORCE					0x0000384C

//AVP Interrupt Status 4 Register
#define AVP_INT4_STATUS					0x00003850
#define AVP_INT4_PAI_PAIR_LAYOUT_CHG_MASK		0x00000001
#define AVP_INT4_PAI_FORMAT_CHG_MASK			0x00000002
#define AVP_INT4_AUDIO_FIFO_OVF_MASK			0x00000010
#define AVP_INT4_I2S_WS_ERR_MASK			0x00000100
#define AVP_INT4_I2S_BBIT_ERR_MASK			0x00000200
#define AVP_INT4_SPDIF_LINE0_PARITY_ERR_MASK		0x00001000
#define AVP_INT4_SPDIF_LINE1_PARITY_ERR_MASK		0x00002000
#define AVP_INT4_SPDIF_LINE2_PARITY_ERR_MASK		0x00004000
#define AVP_INT4_SPDIF_LINE3_PARITY_ERR_MASK		0x00008000
#define AVP_INT4_SPDIF_LINE0_PROTOCOL_ERR_MASK		0x00010000
#define AVP_INT4_SPDIF_LINE1_PROTOCOL_ERR_MASK		0x00020000
#define AVP_INT4_SPDIF_LINE2_PROTOCOL_ERR_MASK		0x00040000
#define AVP_INT4_SPDIF_LINE3_PROTOCOL_ERR_MASK		0x00080000

//AVP Interrupt Mask 4 Register
#define AVP_INT4_MASK					0x00003854

//AVP Interrupt Clear 4 Register
#define AVP_INT4_CLEAR					0x00003858

//AVP Interrupt Force 4 Register
#define AVP_INT4_FORCE					0x0000385C

//AVP Interrupt Status 5 Register
#define AVP_INT5_STATUS					0x00003860
#define AVP_INT5_VIDIF_VMON_HMEAS_MASK			0x00000001
#define AVP_INT5_VIDIF_VMON_VMEAS_MASK			0x00000002
#define AVP_INT5_VID_FIFO_UDF_MASK			0x00000010

//AVP Interrupt Mask 5 Register
#define AVP_INT5_MASK					0x00003864

//AVP Interrupt Clear 5 Register
#define AVP_INT5_CLEAR					0x00003868

//AVP Interrupt Force 5 Register
#define AVP_INT5_FORCE					0x0000386C

//AVP Interrupt Status 6 Register
#define AVP_INT6_STATUS					0x00003870
#define AVP_INT6_VID_FIFO_OVF_MASK			0x00000001

//AVP Interrupt Mask 6 Register
#define AVP_INT6_MASK					0x00003874

//AVP Interrupt Clear 6 Register
#define AVP_INT6_CLEAR					0x00003878

//AVP Interrupt Force 6 Register
#define AVP_INT6_FORCE					0x0000387C

//CEC Interrupt Status Register
#define CEC_INT_STATUS					0x00004000
#define CEC_INT_TX_DONE_MASK				0x00000001
#define CEC_INT_TX_NACK_MASK				0x00000002
#define CEC_INT_TX_ARBLOST_MASK				0x00000004
#define CEC_INT_TX_LINE_ERR_MASK			0x00000008
#define CEC_INT_TX_NRETRANSMIT_FAIL_MASK		0x00000010
#define CEC_INT_TX_FRAME_DISCARDED_MASK			0x00000020
#define CEC_INT_TX_BUSY_MASK				0x00000100
#define CEC_INT_RX_BUSY_MASK				0x00000200
#define CEC_INT_TX_DRIVE_ERR_MASK			0x00000400
#define CEC_INT_RX_EOM_MASK				0x00000800
#define CEC_INT_RX_NOTIFY_ERR_MASK			0x00001000
#define CEC_INT_ALL_MASK				0x00001F3F

//CEC Interrupt Mask Register
#define CEC_INT_MASK					0x00004004

//CEC Interrupt Clear Register
#define CEC_INT_CLEAR					0x00004008

//CEC Interrupt Force Register
#define CEC_INT_FORCE					0x0000400C

//eARC RX Interrupt Vector Table Register
#define EARC_INTVEC_INDEX				0x00004800
#define EARC_INTVEC_INDEX_0_MASK			0x00000001
#define EARC_INTVEC_INDEX_1_MASK			0x00000002

//eARC RX Interrupt Status 0 Register
#define EARC_INT0_STATUS				0x00004810
#define EARC_INT0_CMDC_DISCOVERY_DONE_MASK		0x00000100
#define EARC_INT0_CMDC_DISCOVERY_TIMEOUT_MASK		0x00000200
#define EARC_INT0_CMDC_HEARTBEAT_LOST_MASK		0x00000400
#define EARC_INT0_CMDC_HEARTBEAT_STAT_CHNG_MASK		0x00000800
#define EARC_INT0_CMDC_HEARTBEAT_CAP_CHNG_MASK		0x00001000
#define EARC_INT0_CMDC_RXPKT_BADPREAMBLE_MASK		0x00002000
#define EARC_INT0_CMDC_RXPKT_BADSYNC_MASK		0x00004000
#define EARC_INT0_CMDC_RXPKT_INCOMPLETE_MASK		0x00008000
#define EARC_INT0_CMDC_RXPKT_BADECC_MASK		0x00010000
#define EARC_INT0_CMDC_RXPKT_BADPAR_MASK		0x00020000
#define EARC_INT0_CMDC_XACT_DONE_MASK			0x00040000
#define EARC_INT0_CMDC_XACT_READ_DATA_MASK		0x00080000
#define EARC_INT0_CMDC_HEARTBEAT_EARCTXSTAT_UPD_MASK	0x00100000
#define EARC_INT0_CMDC_HEARTBEAT_EARCVALID0_MASK	0x00200000
#define EARC_INT0_ALL_MASK				0x003FFF00

//eARC RX Interrupt Mask 0 Register
#define EARC_INT0_MASK					0x00004814

//eARC RX Interrupt Clear 0 Register
#define EARC_INT0_CLEAR					0x00004818

//eARC RX Interrupt Force 0 Register
#define EARC_INT0_FORCE					0x0000481C

//eARC RX Interrupt Status 1 Register
#define EARC_INT1_STATUS				0x00004820
#define EARC_INT1_DMAC_BPRXSAMPLE_PARITY_ERR_MASK	0x00000001
#define EARC_INT1_DMAC_CHSTATUS_BLOCK_CHG_MASK		0x00000002
#define EARC_INT1_DMAC_AUDIO_TYPE_CHG_MASK		0x00000004
#define EARC_INT1_DMAC_AUDIO_MUTE_CHG_MASK		0x00000008
#define EARC_INT1_DMAC_ECC_RX_CHECK_UNCERR_MASK		0x00000010
#define EARC_INT1_DMAC_HDMI_ACP_CHG_MASK		0x00000020
#define EARC_INT1_DMAC_HDMI_ISRC1_CHG_MASK		0x00000040
#define EARC_INT1_DMAC_HDMI_ISRC2_CHG_MASK		0x00000080
#define EARC_INT1_DMAC_GENERIC_CHG_MASK			0x00000100
#define EARC_INT1_ALL_MASK				0x000001FF

//eARC RX Interrupt Mask 1 Register
#define EARC_INT1_MASK					0x00004824

//eARC RX Interrupt Clear 1 Register
#define EARC_INT1_CLEAR					0x00004828

//eARC RX Interrupt Force 1 Register
#define EARC_INT1_FORCE					0x0000482C

/*****************************************************************************
 *                                                                           *
 *                            TILE Registers                            *
 *                                                                           *
 *****************************************************************************/
//Tx PHY Configuration
#define HDMI_PHY_CTRL				    0x00000000
#define SRAM_BOOTLOAD_BYPASS		    0x00000001
#define SRAM_BYPASS    		            0x00000002
#define SRAM_EXT_LD_DONE                0x00000003

#define HDMI_HPD_CTRL                   0x00000060
#define HPD_ENVENT_EN                   0x1
#define HPD_UNPLUG_ERR_EN               (1<<1)
#define HPD_UNPLUG_EN                   (1<<2)
#define HPD_PLUG_EN                     (1<<3)
#define HPD_IRQ_EN                      (1<<4)
#define FORCE_HPD                       (1<<5)

#define HDMI_HPD_INT_STATUS             0x0000006C
#define TILE_HPD_EVENT                  0x00000000
#define TILE_HPD_STATE                  0x00000700
#define TILE_HPD_STATUS                 0x00000800
#define TILE_HPD_OFFSET                 0x0000000B 
#define HPD_STATUS_MASE                 (1 << 11)

#define  ALL_HDMIDP_PHY_IF_CLR  0xa4
#define  ALL_HDMIDP_PHY_IF_RAW  0xa8
#define  ALL_HDMI_PHY_CTRL      0x0
#define  ALL_HDMI_PCS_CTRL      0x8
#define  ALL_HDMI_HPD_CTRL      0x60
#define  ALL_HDMI_HPD_CLR       0x64
#define  ALL_HDMI_HPD_RAW_STATUS  0x68
#define  ALL_HDMI_HPD_INT_STATUS  0x6c

#endif
