#include <linux/version.h>
#include <linux/platform_device.h>
#include <linux/dmaengine.h>
#include <linux/module.h>
#include <linux/dma-mapping.h>
#include <linux/types.h>
#include <linux/of_dma.h>
#include <linux/of_device.h>
#include <linux/kernel.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/uaccess.h>
#include <linux/sched/signal.h>
#include "dma_drv.h"
#include "xdx_dma.h"
#include "img_types.h"

struct dma_chan *get_dma_channel(struct device *dma_dev)
{
	struct dma_chan *vchan;

	if (!dma_dev)
	{
		pr_err("[%s]dma_dev is NULL", __func__);
		return NULL;
	}
	dev_info(dma_dev, "xdx_dma(%s) = 0x%llx\n", dev_name(dma_dev), (IMG_UINT64)(dma_dev));
	vchan = dma_request_chan(dma_dev, "xdx_dma");
	if(IS_ERR_OR_NULL(vchan)) {
		dev_err(dma_dev, "Get DMAchan failed %ld.\n", PTR_ERR(vchan));
		return NULL;
	}
	dev_dbg(dma_dev, "Requested Dma chan id = %d\n", vchan->chan_id);
	return vchan;
}

struct dma_chan *get_tdma_channel(struct device *tdma_dev)
{
	struct dma_chan *vchan;

	if (!tdma_dev)
	{
		pr_err("[%s]tdma_dev is NULL", __func__);
		return NULL;
	}
	pr_debug("xdx_tdma = 0x%llx\n", (IMG_UINT64)(tdma_dev));
	vchan = dma_request_chan(tdma_dev, "xdx_dma");
	if(vchan != NULL)
		pr_debug("Tdma chan id = %d\n", vchan->chan_id);
	return vchan;
}

void release_dma_channel(struct dma_chan *pchan)
{
	dma_release_channel(pchan);
	pr_debug("release_dma_channel ...\n");
	return;
}
