//#define DEBUG 1
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/dma-mapping.h>

#include "sysinfo.h"
#include "xdx_dma.h"
#include "pangu.h"
#include "virt-dma.h"

#include "pvrmodule.h"
#include <linux/dma-mapping.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/dmaengine.h>
#include <linux/hardirq.h>
#include <linux/spinlock.h>
#include <linux/percpu.h>
#include <linux/rcupdate.h>
#include <linux/mutex.h>
#include <linux/jiffies.h>
#include <linux/rculist.h>
#include <linux/idr.h>
#include <linux/slab.h>
#include <linux/acpi.h>
#include <linux/acpi_dma.h>
#include <linux/of_dma.h>
#include <linux/mempool.h>
#include <linux/numa.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))
#include <linux/dma-map-ops.h>
#endif
#include "osfunc.h"
#include "hash.h"
#include "pvr_debug.h"
#include "dma_drv.h"
#include "generated/volcanic/dma_bridge/common_dma_bridge.h"

#include <linux/dmaengine.h>
#include "dma_debug.h"
#include "xdx_device.h"
#include "xdx_drv.h"
#include "syscommon.h"

#ifdef DEBUG
//#define DEBUG_DMA
//#define DEBUG_VDMA
#endif

#define DMA_CHAN_CFG_SRC_WIDTH(x)	(((x) & 0x3) << 9)
#define DMA_CHAN_CFG_DST_WIDTH(x)	(DMA_CHAN_CFG_SRC_WIDTH(x) << 16)
#define DMA_SYNC_CHAN_NUM	(2)
#define VDMA_ADDR_OFFSET    (0x200000)

static struct device *chan2dev(struct dma_chan *chan)
{
	return &chan->dev->device;
}

static inline struct xdx_desc *
to_xdx_desc(struct dma_async_tx_descriptor *tx)
{
	return container_of(tx, struct xdx_desc, vd.tx);
}

static void *xdx_dma_lli_add(struct xdx_dma_lli_a0 *prev,
		struct xdx_dma_lli_a0 *next,
		dma_addr_t next_phy,
		struct xdx_desc *txd)
{
	if ((!prev && !txd) || !next)
		return NULL;

	if (!prev)
	{
		txd->p_lli = next_phy;
		txd->v_lli_a0 = next;
	}
	else
	{
		DBG_ASSERT(TEST_ALIGNED(next_phy, VDMA_DRV_DESC_LINKPTR_ALIGN));	// Linklist use 16Byte Align
		prev->xdma_desc.Dword0.NextDescPtr_47_4 = GET_BITS_U64(next_phy, 47, 4);
		prev->xdma_desc.Dword1.LinkEnd = 0; //modify link end to zero
		prev->v_lli_next = next;
	}

	next->xdma_desc.Dword1.LinkEnd = 1;
	next->v_lli_next = NULL;

	return next;
}

static inline void xdx_dma_dump_lli(struct xdx_vchan *vchan,
				struct xdx_dma_lli_a0 *lli)
{
	dev_dbg(chan2dev(&vchan->vc.chan),
		"\t\tlen - 0x%08x next - 0x%08llx\n",
		lli->len, (uint64_t)lli->v_lli_next);

	VDMA_DRV_DumpVdmaDescData(&lli->xdma_desc);
}

static void xdx_dma_free_desc(struct xdx_virt_dma_desc *vd)
{
	struct xdx_desc *txd = to_xdx_desc(&vd->tx);
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vd->tx.chan->device);
	struct xdx_dma_lli_a0 *v_lli, *v_next;
	dma_addr_t p_lli, p_next;

	if (unlikely(!txd))
		return;

	p_lli = txd->p_lli;
	v_lli = txd->v_lli_a0;

	while (v_lli)
	{
		v_next = v_lli->v_lli_next;
		p_next = (dma_addr_t)(v_lli->xdma_desc.Dword0.NextDescPtr_47_4) << VDMA_DRV_ADDR_SHIFT;
		dma_pool_free(sdev->pool, v_lli, p_lli);
		v_lli = v_next;
		p_lli = p_next;
	}
	kmem_cache_free(sdev->txpool, txd);
}

static uint32_t xdx_dma_get_command_tag(struct xdx_dma_dev *sdev, struct xdx_pchan *pchan)
{
	uint32_t cmd_tag = ~0x0;

	spin_lock(&pchan->lock);
	if(pchan->DmaCmdPoolMask)
	{
		DBG_ASSERT(pchan->free_cmd > 0);

		cmd_tag = GetLeadOnefromU64(pchan->DmaCmdPoolMask);
		BIT_UNSET(pchan->DmaCmdPoolMask, cmd_tag);
		pchan->free_cmd--;
		pchan->submit_cmd++;
#if 0
		pr_debug("task let pchan %d free cmd %d, submit cmd %d, pool mask %llx, sdev->cur_pID %d...\n",
			pchan->idx, pchan->free_cmd, pchan->submit_cmd, pchan->DmaCmdPoolMask, sdev->cur_pID);
#endif
	}
	spin_unlock(&pchan->lock);

	return cmd_tag;
}

static inline void xdx_dma_release_command_tag(struct xdx_pchan *pchan, int cmd_tag)
{
	DBG_ASSERT(pchan->free_cmd < CMD_MAX);
	DBG_ASSERT(!BIT_ISSET(pchan->DmaCmdPoolMask, cmd_tag));
	DBG_ASSERT(pchan->submit_cmd > 0);
	BIT_SET(pchan->DmaCmdPoolMask, cmd_tag);
	pchan->free_cmd++;
	pchan->submit_cmd--;
#ifdef DEBUG_DMA
	//pr_debug("ISR pchan %d free cmd %d, submit cmd %d, pool mask %llx...\n",
				//pchan->idx, pchan->free_cmd, pchan->submit_cmd, pchan->DmaCmdPoolMask);
#endif
}

static inline void SubmitDMACommand(VDMA_CMD_Config_t *pVdmaCfg, struct xdx_vchan *vchan, struct xdx_desc *txd)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	struct xdx_pchan *pchan = &(sdev->pchans[(txd->direct == DMA_MEM_TO_DEV) ? 0 : 1]);
	VDMA_CMD_Data_t VdmaCmd;
	uint32_t cmd_tag;

	//Get_freeCommand
	cmd_tag = xdx_dma_get_command_tag(sdev, pchan);
	DBG_ASSERT(cmd_tag < CMD_MAX);
	DBG_ASSERT(pVdmaCfg->CopyParam.TransferLen);
	pchan->cmdRef[cmd_tag].desc = txd; //record command reference in pchan struct
	pchan->cmdRef[cmd_tag].vchan = vchan; //record command reference in pchan struct

	pVdmaCfg->CommandTag = cmd_tag;
	VDMA_DRV_GenerateVdmaCmd(pVdmaCfg, &VdmaCmd);
#ifdef DEBUG_VDMA
	VDMA_DRV_DumpVdmaCmdCfg(pVdmaCfg);
	VDMA_DRV_DumpVdmaCmdData(&VdmaCmd);
#endif

	spin_lock(&pchan->lock);
	VDMA_HAL_CH_WrCmdBufBurst((uint64_t)pchan->base, (uint64_t*)&(VdmaCmd.CmdDword0), 3);
	spin_unlock(&pchan->lock);
}

static inline void PackageDMACfg(VDMA_CMD_Config_t *pVdmaCfg,
				dma_addr_t dst_addr, dma_addr_t src_addr,
				uint32_t dst_mode, uint32_t src_mode)
{
	pVdmaCfg->CopyParam.DstAddr = dst_addr;
	pVdmaCfg->CopyParam.DstAddrMode = dst_mode;
	pVdmaCfg->CopyParam.SrcAddr = src_addr;
	pVdmaCfg->CopyParam.SrcAddrMode = src_mode;
}

static void TrimHeadNode(struct xdx_desc *txd, struct xdx_vchan *vchan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	struct xdx_dma_lli_a0 *v_lli;
	uint32_t aligned_size;
	dma_addr_t 	aligned_addr, p_lli;
	VDMA_CMD_Config_t VdmaCfg;

	v_lli = txd->v_lli_a0;
	if(v_lli)
	{
		// the first node is not align to 1024
		aligned_addr = PVR_ALIGN(txd->host_addr, (uint64_t)VDMA_ADDR_BEST_ALIGN);
		aligned_size = aligned_addr - txd->host_addr;

#ifdef DEBUG_DMA
		pr_debug("TrimHeadNode aligned_size %d, v_lli len %d, txd len %d, aligned_addr %llx, host_addr %llx...\n", aligned_size, v_lli->len, txd->transfer_len, aligned_addr, txd->host_addr);
#endif
		if(aligned_size)
		{
			// if len < VDMA_MAX_SIZE_8B, use all v_lli size
			if(v_lli->len < VDMA_MAX_SIZE_8B)
			{
				aligned_size =  v_lli->len;
			}

			//send head dma command
			memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
			if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
			{
				PackageDMACfg(&VdmaCfg, txd->dev_addr, txd->host_addr, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
			}
			else //DEV2MEM
			{
				PackageDMACfg(&VdmaCfg, txd->host_addr, txd->dev_addr, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
			}

			VdmaCfg.CopyParam.LengthUnit = VDMA_DRV_LENUNIT_8B;
			VdmaCfg.CopyParam.TransferLen = (aligned_size >> VDMA_DRV_LENUNIT_8B_SHIFT);

			DBG_ASSERT(txd->transfer_len > 0);
			atomic_inc(&txd->Cmd_cnt);
			SubmitDMACommand(&VdmaCfg, vchan, txd);

			v_lli->len -= aligned_size;
			txd->transfer_len -= aligned_size;

			txd->dev_addr += aligned_size;
			txd->host_addr += aligned_size;

			if(v_lli->len)
			{
				//modify data blk address
				v_lli->xdma_desc.Dword0.DataBlkPtr_20_5 = GET_BITS_U64((txd->host_addr), 20, 5);
				v_lli->xdma_desc.Dword1.DataBlkPtr_47_21 = GET_BITS_U64((txd->host_addr), 47, 21);
#ifdef DEBUG_DMA
				pr_debug("TrimHeadNode modify linklist addr...\n");
#endif
			}
			else
			{
				//remove v_lli node
				txd->v_lli_a0 = v_lli->v_lli_next;
				p_lli = txd->p_lli;
				DBG_ASSERT(txd->v_lli_a0);
				txd->p_lli = (dma_addr_t)(v_lli->xdma_desc.Dword0.NextDescPtr_47_4) << VDMA_DRV_ADDR_SHIFT;

				dma_pool_free(sdev->pool, v_lli, p_lli);
#ifdef DEBUG_DMA
				pr_debug("remove head node!!...\n");
#endif
			}
#ifdef DEBUG_DMA
			pr_debug("send head node!!...\n");
#endif
		}
	}
}

static void TrimTailNode(struct xdx_desc *txd, struct xdx_vchan *vchan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	struct xdx_dma_lli_a0 *v_lli, *prev_lli = NULL;
	uint32_t aligned_size, aligned_len;
	VDMA_CMD_Config_t VdmaCfg;
	dma_addr_t p_lli, tail_addr;

	v_lli = txd->v_lli_a0;
	if(v_lli)
	{
		//travel v_lli to fixed LengthUnit & Transfer Len
		while(v_lli->v_lli_next)
		{
			prev_lli = v_lli;
			DBG_ASSERT(TEST_ALIGNED(v_lli->len, VDMA_ADDR_BEST_ALIGN));
			v_lli->xdma_desc.Dword1.LengthUnit = VDMA_DRV_LENUNIT_1K; //use default lenght unit
			v_lli->xdma_desc.Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(v_lli->len >> VDMA_DRV_LENUNIT_1K_SHIFT);
			v_lli = v_lli->v_lli_next;
		}

		aligned_len = PVR_FLOOR_ALIGN(v_lli->len, (uint32_t)VDMA_ADDR_BEST_ALIGN);
		aligned_size = v_lli->len - aligned_len;
#ifdef DEBUG_DMA
		pr_debug("TrimTailNode aligned len %x, txd->transfer_len %x aligend size %d..\n", aligned_len, txd->transfer_len, aligned_size);
#endif

		DBG_ASSERT(1 == v_lli->xdma_desc.Dword1.LinkEnd);
		if(aligned_size)
		{
			// if len < VDMA_MAX_SIZE_8B, use all v_lli size
			if(v_lli->len < VDMA_MAX_SIZE_8B)
			{
				aligned_size = v_lli->len;
			}
#ifdef DEBUG_DMA
			pr_debug("TrimTailNode before last node len %d..\n", v_lli->len);
#endif
			v_lli->len -= aligned_size;
			txd->transfer_len -= aligned_size;

			// find the last node trim addr
			tail_addr = (uint64_t)v_lli->xdma_desc.Dword1.DataBlkPtr_47_21<<21;
			tail_addr |= (uint64_t)v_lli->xdma_desc.Dword0.DataBlkPtr_20_5<<5;
			tail_addr += v_lli->len;

			//send Tail dma command
			memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
			if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
			{
				PackageDMACfg(&VdmaCfg, txd->dev_addr + txd->transfer_len, tail_addr, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
			}
			else //DEV2MEM
			{
				PackageDMACfg(&VdmaCfg, tail_addr, txd->dev_addr + txd->transfer_len, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
			}

			VdmaCfg.CopyParam.LengthUnit = VDMA_DRV_LENUNIT_8B;
			VdmaCfg.CopyParam.TransferLen = (aligned_size >> VDMA_DRV_LENUNIT_8B_SHIFT);

			if(txd->transfer_len)
			{
				atomic_inc(&txd->Cmd_cnt);
			}
			SubmitDMACommand(&VdmaCfg, vchan, txd);

			if(v_lli->len)
			{
				DBG_ASSERT(TEST_ALIGNED(v_lli->len, VDMA_ADDR_BEST_ALIGN));	// Linklist data use 32Byte Align
				v_lli->xdma_desc.Dword1.LengthUnit = VDMA_DRV_LENUNIT_1K; //use default lenght unit
				v_lli->xdma_desc.Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(v_lli->len >> VDMA_DRV_LENUNIT_1K_SHIFT);
			}
			else
			{
				//remove v_lli node
				if(prev_lli)
				{
					prev_lli->v_lli_next = NULL;
					DBG_ASSERT(0 == prev_lli->xdma_desc.Dword1.LinkEnd);
					prev_lli->xdma_desc.Dword1.LinkEnd = 1;
					p_lli = (dma_addr_t)(prev_lli->xdma_desc.Dword0.NextDescPtr_47_4) << VDMA_DRV_ADDR_SHIFT;
				}
				else
				{
					txd->v_lli_a0 = NULL;
					p_lli = txd->p_lli;
					DBG_ASSERT(0 == txd->transfer_len);
				}
				dma_pool_free(sdev->pool, v_lli, p_lli);
#ifdef DEBUG_DMA
				pr_debug("TrimTailNode last node len %d will be removed..\n", v_lli->len);
#endif
			}
#ifdef DEBUG_DMA
			pr_debug("TrimTailNode send tail list done...\n");
#endif
		}
		else
		{
#ifdef DEBUG_DMA
			pr_debug("TrimTailNode set 1K len...\n"); //set 1K
#endif
			DBG_ASSERT(TEST_ALIGNED(v_lli->len, VDMA_ADDR_BEST_ALIGN));
			v_lli->xdma_desc.Dword1.LengthUnit = VDMA_DRV_LENUNIT_1K; //use default lenght unit
			v_lli->xdma_desc.Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(v_lli->len >> VDMA_DRV_LENUNIT_1K_SHIFT);
		}
	}
	else
	{
		aligned_len = PVR_FLOOR_ALIGN(txd->transfer_len, (uint32_t)VDMA_ADDR_BEST_ALIGN);
		aligned_size =  (txd->transfer_len - aligned_len);
#ifdef DEBUG_DMA
		pr_debug("TrimTailNode direct... aligned_len %x, txd->transfer_len %x, aligned_size %d\n", aligned_len, txd->transfer_len, aligned_size);
#endif
		if(aligned_size)
		{
			if(txd->transfer_len < VDMA_MAX_SIZE_8B)
			{
				aligned_size = txd->transfer_len;
			}

			txd->transfer_len -= aligned_size; //trim txd transfer length

			//send tail dma command
			memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
			if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
			{
				PackageDMACfg(&VdmaCfg, txd->dev_addr + txd->transfer_len, txd->host_addr + txd->transfer_len, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
			}
			else //DEV2MEM
			{
				PackageDMACfg(&VdmaCfg, txd->host_addr + txd->transfer_len, txd->dev_addr + txd->transfer_len, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
			}

			VdmaCfg.CopyParam.LengthUnit = VDMA_DRV_LENUNIT_8B;
			VdmaCfg.CopyParam.TransferLen = (aligned_size >> VDMA_DRV_LENUNIT_8B_SHIFT);
			if(txd->transfer_len)
			{
				atomic_inc(&txd->Cmd_cnt);
			}
			SubmitDMACommand(&VdmaCfg, vchan, txd);
#ifdef DEBUG_DMA
			pr_debug("TrimTailNode direct send tail done...\n");
#endif
		}
	}
}

static void SendNormalNode(struct xdx_desc *txd, struct xdx_vchan *vchan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	VDMA_CMD_Config_t VdmaCfg;

#ifdef DEBUG_DMA
	pr_debug("SendNormalNode txd->transfer_len %d, v_lli is %d\n", txd->transfer_len, (txd->v_lli_a0 != NULL));
#endif
	if(txd->v_lli_a0)
	{
		//use linklist
		if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
		{
			memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_LSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
			PackageDMACfg(&VdmaCfg, txd->dev_addr, txd->p_lli, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
		}
		else
		{
			memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_LDST_CFG].RawData, sizeof(VdmaCfg.RawData));
			PackageDMACfg(&VdmaCfg, txd->p_lli, txd->dev_addr, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
		}
#if defined(SUPPORT_DONT_SNOOP) || defined(__arm64__) || defined(__aarch64__)
		{
			IMG_CPU_PHYADDR addr = {0};
			struct xdx_dma_lli_a0 *v_lli;
			for (v_lli = txd->v_lli_a0; v_lli; v_lli = v_lli->v_lli_next)
			{
				OSCPUCacheFlushRangeKM(NULL,
						(IMG_BYTE*)v_lli,
						(IMG_BYTE*)v_lli + sizeof(VDMA_DESC_Data_t),
						addr, addr);
			}
		}
#endif
	}
	else
	{
		//use direct
		memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
		if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
		{
			PackageDMACfg(&VdmaCfg, txd->dev_addr, txd->host_addr, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
		}
		else //DEV2MEM
		{
			PackageDMACfg(&VdmaCfg, txd->host_addr, txd->dev_addr, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
		}
	}

	if(txd->transfer_len < VDMA_MAX_SIZE_8B)
	{
		DBG_ASSERT(TEST_ALIGNED(txd->transfer_len, VDMA_DRV_CMD_COPY_ADDR_ALIGN));	// Linklist data use 32Byte Align
		VdmaCfg.CopyParam.LengthUnit = VDMA_DRV_LENUNIT_8B;
		VdmaCfg.CopyParam.TransferLen = (txd->transfer_len >> VDMA_DRV_LENUNIT_8B_SHIFT); //TransferLen;
	}
	else
	{
		DBG_ASSERT(TEST_ALIGNED(txd->transfer_len, VDMA_ADDR_BEST_ALIGN));	// Linklist data use 32Byte Align
		VdmaCfg.CopyParam.LengthUnit = VDMA_DRV_LENUNIT_1K;
		VdmaCfg.CopyParam.TransferLen = (txd->transfer_len >> VDMA_DRV_LENUNIT_1K_SHIFT); //TransferLen;
	}


	SubmitDMACommand(&VdmaCfg, vchan, txd);
}

static void SendDirect_iommu(struct xdx_desc *txd, struct xdx_vchan *vchan,uint32_t transfer_size,uint16_t LengthUnit)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	VDMA_CMD_Config_t VdmaCfg;

	DBG_ASSERT(transfer_size > 0);
	{
		//send head dma command
		memcpy(VdmaCfg.RawData, sdev->cmd_cfg[VDMA_COPY_DSRC_DDST_CFG].RawData, sizeof(VdmaCfg.RawData));
		if(txd->direct == DMA_MEM_TO_DEV) //MEM2DEV
		{
			PackageDMACfg(&VdmaCfg, txd->dev_addr, txd->host_addr, VDMA_DRV_ADDRMODE_GPU_SVA, VDMA_DRV_ADDRMODE_HOST_SVA);
		}
		else //DEV2MEM
		{
			PackageDMACfg(&VdmaCfg, txd->host_addr, txd->dev_addr, VDMA_DRV_ADDRMODE_HOST_SVA, VDMA_DRV_ADDRMODE_GPU_SVA);
		}

		VdmaCfg.CopyParam.LengthUnit = LengthUnit;

		if(LengthUnit == VDMA_DRV_LENUNIT_8B)
			VdmaCfg.CopyParam.TransferLen = (transfer_size >> VDMA_DRV_LENUNIT_8B_SHIFT);
		else
			VdmaCfg.CopyParam.TransferLen = (transfer_size >> VDMA_DRV_LENUNIT_1K_SHIFT);

		DBG_ASSERT(txd->transfer_len > 0);
		SubmitDMACommand(&VdmaCfg, vchan, txd);

	}
}

static inline int GetFreeTotalCommand(struct xdx_dma_dev *sdev, uint32_t Index)
{
	struct xdx_pchan *pchan;

	pchan = &(sdev->pchans[Index]);
	return pchan->free_cmd;
}


static int xdx_dma_start_desc_iommu(struct xdx_dma_dev *sdev, struct xdx_vchan *vchan)
{
	struct xdx_virt_dma_desc *desc;
	struct xdx_desc	*txd = NULL;
	uint32_t transfer_size;
	int iRet = -EAGAIN;
	do
	{
		desc = xdx_vchan_next_desc(&vchan->vc);
		if(!desc)
		{
			break;
		}
		txd = to_xdx_desc(&desc->tx);
		if(GetFreeTotalCommand(sdev, (txd->direct == DMA_MEM_TO_DEV) ? 0 : 1) < 2)
		{
			iRet = 0;
			break;
		}

		list_del(&desc->node);
		txd->dev_addr = desc->tx.phys;

		if(txd->transfer_len < VDMA_MAX_SIZE_8B)
		{
			atomic_set(&(txd->Cmd_cnt), 1);
			SendDirect_iommu(txd, vchan,txd->transfer_len,VDMA_DRV_LENUNIT_8B);
		}
		else
		{
			transfer_size = MOD_2(txd->transfer_len, VDMA_ADDR_BEST_ALIGN);
			if(transfer_size)
			{
				atomic_set(&(txd->Cmd_cnt), 2);
				SendDirect_iommu(txd, vchan,transfer_size,VDMA_DRV_LENUNIT_8B);

				txd->dev_addr += transfer_size;
				txd->host_addr += transfer_size;
				SendDirect_iommu(txd, vchan, txd->transfer_len - transfer_size,VDMA_DRV_LENUNIT_1K);
			}
			else
			{
				atomic_set(&(txd->Cmd_cnt), 1);
				SendDirect_iommu(txd, vchan,txd->transfer_len,VDMA_DRV_LENUNIT_1K);
			}
		}
	}while(1);


	return iRet;
}

static int xdx_dma_start_desc(struct xdx_dma_dev *sdev, struct xdx_vchan *vchan)
{
	struct xdx_virt_dma_desc *desc;
	struct xdx_desc	*txd;
	int32_t nFreeCmd;
	int iRet = -EAGAIN;

	do
	{
		desc = xdx_vchan_next_desc(&vchan->vc);
		if(!desc)
		{
			break;
		}

		txd = to_xdx_desc(&desc->tx);
		nFreeCmd = GetFreeTotalCommand(sdev, (txd->direct == DMA_MEM_TO_DEV) ? 0 : 1);

		if(txd->Cmd_cnt.counter > nFreeCmd)
		{
			pr_debug("free command %d less than %d\n", nFreeCmd, txd->Cmd_cnt.counter);
			iRet = 0;
			break;
		}

		list_del(&desc->node);

		txd->dev_addr = desc->tx.phys;

		if(txd->Cmd_cnt.counter > 1)
		{
			txd->Cmd_cnt.counter = 1;
			TrimHeadNode(txd, vchan);
			TrimTailNode(txd, vchan);
		}

		if(txd->transfer_len)
		{
			SendNormalNode(txd, vchan);
		}

#if defined(DEBUG_VDMA)
//		VDMA_DRV_CH_Monitor(0xffff0002);
#endif
	}while(1);

#ifdef NOHW_DEBUG
		pr_debug("xdx_dma_start_desc done ...\n");
		sdev->timer.expires = jiffies + 1*HZ;
		timer_setup(&sdev->timer,timer_handler,0);
		add_timer(&sdev->timer);
#endif
	return iRet;
}

static void xdx_dma_tasklet(unsigned long data)
{
	struct xdx_dma_dev *sdev = (struct xdx_dma_dev *)data;
	struct xdx_vchan *vchan;
	unsigned int vchan_idx, vcMask = sdev->pendingMask;
	unsigned long flags;

	while(vcMask)
	{
		vchan_idx = GetLeadOnefromU32(vcMask);
		BIT_UNSET(vcMask, vchan_idx);
		vchan = &sdev->vchans[vchan_idx];

		spin_lock_irqsave(&vchan->vc.lock, flags);
		if(sdev->IommuEnable)
		{
			if(0 == xdx_dma_start_desc_iommu(sdev, vchan))
			{
				pr_debug("xdx_dma_tasklet ... vc %d is full!!!\n", vchan->tag);
			}
		}
		else
		{
			if(0 == xdx_dma_start_desc(sdev, vchan))
			{
				pr_debug("xdx_dma_tasklet ... vc %d is full!!!\n", vchan->tag);
			}
		}
		spin_unlock_irqrestore(&vchan->vc.lock, flags);
	}

}

int xdx_dma_a0_interrupt(struct xdx_dma_dev *sdev)
{
	struct xdx_vchan *vchan;
	struct xdx_pchan *pchan;
	struct xdx_desc	*txd;
	uint32_t cmd_tag;
	uint64_t CmdCmplRawSts;
	unsigned int i;

//	dev_dbg(sdev->dev, "xdx_dma_a0_interrupt new ...\n");
	for (i = 0; i < sdev->num_pchans; i++) {
		pchan = &sdev->pchans[i];
		if(pchan->submit_cmd) {

			CmdCmplRawSts = VDMA_HAL_CH_RdAndClrCmdCmplRawSts((uint64_t)pchan->base);
//			dev_dbg(sdev->dev, "interrupt status = %lld %lld %lld i = %d\n",
//				VDMA_DRV_CH_GetCmdCmplIntSts(i),VDMA_DRV_CH_GetPollingTimeoutIntSts(i),VDMA_DRV_CH_GetBusErrIntSts(i),i);
			while(CmdCmplRawSts) {
				cmd_tag = GetLeadOnefromU64(CmdCmplRawSts);
				BIT_UNSET(CmdCmplRawSts, cmd_tag);

				txd = pchan->cmdRef[cmd_tag].desc;
				vchan = pchan->cmdRef[cmd_tag].vchan;

				spin_lock(&pchan->lock);
				xdx_dma_release_command_tag(pchan, cmd_tag);
				spin_unlock(&pchan->lock);

				if(atomic_dec_and_test(&txd->Cmd_cnt)) {
#if defined(DEBUG_VDMA)
//					VDMA_DRV_CH_Monitor(0xffff0009);
#endif
					spin_lock(&vchan->vc.lock);
					xdx_vchan_cookie_complete(&txd->vd);
					spin_unlock(&vchan->vc.lock);
				}
			}
		}

		if(!atomic_read(&sdev->tasklet_shutdown) && (sdev->pendingMask)) {
			tasklet_schedule(&sdev->task);
		}
	}
	return 0;
}
static inline s8 convert_burst(u32 maxburst)
{
	switch (maxburst) {
	case 1:
		return 0;
	case 4:
		return 1;
	case 8:
		return 2;
	case 16:
		return 3;
	default:
		return -EINVAL;
	}
}

static inline s8 convert_buswidth(enum dma_slave_buswidth addr_width)
{
	return ilog2(addr_width);
}


#ifdef GENERATE_LINKLIST
IMG_UINT64 xdx_dma_prep_sg2(
		struct scatterlist *sgl,
		unsigned int sg_len)
{
	struct scatterlist *sg;
	int i;
	VDMA_DESC_Config_t VdmaDescCfg;
	VDMA_DESC_Data_t *VdmaDesc_vir;
	dma_addr_t VdmaDesc_phy;
	const uint32_t LengthUnit = VDMA_DRV_LENUNIT_8B;
	VDMA_DESC_LinkHeadTail_t VdmaHostDescList;
	VDMA_DRV_DescLink_Init(&VdmaHostDescList);

	if (!sgl)
		return 0;

	pr_info("sg_len = %d\n",sg_len);
	for_each_sg(sgl, sg, sg_len, i) {
		VdmaDesc_vir = dma_pool_alloc(linklistpool, GFP_KERNEL, &VdmaDesc_phy);
		memset(VdmaDesc_vir,0x00,sizeof(VDMA_DESC_Data_t));
		VdmaDescCfg.DataBlkPtr = (uint64_t)sg_dma_address(sg);
		VdmaDescCfg.DataPtrAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
		VdmaDescCfg.LengthUnit = LengthUnit;
		VdmaDescCfg.TransferLen = (sg_dma_len(sg)+8-1)/8;
		if((i+1)==sg_len)
			VdmaDescCfg.LinkEnd = 1;
		else
			VdmaDescCfg.LinkEnd = 0;
		VDMA_DRV_GenerateVdmaDesc(&VdmaDescCfg, VdmaDesc_vir);
		VDMA_DRV_DescLink_AddToTail(&VdmaHostDescList,VdmaDesc_vir);
	}
	VDMA_DRV_DumpVdmaDescList(VdmaHostDescList.head);
	return virt_to_phys(VdmaHostDescList.head);
}
EXPORT_SYMBOL_GPL(xdx_dma_prep_sg2);
IMG_UINT64 Get_Direct_addr(IMG_DEV_PHYADDR Linklist_Head)
{
#if 1
	VDMA_DESC_Data_t* VdmaDesc;
	IMG_UINT64 DataBlkPtr;

	VdmaDesc = (VDMA_DESC_Data_t *)phys_to_virt(Linklist_Head.uiAddr);

	DataBlkPtr = (uint64_t)VdmaDesc->Dword1.DataBlkPtr_47_21<<21;
	DataBlkPtr += (uint64_t)VdmaDesc->Dword0.DataBlkPtr_20_5<<5;
	return DataBlkPtr;
#else
	pr_info("Get_Direct_addr 0x%llx\n",Linklist_Head.uiAddr);
	return 0;
#endif

}
EXPORT_SYMBOL_GPL(Get_Direct_addr);
#endif


static struct dma_async_tx_descriptor *xdx_dma_prep_dma_memcpy(
		struct dma_chan *chan, dma_addr_t dest, dma_addr_t src,
		size_t len, unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_dma_lli_a0 *v_lli;
	struct xdx_desc *txd;
	dma_addr_t p_lli;

	if(0 == dest)
	{
		txd =  kmem_cache_alloc(sdev->txpool, GFP_KERNEL);

		if (!txd)
			return NULL;

		memset(txd, 0x00, sizeof(struct xdx_desc));
		txd->host_addr = src;
		txd->transfer_len = len;
	}
	else
	{
		txd = (struct xdx_desc *)dest;

		v_lli = dma_pool_alloc(sdev->pool, GFP_KERNEL, &p_lli);
		if (!v_lli)
			return NULL;

		v_lli->len = len;
		txd->transfer_len += len; //accumulate transfer_len

		// always set NextPtr to 0 when generating VDMA Descriptor
		// linklist operation will be handled by other function later

		v_lli->xdma_desc.Dword0.DataBlkPtr_20_5 = GET_BITS_U64(src, 20, 5);
		v_lli->xdma_desc.Dword1.DataBlkPtr_47_21 = GET_BITS_U64(src, 47, 21);
		v_lli->xdma_desc.Dword1.DataPtrAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;

		if(v_lli->len < VDMA_MAX_SIZE_8B)
		{
			DBG_ASSERT(TEST_ALIGNED(v_lli->len, VDMA_DRV_CMD_COPY_ADDR_ALIGN));
			v_lli->xdma_desc.Dword1.LengthUnit = VDMA_DRV_LENUNIT_8B; //use default lenght unit
			v_lli->xdma_desc.Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(v_lli->len >> VDMA_DRV_LENUNIT_8B_SHIFT);
		}

		txd->v_lli_next_a0 = xdx_dma_lli_add(txd->v_lli_next_a0, v_lli, p_lli, txd);
	}

	//here, we need consider align to 8 Bytes and all in on IOMMU
	txd->Cmd_cnt.counter = (txd->transfer_len >= VDMA_MAX_SIZE_8B) ? 3 : 1;
	txd->direct = flags;

	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);

}

static struct dma_async_tx_descriptor *xdx_dma_prep_slave_sg(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int sg_len, enum dma_transfer_direction dir,
		unsigned long flags, void *context)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_dma_lli_a0 *v_lli, *v_next = NULL;
	struct xdx_desc *txd;
	dma_addr_t p_lli, p_next;

	if (!sgl)
		return NULL;

	txd = kmem_cache_alloc(sdev->txpool, GFP_KERNEL);
	if (!txd)
		return NULL;
	memset(txd, 0x00, sizeof(struct xdx_desc));

	txd->host_addr = sg_dma_address(sgl);

	if(sg_len > 1)
	{
		struct scatterlist *sg;
		unsigned int i;

		for_each_sg(sgl, sg, sg_len, i)
		{
			v_lli = dma_pool_alloc(sdev->pool, GFP_KERNEL, &p_lli);
			if (!v_lli)
				goto err_lli_free;

			v_lli->len = sg_dma_len(sg);
			txd->transfer_len += v_lli->len; //accumulate transfer_len

			// always set NextPtr to 0 when generating VDMA Descriptor
			// linklist operation will be handled by other function later

			v_lli->xdma_desc.Dword0.DataBlkPtr_20_5 = GET_BITS_U64(sg_dma_address(sg), 20, 5);
			v_lli->xdma_desc.Dword1.DataBlkPtr_47_21 = GET_BITS_U64(sg_dma_address(sg), 47, 21);
			v_lli->xdma_desc.Dword1.DataPtrAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;

			if(v_lli->len < VDMA_MAX_SIZE_8B)
			{
				DBG_ASSERT(TEST_ALIGNED(v_lli->len, VDMA_DRV_CMD_COPY_ADDR_ALIGN));
				v_lli->xdma_desc.Dword1.LengthUnit = VDMA_DRV_LENUNIT_8B; //use default lenght unit
				v_lli->xdma_desc.Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(v_lli->len >> VDMA_DRV_LENUNIT_8B_SHIFT);
			}

			v_next = xdx_dma_lli_add(v_next, v_lli, p_lli, txd);
		}
	}
	else
	{
		txd->transfer_len = sg_dma_len(sgl);
	}

	//here, we need consider align to 8 Bytes and all in on IOMMU
	txd->Cmd_cnt.counter = ((txd->transfer_len >= VDMA_MAX_SIZE_8B) ||
						((sg_len > 1) && (MOD_2(txd->host_addr, (uint64_t)VDMA_DRV_DESC_DATAPTR_ALIGN))))
						 ? 3 : 1;
	txd->direct = dir;

#ifdef DEBUG_DMA
	dev_dbg(chan2dev(chan), "First link: %llx\n", txd->p_lli);
	for (v_next = txd->v_lli_a0; v_next; v_next = v_next->v_lli_next)
	{
		xdx_dma_dump_lli(vchan, v_next);
	}
#endif
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);

err_lli_free:

	p_lli = txd->p_lli;
	v_lli = txd->v_lli_a0;

	while (v_lli)
	{
		v_next = v_lli->v_lli_next;
		p_next = (dma_addr_t)(v_lli->xdma_desc.Dword0.NextDescPtr_47_4) << VDMA_DRV_ADDR_SHIFT;
		dma_pool_free(sdev->pool, v_lli, p_lli);
		v_lli = v_next;
		p_lli = p_next;
	}

	kmem_cache_free(sdev->txpool, txd);
	return NULL;
}


static struct dma_async_tx_descriptor *xdx_dma_prep_slave_sg_iommu(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int sg_len, enum dma_transfer_direction dir,
		unsigned long flags, void *context)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;
	struct scatterlist *sg;
	unsigned int i;

	if (!sgl)
		return NULL;

	txd = kmem_cache_alloc(sdev->txpool, GFP_KERNEL);
	if (!txd)
		return NULL;
	memset(txd, 0x00, sizeof(struct xdx_desc));

	txd->host_addr = sg_dma_address(sgl);

	for_each_sg(sgl, sg, sg_len, i)
	{
		txd->transfer_len += sg_dma_len(sg); //accumulate transfer_len
	}

	//here, we need consider align to 8 Bytes and all in on IOMMU
	txd->direct = dir;

	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

static int xdx_dma_config(struct dma_chan *chan,
			    struct dma_slave_config *config)
{
	return 0;
}

static int xdx_dma_pause(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);

	dev_dbg(chan2dev(chan), "vchan %p: pause\n", &vchan->vc);
	{
		spin_lock_bh(&sdev->lock);
		BIT_UNSET(sdev->pendingMask, vchan->tag);

		spin_unlock_bh(&sdev->lock);
	}

	return 0;
}

static int xdx_dma_resume(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	unsigned long flags;

	dev_dbg(chan2dev(chan), "vchan %p: resume\n", &vchan->vc);
	spin_lock_irqsave(&vchan->vc.lock, flags);
	if (!list_empty(&vchan->vc.desc_issued)) {
		spin_lock(&sdev->lock);
		BIT_SET(sdev->pendingMask, vchan->tag);
		spin_unlock(&sdev->lock);
	}
	spin_unlock_irqrestore(&vchan->vc.lock, flags);
	return 0;
}

static int xdx_dma_terminate_all(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	unsigned long flags;
	LIST_HEAD(head);

	spin_lock_bh(&sdev->lock);
	BIT_UNSET(sdev->pendingMask, vchan->tag);
	spin_unlock_bh(&sdev->lock);

	spin_lock_irqsave(&vchan->vc.lock, flags);

	if (vchan->cyclic) {
		vchan->cyclic = false;
	}

	xdx_vchan_get_all_descriptors(&vchan->vc, &head);

	spin_unlock_irqrestore(&vchan->vc.lock, flags);

	xdx_vchan_dma_desc_free_list(&vchan->vc, &head);

	return 0;
}


static enum dma_status xdx_dma_tx_status(struct dma_chan *chan,
					   dma_cookie_t cookie,
					   struct dma_tx_state *state)
{
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_dma_lli_a0 *lli;
	struct xdx_virt_dma_desc *vd;
	struct xdx_desc *txd;
	enum dma_status ret;
	unsigned long flags;
	size_t bytes = 0;

	ret = dma_cookie_status(chan, cookie, state);
	if (ret == DMA_COMPLETE || !state)
		return ret;
	spin_lock_irqsave(&vchan->vc.lock, flags);
	vd = xdx_vchan_find_desc(&vchan->vc, cookie);
	txd = to_xdx_desc(&vd->tx);
	if (vd) {
		for (lli = txd->v_lli_a0; lli != NULL; lli = lli->v_lli_next)
			bytes += lli->len;
	}
	spin_unlock_irqrestore(&vchan->vc.lock, flags);
	dma_set_residue(state, bytes);

	return ret;
}

static void xdx_dma_issue_pending(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	unsigned long flags;

	spin_lock_irqsave(&vchan->vc.lock, flags);

	if (xdx_vchan_issue_pending(&vchan->vc))
	{
		spin_lock(&sdev->lock);
		if(!BIT_ISSET(sdev->pendingMask, vchan->tag))
		{
			BIT_SET(sdev->pendingMask, vchan->tag);
		}
		spin_unlock(&sdev->lock);
		tasklet_schedule(&sdev->task);
	} else {
		dev_dbg(chan2dev(chan), "vchan %p: nothing to issue\n",
			&vchan->vc);
	}

	spin_unlock_irqrestore(&vchan->vc.lock, flags);
}

static void xdx_dma_free_chan_resources(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);

	spin_lock_bh(&sdev->lock);
	BIT_UNSET(sdev->pendingMask, vchan->tag);
	spin_unlock_bh(&sdev->lock);
	xdx_vchan_free_chan_resources(&vchan->vc);
}

static inline void xdx_kill_tasklet(struct xdx_dma_dev *sdev)
{

	/* Prevent spurious interrupts from scheduling the tasklet */
	atomic_inc(&sdev->tasklet_shutdown);
	/* Actually prevent the tasklet from being scheduled */
	tasklet_kill(&sdev->task);
}

static inline void xdx_dma_free(struct xdx_dma_dev *sdev)
{
	unsigned int i;

	for (i = 0; i < sdev->num_vchans; i++) {
		struct xdx_vchan *vchan = &sdev->vchans[i];
		list_del(&vchan->vc.chan.device_node);
		tasklet_kill(&vchan->vc.task);
	}
}

static IMG_BOOL Is_xdx_iommu_enable(struct platform_device *xdx_dev)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0))
	return 0;
#else
	const struct dma_map_ops *ops;

	ops = get_dma_ops(xdx_dev->dev.parent);
	if(ops == NULL) {
		pr_err("iommu disable .\n");
		return IMG_FALSE;
	} else {
		pr_err("iommu enable .\n");
		return IMG_TRUE;
	}
#endif
}

int xdx_dma_slave_a0_init(struct platform_device *pdev, struct xdx_dma_dev *sdc)
{
	INIT_LIST_HEAD(&sdc->slave.channels);
	sdc->slave.device_free_chan_resources	= xdx_dma_free_chan_resources;
	sdc->slave.device_tx_status 	= xdx_dma_tx_status;
	sdc->slave.device_issue_pending 	= xdx_dma_issue_pending;
	sdc->IommuEnable = Is_xdx_iommu_enable(pdev);
	if(sdc->IommuEnable)
	{
		sdc->slave.device_prep_slave_sg     = xdx_dma_prep_slave_sg_iommu;
	}
	else
	{
		sdc->slave.device_prep_slave_sg 	= xdx_dma_prep_slave_sg;
	}

	sdc->slave.device_prep_dma_memcpy	= xdx_dma_prep_dma_memcpy;
	sdc->slave.copy_align			= DMAENGINE_ALIGN_4_BYTES;
	sdc->slave.device_config		= xdx_dma_config;
	sdc->slave.device_pause 		= xdx_dma_pause;
	sdc->slave.device_resume		= xdx_dma_resume;
	sdc->slave.device_terminate_all 	= xdx_dma_terminate_all;
//	sdc->slave.src_addr_widths		= sdc->cfg->src_addr_widths;
//	sdc->slave.dst_addr_widths		= sdc->cfg->dst_addr_widths;
	sdc->slave.directions			= BIT(DMA_DEV_TO_MEM) | BIT(DMA_MEM_TO_DEV);
	sdc->slave.residue_granularity		= DMA_RESIDUE_GRANULARITY_BURST;
	sdc->slave.dev = &pdev->dev;
	dev_dbg(&pdev->dev, "sdc->slave.dev = 0x%llx\n",(IMG_UINT64)sdc->slave.dev);
	tasklet_init(&sdc->task, xdx_dma_tasklet, (unsigned long)sdc);
	return 0;
}

int xdx_vdma_vchan_a0_init(struct xdx_dma_dev *sdc, struct xdx_vchan *vchan)
{
	vchan->vc.desc_free = xdx_dma_free_desc;
	return 0;
}
