#include <linux/device.h>
#include "xdx_device.h"
#include "xdx_vdma.h"
#include "vdma_irq.h"
#include "rgxdevice.h"
#include "xdx_dma_b0.h"
#include "pangu.h"

extern const struct vdma_funcs tdma_funcs_instance;
extern int xdx_tdma_module_init(void __iomem	*base);

int xdx_tdma_init(struct xdx_device *xdev)
{
	struct xdx_tdma *tdma;
	PVRSRV_DEVICE_NODE *psDeviceNode; 
	struct platform_device_info dma_device_info; 
	int ret = -1;
	dev_info(xdev->dev,"tdma init start"); 
	tdma = kmalloc(sizeof(*tdma), GFP_KERNEL);
	if (!tdma)
		return -ENOMEM;

	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;

	
	dma_device_info = (struct platform_device_info){
		.parent = &xdev->pdev->dev,
		.name = "xdx-dma",
		.id = -2,
		.dma_mask = DMA_BIT_MASK(64),
	};
		
	tdma->tdma_rmmio_base = xdev->rmmio_base + SYS_PANGU_REG_TDMA_BASE;
	tdma->tdma_rmmio_size = 3*SYS_PANGU_REG_TDMA_SIZE;
	pr_err("tdma_rmmio_base = 0x%llx\n",tdma->tdma_rmmio_base);
	tdma->tdma_rmmio = ioremap(tdma->tdma_rmmio_base, tdma->tdma_rmmio_size); 
	if (tdma->tdma_rmmio == NULL) {
		dev_err(xdev->dev, "vdma0 failed to map mmio registers\n");
		goto err1;
	}
	tdma->funcs = &tdma_funcs_instance;	
	xdev->tdma = *tdma;
	pr_err("%s line:%d\n",__func__,__LINE__);
	ret = xdx_tdma_module_init(tdma->tdma_rmmio);	
	if (ret != 0){
		dev_err(xdev->dev, "xdx_dma_module_init failed \n");
		goto err2;
	}		
	pr_err("%s line:%d\n",__func__,__LINE__);
	#if 0
	dma_dev = platform_device_register_full(&dma_device_info);
	if (IS_ERR(dma_dev)) {
			ret = PTR_ERR(dma_dev);
			dev_err(&xdev->pdev->dev,
					"Failed to register rogue device (%d)\n", ret);
			dma_dev = NULL;
		goto err3;
	}
	#endif
//	ops = get_dma_ops(dma_dev->dev.parent);	
//	pr_err("%s line:%d\n",__func__,__LINE__);
//	((PVRSRV_RGXDEV_INFO	*)(psDeviceNode->pvDevice))->dma_dev = (NULL == ops) ? psDeviceNode->psDevConfig->pvOSDevice : dma_dev->dev.parent;
//	xdev->tdma.sdc = platform_get_drvdata(dma_dev);
//	xdev->tdma.dma_dev = dma_dev;
	xdev->tdma.sdc = xdev->vdma.sdc;
	xdev->tdma.dma_dev = xdev->vdma.dma_dev;

	ret = tdma_isr_install(xdev);
	if (ret){
		dev_err(xdev->dev, "tdma_isr_install failed \n");
		goto err4;
	}
	pr_err("%s line:%d\n",__func__,__LINE__);

	dev_info(xdev->dev,"tdma init success");
	return 0;

err4:
	platform_device_unregister(xdev->tdma.dma_dev);
//err3:
	xdx_dma_module_exit();
err2:
	iounmap(xdev->tdma.tdma_rmmio);
err1:
	kfree(&xdev->tdma);
	return ret;
}

int xdx_tdma_fini(struct xdx_device *xdev)
{
	int ret = -1;
	ret = tdma_isr_uninstall(xdev);
	if (ret){
		dev_err(xdev->dev, "vdma_isr_uninstall failed \n");
		return ret;
	}
//	PVRSRVDeInitialiseDMA(psDeviceNode);
	platform_device_unregister(xdev->tdma.dma_dev);
//    xdx_dma_module_exit();
	iounmap(xdev->tdma.tdma_rmmio);
	kfree(&xdev->tdma);

	return 0;

}

