#include <linux/device.h>
#include "xdx_device.h"
#include "xdx_vdma.h"
#include "vdma_irq.h"
#include "rgxdevice.h"
#include "xdx_dma.h"
#include "pangu.h"

int xdx_vdma_init(struct xdx_device *xdev)
{
	struct xdx_vdma *vdma;
	PVRSRV_DEVICE_NODE *psDeviceNode; 
	const struct dma_map_ops *ops; 
	struct platform_device_info dma_device_info; 
	int ret = -1;

	struct platform_device *dma_dev;
	dev_info(xdev->dev,"vdma init start"); 
	vdma = kmalloc(sizeof(*vdma), GFP_KERNEL);
	if (!vdma)
		return -ENOMEM;

	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
	
	dma_device_info = (struct platform_device_info){
		.parent = &xdev->pdev->dev,
		.name = "xdx-dma",
		.id = -2,
		.dma_mask = DMA_BIT_MASK(48),
	};
		
	vdma->vdma0_rmmio_base = xdev->rmmio_base + PANGU_REG_VDMA0_BASE;
	vdma->vdma0_rmmio_size = 3*PANGU_REG_VDMA0_SIZE;
	
	vdma->vdma0_rmmio = ioremap(vdma->vdma0_rmmio_base, vdma->vdma0_rmmio_size); 
	if (vdma->vdma0_rmmio == NULL) {
		dev_err(xdev->dev, "vdma0 failed to map mmio registers\n");
		goto err1;
	}
	vdma->funcs = &vdma_funcs_instance;	
	xdev->vdma = *vdma;

	ret = xdx_dma_module_init(vdma->vdma0_rmmio);	
	if (ret != 0){
		dev_err(xdev->dev, "xdx_dma_module_init failed \n");
		goto err2;
	}		
	
	dma_dev = platform_device_register_full(&dma_device_info);
	if (IS_ERR(dma_dev)) {
			ret = PTR_ERR(dma_dev);
			dev_err(&xdev->pdev->dev,
					"Failed to register rogue device (%d)\n", ret);
			dma_dev = NULL;
		goto err3;
	}
	ops = get_dma_ops(dma_dev->dev.parent);	
	
	((PVRSRV_RGXDEV_INFO	*)(psDeviceNode->pvDevice))->dma_dev = (NULL == ops) ? psDeviceNode->psDevConfig->pvOSDevice : dma_dev->dev.parent;
	xdev->vdma.sdc = platform_get_drvdata(dma_dev);
	xdev->vdma.dma_dev = dma_dev;
	ret = vdma_isr_install(xdev);
	if (ret){
		dev_err(xdev->dev, "vdma_isr_install failed \n");
		goto err4;
	}
	dev_info(xdev->dev,"vdma init success");
	return 0;

err4:
	platform_device_unregister(xdev->vdma.dma_dev);
err3:
	xdx_dma_module_exit();
err2:
	iounmap(xdev->vdma.vdma0_rmmio);
err1:
	kfree(&xdev->vdma);
	return ret;
}

int xdx_vdma_fini(struct xdx_device *xdev)
{
	int ret = -1;
	PVRSRV_DEVICE_NODE *psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
	ret = vdma_isr_uninstall(xdev);
	if (ret){
		dev_err(xdev->dev, "vdma_isr_uninstall failed \n");
		return ret;
	}
	PVRSRVDeInitialiseDMA(psDeviceNode);
	platform_device_unregister(xdev->vdma.dma_dev);
    xdx_dma_module_exit();
	iounmap(xdev->vdma.vdma0_rmmio);
	iounmap(xdev->vdma.vdma1_rmmio);
	kfree(&xdev->vdma);

	return 0;

}

