#include "xdx_device.h"
#include "pangu.h"
#include "xdx_irq.h"
#include "xdx_vmm.h"
#include "dc8200_mmu.h"

struct pangu_msix_vec_info {
	uint32_t clientid;
	const char *name;
};

static const uint32_t pangu_a0_msix_vec[] = {
	[MSIX_VD0_CHANNEL_COMB_CH0]      = 0,
	[MSIX_VD0_CHANNEL_COMB_CH1]      = 1,
	[MSIX_VD0_CHANNEL_COMB_CH2]      = 2,
	[MSIX_VD0_CHANNEL_COMB_CH3]      = 3,
	[MSIX_VD1_CHANNEL_COMB_CH0]      = 4,
	[MSIX_VD1_CHANNEL_COMB_CH1]      = 5,
	[MSIX_VD1_CHANNEL_COMB_CH2]      = 6,
	[MSIX_VD1_CHANNEL_COMB_CH3]      = 7,
	[MSIX_VD0_MCU_DOORBELL]          = 8,
	[MSIX_VD1_MCU_DOORBELL]          = 9,
	[MSIX_C_MCU_DOORBELL]            = 10,
	[MSIX_A_CPU_DOORBELL]            = 11,
	[MSIX_GPU_CORE0]                 = 12,
	[MSIX_GPU_CORE1]                 = 13,
	[MSIX_GPU_CORE2]                 = 14,
	[MSIX_GPU_CORE3]                 = 15,
	[MSIX_A_TO_HOST_RESERVE_INT]     = 24,
	[MSIX_VDMCU_MMU_INT]             = 25,
	[MSIX_VDEC_INT]                  = 26,
	[MSIX_VENC_INT]                  = 27,
	[MSIX_DC0_INTERRUPT]             = 16,
	[MSIX_MMU0_INTERRUPT]            = 17,
	[MSIX_DC0_SECURE_INTERRUPT]      = 18,
	[MSIX_DC1_INTERRUPT]             = MSIX_INVALID_IDX,
	[MSIX_MMU1_INTERRUPT]            = MSIX_INVALID_IDX,
	[MSIX_DC1_SECURE_INTERRUPT]      = MSIX_INVALID_IDX,
	[MSIX_DP0_INTERRUPT_O]           = 19,
	[MSIX_HDMI0_OAVP_INT]            = 20,
	[MSIX_HDMI0_OMAIN_INT]           = 21,
	[MSIX_HDMI0_OCEC_INT]            = 22,
	[MSIX_HDMI0_HPD_INT]             = 23,
	[MSIX_AUDIO0_INT]                = MSIX_INVALID_IDX,
	[MSIX_DP1_INTERRUPT_O]           = MSIX_INVALID_IDX,
	[MSIX_HDMI1_OAVP_INT]            = MSIX_INVALID_IDX,
	[MSIX_HDMI1_OMAIN_INT]           = MSIX_INVALID_IDX,
	[MSIX_HDMI1_OCEC_INT]            = MSIX_INVALID_IDX,
	[MSIX_HDMI1_HPD_INT]             = MSIX_INVALID_IDX,
	[MSIX_AUDIO1_INT]                = MSIX_INVALID_IDX,
	[MSIX_TDMAINTR_DMA]              = MSIX_INVALID_IDX,
	[MSIX_TDMMU_COMB]                = MSIX_INVALID_IDX,
	[MSIX_PDMA_HOST_INT]             = MSIX_INVALID_IDX,
	[MSIX_PDMA_HOST_ERR_INT]         = MSIX_INVALID_IDX,
	[MSIX_RESV0_INT]                 = MSIX_INVALID_IDX,
	[43]                             = MSIX_INVALID_IDX,
	[44]                             = MSIX_INVALID_IDX,
	[45]                             = MSIX_INVALID_IDX,
	[46]                             = MSIX_INVALID_IDX,
	[47]                             = MSIX_INVALID_IDX,
	[48]                             = MSIX_INVALID_IDX,
	[49]                             = MSIX_INVALID_IDX,
	[MSIX_PF1_VD0_MCU_DOORBELL]      = MSIX_INVALID_IDX,
	[MSIX_PF1_VD1_MCU_DOORBELL]      = MSIX_INVALID_IDX,
	[MSIX_PF1_C_MCU_DOORBELL]        = MSIX_INVALID_IDX,
	[MSIX_PF1_A_CPU_DOORBELL]        = MSIX_INVALID_IDX,
	[MSIX_PF1_A_TO_HOST_RESERVE_INT] = MSIX_INVALID_IDX,

	[MSIX_VECTOR_MAX]                = 28,
};

static const uint32_t pangu_b0_msix_vec[] = {
	[MSIX_VD0_CHANNEL_COMB_CH0]      = 0,
	[MSIX_VD0_CHANNEL_COMB_CH1]      = 1,
	[MSIX_VD0_CHANNEL_COMB_CH2]      = 2,
	[MSIX_VD0_CHANNEL_COMB_CH3]      = 3,
	[MSIX_VD1_CHANNEL_COMB_CH0]      = 4,
	[MSIX_VD1_CHANNEL_COMB_CH1]      = 5,
	[MSIX_VD1_CHANNEL_COMB_CH2]      = 6,
	[MSIX_VD1_CHANNEL_COMB_CH3]      = 7,
	[MSIX_VD0_MCU_DOORBELL]          = 8,
	[MSIX_VD1_MCU_DOORBELL]          = 9,
	[MSIX_C_MCU_DOORBELL]            = 10,
	[MSIX_A_CPU_DOORBELL]            = 11,
	[MSIX_GPU_CORE0]                 = 12,
	[MSIX_GPU_CORE1]                 = 13,
	[MSIX_GPU_CORE2]                 = MSIX_INVALID_IDX,
	[MSIX_GPU_CORE3]                 = MSIX_INVALID_IDX,
	[MSIX_A_TO_HOST_RESERVE_INT]     = 14,
	[MSIX_VDMCU_MMU_INT]             = 15,
	[MSIX_VDEC_INT]                  = 16,
	[MSIX_VENC_INT]                  = 17,
	[MSIX_DC0_INTERRUPT]             = 18,
	[MSIX_MMU0_INTERRUPT]            = 19,
	[MSIX_DC0_SECURE_INTERRUPT]      = 20,
	[MSIX_DC1_INTERRUPT]             = 21,
	[MSIX_MMU1_INTERRUPT]            = 22,
	[MSIX_DC1_SECURE_INTERRUPT]      = 23,
	[MSIX_DP0_INTERRUPT_O]           = 24,
	[MSIX_HDMI0_OAVP_INT]            = 25,
	[MSIX_HDMI0_OMAIN_INT]           = 26,
	[MSIX_HDMI0_OCEC_INT]            = 27,
	[MSIX_HDMI0_HPD_INT]             = 28,
	[MSIX_AUDIO0_INT]                = 29,
	[MSIX_DP1_INTERRUPT_O]           = 30,
	[MSIX_HDMI1_OAVP_INT]            = 31,
	[MSIX_HDMI1_OMAIN_INT]           = 32,
	[MSIX_HDMI1_OCEC_INT]            = 33,
	[MSIX_HDMI1_HPD_INT]             = 34,
	[MSIX_AUDIO1_INT]                = 35,
	[MSIX_TDMAINTR_DMA]              = 36,
	[MSIX_TDMMU_COMB]                = 37,
	[MSIX_PDMA_HOST_INT]             = 38,
	[MSIX_PDMA_HOST_ERR_INT]         = 39,
	[MSIX_RESV0_INT]                 = 40,
	[43]                             = MSIX_INVALID_IDX,
	[44]                             = MSIX_INVALID_IDX,
	[45]                             = MSIX_INVALID_IDX,
	[46]                             = MSIX_INVALID_IDX,
	[47]                             = MSIX_INVALID_IDX,
	[48]                             = MSIX_INVALID_IDX,
	[49]                             = MSIX_INVALID_IDX,
	[MSIX_PF1_VD0_MCU_DOORBELL]      = 48,
	[MSIX_PF1_VD1_MCU_DOORBELL]      = 49,
	[MSIX_PF1_C_MCU_DOORBELL]        = 50,
	[MSIX_PF1_A_CPU_DOORBELL]        = 51,
	[MSIX_PF1_A_TO_HOST_RESERVE_INT] = 52,

	[MSIX_VECTOR_MAX]                = 53,
};

static struct pangu_msix_vec_info pangu_a0_msix_vec_to_clientid[] = {
	[MSIX_VD0_CHANNEL_COMB_CH0] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0" },
	[MSIX_VD0_CHANNEL_COMB_CH1] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0" },
	[MSIX_VD0_CHANNEL_COMB_CH2] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0" },
	[MSIX_VD0_CHANNEL_COMB_CH3] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0" },
	[MSIX_VD1_CHANNEL_COMB_CH0] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma1" },
	[MSIX_VD1_CHANNEL_COMB_CH1] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma1" },
	[MSIX_VD1_CHANNEL_COMB_CH2] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma1" },
	[MSIX_VD1_CHANNEL_COMB_CH3] = { XDX_IRQ_CLIENTID_AUDIO_DMA, "xdxgpu-audio-dma1" },
	[MSIX_VD0_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vd0mcu" },
	[MSIX_VD1_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vd1mcu" },
	[MSIX_C_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_CMCU, "xdxgpu-cmcu" },
	[MSIX_A_CPU_DOORBELL] = { XDX_IRQ_CLIENTID_ACPU, "xdxgpu-acpu" },
	[MSIX_GPU_CORE0] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx0" },
	[MSIX_GPU_CORE1] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx1" },
	[MSIX_GPU_CORE2] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx2" },
	[MSIX_GPU_CORE3] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx3" },
	[MSIX_DC0_INTERRUPT] = { XDX_IRQ_CLIENTID_DC0, "xdxgpu-dc" },
	[MSIX_MMU0_INTERRUPT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-mmu" },
	[MSIX_DC0_SECURE_INTERRUPT] = { XDX_IRQ_CLIENTID_DC0, "xdxgpu-dcsec" },
	[MSIX_DP0_INTERRUPT_O] = { XDX_IRQ_CLIENTID_DP0, "xdxgpu-dp" },
	[MSIX_HDMI0_OAVP_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi" },
	[MSIX_HDMI0_OMAIN_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi" },
	[MSIX_HDMI0_OCEC_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi" },
	[MSIX_HDMI0_HPD_INT] = { XDX_IRQ_CLIENTID_HDMI0, "xdxgpu-hdmi" },
	[MSIX_A_TO_HOST_RESERVE_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-a2h" },
	[MSIX_VDMCU_MMU_INT] ={ XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vdmcu" },
	[MSIX_VDEC_INT] = { XDX_IRQ_CLIENTID_VPU, "xdxgpu-vdec" },
	[MSIX_VENC_INT] = { XDX_IRQ_CLIENTID_VPU, "xdxgpu-venc" },
};

static struct pangu_msix_vec_info pangu_b0_msix_vec_to_clientid[] = {
	[MSIX_VD0_CHANNEL_COMB_CH0] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0-0" },
	[MSIX_VD0_CHANNEL_COMB_CH1] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-dma0-1" },
	[MSIX_VD0_CHANNEL_COMB_CH2] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma0-2" },
	[MSIX_VD0_CHANNEL_COMB_CH3] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-dma0-3" },
	[MSIX_VD1_CHANNEL_COMB_CH0] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma1-0" },
	[MSIX_VD1_CHANNEL_COMB_CH1] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-dma1-1" },
	[MSIX_VD1_CHANNEL_COMB_CH2] = { XDX_IRQ_CLIENTID_VDMA, "xdxgpu-vdma1-2" },
	[MSIX_VD1_CHANNEL_COMB_CH3] = { XDX_IRQ_CLIENTID_AUDIO_DMA, "xdxgpu-audio-dma1" },
	[MSIX_VD0_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vd0mcu" },
	[MSIX_VD1_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vd1mcu" },
	[MSIX_C_MCU_DOORBELL] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-cmcu" },
	[MSIX_A_CPU_DOORBELL] = { XDX_IRQ_CLIENTID_ACPU, "xdxgpu-acpu" },
	[MSIX_GPU_CORE0] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx0" },
	[MSIX_GPU_CORE1] = { XDX_IRQ_CLIENTID_GPU, "xdxgpu-gfx1" },
	[MSIX_A_TO_HOST_RESERVE_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-a2h" },
	[MSIX_VDMCU_MMU_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-vdmcu" },
	[MSIX_VDEC_INT] = { XDX_IRQ_CLIENTID_VPU, "xdxgpu-vdec" },
	[MSIX_VENC_INT] = { XDX_IRQ_CLIENTID_VPU, "xdxgpu-venc" },
	[MSIX_DC0_INTERRUPT] = { XDX_IRQ_CLIENTID_DC0, "xdxgpu-dc0" },
	[MSIX_MMU0_INTERRUPT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-mmu" },
	[MSIX_DC0_SECURE_INTERRUPT] = { XDX_IRQ_CLIENTID_DC0, "xdxgpu-dcsec0" },
	[MSIX_DC1_INTERRUPT] = { XDX_IRQ_CLIENTID_DC1, "xdxgpu-dc1" },
	[MSIX_MMU1_INTERRUPT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-mmu1" },
	[MSIX_DC1_SECURE_INTERRUPT] = { XDX_IRQ_CLIENTID_DC1, "xdxgpu-dcsec1" },
	[MSIX_DP0_INTERRUPT_O] = { XDX_IRQ_CLIENTID_DP0, "xdxgpu-dp0" },
	[MSIX_HDMI0_OAVP_INT] ={ XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi0-ovap" },
	[MSIX_HDMI0_OMAIN_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi0-omain" },
	[MSIX_HDMI0_OCEC_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi0-ocec" },
	[MSIX_HDMI0_HPD_INT] = { XDX_IRQ_CLIENTID_HDMI0, "xdxgpu-hdmi0-hpd" },
	[MSIX_AUDIO0_INT] = { XDX_IRQ_CLIENTID_AUDIO, "xdxgpu-audio0" },
	[MSIX_DP1_INTERRUPT_O] = { XDX_IRQ_CLIENTID_DP1, "xdxgpu-dp1" },
	[MSIX_HDMI1_OAVP_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi1-ovap" },
	[MSIX_HDMI1_OMAIN_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi1-omain" },
	[MSIX_HDMI1_OCEC_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-hdmi1-ocec" },
	[MSIX_HDMI1_HPD_INT] = { XDX_IRQ_CLIENTID_HDMI1, "xdxgpu-hdmi1-hpd" },
	[MSIX_AUDIO1_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-audio1" },
	[MSIX_TDMAINTR_DMA] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-tdma" },
	[MSIX_TDMMU_COMB] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-tdma-mmu" },
	[MSIX_PDMA_HOST_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-pdma-host" },
	[MSIX_PDMA_HOST_ERR_INT] = { XDX_IRQ_CLIENTID_INVALID, "xdxgpu-pdma-hosterr" },
};

uint32_t xdx_get_msix_idx(enum xdx_asic_type type, enum msix_vector id)
{
	uint32_t idx = MSIX_INVALID_IDX;

	if (type == CHIP_PANGU_A0)
		idx = pangu_a0_msix_vec[id];
	else
		idx = pangu_b0_msix_vec[id];

	return idx;
}

static uint32_t pangu_get_irq_client_id(struct xdx_irq_entry *irq_entry)
{
	struct xdx_device *xdev = irq_entry->xdev;
	struct pangu_msix_vec_info *info;
	int i;

	if (irq_entry->vec_idx >= xdx_get_msix_idx(xdev->asic_type, MSIX_VECTOR_MAX))
		return XDX_IRQ_CLIENTID_INVALID;

	if (xdev->asic_type == CHIP_PANGU_A0) {
		if ((xdev->flags.virtual_vblank == 1) &&
			(irq_entry->vec_idx == xdx_get_msix_idx(xdev->asic_type, MSIX_DC0_INTERRUPT)))
			return XDX_IRQ_CLIENTID_INVALID;
	}

	if (xdev->asic_type == CHIP_PANGU_B0) {
		if ((xdev->flags.virtual_vblank == 1) &&
			((irq_entry->vec_idx == xdx_get_msix_idx(xdev->asic_type, MSIX_DC0_INTERRUPT)) ||
			(irq_entry->vec_idx == xdx_get_msix_idx(xdev->asic_type, MSIX_DC1_INTERRUPT))))
			return XDX_IRQ_CLIENTID_INVALID;
	}

	if (xdev->asic_type == CHIP_PANGU_A0)
		info = pangu_a0_msix_vec_to_clientid;
	else
		info = pangu_b0_msix_vec_to_clientid;

	for (i = 0; i < MSIX_VECTOR_MAX; i++) {
		if (irq_entry->vec_idx == xdx_get_msix_idx(xdev->asic_type, i))
			return info[i].clientid;
	}

	BUG_ON(1);
	return XDX_IRQ_CLIENTID_INVALID;
}

static const char *pangu_get_irq_vector_name(struct xdx_irq_entry *irq_entry)
{
	struct xdx_device *xdev = irq_entry->xdev;
	struct pangu_msix_vec_info *info;
	int i;

	if (irq_entry->vec_idx >= xdx_get_msix_idx(xdev->asic_type, MSIX_VECTOR_MAX))
		return (char *)XDX_IRQ_CLIENTID_INVALID;

	if (xdev->asic_type == CHIP_PANGU_A0)
		info = pangu_a0_msix_vec_to_clientid;
	else
		info = pangu_b0_msix_vec_to_clientid;

	for (i = 0; i < MSIX_VECTOR_MAX; i++) {
		if (irq_entry->vec_idx == xdx_get_msix_idx(xdev->asic_type, i))
			return info[i].name;
	}

	BUG_ON(1);
	return (char *)XDX_IRQ_CLIENTID_INVALID;
}

static int pangu_intx_handler(struct xdx_device *xdev,
			      struct xdx_irq_entry *irq_entry)
{
	struct xdx_dm *dm = &xdev->dm;
	struct pangu_interrupt_status intr_status;
	unsigned long flags;

	intr_status.e1.all = xdx_mmio_readl(xdev, PANGU_REG_DOORBELL_126);
	intr_status.e2.all = xdx_mmio_readl(xdev, PANGU_REG_DOORBELL_127);

	spin_lock_irqsave(&xdev->irq.lock, flags);

	if (intr_status.e1.intr_idx[0] &&
	    xdev->irq.clients[XDX_IRQ_CLIENTID_GPU])
		xdev->irq.clients[XDX_IRQ_CLIENTID_GPU]->funcs->process(
			xdev, xdev->irq.clients[XDX_IRQ_CLIENTID_GPU],
			irq_entry);

	if (intr_status.e1.intr_idx[1] &&
	    xdev->irq.clients[XDX_IRQ_CLIENTID_VDMA])
		xdev->irq.clients[XDX_IRQ_CLIENTID_VDMA]->funcs->process(
			xdev, xdev->irq.clients[XDX_IRQ_CLIENTID_VDMA],
			irq_entry);

	if (intr_status.e1.intr_idx[2] &&
	    xdev->irq.clients[XDX_IRQ_CLIENTID_DC0]) {
			xdx_mmio_writeb(xdev, PANGU_REG_DOORBELL_126 + 2, 0);
			xdev->irq.clients[XDX_IRQ_CLIENTID_DC0]->funcs->process(
				xdev, xdev->irq.clients[XDX_IRQ_CLIENTID_DC0],
				irq_entry);
		}
	if (dm->dc_num == 2) {
		if (intr_status.e1.intr_idx[2] &&
			xdev->irq.clients[XDX_IRQ_CLIENTID_DC1]) {
				xdx_mmio_writeb(xdev, PANGU_REG_DOORBELL_126 + 3, 0);
				xdev->irq.clients[XDX_IRQ_CLIENTID_DC1]->funcs->process(
					xdev, xdev->irq.clients[XDX_IRQ_CLIENTID_DC1],
					irq_entry);
		}
	}
	spin_unlock_irqrestore(&xdev->irq.lock, flags);

	return 0;
}

static int pangu_set(struct xdx_device *xdev, struct xdx_irq_entry *irq_entry,
		     enum xdx_interrupt_state state)
{
	uint32_t vec_idx = irq_entry->vec_idx;
	uint32_t value;

	if (vec_idx >= xdx_get_msix_idx(xdev->asic_type, MSIX_VECTOR_MAX))
		return XDX_IRQ_CLIENTID_INVALID;

	value = xdx_mmio_readl(xdev, PANGU_REG_HOST_MISC_BASE +
					     mmMSI_MISX_HW_ENABLE_PF0);

	if (state == XDX_IRQ_STATE_DISABLE)
		value &= ~(1 << vec_idx);
	else if (state == XDX_IRQ_STATE_ENABLE)
		value |= (1 << vec_idx);

	xdx_mmio_writel(xdev,
			PANGU_REG_HOST_MISC_BASE + mmMSI_MISX_HW_ENABLE_PF0,
			value);

	return 0;
}

static const struct xdx_ih_funcs pangu_ih_funcs = {
	.get_client_id = pangu_get_irq_client_id,
	.get_irq_vector_name = pangu_get_irq_vector_name,
	.intx_handler = pangu_intx_handler,
	.set = pangu_set
};

static void pangu_ih_set_interrupt_funcs(struct xdx_device *xdev)
{
	xdev->irq.ih_funcs = &pangu_ih_funcs;
}

static void pangu_get_device_info(struct xdx_device *xdev)
{
	uint32_t i = 0;
	union primary_core_id primary_core_id;
	uint32_t primary_id = 0, tmp = 0;
	uint32_t core_count = 0;
	uint32_t dc_num = 0;

	struct xdx_dm *dm = &xdev->dm;
	xdev->hw_info.gddr_dev_phyaddr_base = PANGU_VRAM_DEV_PHYADDR_BASE;
	xdev->hw_info.host_dev_phyaddr_base = PANGU_HOSTRAM_DEV_PHYSICAL_BASE;
	xdev->hw_info.shareram_dev_phyaddr_base = PANGU_SSRAM_DEV_PHYSICAL_BASE;
	xdev->hw_info.alignment = 0x10;
	dm->dc_num =1;

	if (xdev->asic_type == CHIP_PANGU_B0){
		dc_num = xdx_mmio_readl(xdev, PANGU_DC_CHANNEL_INFO_OFFSET);
		dev_info(xdev->dev, "dc_num=0x%x\n", dc_num);
		if ( dc_num == 1)
			dm->dc_num = 1;
		else
			dm->dc_num = 2;
	}
	primary_core_id.value = xdx_mmio_readl(xdev, PANGU_REG_GPU_CORE_MISC_BASE +
					     mmGPU_MISC_PRIMARY_CORE_ID);
	for (i = 0; i < PANGU_GPU_CORE_MAX; i++) {
		tmp = (primary_core_id.value >> 3*i) & GPU_MISC_PRIMARY_CORE_ID_MASK;
		if (tmp != GPU_MISC_PRIMARY_CORE_ID_MASK){
			core_count++;
			if (1 == core_count)
				primary_id = i;
		}
	}

	dev_info(
		xdev->dev,
		"GPU Multi-Core Config: core0 0x%x, core1 0x%x, core2 0x%x, core3 0x%x\n",
		primary_core_id.bitfiles.core0, primary_core_id.bitfiles.core1,
		primary_core_id.bitfiles.core2, primary_core_id.bitfiles.core3);
	dev_info(xdev->dev, "GPU: primary id: %d, core count %d", primary_id, core_count);

	xdev->hw_info.gfx_regs.base = PANGU_REG_GPU_CORE0_BASE +
				      PANGU_REG_GPU_CORE_REG_SIZE * primary_id;
	xdev->hw_info.gfx_regs.size = PANGU_REG_GPU_CORE_REG_SIZE;
	
	xdev->hw_info.dc_regs[0].base = PANGU_REG_DC0_BASE;
	xdev->hw_info.dc_regs[0].size = PANGU_REG_DC0_REG_SIZE;
	xdev->hw_info.dc_mmu_regs[0].base = PANGU_REG_DC0_MMU_BASE;
	xdev->hw_info.dc_mmu_regs[0].size = PANGU_REG_DC0_MMU_REG_SIZE;
	
	xdev->hw_info.hdmi_core_regs[0].base = PROTO_HDMIQP0_TX_CORE_START;
	xdev->hw_info.hdmi_core_regs[0].size = PROTO_HDMIQP0_TX_CORE_SIZE;
	xdev->hw_info.hdmi_tile_regs[0].base = PROTO_HDMIQP0_TX_TILE_START;
	xdev->hw_info.hdmi_tile_regs[0].size = PROTO_HDMIQP0_TX_TILE_SIZE;
	xdev->hw_info.hdmi_pcs_regs[0].base = PROTO_HDMIQP0_TX_PCS_START;
	xdev->hw_info.hdmi_pcs_regs[0].size = PROTO_HDMIQP0_TX_PCS_SIZE;

	xdev->hw_info.dp_regs[0].base = PROTO_DP0_CTL_START;
	xdev->hw_info.dp_regs[0].size = PROTO_DP0_CTL_SIZE;
	xdev->hw_info.dp_pcs_regs[0].base = PROTO_DP0_PCS_START;
	xdev->hw_info.dp_pcs_regs[0].size = PROTO_DP0_PCS_SIZE;

	if (dm->dc_num == 2){
		xdev->hw_info.dc_regs[1].base = PANGU_REG_DC1_BASE;
		xdev->hw_info.dc_regs[1].size = PANGU_REG_DC1_REG_SIZE;
		xdev->hw_info.dc_mmu_regs[1].base = PANGU_REG_DC1_MMU_BASE;
		xdev->hw_info.dc_mmu_regs[1].size = PANGU_REG_DC1_MMU_REG_SIZE;
	
		xdev->hw_info.hdmi_core_regs[1].base = PROTO_HDMIQP1_TX_CORE_START;
		xdev->hw_info.hdmi_core_regs[1].size = PROTO_HDMIQP1_TX_CORE_SIZE;
		xdev->hw_info.hdmi_tile_regs[1].base = PROTO_HDMIQP1_TX_TILE_START;
		xdev->hw_info.hdmi_tile_regs[1].size = PROTO_HDMIQP1_TX_TILE_SIZE;
		xdev->hw_info.hdmi_pcs_regs[1].base = PROTO_HDMIQP1_TX_PCS_START;
		xdev->hw_info.hdmi_pcs_regs[1].size = PROTO_HDMIQP1_TX_PCS_SIZE;

		xdev->hw_info.dp_regs[1].base = PROTO_DP1_CTL_START;
		xdev->hw_info.dp_regs[1].size = PROTO_DP1_CTL_SIZE;
		xdev->hw_info.dp_pcs_regs[1].base = PROTO_DP1_PCS_START;
		xdev->hw_info.dp_pcs_regs[1].size = PROTO_DP1_PCS_SIZE;
	}
	xdev->hw_info.reserved_vram_size = 64 * 1024 * 1024; /* Reserve hign 64M vram for ACPU */
}

static void pangu_soc_init(struct xdx_device *xdev)
{
	xdx_mmio_writel(xdev, PANGU_REG_HOST_MISC_BASE + mmPCIE_HIGH_ADDR_CNTL, 0x00);
	xdx_mmio_writel(xdev, PANGU_REG_HOST_MISC_BASE + mmG2H_BASE_ADDR0_GRP0, 0x26);
	xdx_mmio_writel(xdev, PANGU_REG_HOST_MISC_BASE + mmG2H_BASE_ADDR1_GRP0, 0x80);
	xdx_mmio_writel(xdev, PANGU_REG_HOST_MISC_BASE + mmG2H_TARGET_ADDR0_GRP0, 0x00);
	xdx_mmio_writel(xdev, PANGU_REG_HOST_MISC_BASE + mmG2H_TARGET_ADDR1_GRP0, 0x00);
}

static const struct xdx_soc_funcs pangu_soc_funcs = {
	.hw_init = pangu_soc_init,
	.set_interrupt_funcs = pangu_ih_set_interrupt_funcs,
	.get_device_info = pangu_get_device_info,
};

int pangu_soc_setup(struct xdx_device *xdev)
{
	xdev->soc_funcs = pangu_soc_funcs;

	/* Add ACPU block */
#ifdef SUPPORT_PANGU_ACPU
	xdev->acpu.base.version = acpu_ip_block_version;
	xdx_device_ip_block_add(xdev, &xdev->acpu.base);
#endif

	/* Add GFX block */
	xdev->gfx.base.version = volcanic_block_version;
	xdx_device_ip_block_add(xdev, &xdev->gfx.base);

#ifdef 	SUPPORT_VPU
	/* Add VPU block */
	xdev->vpu.base.version = vpu_ip_block_version;
	xdx_device_ip_block_add(xdev, &xdev->vpu.base);

	xdev->vmm.vpu.funcs = (struct xdx_mmu_funcs *)&dc8200_mmu_funcs;
#endif

	/* Add display block */
	if (!xdev->flags.virtual_display) {
		xdev->vmm.dc.funcs = (struct xdx_mmu_funcs *)&dc8200_mmu_funcs;
		xdev->dm.base.version = dm_ip_block_version;
		xdx_device_ip_block_add(xdev, &xdev->dm.base);
	} else {
		xdev->vkmsdev.base.version = vkms_ip_block_version;
		xdx_device_ip_block_add(xdev, &xdev->vkmsdev.base);
	}
	return 0;
}
