#include "xdx_fw.h"
#include "xdx_device.h"

#include "fwshare/ParameterPage.h"
#include "fwshare/EventLogID.h"
#include "fwshare/SharedVars.h"

const static u32 fw_bootup_gpu_freq_list[] = {
	800000000,
	700000000,
	600000000,
	400000000,
	900000000,
	1000000000,
	1050000000,
	1100000000,
	1150000000,
	1200000000,
	1250000000,
	1300000000,
	1350000000,
	1400000000,
	1500000000,
	500000000,
	300000000,
	200000000,
	100000000,
	50000000
};

u32 update_gddr_temp(struct xdx_device *xdev)
{
	return *((u32*)(xdev->share_ram + ACPU_VARS_OFFSET + offsetof(FwGlobalStruct_T, gddrTemp)));
}

int cache_fw_static_info(struct xdx_device *xdev)
{
	char *pcieMmbarType[] = {"64bit-ResizeBar", "64bit-NoneResizeBar", "32bit"};
	int gddrFreq[5] = {1000, 1500, 1750, 1900, 2000};
	int i;
	int j=0;
	FwGlobalStruct_T *fw_global_struct;
	u32 gpuDis; 
	ParameterPage_T *parameter_page = kmalloc(sizeof(ParameterPage_T), GFP_KERNEL);
	
	memcpy_fromio(parameter_page,
		      xdev->share_ram + PARAMETER_PAGE_OFFSET,
		      sizeof(ParameterPage_T));
	xdev->fw_st_info.default_gpu_freq = parameter_page->fwParameter.gpuClkInMHz * 1000000;
	xdev->fw_st_info.bootup_gpu_freq = fw_bootup_gpu_freq_list[parameter_page->fwParameter.gpuFreq];
	xdev->fw_st_info.acpu_freq = parameter_page->fwParameter.acpuClkInMHz * 1000000;
	xdev->fw_st_info.cmcu_freq = parameter_page->fwParameter.cmcuClkInMHz * 1000000;
	xdev->fw_st_info.sys_freq = parameter_page->fwParameter.sysClkInMHz * 1000000;
	xdev->fw_st_info.ndf_freq = (parameter_page->fwParameter.ndfFreq == 0 ? 1600 : 1500) * 1000000;
	xdev->fw_st_info.fw_ver = parameter_page->fwParameter.externalVer;
	xdev->fw_st_info.chip_id = parameter_page->fuse.chipID[1];
	xdev->fw_st_info.sku = parameter_page->fuse.sku.all;
	xdev->fw_st_info.gddr_count = parameter_page->ddrParameter.gddrCount;
	xdev->fw_st_info.pcie_mmbar_type = pcieMmbarType[parameter_page->fwParameter.hostType];
	xdev->fw_st_info.gddr_freq = gddrFreq[parameter_page->ddrParameter.gddrSpeed] * 1000000;

	fw_global_struct = kmalloc(sizeof(FwGlobalStruct_T), GFP_KERNEL);
	memcpy_fromio(fw_global_struct,
		      xdev->share_ram + ACPU_VARS_OFFSET,
		      sizeof(FwGlobalStruct_T));
	
	memcpy(xdev->fw_st_info.vbiosVerString, fw_global_struct->vbiosVersionString, 64);

	//GPU core info
	gpuDis = parameter_page->fuse.func&0xF;
    for(i=0;i<4;++i){
        if(!(gpuDis&(1<<i))){
			xdev->fw_st_info.gpu_core[j] = i + '0';
			++j;
        }
    }

	xdev->fw_st_info.pcie_event = fw_global_struct->pcieEvent.all;
	xdev->fw_st_info.gpu_primary_core_id = fw_global_struct->gpuPrimaryCoreID;

	kfree(fw_global_struct);
	kfree(parameter_page);
	return 0;
}
