#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0))
#include <linux/hwmon-sysfs.h>
#else
#include "hwmon-sysfs.h"
#endif
#include <linux/hwmon.h>
#include "xdx_hwmon.h"
#include "xdx_device.h"
#include "xdxacpu.h"
#include "xdx_pm.h"

#define GET_B31_24(X)           ((uint8_t)((X)>>24))
#define GET_B23_16(X)           ((uint8_t)((X)>>16))
#define GET_B15_00(X)           ((uint16_t)(X))

/* TODO: replace the dummy implement with host to cmcu doorbell */
struct xdxgpu_hwmon_dummy_record {
	long temp_crit;
	long temp_emergency;
	long power_cap;
	long pwm;
	long pwm_enable;
	long fan_enable;
	long fan_target;
};

static struct xdxgpu_hwmon_dummy_record dummy = {
	.temp_crit = 0,
	.temp_emergency = 0,
	.power_cap = 100,
	.pwm = 100,
	.pwm_enable = 0,
	.fan_enable = 1,
	.fan_target = 100
};

#if 0
static int xdxgpu_get_temp_input(struct xdx_device *xdev, long *val)
{
	*val = 0;
	return 0;
}

static int xdxgpu_get_temp_highest(struct xdx_device *xdev, long *val)
{
	*val = 0;
	return 0;
}

static int xdxgpu_get_temp_crit(struct xdx_device *xdev, long *val)
{
	*val = dummy.temp_crit;
	return 0;
}

static int xdxgpu_get_temp_emergency(struct xdx_device *xdev, long *val)
{
	*val = dummy.temp_emergency;
	return 0;
}

static int xdxgpu_get_in_input(struct xdx_device *xdev, long *val)
{
	*val = pangu_default_voltage;
	return 0;
}

static int xdxgpu_get_curr_input(struct xdx_device *xdev, long *val)
{
	*val = 500;
	return 0;
}

static int xdxgpu_get_power_average(struct xdx_device *xdev, long *val)
{
	*val = 50000;
	return 0;
}
#endif

static int xdxgpu_get_power_min(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_power_max(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_power_cap(struct xdx_device *xdev, long *val)
{
	*val = dummy.power_cap;
	return 0;
}

#if 0
static int xdxgpu_get_pwm(struct xdx_device *xdev, long *val)
{
	*val = dummy.pwm;
	return 0;
}
#endif

static int xdxgpu_get_pwm_enable(struct xdx_device *xdev, long *val)
{
	*val = dummy.pwm_enable;
	return 0;
}
#if 0
static int xdxgpu_get_fan_input(struct xdx_device *xdev, long *val)
{
	*val = 200;
	return 0;
}

static int xdxgpu_get_fan_min(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_fan_max(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_fan_target(struct xdx_device *xdev, long *val)
{
	*val = dummy.fan_target;
	return 0;
}

static int xdxgpu_set_temp_crit(struct xdx_device *xdev, long val)
{
	dummy.temp_crit = val;
	return 0;
}

static int xdxgpu_set_temp_emergency(struct xdx_device *xdev, long val)
{
	dummy.temp_emergency = val;
	return 0;
}
#endif

static int xdxgpu_set_power_cap(struct xdx_device *xdev, long val)
{
	dummy.power_cap = val;
	return 0;
}

#if 0
static int xdxgpu_set_pwm(struct xdx_device *xdev, long val)
{
	dummy.pwm = val;
	return 0;
}
#endif

static int xdxgpu_set_pwm_enable(struct xdx_device *xdev, long val)
{
	dummy.pwm_enable = val;
	return 0;
}

#if 0
static int xdxgpu_set_fan_target(struct xdx_device *xdev, long val)
{
	dummy.fan_target = val;
	return 0;
}
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0))
#define HWMON_CHANNEL_INFO(stype, ...)	\
	(&(struct hwmon_channel_info) {	\
		.type = hwmon_##stype,	\
		.config = (u32 []) {	\
			__VA_ARGS__, 0	\
		}			\
	})
#endif
static const struct hwmon_channel_info *xdxgpu_hwmon_info[] = {
	HWMON_CHANNEL_INFO(temp,
			   HWMON_T_INPUT | HWMON_T_HIGHEST | HWMON_T_CRIT | \
			   HWMON_T_EMERGENCY | HWMON_T_MAX | HWMON_T_MIN),
	HWMON_CHANNEL_INFO(in, HWMON_I_INPUT),
	HWMON_CHANNEL_INFO(curr, HWMON_C_INPUT),
	HWMON_CHANNEL_INFO(power,
			   HWMON_P_AVERAGE | HWMON_P_CAP_MIN | \
			   HWMON_P_CAP_MAX | HWMON_P_CAP),
	HWMON_CHANNEL_INFO(pwm,
			   HWMON_PWM_INPUT | HWMON_PWM_ENABLE),
	HWMON_CHANNEL_INFO(fan,
			   HWMON_F_INPUT | \
			   HWMON_F_MIN | HWMON_F_MAX | HWMON_F_TARGET),
	NULL
};

static umode_t xdxgpu_is_visible(const void *drvdata, enum hwmon_sensor_types type, u32 attr, int channel)
{
	switch(type) {
	case hwmon_temp:
		switch(attr) {
		case hwmon_temp_input:
		case hwmon_temp_highest:
			return S_IRUGO;
		case hwmon_temp_crit:
		case hwmon_temp_max:
		case hwmon_temp_min:
		case hwmon_temp_emergency:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	case hwmon_power:
		switch(attr) {
		case hwmon_power_average:
		case hwmon_power_cap_min:
		case hwmon_power_cap_max:
			return S_IRUGO;
		case hwmon_power_cap:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	case hwmon_pwm:
		switch(attr) {
		case hwmon_pwm_input:
		case hwmon_pwm_enable:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	case hwmon_in:
		switch(attr) {
		case hwmon_in_input:
			/* All A0 cards are prohibited from setting their voltage */
			return S_IRUGO;
		default:
			return (umode_t)0;
		}
	case hwmon_curr:
		switch(attr) {
		case hwmon_curr_input:
			return S_IRUGO;
		default:
			return (umode_t)0;
		}
	case hwmon_fan:
		switch(attr) {
		case hwmon_fan_input:
		case hwmon_fan_min:
		case hwmon_fan_max:
			return S_IRUGO;
		case hwmon_fan_target:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	default:
		return (umode_t)0;
	}
}

int xdxgpu_hwmon_read(struct device *dev, enum hwmon_sensor_types type,
		      u32 attr, int channel, long *val)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;

	xdev = dev_get_drvdata(dev);

	switch(type) {
	case hwmon_temp:
		subcmd = cDDKPowerTemp;
		switch(attr) {
		case hwmon_temp_input:
			cmd_type = TEMP_GET;
			break;
		case hwmon_temp_highest:
			cmd_type = TEMP_HIGHEST_GET;
			break;
		case hwmon_temp_crit:
			cmd_type = TEMP_GET_CRITICAL;
			break;
		case hwmon_temp_max:
			cmd_type = TEMP_GET_MAX;
			break;
		case hwmon_temp_min:
			cmd_type = TEMP_GET_MIN;
			break;
		case hwmon_temp_emergency:
			cmd_type = TEMP_GET_EMERGENCY;
			break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_power:
		subcmd = cDDKPowerPower;
		switch(attr) {
		case hwmon_power_average:
			cmd_type = POWER_GET_AVG;
			break;
		
		//There aren't these attributes in firmware (HostEvent.h)
		case hwmon_power_cap_min:
			return xdxgpu_get_power_min(xdev, val);
		case hwmon_power_cap_max:
			return xdxgpu_get_power_max(xdev, val);
		case hwmon_power_cap:
			return xdxgpu_get_power_cap(xdev, val);

		default:
			return -EINVAL;
		}
		break;
	case hwmon_in:
		subcmd = cDDKPowerVolt;
		switch(attr) {
		case hwmon_in_input:
			cmd_type = VOLTAGE_GET;
			break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_curr:
		subcmd = cDDKPowerCurr;
		switch(attr) {
		case hwmon_curr_input:
			cmd_type = CURRENT_GET;
			break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_pwm:
		subcmd = cDDKPowerPWM;
		switch(attr) {
		case hwmon_pwm_input:
			cmd_type = PWM_GET;
			break;
		case hwmon_pwm_enable:
			/* h02c doorbell interface not ready for now, using dummy data */
			return xdxgpu_get_pwm_enable(xdev, val);
		default:
			return -EINVAL;
		}
		break;
	case hwmon_fan:
		subcmd = cDDKPowerFan;
		switch(attr) {
		case hwmon_fan_input:
		case hwmon_fan_target:
			cmd_type = FAN_RPM_GET;
			break;
		case hwmon_fan_min:
			cmd_type = FAN_RPM_GET_MIN;
			break;
			//return xdxgpu_get_fan_min(xdev, val);
		case hwmon_fan_max:
			cmd_type = FAN_RPM_GET_MAX;
			break;
			//return xdxgpu_get_fan_max(xdev, val);
		default:
			return -EINVAL;
		}
		break;
	default:
		return -EINVAL;
	}

	*val = xdx_power_get(xdev, subcmd, cmd_type, channel);
	return 0;
}

int xdxgpu_hwmon_write(struct device *dev, enum hwmon_sensor_types type,
		       u32 attr, int channel, long val)
{
	uint8_t subcmd, cmd_type;
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);

	switch(type) {
	case hwmon_temp:
		subcmd = cDDKPowerTemp;
		switch(attr) {
		case hwmon_temp_crit:
			cmd_type = TEMP_SET_CRITICAL;
			break;
		case hwmon_temp_max:
			cmd_type = TEMP_SET_MAX;
			break;
		case hwmon_temp_min:
			cmd_type = TEMP_SET_MIN;
			break;
		case hwmon_temp_emergency:
			cmd_type = TEMP_SET_EMERGENCY;
			break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_power:
		/* h02c doorbell interface not ready for now, using dummy data */
		switch(attr) {
		case hwmon_power_cap:
			return xdxgpu_set_power_cap(xdev, val);
		default:
			return -EINVAL;
		}
		break;
	/* All A0 cards are prohibited from setting their voltage */
	#if 0
	case hwmon_in:
		subcmd = cDDKPowerDVFS;
		switch(attr) {
		case hwmon_in_input:
			cmd_type = DVFS_SET_VOLT;
			break;
		default:
			return -EINVAL;
		}
		break;
	#endif
	case hwmon_pwm:
		subcmd = cDDKPowerPWM;
		switch(attr) {
		case hwmon_pwm_input:
			cmd_type = PWM_SET;
			break;
		case hwmon_pwm_enable:
			/* h02c doorbell interface not ready for now, using dummy data */
			return xdxgpu_set_pwm_enable(xdev, val);
		default:
			return -EINVAL;
		}
		break;
	case hwmon_fan:
		subcmd = cDDKPowerFan;
		switch(attr) {
		case hwmon_fan_target:
			cmd_type = FAN_RPM_SET;
			break;
		default:
			return -EINVAL;
		}
		break;
	default:
		return -EINVAL;
	}

	xdx_power_set(xdev, subcmd, cmd_type, channel, val);

	return 0;
}

static const struct hwmon_ops xdxgpu_hwmon_ops = {
	.is_visible = xdxgpu_is_visible,
	.read = xdxgpu_hwmon_read,
	.write = xdxgpu_hwmon_write
};

static const struct hwmon_chip_info xdxgpu_hwmon_chip_info = {
	.ops = &xdxgpu_hwmon_ops,
	.info = xdxgpu_hwmon_info
};

static ssize_t xdx_gddr_phys_freq_mhz_show(struct device *dev,
					   struct device_attribute *attr,
					   char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.gddr_freq);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.gddr_freq);
#endif
}

static ssize_t xdx_acpu_freq_mhz_show(struct device *dev,
				      struct device_attribute *attr,
				      char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.acpu_freq);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.acpu_freq);
#endif
}

static ssize_t xdx_cmcu_freq_mhz_show(struct device *dev,
				      struct device_attribute *attr,
				      char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.cmcu_freq);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.cmcu_freq);
#endif
}

static ssize_t xdx_ndf_freq_mhz_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.ndf_freq);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.ndf_freq);
#endif
}

static ssize_t xdx_host_freq_mhz_show(struct device *dev,
				      struct device_attribute *attr,
				      char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.sys_freq);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.sys_freq);
#endif
}

static ssize_t xdx_gfx_freq_mhz_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	IMG_UINT32 ret;

	xdev = dev_get_drvdata(dev);
	
	subcmd = cDDKPowerDVFS;
	cmd_type = DVFS_GET_FREQ;

	ret = xdx_power_get(xdev, subcmd, cmd_type, 1);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", ret*1000000);
#else
	return sprintf(buf, "%u\n", ret*1000000);
#endif
}

static ssize_t xdx_gfx_freq_mhz_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	u32 value;

	int err;
	err = kstrtou32(buf, 10, &value);
	if (err)
		return err;
	
	xdev = dev_get_drvdata(dev);

	xdx_dvfs_set_freq(xdev, value);

	return count;
}

static ssize_t xdx_supported_gfx_freq_mhz_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->pm.current_gfxclk);
#else	
	return sprintf(buf, "%u\n", xdev->pm.current_gfxclk);
#endif
}

static ssize_t temp1_ctrl_mode_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	int value;
	int err;
	xdev = dev_get_drvdata(dev);

	subcmd = cDDKPowerTemp;
	cmd_type = TEMP_SET_CTRL_MODE;
	
	err = kstrtoint(buf, 10, &value);

	if (err)
		return err;

	switch (value) {
		case 0:  /* auto */
			xdx_power_set(xdev, subcmd, cmd_type, 1, value);
			break;
		case 1:  /* manual */
			xdx_power_set(xdev, subcmd, cmd_type, 1, value);
			break;
		default: /* disable */
			break;
	}

	return count;
}

static ssize_t temp1_ctrl_mode_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	IMG_UINT32 ret;

	xdev = dev_get_drvdata(dev);

	subcmd = cDDKPowerTemp;
	cmd_type = TEMP_GET_CTRL_MODE;

	ret = xdx_power_get(xdev, subcmd, cmd_type, 1);

	return sprintf(buf, "%i\n", ret);
}


static ssize_t fan1_ctrl_mode_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	int value;
	int err;
	
	xdev = dev_get_drvdata(dev);

	subcmd = cDDKPowerFan;
	cmd_type = FAN_CTRL_MODE_SET;
	
	err = kstrtoint(buf, 10, &value);

	if (err)
		return err;

	switch (value) {
		case 0:  /* auto */
			xdx_power_set(xdev, subcmd, cmd_type, 1, value);
			break;
		case 1:  /* manual */
			xdx_power_set(xdev, subcmd, cmd_type, 1, value);
			break;
		default: /* disable */
			break;
	}

	return count;
}

static ssize_t fan1_ctrl_mode_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	IMG_UINT32 ret;
	xdev = dev_get_drvdata(dev);

	subcmd = cDDKPowerFan;
	cmd_type = FAN_CTRL_MODE_GET;

	ret = xdx_power_get(xdev, subcmd, cmd_type, 1);

	return sprintf(buf, "%i\n", ret);
}

static ssize_t xdx_fw_ver_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.fw_ver), GET_B23_16(xdev->fw_st_info.fw_ver), GET_B15_00(xdev->fw_st_info.fw_ver));
#else
	return sprintf(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.fw_ver), GET_B23_16(xdev->fw_st_info.fw_ver), GET_B15_00(xdev->fw_st_info.fw_ver));
#endif
}

static ssize_t xdx_vbios_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.vbiosVerString);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.vbiosVerString);
#endif
}

static ssize_t xdx_chip_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.chip_id);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.chip_id);
#endif
}

static ssize_t xdx_gpu_core_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.gpu_core);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.gpu_core);
#endif
}

static ssize_t xdx_gpu_primary_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.gpu_primary_id);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.gpu_primary_id);
#endif
}

static ssize_t xdx_gddr_count_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.gddr_count);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.gddr_count);
#endif
}

static ssize_t xdx_pcie_mmbar_type_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.pcie_mmbar_type);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.pcie_mmbar_type);
#endif
}

static ssize_t xdx_asci_type_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.asci_type);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.asci_type);
#endif
}

static ssize_t pwm1_min_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	IMG_UINT32 ret;
	
	xdev = dev_get_drvdata(dev);
	subcmd = cDDKPowerPWM;
	cmd_type = PWM_MIN_GET;

	ret = xdx_power_get(xdev, subcmd, cmd_type, 1);

	return sprintf(buf, "%i\n", ret);
}


static ssize_t pwm1_max_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	uint8_t subcmd, cmd_type;
	IMG_UINT32 ret;
	
	xdev = dev_get_drvdata(dev);
	subcmd = cDDKPowerPWM;
	cmd_type = PWM_MAX_GET;

	ret = xdx_power_get(xdev, subcmd, cmd_type, 1);

	return sprintf(buf, "%i\n", ret);
}

static ssize_t xdx_pcie_event_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.pcie_event);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.pcie_event);
#endif
}

static ssize_t xdx_gpu_primary_core_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.gpu_primary_core_id);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.gpu_primary_core_id);
#endif
}

static ssize_t xdx_gddr_temp_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", update_gddr_temp(xdev));
#else
	return sprintf(buf, "%u\n", update_gddr_temp(xdev));
#endif
}

static SENSOR_DEVICE_ATTR_RO(xdx_gddr_phys_freq_mhz, xdx_gddr_phys_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_acpu_freq_mhz, xdx_acpu_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_cmcu_freq_mhz, xdx_cmcu_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_ndf_freq_mhz, xdx_ndf_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_host_freq_mhz, xdx_host_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_gfx_freq_mhz, xdx_gfx_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_supported_gfx_freq_mhz, xdx_supported_gfx_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RW(temp1_ctrl_mode, temp1_ctrl_mode, 0);
static SENSOR_DEVICE_ATTR_RW(fan1_ctrl_mode, fan1_ctrl_mode, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_fw_ver, xdx_fw_ver, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_vbios, xdx_vbios, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_chip_id, xdx_chip_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gpu_core, xdx_gpu_core, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gpu_primary_id, xdx_gpu_primary_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gddr_count, xdx_gddr_count, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_pcie_mmbar_type, xdx_pcie_mmbar_type, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_asci_type, xdx_asci_type, 0);
static SENSOR_DEVICE_ATTR_RO(pwm1_min, pwm1_min, 0);
static SENSOR_DEVICE_ATTR_RO(pwm1_max, pwm1_max, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_pcie_event, xdx_pcie_event, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gpu_primary_core_id, xdx_gpu_primary_core_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gddr_temp, xdx_gddr_temp, 0);

static struct attribute * xdx_extra_attrs[] = {
	&sensor_dev_attr_xdx_gddr_phys_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_acpu_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_cmcu_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_ndf_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_host_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_gfx_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_supported_gfx_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_temp1_ctrl_mode.dev_attr.attr,
	&sensor_dev_attr_fan1_ctrl_mode.dev_attr.attr,
	&sensor_dev_attr_xdx_fw_ver.dev_attr.attr,
	&sensor_dev_attr_xdx_vbios.dev_attr.attr,
	&sensor_dev_attr_xdx_chip_id.dev_attr.attr,
	&sensor_dev_attr_xdx_gpu_core.dev_attr.attr,
	&sensor_dev_attr_xdx_gpu_primary_id.dev_attr.attr,
	&sensor_dev_attr_xdx_gddr_count.dev_attr.attr,
	&sensor_dev_attr_xdx_pcie_mmbar_type.dev_attr.attr,
	&sensor_dev_attr_xdx_asci_type.dev_attr.attr,
	&sensor_dev_attr_pwm1_min.dev_attr.attr,
	&sensor_dev_attr_pwm1_max.dev_attr.attr,
	&sensor_dev_attr_xdx_pcie_event.dev_attr.attr,
	&sensor_dev_attr_xdx_gpu_primary_core_id.dev_attr.attr,
	&sensor_dev_attr_xdx_gddr_temp.dev_attr.attr,
	NULL
};

static const struct attribute_group hwmon_attrgroup = {
	.attrs = xdx_extra_attrs,
};

static const struct attribute_group *hwmon_groups[] = {
	&hwmon_attrgroup,
	NULL
};

int xdxgpu_pm_sysfs_init(struct xdx_device *xdev)
{
	int ret;

	xdev->hwmon_dev = devm_hwmon_device_register_with_info(xdev->dev,
							 "xdxgpu",
							 xdev,
							 &xdxgpu_hwmon_chip_info,
							 hwmon_groups);
	if (IS_ERR(xdev->hwmon_dev)) {
		ret = PTR_ERR(xdev->hwmon_dev);
		dev_err(xdev->dev,
			"Unable to register hwmon device: %d\n", ret);
		return ret;
	}
	return 0;
}

void xdxgpu_pm_sysfs_fini(struct xdx_device *xdev)
{
	devm_hwmon_device_unregister(xdev->dev);
}
