#include "xdx_pm.h"
#include "xdx_device.h"
#include "xdx_irq.h"
#include "xdxthm_irq.h"
#include "osfunc.h"
#include "xdx_shared.h"
#include <linux/delay.h>

static int xdxthm_irq_set(struct xdx_device *xdev,
			   struct xdx_irq_src *src,
			   unsigned type,
			   enum xdx_interrupt_state state)
{
	// TODO: enable/disable IRQ
	return -1;
}

static u64 c2h0_doorbell_rreg64(void *reg_base, u32 off)
{
	u64 data;
	wmb();
	data = OSReadHWReg64(reg_base, off);
	mb();
	return data;
}

static void c2h0_doorbell_wreg64(void  *reg_base, u32 off, u64 value)
{
	wmb();
	OSWriteHWReg64(reg_base, off, value);
	mb();
}

void down_gfx(IMG_UINT32 clk, struct xdx_device *xdev) {
	int i;
	IMG_UINT32 tgt;

	if(clk <= 400000000)
		return;

	for(i=0; i<CLK_NUM; ++i) {
		if(xdx_gpu_freq_list[i] == clk) {
			tgt = xdx_gpu_freq_list[i+1];
			xdx_dvfs_set_freq(xdev, tgt);
			return;
		}
	}
}

static void down_temp(struct work_struct *para) {
	struct work_data *data = container_of(para, struct work_data, work);

	if(data->ddkTempCmd.temp.subCmd == TEMP_OVER_PROTECT) {
		IMG_UINT32 clk;
		clk = xdx_power_get(data->xdev, cDDKPowerDVFS, DVFS_GET_FREQ, 1)*1000000;

		data->temp1 = data->temp2;
		data->temp2 = xdx_power_get(data->xdev, cDDKPowerTemp, TEMP_GET, 0);
		if(data->temp2 < 75)
			data->temp2 = 75;

		if(data->temp1 == data->temp2) {
			if(data->temp2 > 75)
				down_gfx(clk, data->xdev);
		}
		else if(data->temp1 < data->temp2) {
			down_gfx(clk, data->xdev);
		}
		else if(data->temp1 > data->temp2) {
			if(data->temp2 == 75)
				xdx_dvfs_set_freq(data->xdev, data->gfx_default);
		}
	}
	else if(data->ddkTempCmd.temp.subCmd == TEMP_OVER_Fatal) {
		/* Need to be modified */
		/* 1. xdx_switch_power_state() cannot make GPU go back PS0 from PS2 */
		/* 2. when over 115C, GPU should be dealed by kernel thread, not work queue */
		int r;
		r = xdx_switch_power_state(data->xdev, XDX_PS2);
		if(r) {
			dev_err(data->xdev->dev, "failed to switch xgpu power state to PS%u\n", XDX_PS2);
		}
	}
}

static int xdxthm_irq_process(struct xdx_device *xdev,
			       struct xdx_irq_src *src,
			       struct xdx_irq_entry *entry)
{
	void __iomem * reg_base;
	uint64_t int_status;

	reg_base = (void __iomem *)xdev->thm.thm_rmmio;

	xdev->thm.data.ddkTempCmd.all = c2h0_doorbell_rreg64(reg_base, CMCU_DOORBELLS_DOORBELL46_OFFSET);
    c2h0_doorbell_wreg64(reg_base, CMCU_DOORBELLS_DOORBELL46_OFFSET,1);
	
	int_status = c2h0_doorbell_rreg64(reg_base, CMCU_DOORBELLS_C2H0_STATS_REGS_OFFSET);
	if(int_status != 0) {
		c2h0_doorbell_wreg64(reg_base, CMCU_DOORBELLS_C2H0_CLR_REGS_OFFSET, int_status);
	}

	xdev->thm.data.xdev = xdev;
	schedule_work(&xdev->thm.data.work);

	return 0;
}

static struct xdx_irq_src_funcs xdxthm_src_funcs = {
	.set = xdxthm_irq_set,
	.process = xdxthm_irq_process,
};

int xdxthm_isr_install(struct xdx_device *xdev)
{
	struct xdx_irq_src *irq_src;
	irq_src = kmalloc(sizeof(*irq_src), GFP_KERNEL);
	if (!irq_src)
		return -ENOMEM;

	irq_src->id = XDX_IRQ_CLIENTID_CMCU;
	irq_src->funcs = &xdxthm_src_funcs;

	xdx_irq_register_src(xdev, irq_src);

	xdev->thm.irq_src = irq_src;

	xdev->thm.data.temp1 = 75;
	xdev->thm.data.temp2 = 75;
	xdev->thm.data.gfx_default = xdev->fw_st_info.bootup_gpu_freq;

	INIT_WORK(&xdev->thm.data.work, down_temp);
	
	return 0;
}

int xdxthm_isr_uninstall(struct xdx_device *xdev)
{
	int ret;

	ret = xdx_irq_unregister_src(xdev, xdev->thm.irq_src);
	if (ret) {
		dev_err(xdev->dev, "failed to unregister irq source\n");
		return ret;
	}
	iounmap(xdev->thm.thm_rmmio);

	kfree(xdev->thm.irq_src);
	xdev->thm.irq_src = NULL;

	return 0;
}
