/* SPDX-License-Identifier: GPL-2.0 */
/*
 *    Hantro mmu driver header file.
 *
 *    Copyright (c) 2017, VeriSilicon Inc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License, version 2, as
 *    published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License version 2 for more details.
 *
 *    You may obtain a copy of the GNU General Public License
 *    Version 2 at the following locations:
 *    https://opensource.org/licenses/gpl-2.0.php
 */

#ifndef _HANTROMMU_H_
#define _HANTROMMU_H_

#include "hantro_priv.h"
#include "xdx_device.h"
#define REGION_IN_START 0x0
#define REGION_IN_END 0x40000000u
#define REGION_OUT_START 0x40000000u
#define REGION_OUT_END 0x80000000u
#define REGION_PRIVATE_START 0x80000000u
#define REGION_PRIVATE_END 0xc0000000u

#define REGION_IN_MMU_START 0x1000u
#define REGION_IN_MMU_END 0x40002000u
#define REGION_OUT_MMU_START 0x40002000u
#define REGION_OUT_MMU_END 0x80001000u
#define REGION_PRIVATE_MMU_START 0x80001000u
#define REGION_PRIVATE_MMU_END 0xc0000000u

enum MMUStatus {
	MMU_STATUS_OK = 0,

	MMU_STATUS_FALSE = -1,
	MMU_STATUS_INVALID_ARGUMENT = -2,
	MMU_STATUS_INVALID_OBJECT = -3,
	MMU_STATUS_OUT_OF_MEMORY = -4,
	MMU_STATUS_NOT_FOUND = -19,
};

struct addr_desc {
	void *virtual_address; /* buffer virtual address */
	unsigned long bus_address; /* buffer physical address */
	unsigned long mmu_bus_address;
	unsigned long size; /* physical size */
	unsigned int id; /*SLICE, TYPE*/
	unsigned int handle;
};

/* Init MMU, should be called in driver init function. */
int hantroMMUInit(struct xdx_device *xdev, int type);
int hantroMMUMap(struct xdx_device *xdev, struct xdx_bo *bo, unsigned long* vaddr, int type);
int hantroMMUprobe(dtbnode *pnode, int loop, struct mmu_t *pmmu,
		   struct xdx_device *xdev, unsigned long ddr_base, unsigned long reg_base);
int vpu_mmu_probe(dtbnode *pnode, int type, struct mmu_t *pmmu,
		   struct xdx_device *xdev, unsigned long ddr_base, unsigned long reg_base);
/* Clean up all data in MMU */
void hantroMMUCleanup(void);
/* The function should be called in driver realease function */
void hantroMMURelease(void *filp);
/* Memmap the address to hantro mmu */
void hantro_mmu_map(struct addr_desc *addr, int valid_bus_addr);
long hantroMMUIoctl(unsigned int cmd, struct file *filp, unsigned long arg);
/* hw init of MMU */
void MMUEnableStep1(unsigned char *hwregs, struct MMU *mmu);

#endif //#ifndef _HANTROMMU_H_
