#include "xdx_device.h"
#include "xdx_drv.h"
#include "xdx_audio.h"
#include "pangu.h"

//static int index[SNDRV_CARDS] = SNDRV_DEFAULT_IDX;     /* Index 0-MAX */
static char *id[SNDRV_CARDS] = SNDRV_DEFAULT_STR;	   /* ID for this card */
static bool enable[SNDRV_CARDS] = SNDRV_DEFAULT_ENABLE; /* Enable this card */

#define XDX_AUDIO_CARD_DEFAULT_ID                                3

int xdx_audio_init(struct xdx_device *xdev)
{
	struct xdx_dm *dm = xdev_to_dm(xdev);
	static int dev;
	struct snd_xdx_hdmi_audio *xdx0101;
	struct snd_card *card;
	int err;
	void __iomem *audio_rmmio;
	void __iomem *vdma1_rmmio;
	void __iomem *acpu_rmmio;
	void __iomem *dp0_rmmio;
	resource_size_t rmmio_audio_base, rmmio_vdma1_base, rmmio_acpu_base,  rmmio_dp0_base;
	resource_size_t rmmio_audio_size, rmmio_vdma1_size, rmmio_acpu_size,  rmmio_dp0_size;

	dm_to_output(dm);
	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		xdev->audio_pdev = pci_get_domain_bus_and_slot(pci_domain_nr(xdev->pdev->bus), xdev->pdev->bus->number,PCI_DEVFN(0, 1));
		if (xdev->audio_pdev == NULL) {
			pr_info("XDX_HDMI_AUDIO: get the audio device fail.\n");
			return -ENXIO;
		}
	}

	if (dev >= SNDRV_CARDS)
		return -ENODEV;

	if (!enable[dev]) {
		dev++;
		return -ENOENT;
	}

	err = snd_card_new(xdev->dev, XDX_AUDIO_CARD_DEFAULT_ID, id[dev], THIS_MODULE,
	                   0, &card);
	if (err < 0)
		return err;

	err = snd_xdx_hdmi_audio_create(card, xdev, &xdx0101);
	if (err < 0) {
		snd_card_free(card);
		return err;
	}

	if (!xdev) {
		pr_info("XDX_HDMI_AUDIO: get the pointer of xdev fail.\n");
		snd_card_free(card);
		return -1;
	}

	xdev->xdx_hdmi_audio_hotplug_ptr = xdx0101;
	xdx0101->xdev = xdev;

	rmmio_audio_base =  xdev->rmmio_base +  0xE00000;
	rmmio_audio_size = 0xFFFFF;
	audio_rmmio = ioremap(rmmio_audio_base, rmmio_audio_size);
	xdev->xdx_hdmi_audio_hotplug_ptr->iobase_audio0 = audio_rmmio;

	rmmio_vdma1_base =  xdev->rmmio_base +  0xD00000;
	rmmio_vdma1_size = 1 * 1024 *1024;
	vdma1_rmmio = ioremap(rmmio_vdma1_base, rmmio_vdma1_size);
	xdev->xdx_hdmi_audio_hotplug_ptr->iobase_vdma1 = vdma1_rmmio;

	rmmio_acpu_base =  xdev->rmmio_base +  0x0;
	rmmio_acpu_size = 0x1005e0;
	acpu_rmmio = ioremap(rmmio_acpu_base, rmmio_acpu_size);
	xdev->xdx_hdmi_audio_hotplug_ptr->iobase_acpu = acpu_rmmio;


	rmmio_dp0_base =  xdev->rmmio_base +  0x1770000;
	rmmio_dp0_size = 0x80000;
	dp0_rmmio = ioremap(rmmio_dp0_base, rmmio_dp0_size);
	xdev->xdx_hdmi_audio_hotplug_ptr->iobase_dp0 = dp0_rmmio;

	if (xdev->shareram_size == XDX_AUDIO_BAR4_1M) {
		err = xdxaudio_isr_install(xdev);
		if (err != 0)
			return err;
	}

#if 0
#if defined(SUPPORT_PANGU_DC_HDMI)
	if (!hdmi) {
		pr_info("XDX_HDMI_AUDIO: get the pointer of HDMI fail.\n");
	    snd_card_free(card);

		return -1;
	}
	xdx0101->hdmi = hdmi;
#else
	if (!dp) {
		pr_info("XDX_HDMI_AUDIO: get the pointer of DP fail.\n");
		snd_card_free(card);

		return -1;
	}
	xdx0101->dp = dp;
#endif
#endif
	strcpy(card->driver, "xdxgpu_vaduio");
	strcpy(card->shortname, "xdx_vaudio");
	sprintf(card->longname, "%s[0x%04x:0x%04x] ", card->shortname, XDX_VAUDIO_VENDOR, XDX_VAUDIO_DEVICE);
	
	pr_info("XDX_HDMI_AUDIO: %s\n", card->longname);
	err = snd_card_register(card);
	if (err < 0) {
		snd_card_free(card);
		return err;
	}
	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M)
		pci_set_drvdata(xdev->audio_pdev, card);
	dev++;
	return 0;
}


int xdx_audio_fini(struct xdx_device *xdev)
{
	struct snd_card *card;

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		xdev->audio_pdev = pci_get_domain_bus_and_slot(pci_domain_nr(xdev->pdev->bus),xdev->pdev->bus->number,PCI_DEVFN(0, 1));
		if (xdev->audio_pdev == NULL)
			return 0;
		card = pci_get_drvdata(xdev->audio_pdev);
		snd_card_free(card);
	} else
		xdxaudio_isr_uninstall(xdev);

	pr_info("XDX_HDMI_AUDIO fini.\n");
	return 0;
}
