/*
 * Copyright (c) 2016 Synopsys, Inc.
 *
 * Synopsys DP TX Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 *
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 *
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "xdx_dp.h"
#if((LINUX_VERSION_CODE>=KERNEL_VERSION(5,18,0))&&(LINUX_VERSION_CODE<KERNEL_VERSION(5,19,0)))
#include <drm/dp/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif
#include <drm/drm_fixed.h>
#include "reg.h"
#include "avgen.h"
#include "xdx_dp_accel.h"
#include "dm.h"
#include "xdx_device.h"
//#include <asm/fpu/api.h>

u8 dptx_bit_field(const u16 data, u8 shift, u8 width)
{
	return ((data >> shift) & ((((u16)1) << width) - 1));
}

u16 dptx_concat_bits(u8 bhi, u8 ohi, u8 nhi, u8 blo, u8 olo, u8 nlo)
{
	return (dptx_bit_field(bhi, ohi, nhi) << nlo) |
		dptx_bit_field(blo, olo, nlo);
}

u16 dptx_byte_to_word(const u8 hi, const u8 lo)
{
	return dptx_concat_bits(hi, 0, 8, lo, 0, 8);
}

u32 dptx_byte_to_dword(u8 b3, u8 b2, u8 b1, u8 b0)
{
	u32 retval = 0;

	retval |= b0 << (0 * 8);
	retval |= b1 << (1 * 8);
	retval |= b2 << (2 * 8);
	retval |= b3 << (3 * 8);
	return retval;
}

int dptx_dtd_parse(struct dptx *dptx, struct dtd *mdtd, u8 data[18])
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	mdtd->pixel_repetition_input = 0;

	mdtd->pixel_clock = dptx_byte_to_word(data[1], data[0]);
	if (mdtd->pixel_clock < 0x01)
		return -EINVAL;

	mdtd->h_active = dptx_concat_bits(data[4], 4, 4, data[2], 0, 8);
	mdtd->h_blanking = dptx_concat_bits(data[4], 0, 4, data[3], 0, 8);
	mdtd->h_sync_offset = dptx_concat_bits(data[11], 6, 2, data[8], 0, 8);
	mdtd->h_sync_pulse_width = dptx_concat_bits(data[11], 4, 2, data[9],
							0, 8);
	mdtd->h_image_size = dptx_concat_bits(data[14], 4, 4, data[12], 0, 8);

	mdtd->v_active = dptx_concat_bits(data[7], 4, 4, data[5], 0, 8);
	mdtd->v_blanking = dptx_concat_bits(data[7], 0, 4, data[6], 0, 8);
	mdtd->v_sync_offset = dptx_concat_bits(data[11], 2, 2, data[10], 4, 4);
	mdtd->v_sync_pulse_width = dptx_concat_bits(data[11], 0, 2,
							data[10], 0, 4);
	mdtd->v_image_size = dptx_concat_bits(data[14], 0, 4, data[13], 0, 8);
	if (dptx_bit_field(data[17], 4, 1) != 1)
		return -EINVAL;
	if (dptx_bit_field(data[17], 3, 1) != 1)
		return -EINVAL;

	mdtd->interlaced = dptx_bit_field(data[17], 7, 1) == 1;
	mdtd->v_sync_polarity = dptx_bit_field(data[17], 2, 1) == 0;
	mdtd->h_sync_polarity = dptx_bit_field(data[17], 1, 1) == 0;
	if (mdtd->interlaced == 1)
		mdtd->v_active /= 2;
	dev_dbg(xdev->dev, "DTD pixel_clock: %d interlaced: %d\n",
		 mdtd->pixel_clock, mdtd->interlaced);
	dev_dbg(xdev->dev, "h_active: %d h_blanking: %d h_sync_offset: %d\n",
		 mdtd->h_active, mdtd->h_blanking, mdtd->h_sync_offset);
	dev_dbg(xdev->dev, "h_sync_pulse_width: %d h_image_size: %d h_sync_polarity: %d\n",
		 mdtd->h_sync_pulse_width, mdtd->h_image_size,
		 mdtd->h_sync_polarity);
	dev_dbg(xdev->dev, "v_active: %d v_blanking: %d v_sync_offset: %d\n",
		 mdtd->v_active, mdtd->v_blanking, mdtd->v_sync_offset);
	dev_dbg(xdev->dev, "v_sync_pulse_width: %d v_image_size: %d v_sync_polarity: %d\n",
		 mdtd->v_sync_pulse_width, mdtd->v_image_size,
		 mdtd->v_sync_polarity);
	mdtd->pixel_clock *= 10;
	return 0;
}

void dptx_video_reset(struct dptx *dptx, int enable, int stream)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 reg;

	dev_dbg(xdev->dev, "%s \n", __func__);
	reg = dptx_readl(dptx, DPTX_SRST_CTRL);

	if (enable)
		reg |= DPTX_SRST_VIDEO_RESET_N(stream);
	else
		reg &= ~DPTX_SRST_VIDEO_RESET_N(stream);

	dptx_writel(dptx, DPTX_SRST_CTRL, reg);
}

/*
 * Video Generation
 */
#if 0
void dptx_video_timing_change(struct dptx *dptx, int stream)
{
	dev_dbg(xdev->dev, "%s \n", __func__);
	dptx_disable_default_video_stream(dptx, stream);
	//if(dptx->dsc)
	//	dptx_dsc_enable(dptx);
	/*
	int i = 0;
	for (i = 0; i < 32; i++)
	{
		u32 pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, i));
		dev_err(xdev->dev, "DSC_ PPS[%d] = %x\n", i, pps);
	}
	*/
	dptx_video_core_config(dptx, stream);
	dptx_video_ts_change(dptx, stream);
	dptx_enable_default_video_stream(dptx, stream);
	dptx_video_gen_config(dptx, stream);
	dptx_writel(dptx, DPTX_VG_SWRST_N(stream), 0);
}

int dptx_video_mode_change(struct dptx *dptx, u8 vmode, int stream)
{
	int retval;
	struct video_params *vparams;
	struct dtd mdtd;

	vparams = &dptx->vparams;
	if (!dptx_dtd_fill(&mdtd, vmode, vparams->refresh_rate,
				vparams->video_format)) {
		dev_dbg(xdev->dev, "%s: Invalid video mode value %d\n",
			 __func__, vmode);
		return -EINVAL;
	}
	retval = dptx_video_ts_calculate(dptx, dptx->link.lanes,
					 dptx->link.rate, vparams->bpc,
					 vparams->pix_enc, mdtd.pixel_clock);
	if (retval)
		return retval;
	vparams->mdtd = mdtd;
	vparams->mode = vmode;

	/* MMCM */
	dptx_video_reset(dptx, 1, stream);
	retval = dptx_video_pixel_freq_change(dptx, mdtd.pixel_clock, stream);
	if (retval) {
		dptx_video_reset(dptx, 0, stream);
		return retval;
	}
	dptx_video_reset(dptx, 0, stream);
	dptx_video_timing_change(dptx, stream);
	dev_dbg(xdev->dev, "%s: Change video mode to %d\n",
		 __func__, vmode);

	return retval;
}
#endif

int dptx_video_config(struct dptx *dptx, int stream)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;
	struct dtd *mdtd;
	int retVal;

	vparams = &dptx->vparams;
	mdtd = &vparams->mdtd;

	dev_dbg(xdev->dev, "%s mode:%d refresh_rate:%d video_format:%d\n", __func__,vparams->mode,\
			vparams->refresh_rate, vparams->video_format);
#if 0
	if (!dptx_dtd_fill(mdtd, vparams->mode, vparams->refresh_rate, vparams->video_format)) {
		dev_dbg(xdev->dev,"dtd fill fail\n");
		return -EINVAL;
	}
#endif


	retVal = dptx_video_ts_calculate(dptx, dptx->link.lanes, dptx->link.rate, vparams->bpc,
							vparams->pix_enc, mdtd->pixel_clock);
	if (retVal) {
		dev_dbg(xdev->dev, "dptx video ts calculate error, retVal = %d\n", retVal);
		return retVal;
	}
//	if(dptx->dsc) {
//		dptx_dsc_enable(dptx);
//			for (i = 0; i < 32; i++)
//		{
//			u32 pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, i));
//			dev_err(xdev->dev, "DSC_ PPS[%d] = %x\n", i, pps);
//		}
//	}
	dptx_video_core_config(dptx, stream);
	//dptx_video_gen_config(dptx, stream);

	return retVal;
}

void dptx_video_gen_config(struct dptx *dptx, int stream)
{
	u32 reg = 0;
	struct video_params *vparams;
	struct dtd *mdtd;
	u8 vmode;

	vparams = &dptx->vparams;
	mdtd = &vparams->mdtd;
	vmode = vparams->mode;

	reg |= DPTX_VG_CONFIG1_ODE_POL_EN;

	/* Configure video polarity	*/
	if (mdtd->h_sync_polarity == 1)
		reg |= DPTX_VG_CONFIG1_OH_SYNC_POL_EN;

	if (mdtd->v_sync_polarity == 1)
		reg |= DPTX_VG_CONFIG1_OV_SYNC_POL_EN;

	/* Configure Interlaced or Prograssive video  */
	if (mdtd->interlaced == 1)
		reg |= DPTX_VG_CONFIG1_OIP_EN;

	/* Configure BLANK  */

	if (vparams->video_format == VCEA) {
		if (vmode == 5 || vmode == 6 || vmode == 7 ||
			vmode == 10 || vmode == 11 || vmode == 20 ||
			vmode == 21 || vmode == 22 || vmode == 39 ||
			vmode == 25 || vmode == 26 || vmode == 40 ||
			vmode == 44 || vmode == 45 || vmode == 46 ||
			vmode == 50 || vmode == 51 || vmode == 54 ||
			vmode == 55 || vmode == 58 || vmode == 59)
			reg |= DPTX_VG_CONFIG1_BLANK_IN_OSC_EN;
	}

	/* Single, dual, or quad pixel */
	reg &= ~DPTX_VG_CONFIG1_MULTI_PIXEL_MASK;
	reg |= dptx->multipixel << DPTX_VG_CONFIG1_MULTI_PIXEL_SHIFT;
	dptx_writel(dptx, DPTX_VG_CONFIG1_N(stream), reg);

	dptx_video_ycc_mapping_change(dptx, stream);
	dptx_video_set_gen_bpc(dptx, stream);
	dptx_video_pattern_change(dptx, stream);

	/* Configure video_gen2 register */

	reg = 0;
	if(vparams->pix_enc == YCBCR420) {
		 reg |= (mdtd->h_active/2) << DPTX_VG_CONFIG2_H_ACTIVE_SHIFT;
		 reg |= (mdtd->h_blanking/2) << DPTX_VG_CONFIG2_H_BLANK_SHIFT;
	} else {
		 reg |= mdtd->h_active << DPTX_VG_CONFIG2_H_ACTIVE_SHIFT;
		 reg |= mdtd->h_blanking << DPTX_VG_CONFIG2_H_BLANK_SHIFT;
	}

	dptx_writel(dptx, DPTX_VG_CONFIG2_N(stream), reg);

	/* Configure video_gen3 register */
	reg = 0;
	reg |= mdtd->h_sync_offset << DPTX_VIDEO_H_FRONT_PORCH;
	reg |= mdtd->h_sync_pulse_width << DPTX_VIDEO_H_SYNC_WIDTH;
	dptx_writel(dptx, DPTX_VG_CONFIG3_N(stream), reg);

	/* Configure video_gen4 register */
	reg = 0;
	reg |= mdtd->v_active << DPTX_VIDEO_V_ACTIVE_SHIFT;
	reg |= mdtd->v_blanking << DPTX_VIDEO_V_BLANK_SHIFT;
	dptx_writel(dptx, DPTX_VG_CONFIG4_N(stream), reg);

	/* Configure video_gen5 register */
	reg = 0;
	reg |= mdtd->v_sync_offset << DPTX_VIDEO_V_FRONT_PORCH;
	reg |= mdtd->v_sync_pulse_width << DPTX_VIDEO_V_SYNC_WIDTH;
	dptx_writel(dptx, DPTX_VG_CONFIG5_N(stream), reg);

	dptx_enable_default_video_stream(dptx, 0);
}

void dptx_video_ycc_mapping_change(struct dptx *dptx, int stream)
{
	u32 reg = 0;
	enum pixel_enc_type pix_enc;
	struct video_params *vparams;

	vparams = &dptx->vparams;
	pix_enc = vparams->pix_enc;
	reg = dptx_readl(dptx, DPTX_VG_CONFIG1_N(stream));

	if (pix_enc == YCBCR420 || pix_enc  == YCBCR422) {
		reg |= DPTX_VG_CONFIG1_YCC_PATTERN_GEN_EN;
		if (pix_enc == YCBCR420) {
		//	reg |= DPTX_VG_CONFIG1_YCC_420_EN; //sahakyan
			reg &= ~DPTX_VG_CONFIG1_YCC_422_EN;
		} else if (pix_enc == YCBCR422) {
			reg |= DPTX_VG_CONFIG1_YCC_422_EN;
			reg &= ~DPTX_VG_CONFIG1_YCC_420_EN;
		}
	} else {
		reg &= ~DPTX_VG_CONFIG1_YCC_PATTERN_GEN_EN;
		reg &= ~DPTX_VG_CONFIG1_YCC_420_EN;
		reg &= ~DPTX_VG_CONFIG1_YCC_422_EN;
	}
	dptx_writel(dptx, DPTX_VG_CONFIG1_N(stream), reg);
}

int dptx_calculate_hblank_interval(struct dptx* dptx)
{
	struct video_params *vparams;
	int pixel_clk;
	u16 h_blank;
	u32 link_clk;
	u8 rate;
	int hblank_interval;

	if (!dptx->vparams.mdtd.pixel_clock)
		return -EINVAL;

	vparams = &dptx->vparams;
	pixel_clk = vparams->mdtd.pixel_clock;
	h_blank = vparams->mdtd.h_blanking;
	rate = dptx->link.rate;

	switch (rate) {
	case DPTX_PHYIF_CTRL_RATE_RBR:
		link_clk = 40500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR:
		link_clk = 67500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR2:
		link_clk = 135000;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR3:
		link_clk = 202500;
		break;
	default:
		WARN(1, "Invalid rate 0x%x\n", rate);
		return -EINVAL;
	}

	hblank_interval = (int)h_blank * (int)link_clk / pixel_clk;

	return hblank_interval;
}

#if 0
int dptx_sink_color_format_capabilities(struct dptx *dptx)
{
	int retval;
	u8 dsc_color_format;

	dev_dbg(xdev->dev, "%s \n", __func__);

	retval = dptx_read_dpcd(dptx, DP_DSC_DEC_COLOR_FORMAT_CAP,
				&dsc_color_format);
	if (retval) {
		dev_err(xdev->dev,"%s : DPCD read failed\n", __func__);
		//return retval;
	}

	if(dsc_color_format & DP_DSC_RGB)
		dev_dbg(xdev->dev, "%s: Sink supports RGB color format\n", __func__);

	if (dsc_color_format & DP_DSC_YCbCr444)
		dev_dbg(xdev->dev, "%s: Sink supports YCbCr 4:4:4 color format\n", __func__);

	if (dsc_color_format & DP_DSC_YCbCr422_Simple)
		dev_dbg(xdev->dev, "%s: Sink supports YCbCr 4:2:2 SIMPLE color format\n", __func__);

	if (dsc_color_format & DP_DSC_YCbCr422_Native)
		dev_dbg(xdev->dev, "%s: Sink supports YCbCr 4:2:2 NATIVE color format\n", __func__);

	if (dsc_color_format & DP_DSC_YCbCr420_Native)
		dev_dbg(xdev->dev, "%s: Sink supports YCbCr 4:2:0 NATIVE color format\n", __func__);

	return 0;
}

int dptx_sink_color_depth_capabilities(struct dptx *dptx)
{
	int retval;
	u8 dsc_color_depth;

	dev_dbg(xdev->dev, "%s \n", __func__);

	retval = dptx_read_dpcd(dptx, DP_DSC_DEC_COLOR_DEPTH_CAP,
				&dsc_color_depth);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
		//return retval;
	}

	if (dsc_color_depth & DP_DSC_8_BPC)
		dev_dbg(xdev->dev, "%s: Sink supports 8 bpc\n", __func__);

	if (dsc_color_depth & DP_DSC_10_BPC)
		dev_dbg(xdev->dev, "%s: Sink supports 10 bpc\n", __func__);

	if (dsc_color_depth & DP_DSC_12_BPC)
		dev_dbg(xdev->dev, "%s: Sink supports 12 bpc\n", __func__);

	return 0;
}

int dptx_get_slice_count(int ppr)
{
	printk("%s \n", __func__);

	if (ppr <= 340)
		return 1;

	if (ppr > 340 && ppr <= 680)
		return 2;

	if (ppr > 680 && ppr <= 1360)
		return 4;

	if (ppr > 1360 && ppr <= 3200)
		return 8;

	if (ppr > 3200 && ppr <= 4800)
		return 12;

	if (ppr > 4800 && ppr <= 6400)
		return 16;

	if (ppr > 6400 && ppr <= 8000)
		return 20;

	if (ppr > 8000 && ppr <= 9600)
		return 24;

	return -EINVAL;
}

bool dptx_slice_count_supported(struct dptx* dptx, int slice_count)
{
	u8 slice_cap1;
	u8 slice_cap2;
	int retval = 0;

	dev_dbg(xdev->dev, "%s \n", __func__);

	retval = dptx_read_dpcd(dptx, DP_DSC_SLICE_CAP_1, &slice_cap1);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed 1\n", __func__);
	}
	retval = dptx_read_dpcd(dptx, DP_DSC_SLICE_CAP_2, &slice_cap2);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed 2\n", __func__);
	}

	if ((slice_cap1 & DP_DSC_1_PER_DP_DSC_SINK) && (slice_count == 1))
		return true;

	if ((slice_cap1 & DP_DSC_2_PER_DP_DSC_SINK) && (slice_count == 2))
		return true;

	if ((slice_cap1 & DP_DSC_4_PER_DP_DSC_SINK) && (slice_count == 4))
		return true;

	if ((slice_cap1 & DP_DSC_6_PER_DP_DSC_SINK) && (slice_count == 6))
		return true;

	if ((slice_cap1 & DP_DSC_8_PER_DP_DSC_SINK) && (slice_count == 8))
		return true;

	if ((slice_cap1 & DP_DSC_10_PER_DP_DSC_SINK) && (slice_count == 10))
		return true;

	if ((slice_cap1 & DP_DSC_12_PER_DP_DSC_SINK) && (slice_count == 12))
		return true;

	if ((slice_cap2 & DP_DSC_16_PER_DP_DSC_SINK) && (slice_count == 16))
		return true;

	if ((slice_cap2 & DP_DSC_20_PER_DP_DSC_SINK) && (slice_count == 20))
		return true;

	if ((slice_cap2 & DP_DSC_24_PER_DP_DSC_SINK) && (slice_count == 24))
		return true;

	return false;
}

int  dptx_next_slice_count(struct dptx* dptx, int slice_count)
{
	dev_dbg(xdev->dev, "%s \n", __func__);

	if (slice_count == 24)
		return 1;

	if (slice_count == 1)
		slice_count = 2;
	else
		if (slice_count == 2)
			slice_count = 4;
		else
			slice_count += 4;

	return slice_count;
}

/*
 *  Get the slice width based on slice count, which is supported by the sink
 */
u16 dptx_get_slice_width(struct dptx* dptx, int* slice_count)
{
	struct video_params *vparams;
	u16 h_active;
	u8 max_slice_width;
	u16 slice_width;
	int ppr;
	int retval = 0;

	dev_dbg(xdev->dev, "%s \n", __func__);

	vparams = &dptx->vparams;
	ppr = vparams->mdtd.pixel_clock;
	h_active = vparams->mdtd.h_active;

	//TODO max_slice_width is u8, should be u16
	retval = dptx_read_dpcd(dptx, DP_DSC_MAX_SLICE_WIDTH, &max_slice_width);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}

	// Initialize slice width
	slice_width = 4096;

	if (h_active <= 4096)
		slice_width = 2048;

	if (slice_width > max_slice_width)
		slice_width = max_slice_width;

	// Adjust slice count and recalculate again, if slice width > max slice width
	slice_width = (h_active / (*slice_count));
	//while (slice_width > max_slice_width) {
	//	*slice_count = dptx_next_slice_count(dptx, *slice_count);
	//	slice_width = (h_active / *slice_count);
	//}

	return slice_width;
}

int dptx_dsc_encoders_count(struct dptx *dptx)
{
	struct video_params *vparams;
	int retval;
	int ppr;
//	u16 chunk_size;

	vparams = &dptx->vparams;
	ppr = vparams->mdtd.pixel_clock;

	dev_dbg(xdev->dev, "%s \n", __func__);
	retval = dptx_sink_color_format_capabilities(dptx);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}
		

	retval = dptx_sink_color_depth_capabilities(dptx);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}

	printk("%s ppr = %d\n",__func__, ppr);
	// Get the initial slice count based on ppr
//	vparams->encoders = dptx_get_slice_count(ppr/1000); //sahakyan
	vparams->encoders = 2;
	if (vparams->encoders < 0) {
		dev_err(xdev->dev, "Could not determine slice count based on PPR!!!\n");
		//return vparams->encoders;
	}

	dev_dbg(xdev->dev, "Initial slice count based on PPR is %d\n", vparams->encoders);

	vparams->slice_width = dptx_get_slice_width(dptx, &vparams->encoders);
	vparams->chunk_size = vparams->slice_width;

	dev_dbg(xdev->dev, "Slice count after calculating supported slice width is %d\n", vparams->encoders);

	//// Check the slice count is supported or not
	//while (!dptx_slice_count_supported(dptx, vparams->encoders)) {
	//	vparams->encoders = dptx_next_slice_count(dptx, vparams->encoders);
	//	vparams->slice_width = dptx_get_slice_width(dptx, &vparams->encoders);
	//}

	dev_dbg(xdev->dev, "%s: Slice count = %d, slice width = %d, vparams->chunk_size = %d\n",
		__func__, vparams->encoders, vparams->slice_width, vparams->chunk_size);

	return vparams->encoders;
}

void dptx_divide_pixel_clock(struct dptx* dptx)
{
	u32 reg;
	u8 pixels_per_pixelclk;
	int encoders = dptx->vparams.encoders;

	dev_dbg(xdev->dev, "%s \n", __func__);

	// Determine sampled pixel count based on pixel clock
	switch (dptx->multipixel)
	{
	case DPTX_MP_SINGLE_PIXEL:
		pixels_per_pixelclk = 1;
		break;
	case DPTX_MP_DUAL_PIXEL:
		pixels_per_pixelclk = 2;
		break;
	case DPTX_MP_QUAD_PIXEL:
		pixels_per_pixelclk = 4;
		break;
	default:
		break;
	}
	pixels_per_pixelclk = 2;
	// Program DSC_CTL.STREAMn_ENC_CLK_DIVIDED bit
	if (encoders > pixels_per_pixelclk) {
		// Divide pixel clock for DSC encoder
		reg = dptx_readl(dptx, DPTX_DSC_CTL);
		reg |= 1 << DPTX_DSC_STREAM0_ENC_CLK_DIV_SHIFT;
		dptx_writel(dptx, DPTX_DSC_CTL, reg);
	}
}

void dptx_pixel_mode_based_on_encoder_count(struct dptx *dptx)
{
	int encoders = dptx->vparams.encoders;
	u32 reg;
	u8 stream = 0; //TODO SST only

	dev_dbg(xdev->dev, "%s \n", __func__);

	// Change pixel mode based on encoders count
	switch (encoders)
	{
	case 8:
		dptx->multipixel = DPTX_MP_QUAD_PIXEL;
		break;
	case 4:
		dptx->multipixel = DPTX_MP_DUAL_PIXEL;
		//dptx->multipixel = DPTX_MP_QUAD_PIXEL;
		break;
	default:
		break;
	}

	/* Single, dual, or quad pixel */
	reg = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(stream));
	reg &= ~DPTX_VSAMPLE_CTRL_MULTI_PIXEL_MASK;
	reg |= dptx->multipixel << DPTX_VSAMPLE_CTRL_MULTI_PIXEL_SHIFT;
	dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(stream), reg);

	/* Divide pixel clock, if needed */
	dptx_divide_pixel_clock(dptx);
}
int dptx_mark_encoders(struct dptx *dptx)
{
	u8 available_encoders;
	u8 dsc_hwcfg;
	u32 dsc_ctl;
	int encoders;
//	int i;

	dev_dbg(xdev->dev, "%s \n", __func__);

	encoders = dptx->vparams.encoders;
	dsc_hwcfg = dptx_readl(dptx, DPTX_DSC_HWCFG);
	dsc_ctl = dptx_readl(dptx, DPTX_DSC_CTL);
	available_encoders = dsc_hwcfg & DPTX_DSC_NUM_ENC_MSK;

	dsc_ctl = dptx_readl(dptx, DPTX_DSC_CTL);  //sahakyan
	dsc_ctl |= 1<<22;
	dptx_writel(dptx, DPTX_DSC_CTL, dsc_ctl);
	
	dev_dbg(xdev->dev, "Calculated encoder count = %d\n", encoders);
	dev_dbg(xdev->dev, "Available encoders count = %d\n", available_encoders);

	//RAZ TODO check this case
	if (encoders > available_encoders) {
		dev_err(xdev->dev, "Encoder count is greather than available encoders\n");
		return -EINVAL;
	}

	//// Write stream number (0 for SST mode) for encoders to use
	//for (i = 1; i <= encoders; ++i) {
	//	dsc_ctl &= ~(1 << DPTX_DSC_ENC_STREAM_SEL_SHIFT(i - 1));
	//}

	return 0;
}
void dptx_program_dsc_version(struct dptx* dptx)
{
	u8 dsc_rev;
	u8 dsc_rev_min;
	u8 dsc_rev_maj;
	u32 pps;
	int retval = 0;

	retval = dptx_read_dpcd(dptx, DP_DSC_REV, &dsc_rev);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}

	/*dsc_rev_min = dsc_rev & DP_DSC_MINOR_MASK;
	dsc_rev_maj = (dsc_rev & DP_DSC_MAJOR_MASK) >> DP_DSC_MAJOR_SHIFT;*/

	/* Hardcode, DPCD not present */
	dsc_rev_min = 2;
	dsc_rev_maj = 1 >> DP_DSC_MAJOR_SHIFT;

	dev_dbg(xdev->dev, "%s  PPS dsc_rev = %d\n", __func__, dsc_rev);
	dev_dbg(xdev->dev, "%s  PPS dsc_rev_min = %d  dsc_rev_maj = %d\n", __func__, dsc_rev_min, dsc_rev_maj);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0));
	pps |= dsc_rev_min;
	pps |= (dsc_rev_maj << DPTX_DSC_VER_MAJ_SHIFT);
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0),pps);
}
void dptx_program_dsc_buf_bit_depth(struct dptx* dptx)
{
	u8 buf_bit_depth;
	u8 pps_buf_bit_depth;
	u32 pps;
	int retval = 0;

	retval = dptx_read_dpcd(dptx, DP_DSC_LINE_BUF_BIT_DEPTH, &buf_bit_depth);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}
	pps_buf_bit_depth = (buf_bit_depth & DP_DSC_LINE_BUF_BIT_DEPTH_MASK);
	
	dev_dbg(xdev->dev, "%s PPS pps_buf_bit_depth = %d\n", __func__, pps_buf_bit_depth);
	/* Hardcode, DPCD not present */
	pps_buf_bit_depth = 0xE;

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0));
	pps |= (pps_buf_bit_depth << DPTX_DSC_BUF_DEPTH_SHIFT);
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0), pps);
}

void dptx_program_dsc_block_prediction(struct dptx* dptx)
{
	u8 block_pred;
	u32 pps;
	int retval = 0;

	retval = dptx_read_dpcd(dptx, DP_DSC_BLK_PREDICTION_SUPPORT, &block_pred);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}
 
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1));
	//pps |= (block_pred & 1) << DPTX_DSC_BLOCK_PRED_SHIFT;
	// Hardcode, DPCD not present
	//pps |= (1 & 1) << DPTX_DSC_BLOCK_PRED_SHIFT;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1), pps);

	dev_dbg(xdev->dev, "%s PPS block_pred = %d \n", __func__, block_pred);
}

static u8 dptx_calculate_bpc(struct dptx* dptx)
{

	u8 bpc;
	u8 port_cap[4];
	int retval = 0;

	retval = dptx_read_bytes_from_dpcd(dptx, DP_DOWNSTREAM_PORT_0,
						port_cap,
						sizeof(port_cap));
	if (retval)
		return retval;

	bpc = port_cap[2] & DP_DS_MAX_BPC_MASK;

	pr_err("Sahakyan bpc = %x\n", bpc);
		switch (bpc) {
		case DP_DS_8BPC:
			return 8;
		case DP_DS_10BPC:
			return 10;
		case DP_DS_12BPC:
			return 12;
		case DP_DS_16BPC:
			return 16;
		default:
			return 0;
	}
}

//static u16 dptx_calculate_bpp(struct dptx* dptx)
//{
//	u8 bpc;
//	u8 bpp_high, bpp_low;
//	u16 bpp;
//	int retval = 0;


//	retval = dptx_read_dpcd(dptx, DP_DSC_MAX_BITS_PER_PIXEL_LOW, &bpp_low);
//	if (retval)
//		return retval;

//	retval = dptx_read_dpcd(dptx, DP_DSC_MAX_BITS_PER_PIXEL_HI, &bpp_high);
//	if (retval)
//		return retval;

//	bpp =  (bpp_low << 8) & bpp_high;

//	if(!bpp) {
//		bpc = dptx_calculate_bpc(dptx);
//		if(bpc > 0)
//			bpp = 3*bpc;
//	}
//
//	return bpp;
//
//}
void dptx_program_dsc_bits_perpixel(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;
	enum pixel_enc_type;// pix_enc;
	u8 bpp_high, bpp_low;
	u16 bpp;
	u32 pps;
	u32 bpp1;
 
	vparams = &dptx->vparams;
	bpp1=128; 

	bpp = DPTX_BITS_PER_PIXEL; // dptx_calculate_bpp(dptx);
	dptx->vparams.dsc_bpp = bpp;

	/* Get high and low parts of bpp (10 bits) */
	bpp_high = (bpp & DPTX_DSC_BPP_HIGH_MASK) >> 8;
	bpp_low = 128 & DPTX_DSC_BPP_LOW_MASK;

	pr_err("RAZ bpp_low = %x\n",bpp_low);
	pr_err("RAZ bpp_high = %x\n", bpp_high);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1));
	pps |= bpp_high;
	pps |= bpp_low <<8 ;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1), pps);

	dev_dbg(xdev->dev, "%s PPS  bpp = %d\n", __func__, bpp);
}
void dptx_program_dsc_bpc_and_depth(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;
	enum pixel_enc_type pix_enc;
	u8 bpc;
	u32 pps;


	vparams = &dptx->vparams;
	vparams->dsc_bpc =  dptx_calculate_bpc(dptx);
	bpc = vparams->dsc_bpc;
	pix_enc = vparams->pix_enc;

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0));
	switch (bpc)
	{
	case COLOR_DEPTH_8:
		pps |= bpc << DPTX_DSC_BPC_SHIFT;
		break;
	case COLOR_DEPTH_10:
		pps |= bpc << DPTX_DSC_BPC_SHIFT;
		break;
	case COLOR_DEPTH_12:
		pps |= bpc << DPTX_DSC_BPC_SHIFT;
		break;
	case COLOR_DEPTH_16:
		pps |= bpc << DPTX_DSC_BPC_SHIFT;
		break;
	default:
		dev_err(xdev->dev, "Unsupported Color depth by DSC spec\n");
		break;
	}

	dev_dbg(xdev->dev, "%s PPS bpc = %d\n", __func__, bpc);
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0), pps);

	switch (pix_enc)
	{
	case RGB:
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1));
		pps |= 1 << 4;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1), pps);
		break;
	case YCBCR420:
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 23));
		pps |= 1 << 1;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 23), pps);
		break;
	case YCBCR422:
		// TODO check with bala
		break;
	default:
		break;
	}
}

void dptx_program_dsc_slice_width(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 pps;
	struct video_params *vparams;

	dev_dbg(xdev->dev, "%s \n", __func__);
	vparams = &dptx->vparams;

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 3));
	pps |= dptx->vparams.slice_width >> 8;
	pps |= (dptx->vparams.slice_width & GENMASK(7, 0)) << 8;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 3), pps);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 3));
	pps |= (dptx->vparams.chunk_size >> 8) << 16;
	pps |= (dptx->vparams.chunk_size & GENMASK(7, 0)) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 3), pps);

	dev_dbg(xdev->dev, "%s PPS slice_width = %d\n", __func__, dptx->vparams.slice_width);
	dev_dbg(xdev->dev, "%s PPS chunk_size = %d\n", __func__, dptx->vparams.chunk_size);
	dev_dbg(xdev->dev, "%s PPS vbr_enable = %d\n", __func__, 0);

	//vbr_enable = 0
}


void dptx_program_dsc_pic_width_height(struct dptx* dptx)
{
	struct video_params *vparams;
	u32 pps;
	u32 pic_width;
	u32 pic_height;

	vparams = &dptx->vparams;
	pic_width = vparams->mdtd.h_active;
	pic_height = vparams->mdtd.v_active;

	printk("RAZ pic_height=%d\n", pic_height);
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 2));

		pps = 0;

	pps |= (pic_width >> 8);
	pps |= (pic_width & GENMASK(7,0)) << 8;
	printk("RAZ pps=0x%0x -pic_width", pps);
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 2), pps);


	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1));

	pps &= ~GENMASK(31,16);
	pps |= (pic_height >> 8) << 16;

	printk("RAZ pps=0x%0x -pic_height", pps);

	pps |= (pic_height & GENMASK(7,0)) << 24;

	printk("RAZ pps=0x%0x -pic_height", pps);

	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1), pps);

	dev_dbg(xdev->dev, "%s PPS pic_width = %d\n", __func__, pic_width);
	dev_dbg(xdev->dev, "%s PPS pic_height = %d\n", __func__, pic_height);
}


void dptx_program_dsc_slice_height(struct dptx* dptx)
{
	struct video_params *vparams;
//	u32 pps;
	u16 pic_height;
	u16 slice_height;
	u16 dsc_max_num_lines;
	u32 reg;
	u8 first_line_bpg_offset;
	u8 second_line_bpg_offset;

	dev_dbg(xdev->dev, "%s \n", __func__);

	reg = dptx_readl(dptx, DPTX_CONFIG_REG2);
	dsc_max_num_lines = (reg & DSC_MAX_NUM_LINES_MASK) >> DSC_MAX_NUM_LINES_SHIFT;

	dev_dbg(xdev->dev, "%s dsc_max_num_lines = %d\n", __func__, dsc_max_num_lines);

	vparams = &dptx->vparams;
	pic_height = vparams->mdtd.v_active;

	if(pic_height < dsc_max_num_lines)
		slice_height = pic_height;
	else {
		slice_height = pic_height;
		while (!(slice_height < dsc_max_num_lines))
		{
			slice_height = pic_height >> 2; // divide to 2
		}
	}

	dptx->vparams.slice_height = slice_height;
	dev_dbg(xdev->dev, "%s PPS slice_height = %d\n", __func__, slice_height);
	reg = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 2));
	reg |= (slice_height >> 8) << 16;
	reg |= (slice_height & GENMASK(7, 0)) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 2), reg);


	//Calculate first_line_bpg_offset based on slice height
	if (slice_height >= 8)
		first_line_bpg_offset = 12 +  ((9 * min(34, slice_height - 8)) / 100);
		//first_line_bpg_offset = 12 + min(34, slice_height - 8);
	else
		first_line_bpg_offset = 2 * (slice_height - 1);

	reg = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 6));
	reg |= first_line_bpg_offset << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 6), reg);

	dev_dbg(xdev->dev, "%s PPS first_line_bpg_offset = %d\n", __func__, first_line_bpg_offset);
	pr_err("RAZ first_line_bpg_offset = %d\n", first_line_bpg_offset);

	//Calculate second_line_bpg_offset based on slice height only in YcBcR420
	if (dptx->vparams.pix_enc == YCBCR420)
	{
		if (slice_height < 8)
			second_line_bpg_offset = 2 * (slice_height - 1);
	}
	else 
		// Generic value for second_line_bpg_offset
		second_line_bpg_offset = 12;
	
	reg = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 22));
	reg |= second_line_bpg_offset << 8;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 22), reg);

	dptx->vparams.first_line_bpg_offset = first_line_bpg_offset;
}

u8 dptx_dsc_groups_per_line(struct dptx* dptx)
{
	u8 groups_per_line;
	u16 slice_width;
	enum pixel_enc_type pix_enc;
	struct video_params *vparams;
	
	vparams = &dptx->vparams;
	slice_width = dptx->vparams.slice_width;
	pix_enc = vparams->pix_enc;
	
	pr_err("%s: RAZ slice_width = %d ",__func__, slice_width);
	if (pix_enc == RGB)
		groups_per_line = slice_width / PIXELS_PER_GROUP;
	else
		groups_per_line = (slice_width >> 1) / PIXELS_PER_GROUP;

	pr_err("%s: RAZ groups_per_line = %d ", __func__, groups_per_line);

	return groups_per_line;
}

void dptx_program_dsc_min_rate_bufsize(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;
	enum pixel_enc_type pix_enc;
	u32 minRateBufferSize, pps;
	u8 group_pl;
	u8 rc_buf_size;
	u8 rc_buf_block_size;
	u8 first_line_bpg_offset;
	u8 bpc;
	u16 bpp;
	u16 rc_model_size = RC_MODEL_SIZE;
	u16 initial_offset = INITIAL_OFFSET;
	u16 initial_delay = INITIAL_DELAY;
	int retval = 0;

	dev_dbg(xdev->dev, "%s \n", __func__);

	vparams = &dptx->vparams;
	bpc = vparams->dsc_bpc;
	bpp = vparams->dsc_bpp;
	pix_enc = vparams->pix_enc;
	first_line_bpg_offset = dptx->vparams.first_line_bpg_offset;
	group_pl = dptx_dsc_groups_per_line(dptx);

	retval = dptx_read_dpcd(dptx, DP_DSC_RC_BUF_SIZE, &rc_buf_size);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}

	retval = dptx_read_dpcd(dptx, DP_DSC_RC_BUF_BLK_SIZE, &rc_buf_block_size);
	if (retval) {
		dev_err(xdev->dev, "%s : DPCD read failed\n", __func__);
	}
	
	switch (pix_enc)
	{
	case RGB:
		minRateBufferSize = (rc_model_size - initial_offset) +
					(initial_delay * bpp) +
					(group_pl * first_line_bpg_offset);
		pr_err("RAZ minRateBufferSize=%d\n", minRateBufferSize);
		break;
	case YCBCR420:
		//TODO
		break;
	case YCBCR422:
		//TODO
		break;
	default:
		break;
	}

	
	//if (minRateBufferSize > rc_buf_size) {
	//	// TODO reduce silce height by half and continue from 5.12 step 8
	//	dev_err(xdev->dev, " Min rate buf size is greater than sinks rc buf size\n");
	//}
	//else {

		dev_dbg(xdev->dev, " PPS DSC minRateBufferSize is %d\n", minRateBufferSize);
		dev_dbg(xdev->dev, " PPS DSC rc_model_size is %d\n", rc_model_size);
		dev_dbg(xdev->dev, " PPS DSC initial_offset is %d\n", initial_offset);
		dev_dbg(xdev->dev, " PPS DSC initial_delay is %d\n", initial_delay);

		// Program rc_model_size pps
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 9));
		pps |= (rc_model_size >> 8) << 16;
		pps |= (rc_model_size & GENMASK(7,0)) << 24;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 9), pps);

		// Program initial_offset pps
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 8));
		pps |= ((initial_offset & GENMASK(15,8))>> 8);
		pps |= (initial_offset & GENMASK(7, 0)) << 8;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 8), pps);

		// Program initial_delay pps
		pr_err("initial_delay = %d\n", initial_delay);
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 4));
		pps |= ((initial_delay & GENMASK(9, 8)) >> 8);
		pps |= (initial_delay & GENMASK(7, 0)) << 8;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 4), pps);
	//}

	dptx->vparams.minRateBufferSize = minRateBufferSize;
}

u8 dptx_dsc_get_mux_word_size(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u8 bpc;
	u8 muxWordSize = 0;

	bpc = dptx->vparams.dsc_bpc;

	switch (bpc)
	{
	case 12:
	case 14:
	case 16:
		muxWordSize = 64;
		break;
	case 8:
	case 10:
		muxWordSize = 48;
		break;
	default:
		dev_err(xdev->dev, "Cant get muxWordSize based on bpc\n");
		break;
	}

	return muxWordSize;
}

void dptx_program_dsc_hrdelay(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;
	u32 minRateBufferSize;
	u8 muxWordSize, maxSeSize_Y, maxSeSize_C;
	u16 bpp;
	u8 bpc;
	u8 group_per_line;
	u32 pps;
	u32 hrddelay;
	u32 initial_dec_delay;
	u32 slice_bpg_offset;
	u16 initial_scale_value;
	u16 rc_model_size = RC_MODEL_SIZE;
	u16 initial_offset = INITIAL_OFFSET;
	u32 scale_decrement_interval;
	u32 scale_increment_interval;
	u32 numExtraMuxBits;
	int rcxformoffset;
	int final_offset;
	int final_scale_value;
	int nfl_bpg_offset;
	int groupsTotal;

	dev_dbg(xdev->dev, "%s \n", __func__);

	vparams = &dptx->vparams;
	minRateBufferSize = dptx->vparams.minRateBufferSize;
	bpp = dptx->vparams.dsc_bpp;
	bpc = dptx->vparams.dsc_bpc;

	 hrddelay = minRateBufferSize / bpp;

	dev_dbg(xdev->dev, "%s PPS hrddelay = %d\n", __func__, hrddelay);
	pr_err("RAZ hrddelay = %d\n", hrddelay);
	dptx->vparams.hrddelay = hrddelay;

	// Calculate initial_dec_delay and program PPS
	initial_dec_delay = hrddelay - INITIAL_DELAY;
	pr_err("RAZ initial_dec_delay = %d\n", initial_dec_delay);
	//initial_dec_delay = 658;
	dptx->vparams.initial_dec_delay = initial_dec_delay;

	dev_dbg(xdev->dev, "%s PPS initial_dec_delay = %d\n", __func__, initial_dec_delay);

	pr_err("RAZ initial_dec_delay = %d\n", initial_dec_delay);
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 4));
	pr_err("RAZ high part = %x\n", (initial_dec_delay >> 8));
	pr_err("RAZ low part = %lx\n", initial_dec_delay & GENMASK(7, 0));
	pps |= (initial_dec_delay >> 8) << 16;
	pps |= (initial_dec_delay & GENMASK(7, 0)) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 4), pps); //Bala changed 4 to 5

	// Calculate initial_scale_value and program PPS
	initial_scale_value = (rc_model_size / (rc_model_size - initial_offset));
	pr_err("RAZ initial_scale_value = %d (%x)\n", initial_scale_value, initial_scale_value);
	dptx->vparams.initial_scale_value = initial_scale_value;
	dev_dbg(xdev->dev, "%s PPS initial_scale_value = %d\n", __func__, initial_scale_value);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 5));
	pps |= (initial_scale_value << INITIAL_SCALE_VALUE_SHIFT)<< 8;
	//pps |= initial_scale_value ;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 5), pps); //Bala

	 //Calculate scale_decrement_interval and program PPS
	group_per_line = dptx_dsc_groups_per_line(dptx);

	pr_err("RAZ  group_per_line =%d\n", group_per_line);
	pr_err("RAZ  initial_scale_value =%d\n", initial_scale_value);
	

	scale_decrement_interval = group_per_line / (8 * (initial_scale_value - 1));
	dev_dbg(xdev->dev, "%s PPS scale_decrement_interval = %d\n", __func__, scale_decrement_interval);
	dptx->vparams.scale_decrement_interval = scale_decrement_interval;
	pr_err("RAZ  scale_decrement_interval =%d\n", scale_decrement_interval);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 6));
	pps |= (scale_decrement_interval & GENMASK(11,9)) >> 9;
	pps |= (scale_decrement_interval & GENMASK(7, 0)) << 8;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 6), pps);

	//Calculate scale_increment_interval and program PPS
	muxWordSize = dptx_dsc_get_mux_word_size(dptx);
	rcxformoffset = INITIAL_OFFSET - RC_MODEL_SIZE;
	pr_err("RAZ first_line_bpg_offset=%d\n", dptx->vparams.first_line_bpg_offset);
	pr_err("RAZ slice_height=%d\n", dptx->vparams.slice_height);
	
	nfl_bpg_offset = (dptx->vparams.first_line_bpg_offset << OFFSET_FRACTIONAL_BITS)/ (dptx->vparams.slice_height - 1);
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 7));
	pps |= ((nfl_bpg_offset & GENMASK(15, 8)) >> 8);
	pps |= ((nfl_bpg_offset & GENMASK(7, 0))) << 8;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 7), pps);
	pr_err("RAZ nfl_bpg_offset=%d\n", nfl_bpg_offset);
	
	if (bpc == 16) {
		maxSeSize_Y = maxSeSize_C = 64;
	}
	else {
		maxSeSize_Y = bpc * 4 + 4;
		maxSeSize_C = (bpc + 1) * 4; //1 is convert_rgb
	}

	pr_err("RAZ muxWordSize=%d\n", muxWordSize);
	pr_err("RAZ maxSeSize_Y=%d\n", maxSeSize_Y);
	pr_err("RAZ maxSeSize_C=%d\n", maxSeSize_C);

	numExtraMuxBits = (muxWordSize + maxSeSize_Y - 2) + 2 * (muxWordSize + maxSeSize_C - 2);
	pr_err("RAZ numExtraMuxBits = %d \n", numExtraMuxBits);
	final_offset = RC_MODEL_SIZE - INITIAL_DELAY * bpp + numExtraMuxBits;
	
	pr_err("RAZ  final_offset = %d\n", final_offset);
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 8));
	pps |= ((final_offset & GENMASK(15, 8)) >> 8) <<16;
	pps |= ((final_offset & GENMASK(7, 0))) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 8), pps);

	final_scale_value = RC_MODEL_SIZE / (RC_MODEL_SIZE - final_offset);
	pr_err("RAZ  final_scale_value = %d\n", final_scale_value);
	groupsTotal = group_per_line * dptx->vparams.slice_height;
	pr_err("RAZ groupsTotal=%d\n", groupsTotal);
	slice_bpg_offset = ((1 << OFFSET_FRACTIONAL_BITS) *(RC_MODEL_SIZE - INITIAL_OFFSET + numExtraMuxBits) / groupsTotal);
	pr_err("RAZ slice_bpg_offset=%d\n", slice_bpg_offset);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 7));
	pps |= ((slice_bpg_offset & GENMASK(15, 8)) >> 8) << 16;
	pps |= ((slice_bpg_offset & GENMASK(7, 0))) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 7), pps);

	scale_increment_interval = (final_offset / nfl_bpg_offset + slice_bpg_offset) / (8 * (final_scale_value -1));
	//pr_err("RAZ scale_increment_interval = %d\n", scale_increment_interval);
	scale_increment_interval = 11428;

	dev_dbg(xdev->dev,"%s PPS scale_increment_interval = %d\n", __func__, scale_increment_interval);

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 5));
	pps |= (scale_increment_interval >> 8) << 16;
	pps |= ((scale_increment_interval & GENMASK(7, 0))) << 24;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 5), pps); //Bala

	pr_err("RAZ scale_increment_interval = %x\n", scale_increment_interval);
	if (scale_increment_interval > 65535)
		dev_err(xdev->dev, "%s Scale increment interval exceeds 65535\n", __func__); // TODO check with Bala
}

void dptx_program_flatness_qp(struct dptx* dptx)
{
	u32 pps;

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 9));
	pps |= FLATNESS_MIN_QP;
	pps |= FLATNESS_MAX_QP << 8;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 9), pps);
}

/*
 *  Returns 2's complement representation of negative number
 *  Number should be 5 bits in this function
 *  Number passed as integer, for being able have negative values
 */
u8 dptx_get_twos_compl_rep(int number)
{
	u8 res;

	if (number >= 0)
		return number;

	number = -number;

	// Calculate 2's complement representation
	res = (number ^ 0x1f) + 1;

	// Set the sign bit before returning
	return (res | (1 << 5));
}

//static struct rc_range_param rc_range_params[15] = {
//		{0, 4, 2},
//		{0, 4, 0},
//		{1, 5, 0},
//		{1, 6, -2},
//		{3, 7, -4},
//		{3, 7, -6},
//		{3, 7, -8},
//		{3, 8, -8},
//		{3, 9, -8},
//		{3, 10, -10},
//		{5, 10, -10},
//		{5, 11, -12},
//		{5, 11, -12},
//		{9, 12, -12},
//		{12, 13, -12},
//};

void dptx_program_rc_range_parameters(struct dptx* dptx)
{
	u32 pps;
	int i;
	int pps_index;

	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, (14)));
	pps |= ((rc_range_params[0].maxQP & GENMASK(4, 2)) >> 2) << 16;
	pps |= rc_range_params[0].minQP << 19;
	pps |= dptx_get_twos_compl_rep(rc_range_params[0].offset) << 24;
	pps |= (rc_range_params[0].maxQP & GENMASK(1, 0)) << 29;
	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 14), pps);

	// Read once 4 byte and program in there 2  rc_range_parameters
	for (i = 1, pps_index = 60; i < 15; i+=2, pps_index+=4)
	{
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, (pps_index / 4)));
		pps |= ((rc_range_params[i].maxQP & GENMASK(4, 2)) >> 2);
		pps |= (rc_range_params[i].minQP) << 3;
		pps |= dptx_get_twos_compl_rep(rc_range_params[i].offset) << 8;
		pps |= (rc_range_params[i].maxQP & GENMASK(1, 0)) << 14 ;

		pps |= ((rc_range_params[i+1].maxQP & GENMASK(4, 2)) >> 2) << 16;
		pps |= (rc_range_params[i+1].minQP) << 19;
		pps |= dptx_get_twos_compl_rep(rc_range_params[i+1].offset) << 24;
		pps |= (rc_range_params[i+1].maxQP & GENMASK(1, 0)) << 30;

		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, pps_index / 4), pps);
	}
}
static u32 RC_BUF_THRESHOLD[14] = {
		896,1792,2688,3584,4480,5376,6272,6720,7168,7616,7744,7872,8000,8064,
};

void dptx_program_rc_parameter_set(struct dptx* dptx)
{
	u32 pps;
	int i;
	int pps_index;

	// rc_edge_factor
	pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 10));
	pps |= RC_EDGE_FACTOR;

	// rc_quant_incr_limit0
	pps |= RC_QUANT_INCR_LIMIT0 << 8;

	// rc_quant_incr_limit1
	pps |= RC_QUANT_INCR_LIMIT1 << 16;

	// rc_tgt_offset_hi
	pps |= RC_TGT_OFFSET_HIGH << 28;

	// rc_tgt_offset_lo
	pps |= RC_TGT_OFFSET_LOW << 24;

	dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 10), pps);

	// PPS44 - PPS57 rc_buf_threshold values
	for (i = 0, pps_index = 44; i < 14; ++i, ++pps_index)
	{
		pps = dptx_readl(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, (pps_index / 4)));
		pr_err("RAZ %d = %x\n", RC_BUF_THRESHOLD[i], RC_BUF_THRESHOLD[i] >> 6);
		pps |= (RC_BUF_THRESHOLD[i] >> 6) << (pps_index % 4) * 8;
		dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, pps_index / 4), pps);
	}

	// RC_RANGE_PARAMETERS from DSC 1.2 spec
	dptx_program_rc_range_parameters(dptx);
}
void dptx_program_pps_sdps(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	dev_dbg(xdev->dev, "%s \n", __func__);

	dptx_program_dsc_version(dptx);
	dptx_program_dsc_buf_bit_depth(dptx);
	dptx_program_dsc_block_prediction(dptx);
	dptx_program_dsc_bits_perpixel(dptx);
	dptx_program_dsc_bpc_and_depth(dptx);
	dptx_program_dsc_slice_width(dptx);
	dptx_program_dsc_pic_width_height(dptx);
	dptx_program_dsc_slice_height(dptx);
	dptx_program_dsc_min_rate_bufsize(dptx);
	dptx_program_dsc_hrdelay(dptx);
	dptx_program_flatness_qp(dptx);
	dptx_program_rc_parameter_set(dptx);

	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 0), 0x8e000012);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 1), 0xe0018010);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 2), 0xe0018002);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 3), 0x80028002);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 4), 0x92020002);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 5), 0xa42c2000);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 6), 0x0f000800);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 7), 0x2e004100);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 8), 0xf0100018);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 9), 0x00200c03);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 10), 0x330b0b06);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 11), 0x382a1c0e);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 12), 0x69625446);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 13), 0x7b797770);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 14), 0x02017e7d);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 15), 0x40090001);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 16), 0xfc19be09);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 17), 0xf819fa19);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 18), 0x781a381a);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 19), 0x62ab61a);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 20), 0xf42af42a);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 21), 0x7463344b);
	//dptx_writel(dptx, DPTX_DSC_PPS(DPTX_SST_MODE, 22), 0x00000000);
}
#endif

void dptx_calc_dsc_lsteer_xmit_delay(struct dptx* dptx)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int encoder_delay;
	u32 mux_word_size;
	u32 muxer_initial_delay;
	u32 reg;
	u16 h_active;
	u16 h_blanking;
	int lsteer_xmit_delay;
	u8 horizontal_slices;
	u8 vertical_slices;
	u8 bpc;
	u8 multipixel;

	bpc = dptx->vparams.dsc_bpc;
	mux_word_size = (bpc < 12) ? 48 : 64;
	muxer_initial_delay = (mux_word_size + (4 * bpc + 4) - 3 + 32) * 3;
	horizontal_slices = dptx->vparams.encoders;
	multipixel = dptx->multipixel;
	h_active = dptx->vparams.mdtd.h_active;
	h_blanking = dptx->vparams.mdtd.h_blanking;
	vertical_slices = horizontal_slices; // TODO check with Bala

	encoder_delay = (((PIXEL_HOLD_DELAY + PIXEL_FLATNESSBUF_DELAY +
					PIXEL_GROUP_DELAY + muxer_initial_delay +
					MUXER_INITIAL_BUFFERING_DELAY) * horizontal_slices)) /
					(1 << multipixel);

	lsteer_xmit_delay = encoder_delay + (((h_blanking + h_active) * vertical_slices) +
		h_blanking + h_active +  (INITIAL_DELAY * horizontal_slices)) / (1 << multipixel);

	dev_dbg(xdev->dev, "%s muxer inital delay  = %d\n", __func__, muxer_initial_delay);
	dev_dbg(xdev->dev, "%s DSC encoder delay = %d\n", __func__, encoder_delay);
	dev_dbg(xdev->dev, "%s DSC XMIT delay = %d\n", __func__, lsteer_xmit_delay);

//	switch (multipixel)
//	{
//	case DPTX_MP_DUAL_PIXEL:
//		lsteer_xmit_delay /= 2;
//		break;
//	case DPTX_MP_QUAD_PIXEL:
//		lsteer_xmit_delay /= 4;
//		break;
//	default:
//		break;
//	}
//	
	//lsteer_xmit_delay = 2492;
	lsteer_xmit_delay = 1692;
	dev_dbg(xdev->dev, "%s DSC XMIT delay = %d\n", __func__, lsteer_xmit_delay);

	reg = dptx_readl(dptx, DPTX_VIDEO_DSCCFG);
	reg &=~DPTX_DSC_LSTEER_XMIT_DELAY_MASK;
	reg &= ~DPTX_DSC_LSTEER_FRAC_SHIFT_MASK;
	reg |= lsteer_xmit_delay << DPTX_DSC_LSTEER_XMIT_DELAY_SHIFT;
	dev_dbg(xdev->dev, "%s DSCCFG is  = %x\n", __func__, reg);
	dptx_writel(dptx, DPTX_VIDEO_DSCCFG, reg);
}

#if 0
void dptx_dsc_compress_video(struct dptx* dptx)
{
	u32 reg;
	int encoders;
	u16 bpp;
	u32 wait_cnt_int;
	u32 wait_cnt_frac;
	s64 fixp;

	bpp = dptx->vparams.dsc_bpp;
	encoders = dptx->vparams.encoders;

	// Get the integer part
	fixp = drm_fixp_from_fraction(128, (bpp * encoders));
	wait_cnt_int = drm_fixp2int(fixp);
	//wait_cnt_int = 16;

	// Get the fractional part
	fixp &= DRM_FIXED_DECIMAL_MASK;
	fixp *= 64;
	wait_cnt_frac = drm_fixp2int(fixp);
	wait_cnt_frac = 0;

	dev_dbg(xdev->dev, "%s wait_cnt_int = %u, wait_cnt_frac = %u\n", 
		__func__, wait_cnt_int, wait_cnt_frac);

	reg = dptx_readl(dptx, DPTX_VIDEO_DSCCFG);
	reg &=~DPTX_DSC_LSTEER_INT_SHIFT_MASK;
	reg |= wait_cnt_int << DPTX_DSC_LSTEER_INT_SHIFT;
	reg |= wait_cnt_frac << DPTX_DSC_LSTEER_FRAC_SHIFT;
	reg &=~DPTX_DSC_LSTEER_XMIT_DELAY_MASK;
	dptx_writel(dptx, DPTX_VIDEO_DSCCFG, reg);

	dptx_calc_dsc_lsteer_xmit_delay(dptx);
	
	/* Program pps repeat count */
	//reg = dptx_readl(dptx, DPTX_DSC_CTL);
	//reg |= 3 << 16;
	//dptx_writel(dptx, DPTX_DSC_CTL, reg);

	/* Enable compression */
	reg = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(0));
	reg |= DPTX_VSAMPLE_CTRL_ENABLE_DSC;
	//dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(0), reg); BB

}

void dptx_dsc_enable(struct dptx* dptx)
{
	int encoders = 0;

	dev_dbg(xdev->dev, "%s\n",__func__);
	encoders = dptx_dsc_encoders_count(dptx);
	dptx_mark_encoders(dptx);

	// Apply soft reset - stream 0 for SST mode
	dptx_soft_reset(dptx, DPTX_SRST_VIDEO_RESET_N(0));

	dptx_pixel_mode_based_on_encoder_count(dptx);
	dptx_program_pps_sdps(dptx);
	dptx_dsc_compress_video(dptx);
	

}
#endif
void dptx_video_core_config(struct dptx *dptx, int stream)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	u32 reg = 0;
	u8 vmode;

	struct video_params *vparams;
	struct dtd *mdtd;

	vparams = &dptx->vparams;
	mdtd = &vparams->mdtd;
	vmode = vparams->mode;

	dptx_video_set_core_bpc(dptx, stream);

	/* Single, dual, or quad pixel */
	reg = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(stream));
	reg &= ~DPTX_VSAMPLE_CTRL_MULTI_PIXEL_MASK;
	reg |= dptx->multipixel << DPTX_VSAMPLE_CTRL_MULTI_PIXEL_SHIFT;
	dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(stream), reg);

	/* Configure DPTX_VSAMPLE_POLARITY_CTRL register */
	reg = 0;

	if (mdtd->h_sync_polarity == 1)
		reg |= DPTX_POL_CTRL_H_SYNC_POL_EN;
	if (mdtd->v_sync_polarity == 1)
		reg |= DPTX_POL_CTRL_V_SYNC_POL_EN;
	dev_dbg(xdev->dev,"dp hsync pol:%d vsync pol:%d\n",mdtd->h_sync_polarity,mdtd->v_sync_polarity);
	dptx_writel(dptx, DPTX_VSAMPLE_POLARITY_CTRL_N(stream), reg);

	reg = 0;

	/* Configure video_config1 register */
	if (vparams->video_format == VCEA) {
		if (vmode == 5 || vmode == 6 || vmode == 7 ||
			vmode == 10 || vmode == 11 || vmode == 20 ||
			vmode == 21 || vmode == 22 || vmode == 39 ||
			vmode == 25 || vmode == 26 || vmode == 40 ||
			vmode == 44 || vmode == 45 || vmode == 46 ||
			vmode == 50 || vmode == 51 || vmode == 54 ||
			vmode == 55 || vmode == 58 || vmode  == 59)
			reg |= DPTX_VIDEO_CONFIG1_IN_OSC_EN;
	}

	if (mdtd->interlaced == 1){
		reg |= DPTX_VIDEO_CONFIG1_O_IP_EN;
	}
	reg |= mdtd->h_active << DPTX_VIDEO_H_ACTIVE_SHIFT;
	reg |= mdtd->h_blanking << DPTX_VIDEO_H_BLANK_SHIFT;
	dptx_writel(dptx, DPTX_VIDEO_CONFIG1_N(stream), reg);

	/* Configure video_config2 register */
	reg = 0;
	reg |= mdtd->v_active << DPTX_VIDEO_V_ACTIVE_SHIFT;
	reg |= mdtd->v_blanking << DPTX_VIDEO_V_BLANK_SHIFT;
	dptx_writel(dptx, DPTX_VIDEO_CONFIG2_N(stream), reg);

	/* Configure video_config3 register */
	reg = 0;
	reg |= mdtd->h_sync_offset << DPTX_VIDEO_H_FRONT_PORCH;
	reg |= mdtd->h_sync_pulse_width << DPTX_VIDEO_H_SYNC_WIDTH;
	dptx_writel(dptx, DPTX_VIDEO_CONFIG3_N(stream), reg);

	/* Configure video_config4 register */
	reg = 0;
	reg |= mdtd->v_sync_offset << DPTX_VIDEO_V_FRONT_PORCH;
	reg |= mdtd->v_sync_pulse_width << DPTX_VIDEO_V_SYNC_WIDTH;
	dptx_writel(dptx, DPTX_VIDEO_CONFIG4_N(stream), reg);

	/* Configure video_config5 register */
	dptx_video_ts_change(dptx, stream);

	/* Configure video_msa1 register */
	reg = 0;
	reg |= (mdtd->h_blanking - mdtd->h_sync_offset)
		<< DPTX_VIDEO_MSA1_H_START_SHIFT;
	reg |= (mdtd->v_blanking - mdtd->v_sync_offset)
		<< DPTX_VIDEO_MSA1_V_START_SHIFT;
	dptx_writel(dptx, DPTX_VIDEO_MSA1_N(stream), reg);

	dptx_video_set_sink_bpc(dptx, stream);

	reg = dptx_calculate_hblank_interval(dptx);
	reg |= (DPTX_VIDEO_HBLANK_INTERVAL_ENABLE << DPTX_VIDEO_HBLANK_INTERVAL_SHIFT);
	dptx_writel(dptx, DPTX_VIDEO_HBLANK_INTERVAL, reg);
}

u8 dptx_calculate_dsc_init_threshold(struct dptx* dptx)
{
	int link_pixel_clock_ratio;
	int pixle_push_rate;
	int lanes;
	int tu;
	int slot_count;
	int fec_slot_count;
	int link_clk;
	int pixel_clk;
	u16 dsc_bpp;
	u8 rate;

	if (!dptx->vparams.dsc_bpp || !dptx->vparams.mdtd.pixel_clock)
		return (unsigned char)(-EINVAL);

	tu = dptx->vparams.aver_bytes_per_tu;
	lanes = dptx->link.lanes;
	dsc_bpp = dptx->vparams.dsc_bpp;

	if(dptx->fec) {
		if(lanes == 1)
			fec_slot_count = 13;
		else
			fec_slot_count = 7;
	} else {
		fec_slot_count = 0;
	}

	pixle_push_rate = (8 / dsc_bpp) * lanes;
	if(tu > 0)
		slot_count = tu + 1 + fec_slot_count;
	else
		slot_count = tu + fec_slot_count;

	slot_count = ROUND_UP_TO_NEAREST(slot_count, 4);

	pixel_clk = dptx->vparams.mdtd.pixel_clock;

	rate = dptx->link.rate;

	switch (rate) {
	case DPTX_PHYIF_CTRL_RATE_RBR:
		link_clk = 40500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR:
		link_clk = 67500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR2:
		link_clk = 135000;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR3:
		link_clk = 202500;
		break;
	default:
		WARN(1, "Invalid rate 0x%x\n", rate);
		return (unsigned char)(-EINVAL);
	}

	link_pixel_clock_ratio = link_clk / pixel_clk;

	return (pixle_push_rate * link_pixel_clock_ratio * slot_count);
}

int dptx_video_ts_calculate(struct dptx *dptx, int lane_num, int rate,
				int bpc, int encoding, int pixel_clock)
{
	struct video_params *vparams;
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct dtd *mdtd;
	int link_rate;
	int link_clk;
	int retval = 0;
	int ts;
	int T1;
	int T2;
	int tu;
	int tu_frac;
	int color_dep;

	if (!lane_num || !pixel_clock)
		return -EINVAL;

	vparams = &dptx->vparams;
	mdtd = &vparams->mdtd;

	switch (rate) {
	case DPTX_PHYIF_CTRL_RATE_RBR:
		link_rate = 162;
		link_clk = 40500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR:
		link_rate = 270;
		link_clk = 67500;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR2:
		link_rate = 540;
		link_clk = 135000;
		break;
	case DPTX_PHYIF_CTRL_RATE_HBR3:
		link_rate = 810;
		link_clk = 202500;
		break;
	default:	//sahakyan
		dev_dbg(xdev->dev, "Invalid rate param = %d\n" , rate);
		return -EINVAL;
		break;
	}

	switch (bpc) {
		case COLOR_DEPTH_6:
			color_dep = 18;
			break;
		case COLOR_DEPTH_8:
			if (encoding == YCBCR420)
				color_dep  = 12;
			else if (encoding == YCBCR422)
				color_dep = 16;
			else if (encoding == YONLY)
				color_dep = 8;
			else
				color_dep = 24;
			break;
		case COLOR_DEPTH_10:
			if (encoding == YCBCR420)
				color_dep = 15;
			else if (encoding == YCBCR422)
				color_dep = 20;
			else if (encoding  == YONLY)
				color_dep = 10;
			else
				color_dep = 30;
			break;

		case COLOR_DEPTH_12:
			if (encoding == YCBCR420)
				color_dep = 18;
			else if (encoding == YCBCR422)
				color_dep = 24;
			else if (encoding == YONLY)
				color_dep = 12;
			else
				color_dep = 36;
			break;

		case COLOR_DEPTH_16:
			if (encoding == YCBCR420)
				color_dep = 24;
			else if (encoding == YCBCR422)
				color_dep = 32;
			else if (encoding == YONLY)
				color_dep = 16;
			else
				color_dep = 48;
			break;
		default:
			color_dep = 18;
			break;
	}

	// Calculate average_bytes_per_tu based on compressed bpp
	if (dptx->dsc)
		color_dep = dptx->vparams.dsc_bpp;
	dev_dbg(xdev->dev, "color_dep:%d pixel_clock:%d lane_num:%d link_rate:%d\n",color_dep,pixel_clock,lane_num,link_rate);
	ts = (8 * color_dep * pixel_clock) / (lane_num * link_rate);

	tu  = ts / 1000;
	if (tu >= 65) {
		dev_dbg(xdev->dev, "%s: tu(%d) > 65", __func__, tu);
		return -EINVAL;
	}

	tu_frac = ts / 100 - tu * 10;
	
	// Calculate init_threshold for DSC mode
	if (dptx->dsc) {
		vparams->init_threshold = dptx_calculate_dsc_init_threshold(dptx);
		dev_dbg(xdev->dev, "calculated init_threshold for dsc = %d\n", vparams->init_threshold);
		if (vparams->init_threshold < 32) {
			vparams->init_threshold = 32;
			dev_dbg(xdev->dev, "Set init_threshold for dsc to %d\n", vparams->init_threshold);
		}
	// Calculate init_threshold for non DSC mode
	} else {
		T1 = 0;
		T2 = 0;
		//Single Pixel Mode
		if (dptx->multipixel == DPTX_MP_SINGLE_PIXEL) {
			if (tu < 6)
				vparams->init_threshold = 32;
			else if (mdtd->h_blanking <= 40 && encoding == YCBCR420)
				vparams->init_threshold = 3;
			else if (mdtd->h_blanking <= 80  && encoding != YCBCR420)
				vparams->init_threshold = 12;
			else
				vparams->init_threshold = 15;
			dev_dbg(xdev->dev, "TDB: single mode threshold = %d",vparams->init_threshold);
			//Multiple Pixel Mode
		} else {
			dev_dbg(xdev->dev, "harutk---- enter MP switch \n");
				switch (bpc) {
					case COLOR_DEPTH_6:
						T1 = (4*1000/9)*lane_num;
						dev_dbg(xdev->dev, "harutk---- enter MP switch T1= %d\n", T1);
						break;
					case COLOR_DEPTH_8:
						if(encoding == YCBCR422)
							T1 = (1000/2)*lane_num;
						else if (encoding == YONLY)
							T1 = lane_num*1000;
						else
							if(dptx->multipixel == DPTX_MP_DUAL_PIXEL)
								T1 = (1000/3)*lane_num;
							else
								T1 = (3000/16)*lane_num;
						break;
					case COLOR_DEPTH_10:
						if(encoding == YCBCR422)
								T1 = (2000/5)*lane_num;
						  else if (encoding == YONLY)
								T1 = (4000/5)*lane_num;
						  else
								T1 = (4000/15)*lane_num;
						break;
					case COLOR_DEPTH_12:
						if(encoding == YCBCR422)
							if(dptx->multipixel == DPTX_MP_DUAL_PIXEL)
								T1 = (1000/6)*lane_num;
							else
								T1 = (1000/3)*lane_num;
						  else if (encoding == YONLY)
								T1 = (2000/3)*lane_num;
						else
							T1 = (2000/9)*lane_num;
						break;
					case COLOR_DEPTH_16:
						 if(encoding == YONLY)
							T1 = (1000/2)*lane_num;
						 if((encoding != YONLY) && (encoding != YCBCR422) && (dptx->multipixel == DPTX_MP_DUAL_PIXEL))
							T1 = (1000/6)*lane_num;
						 else
							T1 = (1000/4)*lane_num;
						break;
					default:
						dev_dbg(xdev->dev, "Invalid param BPC = %d\n" , bpc);
						return -EINVAL;
						break;
				}

				if (encoding == YCBCR420)
					pixel_clock = pixel_clock /2;

				T2 = (link_clk*1000 /  pixel_clock);

				vparams->init_threshold = T1 * T2 * tu / (1000*1000);
				dev_dbg(xdev->dev, "TDB: threshold = %d",vparams->init_threshold);
		}
	}

	dev_dbg(xdev->dev, "tu = %d, tu_frac = %d\n", tu, tu_frac);
	vparams->aver_bytes_per_tu = tu;
	vparams->aver_bytes_per_tu_frac = tu_frac;

	if (dptx->mst) {
		u32 tu_mst;
		u32 tu_frac_mst;

		int numerator;
		int denominator;
		s64 fixp;
		int lane_count;
		int fec_slot_count;
		int slot_count;
		int num_lanes_divisor;
		int slot_count_adjust;
		int mst_T1 = 0;
		int mst_T2 = 0;
		int mst_T3 = 0;

		dev_dbg(xdev->dev, "MST: pixel_clock=%d\n", mdtd->pixel_clock);
		numerator = 74250 * 3 * 8;
		dev_dbg(xdev->dev, "MST: numerator=%d\n", numerator);
		denominator = (link_rate) * lane_num * 100 * 1000/10;
		dev_dbg(xdev->dev, "MST: denominator=%d\n", denominator);
		fixp = drm_fixp_from_fraction(numerator * 64, denominator);
		tu_mst = drm_fixp2int(fixp);
		

		lane_count = dptx->link.lanes;
		if(dptx->fec) {
			if(lane_count == 1)
				fec_slot_count = 13;
			else
				fec_slot_count = 7;
		} else {
			fec_slot_count = 0;
		}

		fixp &= DRM_FIXED_DECIMAL_MASK;
		fixp *= 64;
		tu_frac_mst = drm_fixp2int(fixp);

		dev_dbg(xdev->dev, "MST: tu = %d, tu_frac = %d\n", tu_mst, tu_frac_mst);
		vparams->aver_bytes_per_tu = tu_mst;
		vparams->aver_bytes_per_tu_frac = tu_frac_mst;

		if(tu_mst > 0)
			slot_count = tu_mst + 1 + fec_slot_count;
		else
			slot_count = tu_mst + fec_slot_count;

		switch (bpc) {
			case COLOR_DEPTH_6:
				if (encoding == RGB || encoding == YCBCR420)
						mst_T1 = 72000/36;
				 break;
			case COLOR_DEPTH_8:
				if (encoding == RGB || encoding == YCBCR420)
					mst_T1 = 16000/12;
				else if(encoding == YCBCR422)
						mst_T1 = 8000/4;
				break;
			case COLOR_DEPTH_10:
				if (encoding == RGB || encoding == YCBCR420)
					mst_T1 = 64000/60;
				else if(encoding == YCBCR422)
						mst_T1 = 32000/20;
				break;
			case COLOR_DEPTH_12:
				if (encoding == RGB || encoding == YCBCR420)
					mst_T1 = 32000/36;
				else if(encoding == YCBCR422)
						mst_T1 = 16000/12;
				break;
			case COLOR_DEPTH_16:
				if (encoding == RGB || encoding == YCBCR420)
					mst_T1 = 8000/12;
				else if(encoding == YCBCR422)
						mst_T1 = 4000/4;
				break;
			default:
				dev_dbg(xdev->dev, "Invalid param BPC = %d\n" , bpc);
				return -EINVAL;
				break;
		}

		if (encoding == YCBCR420)
			pixel_clock = pixel_clock /2;

		mst_T2 = (link_clk*1000 /  pixel_clock);

		if(lane_count == 1) {
			num_lanes_divisor = 4;
			slot_count_adjust = 3;
		} else if(lane_count == 2) { 
			num_lanes_divisor = 2;
			slot_count_adjust = 1;
		} else {
			num_lanes_divisor = 1;
			slot_count_adjust = 0;
		}

		mst_T3 = ((slot_count + slot_count_adjust)/num_lanes_divisor) + 8;

		vparams->init_threshold = mst_T1 * mst_T2 * mst_T3 / (1000*1000);


		pr_err("\n\n\nSahakyan: T1 = %d, T2 =%d, tu = %d, vparams->init_threshold = %d\n\n\n", mst_T1, mst_T2, mst_T3, vparams->init_threshold);
#if 0
		/* TODO this is a duplicate calculation from above */
		if (tu_mst < 6) {
			vparams->init_threshold = 32;
		} else if ((encoding == RGB || encoding == YCBCR444) && mdtd->h_blanking <= 80) {
			if (dptx->multipixel == DPTX_MP_QUAD_PIXEL)
				vparams->init_threshold = 4;
			else
				vparams->init_threshold = 12;
		} else {
			vparams->init_threshold = 15;
		}
#endif
	}

	return retval;
}

void dptx_video_ts_change(struct dptx *dptx, int stream)
{
	u32 reg;
	struct video_params *vparams;

	vparams = &dptx->vparams;

	reg = dptx_readl(dptx, DPTX_VIDEO_CONFIG5_N(stream));
	reg = reg & (~DPTX_VIDEO_CONFIG5_TU_MASK);
	reg = reg | (vparams->aver_bytes_per_tu <<
			DPTX_VIDEO_CONFIG5_TU_SHIFT);
	if(dptx->mst) {
		reg = reg & (~DPTX_VIDEO_CONFIG5_TU_FRAC_MASK_MST);
		reg = reg | (vparams->aver_bytes_per_tu_frac <<
				DPTX_VIDEO_CONFIG5_TU_FRAC_SHIFT_MST);
	}
	else {
		reg = reg & (~DPTX_VIDEO_CONFIG5_TU_FRAC_MASK_SST);
		reg = reg | (vparams->aver_bytes_per_tu_frac <<
				DPTX_VIDEO_CONFIG5_TU_FRAC_SHIFT_SST);
	}
	reg = reg & (~DPTX_VIDEO_CONFIG5_INIT_THRESHOLD_MASK);
	reg = reg | (vparams->init_threshold <<
			DPTX_VIDEO_CONFIG5_INIT_THRESHOLD_SHIFT);
	dptx_writel(dptx, DPTX_VIDEO_CONFIG5_N(stream), reg);
//	dptx_writel(dptx, DPTX_VIDEO_CONFIG5_N(stream), 0x80388);
}

void dptx_video_bpc_change(struct dptx *dptx, int stream)
{
	dptx_video_set_core_bpc(dptx, stream);
	dptx_video_set_sink_bpc(dptx, stream);
	//dptx_video_set_gen_bpc(dptx, stream);
}

void dptx_video_set_core_bpc(struct dptx *dptx, int stream)
{
	u32 reg;
	u8 bpc_mapping = 0, bpc = 0;
	enum pixel_enc_type pix_enc;
	struct video_params *vparams;

	vparams = &dptx->vparams;
	bpc = vparams->bpc;
	pix_enc = vparams->pix_enc;

	reg = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(stream));
	reg &= ~DPTX_VSAMPLE_CTRL_VMAP_BPC_MASK;

	switch (pix_enc) {
	case RGB:
		if (bpc == COLOR_DEPTH_6)
			bpc_mapping = 0;
		else if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 2;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 3;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 4;
		break;
	case YCBCR444:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 5;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 6;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 7;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 8;
		break;
	case YCBCR422:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 9;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 10;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 11;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 12;
		break;
	case YCBCR420:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 13;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 14;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 15;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 16;
		break;
	case YONLY:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 17;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 18;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 19;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 20;
		break;
	case RAW:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 23;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 24;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 25;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 27;
		break;
	}

	/* TODO only for RGB 8bpc */
	if (dptx->dsc)
		reg |= (1 << DPTX_VG_CONFIG1_BPC_SHIFT); //sahakyan
	//	reg |= 2 << DPTX_VSAMPLE_CTRL_VMAP_BPC_SHIFT; // check DSC-2slices for 10bpc -harutk
	else
		reg |= (bpc_mapping << DPTX_VSAMPLE_CTRL_VMAP_BPC_SHIFT);
	dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(stream), reg);
}

void dptx_video_set_gen_bpc(struct dptx *dptx, int stream)
{
	u32 reg;
	u8 bpc_mapping = 0, bpc = 0;
	struct video_params *vparams;

	vparams = &dptx->vparams;
	bpc = vparams->bpc;
	reg = dptx_readl(dptx, DPTX_VG_CONFIG1_N(stream));
	reg &= ~DPTX_VG_CONFIG1_BPC_MASK;

	switch (bpc) {
	case COLOR_DEPTH_6:
		bpc_mapping = 0;
		break;
	case COLOR_DEPTH_8:
		bpc_mapping = 1;
		break;
	case COLOR_DEPTH_10:
		bpc_mapping = 2;
		break;
	case COLOR_DEPTH_12:
		bpc_mapping = 3;
		break;
	case COLOR_DEPTH_16:
		bpc_mapping = 4;
		break;
	}

	if(dptx->dsc)
		reg |= (1 << DPTX_VG_CONFIG1_BPC_SHIFT); // 10 bpc DSC harutk
	else
		reg |= (bpc_mapping << DPTX_VG_CONFIG1_BPC_SHIFT);

	dptx_writel(dptx, DPTX_VG_CONFIG1_N(stream), reg);
}

void dptx_video_set_sink_col(struct dptx *dptx, int stream)
{
	u32 reg_msa2;
	u8 col_mapping;
	u8 colorimetry;
	u8 dynamic_range;
	struct video_params *vparams;
	enum pixel_enc_type pix_enc;

	vparams = &dptx->vparams;
	pix_enc = vparams->pix_enc;
	colorimetry = vparams->colorimetry;
	dynamic_range = vparams->dynamic_range;

	reg_msa2 = dptx_readl(dptx, DPTX_VIDEO_MSA2_N(stream));
	reg_msa2 &= ~DPTX_VIDEO_VMSA2_COL_MASK;

	col_mapping = 0;

	/* According to Table 2-94 of DisplayPort spec 1.3 */
	switch (pix_enc) {
	case RGB:
		if (dynamic_range == CEA)
			col_mapping = 4;
		else if (dynamic_range == VESA)
			col_mapping = 0;
		break;
	case YCBCR422:
		if (colorimetry == ITU601)
			col_mapping = 5;
		else if (colorimetry == ITU709)
			col_mapping = 13;
		break;
	case YCBCR444:
		if (colorimetry == ITU601)
			col_mapping = 6;
		else if (colorimetry == ITU709)
			col_mapping = 14;
		break;
	case RAW:
		col_mapping = 1;
		break;
	case YCBCR420:
	case YONLY:
		break;
	}

	reg_msa2 |= (col_mapping << DPTX_VIDEO_VMSA2_COL_SHIFT);
	dptx_writel(dptx, DPTX_VIDEO_MSA2_N(stream), reg_msa2);
}

void dptx_video_set_sink_bpc(struct dptx *dptx, int stream)
{
	u32 reg_msa2, reg_msa3;
	u8 bpc_mapping = 0, bpc = 0;
	struct video_params *vparams;
	enum pixel_enc_type pix_enc;

	vparams = &dptx->vparams;
	pix_enc = vparams->pix_enc;
	bpc = vparams->bpc;

	reg_msa2 = dptx_readl(dptx, DPTX_VIDEO_MSA2_N(stream));
	reg_msa3 = dptx_readl(dptx, DPTX_VIDEO_MSA3_N(stream));

	reg_msa2 &= ~DPTX_VIDEO_VMSA2_BPC_MASK;
	reg_msa3 &= ~DPTX_VIDEO_VMSA3_PIX_ENC_MASK;

	switch (pix_enc) {
	case RGB:
		if (bpc == COLOR_DEPTH_6)
			bpc_mapping = 0;
		else if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 2;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 3;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 4;
		break;
	case YCBCR444:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 2;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 3;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 4;
		break;
	case YCBCR422:
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 2;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 3;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 4;
		break;
	case YCBCR420:
		reg_msa3 |= 1 << DPTX_VIDEO_VMSA3_PIX_ENC_YCBCR420_SHIFT;
		break;
	case YONLY:
		/* According to Table 2-94 of DisplayPort spec 1.3 */
		reg_msa3 |= 1U << DPTX_VIDEO_VMSA3_PIX_ENC_SHIFT;
		if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 2;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 3;
		if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 4;
		break;
	case RAW:
		/* According to Table 2-94 of DisplayPort spec 1.3 */
		reg_msa3 |= (1U << DPTX_VIDEO_VMSA3_PIX_ENC_SHIFT);

		if (bpc == COLOR_DEPTH_6)
			bpc_mapping = 1;
		else if (bpc == COLOR_DEPTH_8)
			bpc_mapping = 3;
		else if (bpc == COLOR_DEPTH_10)
			bpc_mapping = 4;
		else if (bpc == COLOR_DEPTH_12)
			bpc_mapping = 5;
		else if (bpc == COLOR_DEPTH_16)
			bpc_mapping = 7;
		break;
	}

//	if(dptx->dsc)
//		reg_msa2 |= (1 << DPTX_VIDEO_VMSA2_BPC_SHIFT); // DSC 10bpc harutk
//	else
		reg_msa2 |= (bpc_mapping << DPTX_VIDEO_VMSA2_BPC_SHIFT);

	dptx_writel(dptx, DPTX_VIDEO_MSA2_N(stream), reg_msa2);
	dptx_writel(dptx, DPTX_VIDEO_MSA3_N(stream), reg_msa3);
	dptx_video_set_sink_col(dptx, stream);
}

void dptx_set_ram_ctr(struct dptx *dptx, int stream)
{
#if 0
	u32 reg;

	reg = dptx_readl(dptx, DPTX_VG_RAM_ADDR_N(stream));
	reg &= ~DPTX_VG_RAM_ADDR_START_MASK;
	dptx_writel(dptx, DPTX_VG_RAM_ADDR_N(stream), reg);

	reg = dptx_readl(dptx, DPTX_VG_WRT_RAM_CTR_N(stream));
	reg |= DPTX_VG_RAM_CTR_START_MASK;
	dptx_writel(dptx, DPTX_VG_WRT_RAM_CTR_N(stream), reg);

	reg = dptx_readl(dptx, DPTX_VG_WRT_RAM_CTR_N(stream));
	reg &= ~DPTX_VG_RAM_CTR_START_MASK;
	dptx_writel(dptx, DPTX_VG_WRT_RAM_CTR_N(stream), reg);
#endif
}

void dptx_set_ram_data(struct dptx *dptx, u32 value, int count, int stream)
{
	int i;

	for (i = 0; i < count; i++)
		dptx_writel(dptx, DPTX_VG_WRT_RAM_DATA_N(stream), value);
}

void dptx_set_ramp_pattern(struct dptx *dptx, int column_num, int stream)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	int i, step, range, shift1, shift2, value;
	u8 bpc, mult;
	struct video_params *vparams;

	vparams = &dptx->vparams;
	bpc = vparams->bpc;
	dev_dbg(xdev->dev, "%s \n", __func__);

	dptx_set_ram_ctr(dptx, stream);

	switch (dptx->multipixel) {
	default:
	case DPTX_MP_SINGLE_PIXEL:
		dptx_set_ram_data(dptx, 0, 6, stream);
		break;
	case DPTX_MP_DUAL_PIXEL:
		dptx_set_ram_data(dptx, 0, 12, stream);
		break;
	case DPTX_MP_QUAD_PIXEL:
		dptx_set_ram_data(dptx, 0, 24, stream);
		break;
	}

	range = 0;
	shift1 = 0;
	shift2 = 0;
	step = 0;
	value = 0;
	mult = 1;
		
	if (bpc == COLOR_DEPTH_6 || bpc == COLOR_DEPTH_8) {
		if(!dptx->dsc){
		 if (bpc == COLOR_DEPTH_6)
			mult = 4;
		}

		/* Program RED color */
		for (i = 0; i < column_num; i++) {
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 5, stream);
		}
		/* Program GREEN color */
		for (i = 0; i < column_num; i++) {
			dptx_set_ram_data(dptx, 0, 2, stream);
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 3, stream);
		}
		/* Program BLUE color */
		for (i = 0; i < column_num; i++) {
			dptx_set_ram_data(dptx, 0, 4, stream);
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
		}
		/* Program WHITE color */
		for (i = 0; i < column_num; i++) {
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, i * mult, 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
		}
	} else if (bpc == COLOR_DEPTH_10 || bpc == COLOR_DEPTH_12 ||
			bpc == COLOR_DEPTH_16) {
		if (bpc == COLOR_DEPTH_10) {
			range = 384;
			shift1 = 2;
			shift2 = 6;
			step = 4;
		} else if (bpc == COLOR_DEPTH_12) {
			range = 1920;
			shift1 = 1;
			shift2 = 7;
			step = 16;
		} else if (bpc == COLOR_DEPTH_16) {
			range = 32640;
			shift1 = 0;
			shift2 = 8;
			step = 256;
		}

		for (i = 0; i < column_num; i++) {
			value = range + i;
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
			dptx_set_ram_data(dptx, 0, 4, stream);
		}
		for (i = 0; i < column_num * step; i = i + step) {
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
			dptx_set_ram_data(dptx, 0, 4, stream);
		}
		for (i = 0; i < column_num; i++) {
			value = range + i;
			dptx_set_ram_data(dptx, 0, 2, stream);
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
			dptx_set_ram_data(dptx, 0, 2, stream);
		}
		for (i = 0; i < column_num * step; i = i + step) {
			dptx_set_ram_data(dptx, 0, 2, stream);
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
			dptx_set_ram_data(dptx, 0, 2, stream);
		}
		for (i = 0; i < column_num; i++) {
			value = range + i;
			dptx_set_ram_data(dptx, 0, 4, stream);
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
		}
		for (i = 0; i < column_num * step; i = i + step) {
			dptx_set_ram_data(dptx, 0, 4, stream);
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
		}
		for (i = 0; i < column_num; i++) {
			value = range + i;
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
			dptx_set_ram_data(dptx, (value >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (value << shift2), 1, stream);
		}
		for (i = 0; i < column_num * step; i = i + step) {
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
			dptx_set_ram_data(dptx, (i >> shift1), 1, stream);
			dptx_set_ram_data(dptx, (i << shift2), 1, stream);
		}
	}
}

void dptx_set_col_ramp_pattern(struct dptx *dptx, int stream)
{
	int i, j;
	u8 ram_pix_cnt;
	static int white[3] = {235, 235, 235};
	static int yellow[3] = {235, 235, 16};
	static int cyan[3] = {16, 235, 235};
	static int green[3] = {16, 235, 16};
	static int magenta[3] = {235, 16, 235};
	static int red[3] = {235, 16, 16};
	static int blue[3] = {16, 16, 235};
	static int black[3] = {16, 16, 16};
	static int *colors_row1[8] = {
		white, yellow, cyan, green,
		magenta, red, blue, black
	};
	static int *colors_row2[8] = {
		blue, red, magenta, green,
		cyan, yellow, white, black
	};

	dptx_set_ram_ctr(dptx, stream);

	switch (dptx->multipixel) {
	default:
	case DPTX_MP_SINGLE_PIXEL:
		dptx_set_ram_data(dptx, 0, 6, stream);
		ram_pix_cnt = 1;
		break;
	case DPTX_MP_DUAL_PIXEL:
		dptx_set_ram_data(dptx, 0, 12, stream);
		ram_pix_cnt = 2;
		break;
	case DPTX_MP_QUAD_PIXEL:
		dptx_set_ram_data(dptx, 0, 24, stream);
		ram_pix_cnt = 4;
		break;
	}

	for (i = 0; i < 8; i++) {
		for (j = 0; j < ram_pix_cnt; j++) {
			dptx_set_ram_data(dptx, colors_row1[i][0], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, colors_row1[i][1], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, colors_row1[i][2], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
		}
	}
	for (i = 0; i < 8; i++) {
		for (j = 0; j < ram_pix_cnt; j++) {
			dptx_set_ram_data(dptx, colors_row2[i][0], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, colors_row2[i][1], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
			dptx_set_ram_data(dptx, colors_row2[i][2], 1, stream);
			dptx_set_ram_data(dptx, 0, 1, stream);
		}
	}
}

void dptx_set_col_ycbcr_ramp_pattern(struct dptx *dptx, int stream)
{
	struct video_params *vparams;
	u8 colorimetry, bpc, shift1, shift2;
	int i, j, mult;
	u8 ram_pix_cnt;

	static int white[3] = {235, 128, 128};
	static int yellow[3] = {210, 16, 146};
	static int cyan[3] = {170, 166, 16};
	static int green[3] = {145, 54, 34};
	static int magenta[3] = {106, 202, 222};
	static int red[3] = {81, 90, 240};
	static int blue[3] = {41, 240, 110};
	static int black[3] = {16, 128, 128};
	static int *colors_row1[8] = {
		white, yellow, cyan, green,
		magenta, red, blue, black
	};

	dptx_set_ram_ctr(dptx, stream);

	switch (dptx->multipixel) {
	default:
	case DPTX_MP_SINGLE_PIXEL:
		dptx_set_ram_data(dptx, 0, 6, stream);
		ram_pix_cnt = 1;
		break;
	case DPTX_MP_DUAL_PIXEL:
		dptx_set_ram_data(dptx, 0, 12, stream);
		ram_pix_cnt = 2;
		break;
	case DPTX_MP_QUAD_PIXEL:
		dptx_set_ram_data(dptx, 0, 24, stream);
		ram_pix_cnt = 4;
		break;
	}

	vparams = &dptx->vparams;
	colorimetry = vparams->colorimetry;
	bpc = vparams->bpc;

	if (colorimetry == ITU601) {
		if (bpc == COLOR_DEPTH_10) {
			white[0] = 940;
			white[1] = 512;
			white[2] = 512;
			yellow[0] = 840;
			yellow[1] = 64;
			yellow[2] = 585;
			cyan[0] = 678;
			cyan[1] = 663;
			cyan[2] = 64;
			green[0] = 578;
			green[1] = 215;
			green[2] = 137;
			magenta[0] = 426;
			magenta[1] = 809;
			magenta[2] = 887;
			red[0] = 326;
			red[1] = 361;
			red[2] = 960;
			blue[0] = 164;
			blue[1] = 960;
			blue[2] = 439;
			black[0] = 64;
			black[1] = 512;
			black[2] = 512;
		} else {
			white[0] = 235;
			white[1] = 128;
			white[2] = 128;
			yellow[0] = 210;
			yellow[1] = 16;
			yellow[2] = 146;
			cyan[0] = 170;
			cyan[1] = 166;
			cyan[2] = 16;
			green[0] = 145;
			green[1] = 54;
			green[2] = 34;
			magenta[0] = 106;
			magenta[1] = 202;
			magenta[2] = 222;
			red[0] = 81;
			red[1] = 90;
			red[2] = 240;
			blue[0] = 41;
			blue[1] = 240;
			blue[2] = 110;
			black[0] = 16;
			black[1] = 128;
			black[2] = 128;
		}
	} else {
		if (bpc == COLOR_DEPTH_10) {
			white[0] = 940;
			white[1] = 512;
			white[2] = 512;
			yellow[0] = 877;
			yellow[1] = 64;
			yellow[2] = 553;
			cyan[0] = 753;
			cyan[1] = 614;
			cyan[2] = 64;
			green[0] = 690;
			green[1] = 167;
			green[2] = 106;
			magenta[0] = 314;
			magenta[1] = 857;
			magenta[2] = 918;
			red[0] = 251;
			red[1] = 410;
			red[2] = 960;
			blue[0] = 127;
			blue[1] = 960;
			blue[2] = 471;
			black[0] = 64;
			black[1] = 512;
			black[2] = 512;
		} else {
				white[0] = 235;
			white[1] = 128;
			white[2] = 128;
			yellow[0] = 219;
			yellow[1] = 16;
			yellow[2] = 138;
			cyan[0] = 188;
			cyan[1] = 154;
			cyan[2] = 16;
			green[0] = 173;
			green[1] = 42;
			green[2] =  26;
			magenta[0] = 78;
			magenta[1] = 214;
			magenta[2] = 230;
			red[0] = 63;
			red[1] = 102;
			red[2] = 240;
			blue[0] = 32;
			blue[1] = 240;
			blue[2] = 118;
			black[0] = 16;
			black[1] = 128;
			black[2] = 128;
		}
	}

	if (bpc == COLOR_DEPTH_10) {
		shift1 = 2;
		shift2 = 6;
		mult = 1;
	} else if (bpc == COLOR_DEPTH_12) {
		shift1 = 4;
		shift2 = 4;
		mult = 16;
	} else if (bpc == COLOR_DEPTH_16) {
		shift1 = 8;
		shift2 = 0;
		mult = 16 * 16;
	} else {
		shift1 = 0;
		shift2 = 0;
		mult = 1;
	}

	colors_row1[0] = white;
	colors_row1[1] = yellow;
	colors_row1[2] = cyan;
	colors_row1[3] = green;
	colors_row1[4] = magenta;
	colors_row1[5] = red;
	colors_row1[6] = blue;
	colors_row1[7] = black;

	for (i = 0; i < 8; i++) {
		for(j = 0; j< ram_pix_cnt; j++) { // for quad/dual pixel the same pixel should be programmed 4/2 times
			dptx_set_ram_data(dptx, (colors_row1[i][0]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][0]*mult) << shift2, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][1]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][1]*mult) << shift2, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][2]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][2]*mult) << shift2, 1, stream);
		}
	}

	colors_row1[0] = blue;
	colors_row1[1] = red;
	colors_row1[2] = magenta;
	colors_row1[4] = cyan;
	colors_row1[5] = yellow;
	colors_row1[6] = white;
	colors_row1[7] = black;

	for (i = 0; i < 8; i++) {
		for(j = 0; j< ram_pix_cnt; j++) { // for quad/dual pixel the same pixel should be programmed 4/2 times
			dptx_set_ram_data(dptx, (colors_row1[i][0]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][0]*mult) << shift2, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][1]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][1]*mult) << shift2, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][2]*mult) >> shift1, 1, stream);
			dptx_set_ram_data(dptx, (colors_row1[i][2]*mult) << shift2, 1, stream);
		}
	}
}

void dptx_video_pattern_change(struct dptx *dptx, int stream)
{
	struct video_params *vparams;
	u32 reg;
	u8 pattern, bpc, encoding;
	int column_num;

	vparams = &dptx->vparams;
	pattern = vparams->pattern_mode;
	bpc = vparams->bpc;
	encoding = vparams->pix_enc;

	column_num = 256;

#if 0
	/* TODO - This is temporary code for mst. Just program the pattern
	 * code and not the pattern in the VG RAM. */
	reg = dptx_readl(dptx, DPTX_VG_CONFIG1_N(stream));
	reg &= ~DPTX_VG_CONFIG1_PATTERN_MASK;
	reg |= (0) << DPTX_VG_CONFIG1_PATTERN_SHIFT;
	dptx_writel(dptx, DPTX_VG_CONFIG1_N(stream), reg);

	return;
#else

	if (pattern == RAMP) {
		if(!dptx->dsc) {
		  if (bpc == COLOR_DEPTH_6) 
			column_num = 64; 
		}
		dptx_set_ramp_pattern(dptx, column_num, stream);
	} else if (pattern == COLRAMP) {
		if (encoding == YCBCR422 || encoding == YCBCR444)
			dptx_set_col_ycbcr_ramp_pattern(dptx, stream);
		else
			dptx_set_col_ramp_pattern(dptx, stream);
	}

	reg = dptx_readl(dptx, DPTX_VG_CONFIG1_N(stream));
	reg &= ~DPTX_VG_CONFIG1_PATTERN_MASK;
	reg |= (vparams->pattern_mode << DPTX_VG_CONFIG1_PATTERN_SHIFT);
	dptx_writel(dptx, DPTX_VG_CONFIG1_N(stream), reg);
#endif
}

void dptx_video_pattern_set(struct dptx *dptx, u8 pattern, int stream)
{
	struct xdx_output *output = dptx->output;
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct video_params *vparams;

	vparams = &dptx->vparams;
	vparams->pattern_mode = pattern;

	dptx_disable_default_video_stream(dptx, stream);
	dptx_video_pattern_change(dptx, stream);
	dptx_writel(dptx, DPTX_VG_SWRST_N(stream), 0);
	dptx_enable_default_video_stream(dptx, stream);
	dev_dbg(xdev->dev, "%s: Change video pattern to %d\n",
		 __func__, pattern);
}

void dptx_disable_default_video_stream(struct dptx *dptx, int stream)
{
	u32 vsamplectrl;

	vsamplectrl = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(stream));
	vsamplectrl &= ~DPTX_VSAMPLE_CTRL_STREAM_EN;
	dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(stream), vsamplectrl);
}

void dptx_enable_default_video_stream(struct dptx *dptx, int stream)
{
	u32 vsamplectrl;

	vsamplectrl = dptx_readl(dptx, DPTX_VSAMPLE_CTRL_N(stream));
	vsamplectrl |= DPTX_VSAMPLE_CTRL_STREAM_EN;
	dptx_writel(dptx, DPTX_VSAMPLE_CTRL_N(stream), vsamplectrl);
}

/*
 * Audio/Video Parameters
 */

void dptx_audio_params_reset(struct audio_params *params)
{
	params->iec_channel_numcl0 = 8;
	params->iec_channel_numcr0 = 4;
	params->use_lut = 1;
	params->iec_samp_freq = 3;
	params->iec_word_length = 11;
	params->iec_orig_samp_freq = 12;
	params->data_width = 24;
	params->num_channels = 2;
	params->inf_type = 0;
	params->ats_ver = 17;
	params->mute = 0;
}

void dptx_video_params_reset(struct dptx *dptx)
{
	struct video_params *params = &dptx->vparams;

	params->pix_enc = RGB;
	params->mode = 1;

	if (dptx->mst || dptx->dsc) {
		/* TODO 6 bpc should be default - use 8 bpc for MST calculation */
		params->bpc = COLOR_DEPTH_6;
	} else {
		params->bpc = COLOR_DEPTH_6;
	}

	params->colorimetry = ITU601;
	params->dynamic_range = CEA;
	params->aver_bytes_per_tu = 30;
	params->aver_bytes_per_tu_frac = 0;
	params->init_threshold = 15;
	params->pattern_mode = RAMP;
	params->refresh_rate = 60000;
	params->video_format = VCEA;
}

/*
 * DTD
 */

void dwc_dptx_dtd_reset(struct dtd *mdtd)
{
	mdtd->pixel_repetition_input = 0;
	mdtd->pixel_clock  = 0;
	mdtd->h_active = 0;
	mdtd->h_blanking = 0;
	mdtd->h_sync_offset = 0;
	mdtd->h_sync_pulse_width = 0;
	mdtd->h_image_size = 0;
	mdtd->v_active = 0;
	mdtd->v_blanking = 0;
	mdtd->v_sync_offset = 0;
	mdtd->v_sync_pulse_width = 0;
	mdtd->v_image_size = 0;
	mdtd->interlaced = 0;
	mdtd->v_sync_polarity = 0;
	mdtd->h_sync_polarity = 0;
}

int dptx_dtd_fill(struct dtd *mdtd, u8 code, u32 refresh_rate,
		  u8 video_format)
{
	dwc_dptx_dtd_reset(mdtd);

	mdtd->h_image_size = 16;
	mdtd->v_image_size = 9;
#ifndef FALLTHROUGH_INTENDED
#define FALLTHROUGH_INTENDED __fallthrough
#endif

	if (video_format == VCEA) {
		switch (code) {
		case 1: /* 640x480p @ 59.94/60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 640;
			mdtd->v_active = 480;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 16;
			mdtd->v_sync_offset = 10;
			mdtd->h_sync_pulse_width = 96;
			mdtd->v_sync_pulse_width = 2;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			//mdtd->pixel_clock = 25175;
			mdtd->pixel_clock = 25200;
			break;
		case 2: /* 720x480p @ 59.94/60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 3: /* 720x480p @ 59.94/60Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 480;
			mdtd->h_blanking = 138;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 16;
			mdtd->v_sync_offset = 9;
			mdtd->h_sync_pulse_width = 62;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 27000;
			break;
		case 69:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 4: /* 1280x720p @ 59.94/60Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 370;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 110;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 5: /* 1920x1080i @ 59.94/60Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 540;
			mdtd->h_blanking = 280;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 74250;
			break;
		case 6: /* 720(1440)x480i @ 59.94/60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 7: /* 720(1440)x480i @ 59.94/60Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 240;
			mdtd->h_blanking = 276;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 38;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 124;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 27000;
			break;
		case 8: /* 720(1440)x240p @ 59.826/60.054/59.886/60.115Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 9: /* 720(1440)x240p @59.826/60.054/59.886/60.115Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 240;
			mdtd->h_blanking = 276;
			mdtd->v_blanking = (refresh_rate == 59940) ? 22 : 23;
			mdtd->h_sync_offset = 38;
			mdtd->v_sync_offset = (refresh_rate == 59940) ? 4 : 5;
			mdtd->h_sync_pulse_width = 124;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 27000;
			break;
		case 10: /* 2880x480i @ 59.94/60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 11: /* 2880x480i @ 59.94/60Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 240;
			mdtd->h_blanking = 552;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 76;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 248;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 54000;
			break;
		case 12: /* 2880x240p @ 59.826/60.054/59.886/60.115Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 13: /* 2880x240p @ 59.826/60.054/59.886/60.115Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 240;
			mdtd->h_blanking = 552;
			mdtd->v_blanking = (refresh_rate == 60054) ? 22 : 23;
			mdtd->h_sync_offset = 76;
			mdtd->v_sync_offset = (refresh_rate == 60054) ? 4 : 5;
			mdtd->h_sync_pulse_width = 248;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 14: /* 1440x480p @ 59.94/60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 15: /* 1440x480p @ 59.94/60Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 480;
			mdtd->h_blanking = 276;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 32;
			mdtd->v_sync_offset = 9;
			mdtd->h_sync_pulse_width = 124;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 76:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 16: /* 1920x1080p @ 59.94/60Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 280;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			//mdtd->h_sync_polarity = 1;
			//mdtd->v_sync_polarity = 1;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 148500;
			break;
		case 17: /* 720x576p @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 18: /* 720x576p @ 50Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 576;
			mdtd->h_blanking = 144;
			mdtd->v_blanking = 49;
			mdtd->h_sync_offset = 12;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 64;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 27000;
			break;
		case 68:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 19: /* 1280x720p @ 50Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 700;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 440;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 20: /* 1920x1080i @ 50Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 540;
			mdtd->h_blanking = 720;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 528;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 74250;
			break;
		case 21: /* 720(1440)x576i @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 22: /* 720(1440)x576i @ 50Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 288;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = 24;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 126;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 27000;
			break;
		case 23: /* 720(1440)x288p @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 24: /* 720(1440)x288p @ 50Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 288;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = (refresh_rate == 50080) ? 24
				: ((refresh_rate == 49920) ? 25 : 26);
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = (refresh_rate == 50080) ? 2
				: ((refresh_rate == 49920) ? 3 : 4);
			mdtd->h_sync_pulse_width = 126;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 27000;
			break;
		case 25: /* 2880x576i @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 26: /* 2880x576i @ 50Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 288;
			mdtd->h_blanking = 576;
			mdtd->v_blanking = 24;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 252;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 54000;
			break;
		case 27: /* 2880x288p @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 28: /* 2880x288p @ 50Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 288;
			mdtd->h_blanking = 576;
			mdtd->v_blanking = (refresh_rate == 50080) ? 24
				: ((refresh_rate == 49920) ? 25 : 26);
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = (refresh_rate == 50080) ? 2
				: ((refresh_rate == 49920) ? 3 : 4);
			mdtd->h_sync_pulse_width = 252;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 29: /* 1440x576p @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 30: /* 1440x576p @ 50Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 576;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = 49;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 75:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 31: /* 1920x1080p @ 50Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 720;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 528;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 148500;
			break;
		case 72:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 32: /* 1920x1080p @ 23.976/24Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 830;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 638;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 73:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 33: /* 1920x1080p @ 25Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 720;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 528;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 74:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 34: /* 1920x1080p @ 29.97/30Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 280;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 35: /* 2880x480p @ 60Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 36: /* 2880x480p @ 60Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 480;
			mdtd->h_blanking = 552;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 9;
			mdtd->h_sync_pulse_width = 248;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 108000;
			break;
		case 37: /* 2880x576p @ 50Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 38: /* 2880x576p @ 50Hz 16:9 */
			mdtd->h_active = 2880;
			mdtd->v_active = 576;
			mdtd->h_blanking = 576;
			mdtd->v_blanking = 49;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 256;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 108000;
			break;
		case 39: /* 1920x1080i (1250 total) @ 50Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 540;
			mdtd->h_blanking = 384;
			mdtd->v_blanking = 85;
			mdtd->h_sync_offset = 32;
			mdtd->v_sync_offset = 23;
			mdtd->h_sync_pulse_width = 168;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 72000;
			break;
		case 40: /* 1920x1080i @ 100Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 540;
			mdtd->h_blanking = 720;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 528;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 148500;
			break;
		case 70:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 41: /* 1280x720p @ 100Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 700;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 440;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 148500;
			break;
		case 42: /* 720x576p @ 100Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 43: /* 720x576p @ 100Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 576;
			mdtd->h_blanking = 144;
			mdtd->v_blanking = 49;
			mdtd->h_sync_offset = 12;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 64;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 44: /* 720(1440)x576i @ 100Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 45: /* 720(1440)x576i @ 100Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 288;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = 24;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 126;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 54000;
			break;
		case 46: /* 1920x1080i @ 119.88/120Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 540;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 148500;
			break;
		case 71:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 47: /* 1280x720p @ 119.88/120Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 370;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 110;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 148500;
			break;
		case 48: /* 720x480p @ 119.88/120Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 49: /* 720x480p @ 119.88/120Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 480;
			mdtd->h_blanking = 138;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 16;
			mdtd->v_sync_offset = 9;
			mdtd->h_sync_pulse_width = 62;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 54000;
			break;
		case 50: /* 720(1440)x480i @ 119.88/120Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 51: /* 720(1440)x480i @ 119.88/120Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 240;
			mdtd->h_blanking = 276;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 38;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 124;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 54000;
			break;
		case 52: /* 720X576p @ 200Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 53: /* 720X576p @ 200Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 576;
			mdtd->h_blanking = 144;
			mdtd->v_blanking = 49;
			mdtd->h_sync_offset = 12;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 64;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 108000;
			break;
		case 54: /* 720(1440)x576i @ 200Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 55: /* 720(1440)x576i @ 200Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 288;
			mdtd->h_blanking = 288;
			mdtd->v_blanking = 24;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 126;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 108000;
			break;
		case 56: /* 720x480p @ 239.76/240Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 57: /* 720x480p @ 239.76/240Hz 16:9 */
			mdtd->h_active = 720;
			mdtd->v_active = 480;
			mdtd->h_blanking = 138;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 16;
			mdtd->v_sync_offset = 9;
			mdtd->h_sync_pulse_width = 62;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 108000;
			break;
		case 58: /* 720(1440)x480i @ 239.76/240Hz 4:3 */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 59: /* 720(1440)x480i @ 239.76/240Hz 16:9 */
			mdtd->h_active = 1440;
			mdtd->v_active = 240;
			mdtd->h_blanking = 276;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 38;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 124;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 1;
			mdtd->pixel_clock = 108000;
			break;
		case 65:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 60: /* 1280x720p @ 23.97/24Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 2020;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 1760;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 66:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 61: /* 1280x720p @ 25Hz 16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 2680;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 2420;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 67:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 62: /* 1280x720p @ 29.97/30Hz  16:9 */
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 2020;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 1760;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 74250;
			break;
		case 78:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 63: /* 1920x1080p @ 119.88/120Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 280;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 77:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 64: /* 1920x1080p @ 100Hz 16:9 */
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 720;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 528;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 79:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 1620;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 1360;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 80:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 1488;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 1228;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 81:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 960;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 700;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 82:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 520;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 260;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 82500;
			break;
		case 83:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 520;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 260;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 99000;
			break;
		case 84:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 320;
			mdtd->v_blanking = 105;
			mdtd->h_sync_offset = 60;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 165000;
			break;
		case 85:
			mdtd->h_active = 1680;
			mdtd->v_active = 720;
			mdtd->h_blanking = 320;
			mdtd->v_blanking = 105;
			mdtd->h_sync_offset = 60;
			mdtd->v_sync_offset = 5;
			mdtd->h_sync_pulse_width = 40;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 198000;
			break;
		case 86:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 1190;
			mdtd->v_blanking = 20;
			mdtd->h_sync_offset = 998;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 99000;
			break;
		case 87:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 640;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 448;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 90000;
			break;
		case 88:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 960;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 768;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 118800;
			break;
		case 89:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 740;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 548;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 185625;
			break;
		case 90:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 440;
			mdtd->v_blanking = 20;
			mdtd->h_sync_offset = 248;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 198000;
			break;
		case 91:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 410;
			mdtd->v_blanking = 170;
			mdtd->h_sync_offset = 218;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 371250;
			FALLTHROUGH_INTENDED;
		case 92:
			mdtd->h_active = 2560;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 740;
			mdtd->v_blanking = 170;
			mdtd->h_sync_offset = 548;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 495000;
			break;
		case 101:
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1184;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 968;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 59400;
			break;
		case 100:
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 304;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 99:
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1184;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 968;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 102:
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 304;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 103:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 93:		/* 4k x 2k, 30Hz */
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1660;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 1276;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 104:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 94:
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1440;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 1056;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 105:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 95:
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 560;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 176;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		case 106:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 96:
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1440;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 1056;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 107:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			FALLTHROUGH_INTENDED;
		case 97:
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 560;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 176;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 594000;
			break;
		case 98:
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 1404;
			mdtd->v_blanking = 90;
			mdtd->h_sync_offset = 1020;
			mdtd->v_sync_offset = 8;
			mdtd->h_sync_pulse_width = 88;
			mdtd->v_sync_pulse_width = 10;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;
			mdtd->pixel_clock = 297000;
			break;
		default:
			return false;
		}
	} else if (video_format == CVT) {
		switch (code) {
		case 1:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 640;
			mdtd->v_active = 480;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 20;
			mdtd->h_sync_offset = 8;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 8;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 23750;
			break;
		case 2:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 800;
			mdtd->v_active = 600;
			mdtd->h_blanking = 224;
			mdtd->v_blanking = 24;
			mdtd->h_sync_offset = 31;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 81;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 38250;
			break;
		case 3:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1024;
			mdtd->v_active = 768;
			mdtd->h_blanking = 304;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 104;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 63500;
			break;
		case 4:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 960;
			mdtd->h_blanking = 416;
			mdtd->v_blanking = 36;
			mdtd->h_sync_offset = 80;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 101250;
			break;
		case 5:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1400;
			mdtd->v_active = 1050;
			mdtd->h_blanking = 464;
			mdtd->v_blanking = 39;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 144;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 121750;
			break;
		case 6:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1600;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 560;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 112;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 68;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 161000;
			break;
		case 12:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 1024;
			mdtd->h_blanking = 432;
			mdtd->v_blanking = 39;
			mdtd->h_sync_offset = 80;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 136;
			mdtd->v_sync_pulse_width = 7;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 109000;
			break;
		case 13:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 768;
			mdtd->h_blanking = 384;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 7;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 79500;
			break;
		case 16:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 720;
			mdtd->h_blanking = 384;
			mdtd->v_blanking = 28;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 74500;
			break;
		case 17:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1360;
			mdtd->v_active = 768;
			mdtd->h_blanking = 416;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 72;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 136;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 84750;
			break;
		case 20:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 656;
			mdtd->v_blanking = 40;
			mdtd->h_sync_offset = 128;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 200;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 173000;
			break;
		case 22:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 2560;
			mdtd->v_active = 1440;
			mdtd->h_blanking = 928;
			mdtd->v_blanking = 53;
			mdtd->h_sync_offset = 192;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 272;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 312250;
			break;
		case 28:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 800;
			mdtd->h_blanking = 400;
			mdtd->v_blanking = 31;
			mdtd->h_sync_offset = 72;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 83500;
			break;
		case 34:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 672;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 136;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 200;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 193250;
			break;
		case 38:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 3840;
			mdtd->v_active = 2400;
			mdtd->h_blanking = 80;
			mdtd->v_blanking = 69;
			mdtd->h_sync_offset = 320;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 424;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 580128;
			break;
		case 40:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1600;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 35;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 124076;
			break;
		case 41:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 2048;
			mdtd->v_active = 1536;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 44;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 208000;
			break;
		default:
			return false;
		}
	} else if (video_format == DMT) {
		switch (code) {
		case 1: // HISilicon timing
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 3600;
			mdtd->v_active = 1800;
			mdtd->h_blanking = 120;
			mdtd->v_blanking = 128;
			mdtd->h_sync_offset = 20;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 20;
			mdtd->v_sync_pulse_width = 2;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 645500;
			break;
		case 2:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 3840;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 62;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 533000;
			break;
		case 4:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 640;
			mdtd->v_active = 480;
			mdtd->h_blanking = 144;
			mdtd->v_blanking = 29;
			mdtd->h_sync_offset = 8;
			mdtd->v_sync_offset = 2;
			mdtd->h_sync_pulse_width = 96;
			mdtd->v_sync_pulse_width = 2;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			//mdtd->pixel_clock = 25175;
			mdtd->pixel_clock = 25200;
			break;
		case 13:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 800;
			mdtd->v_active = 600;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 36;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 73250;
			break;
		case 14: /* 848x480p@60Hz */
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 848;
			mdtd->v_active = 480;
			mdtd->h_blanking = 240;
			mdtd->v_blanking = 37;
			mdtd->h_sync_offset = 16;
			mdtd->v_sync_offset = 6;
			mdtd->h_sync_pulse_width = 112;
			mdtd->v_sync_pulse_width = 8;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI)  */;
			mdtd->pixel_clock = 33750;
			break;
		case 22:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 768;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 22;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 7;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 68250;
			break;
		case 35:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 1024;
			mdtd->h_blanking = 408;
			mdtd->v_blanking = 42;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 112;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 108000;
			break;
		case 39:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1360;
			mdtd->v_active = 768;
			mdtd->h_blanking = 432;
			mdtd->v_blanking = 27;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 112;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 85500;
			break;
		case 40:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1360;
			mdtd->v_active = 768;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 148250;
			break;
		case 81:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1366;
			mdtd->v_active = 768;
			mdtd->h_blanking = 426;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 70;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 142;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 85500;
			break;
		case 86:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1366;
			mdtd->v_active = 768;
			mdtd->h_blanking = 134;
			mdtd->v_blanking = 32;
			mdtd->h_sync_offset = 14;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 56;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 72000;
			break;
		case 87:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 80;
			mdtd->v_blanking = 62;
			mdtd->h_sync_offset = 8;
			mdtd->v_sync_offset = 48;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 8;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 556744;
			break;
		case 88:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 4096;
			mdtd->v_active = 2160;
			mdtd->h_blanking = 80;
			mdtd->v_blanking = 62;
			mdtd->h_sync_offset = 8;
			mdtd->v_sync_offset = 48;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 8;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 556188;
			break;
		case 41:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1400;
			mdtd->v_active = 1050;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 101000;
			break;
		case 42:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1400;
			mdtd->v_active = 1050;
			mdtd->h_blanking = 464;
			mdtd->v_blanking = 39;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 144;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 121750;
			break;
		case 46:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1440;
			mdtd->v_active = 900;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 26;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 88750;
			break;
		case 47:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1440;
			mdtd->v_active = 900;
			mdtd->h_blanking = 464;
			mdtd->v_blanking = 34;
			mdtd->h_sync_offset = 80;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 152;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 106500;
			break;
		case 51:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1600;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 560;
			mdtd->v_blanking = 50;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 192;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 162000;
			break;
		case 57:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1680;
			mdtd->v_active = 1050;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 119000;
			break;
		case 58:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1680;
			mdtd->v_active = 1050;
			mdtd->h_blanking = 560;
			mdtd->v_blanking = 39;
			mdtd->h_sync_offset = 104;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 176;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 146250;
			break;
		case 68:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 35;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 154000;
			break;
		case 69:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1200;
			mdtd->h_blanking = 672;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 136;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 200;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 193250;
			break;
		case 82:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1080;
			mdtd->h_blanking = 280;
			mdtd->v_blanking = 45;
			mdtd->h_sync_offset = 88;
			mdtd->v_sync_offset = 4;
			mdtd->h_sync_pulse_width = 44;
			mdtd->v_sync_pulse_width = 5;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 148500;
			break;
		case 83:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1600;
			mdtd->v_active = 900;
			mdtd->h_blanking = 200;
			mdtd->v_blanking = 100;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 80;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 108000;
			break;
		case 9:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 800;
			mdtd->v_active = 600;
			mdtd->h_blanking = 256;
			mdtd->v_blanking = 28;
			mdtd->h_sync_offset = 40;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 4;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 40000;
			break;
		case 16:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1024;
			mdtd->v_active = 768;
			mdtd->h_blanking = 320;
			mdtd->v_blanking = 38;
			mdtd->h_sync_offset = 24;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 136;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 65000;
			break;
		case 23:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 768;
			mdtd->h_blanking = 384;
			mdtd->v_blanking = 30;
			mdtd->h_sync_offset = 64;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 128;
			mdtd->v_sync_pulse_width = 7;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 79500;
			break;
		case 62:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active =  1792;
			mdtd->v_active = 1344;
			mdtd->h_blanking = 656;
			mdtd->v_blanking =  50;
			mdtd->h_sync_offset = 128;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 200;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0; /* (progressive_nI) */
			mdtd->pixel_clock = 204750;
			break;
		case 32:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 960;
			mdtd->h_blanking = 520;
			mdtd->v_blanking = 40;
			mdtd->h_sync_offset = 96;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 112;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;  /* (progressive_nI) */
			mdtd->pixel_clock = 108000;
			break;
		case 73:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1920;
			mdtd->v_active = 1440;
			mdtd->h_blanking = 680;
			mdtd->v_blanking = 60;
			mdtd->h_sync_offset = 128;
			mdtd->v_sync_offset = 1;
			mdtd->h_sync_pulse_width = 208;
			mdtd->v_sync_pulse_width = 3;
			mdtd->h_sync_polarity = 0;
			mdtd->v_sync_polarity = 1;
			mdtd->interlaced = 0;  /* (progressive_nI) */
			mdtd->pixel_clock = 234000;
			break;
		case 27:
			mdtd->h_image_size = 4;
			mdtd->v_image_size = 3;
			mdtd->h_active = 1280;
			mdtd->v_active = 800;
			mdtd->h_blanking = 160;
			mdtd->v_blanking = 23;
			mdtd->h_sync_offset = 48;
			mdtd->v_sync_offset = 3;
			mdtd->h_sync_pulse_width = 32;
			mdtd->v_sync_pulse_width = 6;
			mdtd->h_sync_polarity = 1;
			mdtd->v_sync_polarity = 0;
			mdtd->interlaced = 0;  /* (progressive_nI) */
			mdtd->pixel_clock = 71000;
			break;
		default:
			return false;
		}
	}
	return true;
}
